/*
 * Action definition used by Diagram to set Actions as function(parametres) on source(Object) 
 */
package com.jdcm.common;

import java.util.List;
import java.util.ArrayList;

/**
 *
 * @author Marc GIBOIN
 */
public class C_Actiondef {

    private java.lang.reflect.Method method;
    private final D_Utils TUtils = new D_Utils();
    private Object dest = null;
    private List<Object> parametres = null;///should be a list of object
    private String destName = null;
    private String methodName = null;
    private Object source = null;

    public C_Actiondef() {
        methodName = null;
        dest = null;
        method = null;
        destName = null;
        parametres = new ArrayList<>();
    }

    public C_Actiondef(Object destobj, java.lang.reflect.Method meth, Object par) {
        parametres = getParameters(par);
        method = meth;
        if (destobj instanceof String) {
            destName = destobj.toString();
        } else if (destobj instanceof Object) {
            dest = destobj;
            destName = dest.toString();
        }
    }

    public C_Actiondef(Object destobj, Object meth, List<Object> pars, Object src) {
        // for clone
        parametres = pars;
        if (meth instanceof java.lang.reflect.Method) {
            method = (java.lang.reflect.Method) meth;
        } else {
            methodName = meth.toString();
        }
        if (destobj instanceof String) {
            destName = destobj.toString();
        } else if (destobj instanceof Object) {
            dest = destobj;
            destName = dest.toString();
        }
        source = src;
    }

    public C_Actiondef(String destobj, String meth, Object par) {
        methodName = meth;
        dest = null;
        method = null;
        destName = destobj;
        parametres = getParameters(par);
    }

    public C_Actiondef(String destobj, String meth) {
        methodName = meth;
        dest = null;
        method = null;
        destName = destobj;
        parametres = new ArrayList<>();
    }

    public C_Actiondef(Object par) {
        methodName = null;
        dest = null;
        method = null;
        destName = null;
        parametres = getParameters(par);

    }

    public XMLres getXMLformat() {
        XMLres res = new XMLres("C_Actiondef");
        if (dest != null || destName != null) {
            res.addAttrVal("dest");
            if (dest != null) {
                res.setAttrVal("dest", dest);
            } else {
                res.setAttrVal("dest", destName);
            }
        }
        if (method != null || methodName != null) {
            res.addAttrVal("method");
            if (method != null) {
                res.setAttrVal("method", method.getName());
            } else {
                res.setAttrVal("method", methodName);
            }
        }
        if (parametres != null && parametres.isEmpty() == false) {
            res.addAttrVal("parametres");
            if (parametres.size() == 1) {
                res.setAttrVal("parametres", parametres.get(0));
            } else {
                res.setAttrVal("parametres", getParams2string());
            }
        }
        return res;
    }

    /* public void addParameter(Object obj) {
     parametres.add(obj);
     }*/
    public final ArrayList<Object> getParameters(Object obj) {
        ArrayList<Object> res = new ArrayList<>();
        if (obj == null) {
            return res;
        } else if ((obj instanceof String == false || obj.toString().contains(",") == false) && obj instanceof List == false) {
            res.add(obj);
        } else if (obj instanceof ArrayList<?>) {
            res = (ArrayList<Object>) obj;
        } else {
            for (String elt : obj.toString().split(",")) {
                res.add(TUtils.getstr2Obj(elt));
            }
        }
        return res;
    }

    public String getMethodName() {
        return methodName;
    }

    public java.lang.reflect.Method getMethod() {
        return method;
    }

    public Object getDest() {
        if (dest == null) {
            return destName;
        } else {
            return dest;
        }
    }

    public Object getSource() {
        return source;
    }

    public void setSource(Object ob) {
        source = ob;
    }

    public void setDest(Object ob) {
        if (ob instanceof String) {
            destName = ob.toString();
            dest = null;
        } else {
            dest = ob;
            destName = null;
        }
    }

    public Object getParam() {
        if (parametres.size() == 1) {
            return parametres.get(0);
        } else {
            return null;
        }
    }

    public List<Object> getParamList() {
        return parametres;
    }

    private String getParams2string() {
        String p = "";
        if (parametres == null || parametres.isEmpty()) {
            p = "";
        } else if (parametres.size() == 1) {
            p = TUtils.getObj2str(parametres.get(0));
        } else {
            for (Object ob : parametres) {
                if (ob != null) {
                    p = p + TUtils.getObj2str(ob) + ",";
                } else {
                    p = p + ",";
                }
            }
            p = p.substring(0, p.length() - 1);
        }
        return p;
    }

    @Override
    public String toString() {

        if (getMethod() != null) {
            return "=>" + getDest() + "." + getMethod().getName() + "(" + getParams2string() + ")";
        } else {
            if (this.getMethodName() != null) {
                return "=>" + getDest() + "." + getMethodName() + "(" + getParams2string() + ")";
            } else {
                if (parametres == null || parametres.isEmpty()) {
                    return "=>Empty";
                }
                return "=>[" + parametres + "]";
            }
        }
    }

    @Override
    public C_Actiondef clone() {
        Object d = dest;
        Object m = method;
        if (d == null) {
            d = destName;
        }
        if (m == null) {
            m = methodName;
        }
        List<Object> pars = new ArrayList<>();
        for (Object o : parametres) {
            pars.add(o);
        }
        return new C_Actiondef(d, m, pars, source);
    }
}
