/*
 * P_Packet.java
 *
 * Created on 20 fevrier 2007, 22:16
 *
 */
package com.jdcm.common;

import com.jdcm.analyse.Code_Model;
import com.jdcm.eltrx.*;
import java.io.IOException;
import java.util.*;

/**
 * Packet used to carry object from ports to ports<br>
 *
 * @author Marc GIBOIN
 */
public class C_Packet implements Cloneable, java.io.Serializable {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private Object contenu = null;
    private byte[] buffserial = null;
    private boolean ismarked = false;
    private Date createtime = null;
    // private Date WItime=null;
    private Date WOtime = null;
    // private Date ROtime=null;
    private Date Storetime = null;
    private String SQL_req = null;
    private int SQL_upd = -1;
    private Date SQL_start = null;
    private Date SQL_end = null;
    private I_Port source = null;
    private I_Port destination = null;
    private boolean scanDir;
    //should be a tree? of protocols
   
    private javax.swing.tree.DefaultMutableTreeNode protoTree = null;
    // store if packet is getting up 
    private boolean protDir;
    private long Pwtime;
    //use to keep src& dest reference afterb transit=>store...
    private int store_srcidx = -1;
    private int store_dstidx = -1;
    private int store_pos = -1;
    //
    // true=> all the row is displayed in the terminal; false => 16 fist bytes
    private boolean allrow;

    /**
     * Creates a new instance of P_Packet
     * 
     * @param obj the contant of it
     * @param src the port that emited this packet
     */
    public C_Packet(Object obj, I_Port src) {
        source = src;
        contenu = obj;
        createtime = new Date(System.currentTimeMillis());
    }

    public C_Packet(Object obj) {
        contenu = obj;
        createtime = new Date(System.currentTimeMillis());
    }

    public C_Packet(Object obj, int storeidx) {
        store_srcidx = storeidx;
        contenu = obj;
        createtime = new Date(System.currentTimeMillis());
    }

    public void setSQL(String req, Date start, Date end) {
        SQL_req = req;
        SQL_start = start;
        SQL_end = end;
    }

    public void setSQL_upd(int i) {
        SQL_upd = i;
    }

    public int getSQL_upd() {
        return SQL_upd;
    }

    public String getSQLReq() {
        return SQL_req;
    }

    public void setSQLReq(String txt) {
        SQL_req = txt;
    }

    public boolean ismarked() {
        return ismarked;
    }

    public void setmarked(boolean bool) {
        ismarked = bool;
    }

    public boolean isEmpty() {
        return contenu == null;
    }

    public Date getSQLstart() {
        return SQL_start;
    }

    public void setSQLstart(Date dt) {
        SQL_start = dt;
    }

    public int getSQLdelay() {
        if (SQL_start == null || SQL_end == null) {
            return -1;
        }
        return (int) (SQL_end.getTime() - SQL_start.getTime());
    }

    public void setSQLdelay(int i) {
        SQL_end.setTime(SQL_start.getTime() + i);
    }

    /**
     *
     * @return the time the packet is waiting (delay/bandwidth)
     */
    public long get_Pwaitedtime() {
        return Pwtime;
    }

    public boolean getPsens() {
        return scanDir;
    }

    public boolean getStacksens() {
        return protDir;
    }

    public void setStacksens(boolean bool) {
        protDir = bool;
    }
    

    /**
     * 
     * @return the tree of protocols detected in a packet
     */
    public javax.swing.tree.DefaultMutableTreeNode getProtoTree() {
        return protoTree;
    }

    public void setProtoTree(javax.swing.tree.DefaultMutableTreeNode prototree) {
        protoTree = prototree;
    }
    /**
     * For Stack detecting, get the last Node Path as list
     * @return the list of protocol of the last Path
     */
        public java.util.List<com.jdcm.modifiers.stacks.I_stack_elt> getProtocols() {
      if (protoTree==null||protoTree.getLastLeaf()==null){
          return null;
      }
        Object[] os= protoTree.getLastLeaf().getUserObjectPath();
       java.util.List<com.jdcm.modifiers.stacks.I_stack_elt> res=new  java.util.ArrayList<>();
       for (Object o:os){
           if (o instanceof com.jdcm.modifiers.stacks.I_stack_elt ){
               res.add((com.jdcm.modifiers.stacks.I_stack_elt)o);
           }
       }
       return res;
    }
    /*
     * add a protocol at the end of the last leaf for Stack auto-detect
     */
    public void addProtocol(com.jdcm.modifiers.stacks.I_stack_elt p) {
        if (protoTree==null){
            protoTree=new javax.swing.tree.DefaultMutableTreeNode("frame");
        }
            javax.swing.tree.DefaultMutableTreeNode last=protoTree.getLastLeaf();
             last.add(new javax.swing.tree.DefaultMutableTreeNode(p));
    }

    public boolean getallrow() {
        return allrow;
    }

    public void setallrow(boolean bool) {
        allrow = bool;
    }

    /**
     * use o store the packet and free it after a delai
     *
     * @param p the PSCAN to use to free the packet
     * @param s the way of the packet (in or out port) true is comming from
     * interface
     * @param Ti the asked wainting time
     */
    public void setdelayed(C_PScan p, boolean s, long Ti) {
        scanDir = s;
        Pwtime = Ti;
        final C_PScan pscan = p;
        final C_Packet pac = this;
        setStoredtime(new Date(System.currentTimeMillis()));
        Timer timerstor = new Timer();
        timerstor.schedule(new TimerTask() {
            @Override
            public void run() {
                pscan.delayed_free(pac);
            }
        }, Pwtime);
    }

    public int getsourceidx() {
        if (source != null) {
            return source.getIndex();
        } else {
            return store_srcidx;
        }
    }

    public I_Port getsource() {
        return source;
    }

    public void setsource(I_Port port) {
        source = port;
    }

    public void setstore_srcidx(int portidx) {
        store_srcidx = portidx;
    }

    public void setdestination(I_Port dst) {
        destination = dst;
    }

    public void setstore_pos(int pos) {
        store_pos = pos;
    }

    public int getstore_pos() {
        return store_pos;
    }

    public int getdestidx() {
        if (destination != null) {
            return destination.getIndex();
        } else {
            return store_dstidx;
        }
    }

    public I_Port getdestination() {
        return destination;
    }

    public void setstore_dstidx(int i) {
        store_dstidx = i;
    }

    public Date getCreatetime() {
        return createtime;
    }

    public void setCreatetime(Date dt) {
        createtime = dt;
    }

    public boolean isbuffer() {
        return (contenu instanceof byte[]);
    }

    public Object getObj() {
        return contenu;
    }

    public void setObj(Object obj) {
        contenu = obj;
    }

    public void setWOtime(Date t) {
        WOtime = t;
    }

    public void setStoredtime(Date t) {
        Storetime = t;
    }

    public java.util.Date getStoredtime() {
        return Storetime;
    }

    public void store() {
        if (source != null) {
            store_srcidx = source.getIndex();
            source = null;
        }
        if (destination != null) {
            store_dstidx = destination.getIndex();
            destination = null;
        }
    }

    /**
     * provide the printable length of the packet content Byte => lenght() else
     * -1
     *
     * @return the printable lenght
     */
    public int length() {
        if (contenu == null) {
            return 0;
        }
        if (isbuffer()) {
            return getBytes().length;
        }
        if (contenu instanceof Code_Model) {
            return ((Code_Model) contenu).bufferSize();
        }
        try {
            if (contenu instanceof jpcap.packet.Packet) {
                return ((jpcap.packet.Packet) contenu).len;
            }
        } catch (java.lang.NoClassDefFoundError ex) {
        }
        if (this.getSerial() != null) {
            return getSerial().length;
        }

        return -1;
    }

    /**
     * provide Byte array of the packet (if it is)
     *
     * @return the Byte array contant or null
     */
    public byte[] getBytes() {
        if (contenu instanceof byte[]) {
            return (byte[]) contenu;
        }
        return null;
    }

    /**
     * Provide Byte array of the packet contant if serializable
     *
     * @return content object serialized or null
     */
    public byte[] getSerial() {
        if (contenu instanceof java.io.Serializable) {
            //  BytesOutStream
            BytesOutStream str = new BytesOutStream();
            try {
                try (java.io.ObjectOutputStream oo = new java.io.ObjectOutputStream(str)) {
                    oo.writeObject(contenu);
                    oo.flush();
                }
                buffserial = str.toBytes();
            } catch (IOException ex) {
                buffserial = null;
            }
        }
        return buffserial;
    }

    /**
     * provide Object corresponding to serialisation (byte[])
     *
     * @return the serialized Object or null
     */
    public Object getSer2Obj() {
        //r; local class incompatible: stream classdesc serialVersionUID =  => TO SOLVE !!
        byte[] buff = getBytes();
        Object o = null;
        if (buff != null) {
            if (buff.length > 3) {
                if (buff[0] == -84 //0xac
                        && buff[1] == -19 //xed
                        && buff[2] == 0 && buff[3] == 5) {
                    try {
                        BytesInStream fi = new BytesInStream(buff);
                        java.io.ObjectInputStream oi = new java.io.ObjectInputStream(fi);
                        o = oi.readObject();
                    } catch (IOException | ClassNotFoundException ex) {
                        System.err.println("Serial=>Object error : " + ex.getMessage());
                        return null;
                    }
                }
            }
        }
        return o;
    }

    /**
     * provide a Packet Description
     *
     * @return the label of the Packet
     */
    @Override
    public String toString() {
        if (contenu == null) {
            return "null";
        }
        if (contenu instanceof byte[]) {
            return "{Byte[" + getBytes().length + "]}";
        }
        if (contenu instanceof java.sql.ResultSet) {
            return "{SQL result}";
        }
        if (contenu instanceof Code_Model) {
            return "{Protocol result}";
        }
        try {
            if (contenu.getClass().getName().startsWith("jpcap.packet.")) {
                return "{EthPacket}";
            }
        } catch (java.lang.NoClassDefFoundError er) {
        }
        // !!!!!!!!!!!packet!=contenu!!!
        return contenu.toString();
    }

    public int getTransittime() {
        if (WOtime == null || createtime == null) {
            return -1;
        }
        int res = (int) (WOtime.getTime() - createtime.getTime());
        return res;
    }

    public void setTransittime(int i) {
        if (WOtime == null || createtime == null) {
            return;
        }
        WOtime.setTime(createtime.getTime() + i);
    }

    /**
     * create a new C_Packet with same contact/src/...
     * @return
     */
    @Override
    public Object clone() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException e) {
            throw new InternalError(e.getMessage());
        }
    }
}
