package com.jdcm.common;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import java.util.ArrayList;
import java.util.List;
import java.util.Collection;

/**
 * model with modulable cells to add/remove/update data
 * 
 * @author Marc GIBOIN
 */
public class ListTableModel extends AbstractTableModel {

    public boolean isvalidated = false;
    private static final long serialVersionUID = 1L;
    String[] colonnes;
    public boolean ishdrcol = false;
    //la liste du contenu du tableau
    private Collection<cellDataModel[]> Lignes = new ArrayList<>();
    javax.swing.table.AbstractTableModel srcmodel = null;

    // private boolean[] iscoleditable = new boolean[0];
    public ListTableModel(Collection<Object[]> lignes, Collection<String> cols) {
        //test si la liste est vide rajouter une ligne vide
        if (cols != null) {
            colonnes = cols.toArray(new String[0]);
        }
        //---------------idem setcontants but....-----------
        for (Object[] row : lignes) {
            cellDataModel[] managedrow = new cellDataModel[row.length];
            for (int col = 0; col < row.length; col++) {
                managedrow[col] = new cellDataModel(row[col]);
            }
            Lignes.add(managedrow);
        }
        this.fireTableDataChanged();
        init();
    }

    public ListTableModel(java.util.Enumeration<?> enumer, String title) {
        //test si la liste est vide rajouter une ligne vide
        colonnes = new String[]{title};
        while (enumer.hasMoreElements()) {
            Lignes.add(new cellDataModel[]{new cellDataModel(enumer.nextElement())});
        }
        init();
    }

    public ListTableModel(javax.swing.table.AbstractTableModel model) {
        model.addTableModelListener(new javax.swing.event.TableModelListener()  {

            @Override
            public void tableChanged(TableModelEvent e) {
                majModel();
                throw new UnsupportedOperationException("Not supported yet.");
            }
        });
        srcmodel = model;
        majModel();
        init();
    }

    private void init() {
    }

    public final void majModel() {
        if (srcmodel == null) {
            return;
        }
        colonnes = new String[srcmodel.getColumnCount()];
        for (int i = 0; i < srcmodel.getColumnCount(); i++) {
            colonnes[i] = srcmodel.getColumnName(i);
        }
        for (int i = 0; i < srcmodel.getRowCount(); i++) {
            cellDataModel[] cells = new cellDataModel[srcmodel.getColumnCount()];
            for (int j = 0; j < cells.length; j++) {
                cells[j] = new cellDataModel(srcmodel.getValueAt(i, j));
            }
            Lignes.add(cells);
        }
        this.fireTableStructureChanged();
    }

    public void commit() {
        for (cellDataModel[] cells : Lignes) {
            for (cellDataModel cell : cells) {
                if (cell.Status == cellDataModel.ADDED
                        || cell.Status == cellDataModel.CHANGED) {
                    cell.Status = cellDataModel.NONE;
                }
            }
        }
        ArrayList<cellDataModel[]> al = new ArrayList<>(Lignes);
        int i = 0, min = -1, max = -1;
        while (i < al.size()) {
            cellDataModel[] cells = al.get(i);
            if (cells[0].Status == cellDataModel.REMOVED) {
                al.remove(cells);
                if (min < 0) {
                    min = i;
                } else {
                    max = i;
                }
            } else {
                i++;
            }
        }
        Lignes = al;
        this.fireTableRowsDeleted(min, max);
    }

    public Collection<Object[]> getContents() {
        List<Object[]> res = new ArrayList<>();
        for (cellDataModel[] managedrow : Lignes) {
            Object[] objs = new Object[managedrow.length];
            for (int i = 0; i < objs.length; i++) {
                objs[i] = managedrow[i].newValue;
            }
            res.add(objs);
        }
        return res;
    }

    public void setContent(Collection<Object[]> lignes) {
        for (Object[] row : lignes) {
            cellDataModel[] managedrow = new cellDataModel[row.length];
            for (int col = 0; col < row.length; col++) {
                managedrow[col] = new cellDataModel(row[col]);
            }
            Lignes.add(managedrow);
        }
        this.fireTableDataChanged();
    }

    /**
     * set colones editables
     * @param bools array with editable column set to true
     */
    public void setEditable(boolean[] bools) {
        for (cellDataModel[] mods : Lignes) {
            for (int i = 0; i < mods.length; i++) {
                if (i < bools.length) {
                    mods[i].isEditable = bools[i];
                } else {
                    mods[i].isEditable = false;
                }
            }
        }
    }
    /* retourne le nombre d'element de la colonne */

    @Override
    public int getColumnCount() {
        if (colonnes != null) {
            return colonnes.length;
        } else if (Lignes != null && Lignes.size() > 0) {
            return ((Object[]) Lignes.toArray()[0]).length;
        } else {
            return 0;
        }
    }

    /* retourne le nombre de lignes */
    @Override
    public int getRowCount() {
        if (Lignes != null) {
            return Lignes.size();
        } else {
            return 0;
        }
    }

    /* retourne le nom de la colonne */
    @Override
    public String getColumnName(int col) {
        if (colonnes != null && col < colonnes.length) {
            return colonnes[col];
        } else {
            return "Col_" + col;
        }
    }

    /* retourne la valeur à la ligne row et colonne col */
    @Override
    public Object getValueAt(int row, int col) {
        if (Lignes != null && row < Lignes.size()) {
            cellDataModel[] r = (cellDataModel[]) Lignes.toArray()[row];
            if (col < r.length) {
                if (isvalidated) {
                    return r[col];
                } else {
                    return r[col].newValue;
                }
            } else {
                return "no column :" + col + " in " + Lignes.toString();
            }
        } else {
            return "no line :" + row + " in " + Lignes.toString();
        }
    }

    @Override
    public Class<?> getColumnClass(int c) {
        if (getValueAt(0, c) != null) {
            return getValueAt(0, c).getClass();
        } else {
            return null;
        }
    }

    /* definie les cellules éditables ou non */
    @Override
    public boolean isCellEditable(int row, int col) {
        if (Lignes != null && row < Lignes.size()) {
            cellDataModel[] r = (cellDataModel[]) Lignes.toArray()[row];
            if (col < r.length) {
                return r[col].isEditable;
            }
        }
        return false;
    }

    /* permet de modifier l'element en placant value a la ligne row
    colonne col */
    @Override
    public void setValueAt(Object value, int row, int col) {
        if (Lignes != null && row < Lignes.size()) {
            cellDataModel[] r = (cellDataModel[]) Lignes.toArray()[row];
            if (col < r.length) {
                r[col].setNewValue(value);
            }
        }
    }

    public void insertRow() {
        insertRow(new Object[getColumnCount()]);
    }

    public void insertRow(Object[] objs) {
        insertRow(objs, Lignes.size());
    }

    public void insertRow(Object[] objs, int pos) {
        // position SHOULD BE MANAGED...
        cellDataModel[] newRow = new cellDataModel[objs.length];
        for (int i = 0; i < objs.length; i++) {
            newRow[i] = new cellDataModel(objs[i]);
            newRow[i].add();
        }
        ((ArrayList<cellDataModel[]>) Lignes).add(pos, newRow);// HERE !collection!
        this.fireTableDataChanged();
    }

    public void removeRows(int[] rows) {
        if (rows == null || rows.length == 0) {
            return;
        }
        int lost = 0;
        java.util.Arrays.sort(rows);
        for (int row : rows) {
            cellDataModel[] mods = (cellDataModel[]) Lignes.toArray()[row - lost];
            if (isvalidated) {
                for (cellDataModel mod : mods) {
                    mod.remove();
                }
            } else {
                Lignes.remove(mods);
                lost++;
            }
            this.fireTableDataChanged();
        }
    }

    public int getCellStatus(int row, int col) {
        if (Lignes != null && row < Lignes.size()) {
            cellDataModel[] r = (cellDataModel[]) Lignes.toArray()[row];
            if (col < r.length) {
                return r[col].Status;
            }
        }
        return 0;
    }
}
