package com.jdcm.common;

import java.awt.event.MouseEvent;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import javax.swing.DefaultComboBoxModel;

/**
 * Option Model is an List Table linked with a Parent Object - special edition
 * (used/nullOnEmpty/default/units/desc) - setValue=> check and execute "set"
 * function on Parent get/set values are for a displayed table. Values are in
 * super.get/set...
 *
 * @author Marc GIBOIN
 */
public class Options_Model extends ListTableModel {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private final D_Utils TUtil = new D_Utils();
    private static class OM_Label extends javax.swing.JLabel {

        private static final long serialVersionUID = 1L;

        public OM_Label() {
        }
    }
    private Object Parent;
    
    private String title = null;

    public Options_Model(Object p) {
        super(new ArrayList<Object[]>(), java.util.Arrays.asList(new String[]{"Option", "Value", "Default", "Type", "Range", "Editable", "NullIfEmpty", "Comment"}));
        Parent = p;
        if (p != null && p.getClass() != null) {
            // addOption("|Methodes", new DefaultComboBoxModel(p.getClass().getMethods()), "methods", "", true, false, "object methods");
            // addOption("|Field", new DefaultComboBoxModel(p.getClass().getFields()), "fields", "", true, false, "object fields");
            //addOption("|Args", "", "Objects", "", true, false, "object Arguments");
        }
    }

    private int getShowedCol(int i) {
        if (i == 3) {
            //the column showed as third
            return 5;
        }
        return i;
    }

    @Override
    public String getColumnName(int col) {
        return super.getColumnName(getShowedCol(col));
    }

    @Override
    public int getColumnCount() {
        return 2;
    }

    public void changeParent(Object p) {
        Parent = p;
        if (this.getValuefor("|Methodes") != null) {
            setValuefor("|Methodes", new DefaultComboBoxModel<>(p.getClass().getMethods()));
        }
        if (this.getValuefor("_Field") != null) {
            setValuefor("|Field", new DefaultComboBoxModel<>(p.getClass().getFields()));
        }
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String txt) {
        //TODO : need to fire Editors ??
        title = txt;
    }

    /**
     * discover the Parent Object by searching Methods/Fields/... and add it to
     * Options_Model()
     */
    public void autoDiscover() {
        if (Parent == null) {
            addOption("Object", null, "null", "", true, false, "");
            return;
        }
        Class<?> cl = Parent.getClass();
        for (java.lang.reflect.Method m : cl.getMethods()) {
            String name = null;
            if (m.getName().startsWith("get")) {
                name = m.getName().substring(3);
            } else if (m.getName().startsWith("is")) {
                name = m.getName().substring(2);
            }
            if (name != null) {
                if (findRowfor(name) < 0) {
                    if (m.getReturnType() != null && (m.getParameterTypes() == null || m.getParameterTypes().length == 0)) {
                        String type = m.getReturnType().getSimpleName();
                        Object val = null;
                        boolean performed = true;
                        try {
                            val = m.invoke(Parent, (Object) null);
                        } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
                            performed = false;
                        }
                        if (performed) {
                            addOption(name, val, type, "", true, false, "");
                        }
                    }

                }
            }
        }
    }

    /**
     * Used for TCP listner to configure connecting TCP ports
     *
     * @return the list of changed options
     */
    public List<List<Object>> getChangedOptions() {
        Collection<Object[]> cont = getContents();
        List<List<Object>> res = new ArrayList<>();
        for (Object[] objs : cont) {
            if (objs.length > 2 && objs[1] != null) {
                if (isDefault(objs[0].toString()) == false) {
                    res.add(java.util.Arrays.asList(objs));
                }
            }
        }
        return res;
    }

    /**
     * Check if, for an option line, the value is equal to the stored default
     * value this is a .equal() managing ComboBox SelectedValue() items
     *
     * @param Opt_label the label to test
     * @return true if default value is 'equal' to set value
     */
    public boolean isDefault(String Opt_label) {
        int idx = findRowfor(Opt_label);
        if (idx < 0) {
            // should be an exceptio

            return false;
        }
        Object val = super.getValueAt(idx, 1);
        Object stored = super.getValueAt(idx, 2);
        if (val == null) {
            return stored == null;
        }
        if (val instanceof javax.swing.ComboBoxModel && stored instanceof javax.swing.ComboBoxModel) {
            Object tmp2 = ((javax.swing.ComboBoxModel) stored).getSelectedItem();
            Object tmp1 = ((javax.swing.ComboBoxModel) val).getSelectedItem();
            if (tmp1 != null) {
                return tmp1.equals(tmp2);
            } else {
                return tmp2 == null;
            }
        } else {
            return val.equals(stored);
        }

    }

    /**
     * Set all the OptionModel values as defaluts values
     */
    public void setInitials() {
        for (int i = 0; i < this.getRowCount(); i++) {
            setValueAt(getStoreElement(getValueAt(i, 1)), i, 2);
        }
    }

    /**
     * reset all values with stored values
     */
    private void setDefault(int row) {
        setValueAt(getStoreElement(getValueAt(row, 2)), row, 1);
    }

    /**
     * reset a values with stored values
     * 
     * @param opt_label the label to swap stored default as active value
     */
    public void setDefault(String opt_label) {
        int idx = this.findRowfor(opt_label);
        if (idx >= 0) {
            setDefault(idx);
        }
    }

    /**
     * Reset all values with stored values
     */
    public void setDefaults() {
        for (int i = 0; i < this.getRowCount(); i++) {
            setDefault(i);
        }
    }

    /**
     * should be a clone() of the element to get/set store element Only
     * ComboBoxModel implemented
     *
     * @param val the element to get from/to store
     * @return the clone element
     */
    public Object getStoreElement(Object val) {
        if (val instanceof javax.swing.ComboBoxModel<?>) {
            // Clone the combobox to remember selected item & starting elements
            javax.swing.ComboBoxModel<Object> init = (javax.swing.ComboBoxModel<Object>) val;
            java.util.ArrayList<Object> list = new java.util.ArrayList<>();
            for (int idx = 0; idx < init.getSize(); idx++) {
                list.add(init.getElementAt(idx));
            }
            javax.swing.ComboBoxModel<Object> cb = new javax.swing.DefaultComboBoxModel<>(list.toArray(new Object[]{}));
            cb.setSelectedItem(init.getSelectedItem());
            return cb;
        } else {
            return val;
        }
    }
    /*
     * unused
     */

    public void setOptions(List<List<Object>> r) {
        List<Object[]> cont = new ArrayList<>();
        for (List<Object> list : r) {
            cont.add(list.toArray());
        }
        setContent(cont);
    }

    /**
     * remove an input from this option model
     *
     * @param opt_label
     */
    public void removeOption(String opt_label) {
        for (int i = 0; i < this.getRowCount(); i++) {
            if (super.getValueAt(i, 0).toString().compareTo(opt_label) == 0) {
                this.removeRows(new int[]{i});
            }
        }
    }

    public void addOption(String opt_label, Object val, String unit, String range, boolean used, boolean nullonempty, String comment) {
        //If existing => modify
        Object[] newrow;
        boolean found = false;
        for (int i = 0; i < this.getRowCount(); i++) {
            if (super.getValueAt(i, 0).toString().compareTo(opt_label) == 0) {
                setValueAt(val, i, 1);
                setValueAt(getStoreElement(val), i, 2);
                setValueAt(unit, i, 3);
                setValueAt(range, i, 4);
                setValueAt(used, i, 5);
                setValueAt(nullonempty, i, 6);
                setValueAt(comment, i, 7);
                found = true;
            }
        }
        if (found == false) {
            // IF new => create & add in order
            newrow = new Object[]{opt_label, val, val, unit, range, used, nullonempty, comment};
            if (this.getRowCount() == 0) {
                insertRow(newrow);
            } else {
                int point = 0;
                while (super.getValueAt(point, 0).toString().compareTo(newrow[0].toString()) < 0 && point != this.getRowCount() - 1) {
                    point++;
                }
                if (point == this.getRowCount() - 1) {
                    insertRow(newrow, point + 1);
                } else {
                    insertRow(newrow, point);
                }
            }
        }
    }

    /**
     * reurn the row for asked label, if not -1
     *
     * @param opt_label the label of input
     * @return the row or -1 if not found
     */
    private int findRowfor(String opt_label) {
        int idx = -1;
        for (int i = 0; idx < 0 && i < this.getRowCount(); i++) {
            if (super.getValueAt(i, 0).toString().compareTo(opt_label) == 0) {
                idx = i;
            }
        }
        return idx;
    }

    public void setValuefor(String optiontxt, Object value) {
        //System.out.println("Option.setValuefor():"+optiontxt+"=>"+value);
        int idx = findRowfor(optiontxt);
        Object OldObj = getValuefor(optiontxt);
        if (idx == -1) {
            throw new UnsupportedOperationException("setValuefor(): not found '" + optiontxt + "' in " + this.getTitle());
        } else if (value != null && OldObj != null && OldObj.equals(value)) {
            // value didn't change !
            return;
        }
        setValueAt(value, idx, 1);
        this.fireTableCellUpdated(idx, 1);
    }

    /**
     * provide the Value for the label with the asked type
     *
     * @param <U>
     * @param opt_label
     * @param typ
     * @return
     */
    public <U> U getValuefor(String opt_label, U typ) {
        return (U) getobjectfor(opt_label, false);
    }
    public  Object  getValuefor(int row) {
        return  this.getValueAt(row, 1);
    }
    public Object getValuefor(String opt_label) {
        return getobjectfor(opt_label, false);
    }

    public Object getDefaultfor(String opt_label) {
        return getobjectfor(opt_label, true);
    }

    /**
     * test if an option label exist
     *
     * @param opt_label the label in this options
     * @return tru if exists,
     */
    public boolean isPresent(String opt_label) {
        return findRowfor(opt_label) >= 0;
    }

    private Object getobjectfor(String opt_label, boolean isdefault) {
        Object res;
        int idx = findRowfor(opt_label);
        if (idx == -1) {
            throw new UnsupportedOperationException("getValuefor(): not found '" + opt_label + "' in " + this.getTitle());
        }
        int col = 1;
        if (isdefault) {
            col = 2;
        }
        res = super.getValueAt(idx, col);
        if (res != null) {
            // COULD EXISTE A .length interface 'sizable??'
            if (res.getClass().isArray()) {
                boolean b = (Boolean) super.getValueAt(idx, 6);
                if (b && java.lang.reflect.Array.getLength(res) == 0) {
                    res = null;
                }
            } else if (res.equals("") && (Boolean) super.getValueAt(idx, 6)) {
                res = null;

            }
        }
        return res;
    }

    private boolean getused(int rowIndex) {
        if (super.getValueAt(rowIndex, 5) instanceof Boolean) {
            return (Boolean) super.getValueAt(rowIndex, 5);
        }
        return false;
    }

    /**
     * set the label able to be changed in the Option Model
     *
     * @param opt_label the label of the input
     * @param bool true if will be able to be changed
     */
    public void setused(String opt_label, boolean bool) {

        for (int i = 0; i < this.getRowCount(); i++) {
            if (super.getValueAt(i, 0).toString().compareTo(opt_label) == 0) {
                super.setValueAt(bool, i, 5);
            }
        }
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        column = getShowedCol(column);
        return column == 1 && getused(row);
    }

    @Override
    public String toString() {
        String txt = "";
        for (int i = 0; i < this.getRowCount(); i++) {
            if (getValueAt(i, 1) != null) {
                if (getValueAt(i, 1).equals(getValueAt(i, 2)) == false) {
                    txt = txt + super.getValueAt(i, 0) + "=" + this.getValueAt(i, 1);
                }
            }
        }
        return "Option for " + Parent.toString() + " : " + txt;
    }

    /**
     * get the label of this input
     *
     * @param row the row of the model
     * @return the label
     */
    public String getLabelAt(int row) {
        if (row < this.getRowCount() && row >= 0) {
            return super.getValueAt(row, 0).toString();
        } else {
            return null;
        }
    }

    /**
     * get value of the ParentObject or a Managed Lbel for row header (column 0)
     *
     *
     * @param row the row of the model
     * @param column the column 
     * @return the value returned
     */
    @Override
    public Object getValueAt(int row, int column) {
        column = getShowedCol(column);
        Object res = super.getValueAt(row, column);
        if (column == 0) {
            // label Bold if different of default & button to reset value to defalut...???
            //label with comment as Tooltiptop
            OM_Label lbl = new OM_Label();
            if (res != null && super.getValueAt(row, 7) != null) {
                lbl.setText(res.toString());
                lbl.setToolTipText(super.getValueAt(row, 7).toString());
            }
            final Object val = super.getValueAt(row, 1);
            final Object deft = super.getValueAt(row, 2);
            final int r = row;
            if ((val == null && deft != null) || (val != null && isDefault(super.getValueAt(row, 0).toString()) == false)) {
                lbl.setFont(lbl.getFont().deriveFont(java.awt.Font.BOLD));
            }
            lbl.setVisible(true);
            lbl.addMouseListener(new java.awt.event.MouseAdapter() {
                //TODO:should be done by an other way (render is not clickable...)
                @Override
                public void mouseClicked(MouseEvent e) {
                    super.mouseClicked(e);
                    setValueAt(deft, r, 1);
                }
            });
            return lbl;
        }
        return res;
    }

    public void setValues(Options_Model src) {
        for (int optrow = 0; optrow < src.getRowCount(); optrow++) {
            String label = src.getLabelAt(optrow);
            // should be better with a .clone() in Option???
            Object source = src.getValuefor(label);
            Object dest = null;
            if (label.compareTo("stack_start") == 0) {
                javax.swing.JToggleButton but1 = (javax.swing.JToggleButton) source;
                // made for start button !!!!only!!!
                final javax.swing.AbstractButton but2;
                try {
                    but2 = (javax.swing.AbstractButton) source.getClass().newInstance();
                    but2.setSelected(but1.isSelected());
                    but2.setMargin(but1.getMargin());
                    but2.addActionListener(new java.awt.event.ActionListener() {
                        @Override
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            ((com.jdcm.modifiers.stacks.Stack_baseStart) Parent).setStack_isStart();
                        }
                    });
                    dest = but2;
                } catch (InstantiationException | IllegalAccessException ex) {
                }
            } else if (source instanceof javax.swing.DefaultComboBoxModel) {
                javax.swing.DefaultComboBoxModel<Object> srcmb = (javax.swing.DefaultComboBoxModel) source;
                java.util.List<Object> l = new java.util.ArrayList<>();
                for (int i = 0; i < srcmb.getSize(); i++) {
                    l.add(srcmb.getElementAt(i));
                }
                dest = new javax.swing.DefaultComboBoxModel<>(l.toArray());
                ((javax.swing.DefaultComboBoxModel) dest).setSelectedItem(srcmb.getSelectedItem());
            } else {
                dest = source;
            }
            setValuefor(label, dest);
        }
    }

    private Object getValue(Object value, Object prev_val) {
        Object res = value;
        if (value != null) {
            String searched = value.toString();
            if (prev_val instanceof javax.swing.DefaultComboBoxModel) {
                //should be in Stacks?!! for zcheque & zconcert
                javax.swing.DefaultComboBoxModel<Object> cbm = (javax.swing.DefaultComboBoxModel) prev_val;
                for (int i = 0; i < cbm.getSize(); i++) {
                    String valstr = cbm.getElementAt(i).toString();
                    if (cbm.getElementAt(i) instanceof java.io.File) {
                        valstr = ((java.io.File) cbm.getElementAt(i)).getName();
                    }
                    if (valstr.indexOf(":") > 0) {
                        // for option in protocol ex: '0:accepted';'7:refused';...
                        valstr = valstr.substring(0, valstr.indexOf(":"));
                    }
                    if (valstr.compareTo(searched) == 0) {
                        return cbm.getElementAt(i);
                    }
                }
            }
        }
        return res;
    }

    /**
     * set value tothe ParentObject by 1> catch the old value. 2> check if the
     * 'set' function of Parent is calling this update 2.1> if NOT procede the
     * Parent set function with new value 2> set this value in the table if
     * different than old one.
     *
     * @param newvalue
     * @param row
     * @param column
     */
    @Override
    public void setValueAt(Object newvalue, int row, int column) {
        Object oldvalue = getValueAt(row, column);
        Object value = getValue(newvalue, oldvalue);
        boolean iscommingfromfunction = false;
        // boolean isinvoked = false;
        if (column == 1) {//&& oldvalue.equals(value)==false
            String optname = super.getValueAt(row, 0).toString();
            if (optname.compareTo("|Args") == 0) {
                java.lang.reflect.Method m = (java.lang.reflect.Method) getValuefor("|Methodes");
                try {
                    m.invoke(Parent, value);
                } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
                }
                return;
            }
            for (StackTraceElement tr : Thread.currentThread().getStackTrace()) {
                if (tr.getMethodName().endsWith("set" + optname) && tr.getClassName().compareTo(Parent.getClass().getCanonicalName()) == 0) {
                    iscommingfromfunction = true;
                }
            }
            if (iscommingfromfunction == false) {
                // if update comes from UI(/not a method call to avoid a loop...)
                java.lang.Class<?> mclas = null;
                // if list is changed then value is combobox, else value is selected
                if (value instanceof javax.swing.ComboBoxModel
                        && (oldvalue != null)
                        && oldvalue instanceof javax.swing.ComboBoxModel
                        && ((javax.swing.ComboBoxModel) oldvalue).getSize() == ((javax.swing.ComboBoxModel) value).getSize()) {// && oldvalue instanceof javax.swing.ComboBoxModel == false
                    //!!!!!!! TODO : hard find a way to reset a ComboBoxModel and "set_fcts(good_type);" 
                    value = ((javax.swing.ComboBoxModel) value).getSelectedItem();

                }
                if (value != null) {
                    mclas = value.getClass();
                }
                //it will check 'set', primitives,...
                java.lang.reflect.Method m = TUtil.getMethod(Parent.getClass(), optname, mclas);
                if (m != null) {
                    try {
                        m.invoke(Parent, value);
                        // isinvoked=true;
                    } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
                    }
                }
            }
        }
        /*    if (oldvalue != null
         && value instanceof javax.swing.ComboBoxModel == false
         && (oldvalue.equals(newvalue)
         || oldvalue instanceof javax.swing.ComboBoxModel)) {
         //combobox &&value or equals
         return;
         }*/
        if (getValueAt(row, column) != null && getValueAt(row, column).equals(oldvalue)
                || oldvalue != null && oldvalue.equals(getValueAt(row, column))
                || (oldvalue == null && getValueAt(row, column) == null)) {
            //not already updated by invoke,..
            if (value != null && value instanceof javax.swing.ComboBoxModel == false
                    && oldvalue instanceof javax.swing.ComboBoxModel) {
                javax.swing.ComboBoxModel cm = (javax.swing.ComboBoxModel) oldvalue;
                if (cm.getSize() > 0
                        && (cm.getElementAt(0).getClass().isAssignableFrom(value.getClass())
                        || value.getClass().isAssignableFrom(cm.getElementAt(0).getClass()))) {
                    cm.setSelectedItem(value);
                    newvalue = cm;
                }

            }
            super.setValueAt(newvalue, row, column);
        }

        this.fireTableDataChanged();
        if (Parent instanceof com.jdcm.eltrx.C_Port) {
            ((com.jdcm.eltrx.C_Port) Parent).optionChanged(super.getValueAt(row, 0).toString(), value);
        }
    }
}
