package com.jdcm.diagram;

import java.lang.reflect.Field;
import com.jdcm.common.*;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.Color;
import com.jdcm.eltrx.D_Debug;
import java.io.FileNotFoundException;
import java.util.List;
import javax.swing.tree.TreeModel;

/**
 * Manage an event/state table (used for diagram) the acttion is a property of
 * task linking 2 state by an event. The histo (log)/HistoTableModel is a data
 * of this table (an not diagram...)
 *
 * TODO : How/Why to implement a Dart system on the Event ???
 *
 * @author Marc GIBOIN
 */
public class A_Events extends javax.swing.table.AbstractTableModel {

    private D_Debug TDeb = null;
    private static final long serialVersionUID = 1L;
    private java.util.List<A_Element> States = null;
    private java.util.List<A_Element> Events = null;
    private A_Task[][] Tasks = null;
    private A_Element lastRcvd = null;
    private String title = "default";
    private String description = "";
    private A_Task emptytask = new A_Task();
    private histoModel histotabl = new histoModel();
    private boolean ishisto = false;
    private boolean diagramButton = true;
    private int zoom = 30;

    public A_Events() {
        this(3);
    }

    public A_Events(int i) {
        States = new java.util.ArrayList<>();
        Events = new java.util.ArrayList<>();
        Tasks = new A_Task[0][0];
        try {
            createfromFile("evt_base.xml", "mapping");
        } catch (Exception ex) {
            createEVT_matrice(i, i, i);
        }
        init();
    }

    public A_Events(com.jdcm.eltrx.I_Console c) {
        TDeb = new com.jdcm.eltrx.D_Debug("Events: ");
        TDeb.setconsole(c);
        States = new java.util.ArrayList<>();
        Events = new java.util.ArrayList<>();
        Tasks = new A_Task[0][0];
        //TDeb.wri("create inital Events");
        setTitle("new");
        init();
    }

    public A_Events(java.io.InputStream filestr) {
        States = new java.util.ArrayList<>();
        Events = new java.util.ArrayList<>();
        Tasks = new A_Task[0][0];
        createfromStream(filestr);
        //createEVT_matrice(3, 3, 3);
        init();
    }

    public A_Events(java.util.List<A_Element> st, java.util.List<Object> evt, A_Task[][] in) {
        States = st;
        Events = evt2obj(evt);
        Tasks = in;
        check();
        init();
    }

    public A_Events(java.util.List<A_Element> st, java.util.List<Object> evtin, A_Task[][] table, java.util.List<Object> evtout) {
        States = st;
        Events = evt2obj(evtin);
        Events.addAll(evt2obj(evtout));
        Tasks = table;
        check();
        init();
    }

    private java.util.List<A_Element> evt2obj(java.util.List<Object> evt) {
        java.util.List<A_Element> res = new java.util.ArrayList<>();
        for (Object o : evt) {
            if (o instanceof A_Element) {
                res.add((A_Element) o);
            } else {
                res.add(new A_Element(new C_Eventdef(o)));
            }
        }
        return res;
    }

    private void init() {
        reset();
        addTableModelListener(new javax.swing.event.TableModelListener() {
            @Override
            public void tableChanged(javax.swing.event.TableModelEvent e) {
                check();
            }
        });
    }

    public void reset() {
        for (A_Element a : States) {
            if (a.isInitial()) {
                a.setActive(true);
            } else {
                a.setActive(false);
            }
        }
        lastRcvd = null;
        fireTableDataChanged();
        // !!!usefull?=> IndexoutofBounds...
        fireTableStructureChanged();
    }

    public int getZoom() {
        return zoom;
    }

    public void setZoom(int zoom) {
        this.zoom = zoom;
    }

    final public void setTitle(String txt) {
        title = txt;
    }

    public String getTitle() {
        return title;
    }

    public void setDescription(String txt) {
        description = txt;
    }

    public String getDescription() {
        return description;
    }

    public boolean isPanelButton() {
        return diagramButton;
    }

    public void setPanelButton(boolean bool) {
        diagramButton = bool;
    }

    public void setHisto(boolean bool) {
        if (bool != ishisto) {
            ishisto = bool;
        }
    }

    public boolean isHisto() {
        return ishisto;
    }

    public javax.swing.table.AbstractTableModel getHistoModel() {
        return histotabl;
    }

    public java.util.List<A_Element> getStatuslist() {
        return States;
    }

    public java.util.List<A_Element> getEventlist() {
        return Events;
    }

    public A_Task[][] getTasklist() {
        return Tasks;
    }

    private boolean isExist(Object o) {
        List<A_Element> temp = new java.util.ArrayList<>();
        temp.addAll(getStatuslist());
        temp.addAll(getEventlist());
        for (A_Element elt : temp) {
            if (elt.getcontent().equals(o)) {
                return true;
            }
        }
        return false;
    }

    public void addElement(boolean isState, boolean isInit, Object content) {
        addElement(createElement(isState, isInit, content));
    }

    public void addElement(A_Element elt) {
        if (elt.isState()) {
            addStatus(elt);
        } else {
            addEvent(elt);
        }
    }

    private A_Element createElement(boolean isState, boolean isInit, Object content) {
        if (content == null) {
            content = "evt";
            if (isState) {
                content = "st";
            }
        }
        if (content instanceof String && isExist(content)) {
            int i = 1;
            while (isExist(content.toString() + "_" + i)) {
                i++;
            }
            content = content.toString() + "_" + i;
        }
        A_Element element;
        if (isState) {
            element = new A_Element(content);
            element.setInitial(isInit);
            return (element);
        } else {
            if (content instanceof C_Eventdef) {
                element = new A_Element((C_Eventdef) content);
            } else {
                element = new A_Element(new C_Eventdef(content));
            }
            return (element);
        }

    }

    public void setTask(A_Element elt1, A_Element elt2, A_Element elt3) {
        if (elt1.isState() == false || elt2.isState() || elt3.isState() == false) {
            return;
        }
        A_Element event = elt2;
        //if several concurency previous then task of previous should have other second next state
        int idxevt = getEventlist().indexOf(event);
        A_Task tsk = getTasklist()[idxevt][getStatuslist().indexOf(elt1)];
        if (tsk == null || tsk.isEmpty()) {
            tsk = new A_Task();
        }
        if (tsk.getColor() == null) {
            Color[] cols = new Color[]{Color.BLACK, Color.RED, Color.BLUE, Color.CYAN.darker(), Color.GREEN.darker(), Color.YELLOW.darker(), Color.MAGENTA, Color.PINK, Color.ORANGE, new Color(255, 0, 255)};
            int i = (int) Math.round(Math.random() * (cols.length - 1));
            tsk.setColor(cols[i]);
        }
        if (tsk.getPrevState() == null) {
            tsk.setPrevState(elt1);
        }
        if (tsk.getNextStates().contains(elt3) == false) {
            tsk.addNextState(elt3);
        }
        if (tsk.getActions().isEmpty()) {
            tsk.getActions().add(new com.jdcm.common.C_Actiondef());
        }
        getTasklist()[idxevt][getStatuslist().indexOf(elt1)] = tsk;
        //if concurent then check that other previous have a new next &this have other 'previous nexts'
        ///?????????concurent manage previous but next ????? should be something else...
        if (elt2.getConcurent() != A_Element.CONCUR_FULL) {
            for (int l = 0; l < getStatuslist().size(); l++) {
                if (tsk.equals(getTasklist()[idxevt][l]) == false) {
                    if (getTasklist()[idxevt][l].isEmpty() == false) {
                        for (A_Element nxtl : getTasklist()[idxevt][l].getNextStates()) {
                            if (tsk.getNextStates().contains(nxtl) == false) {
                                tsk.addNextState(nxtl);
                            }
                        }
                        if (getTasklist()[idxevt][l].getNextStates().contains(elt3) == false) {
                            getTasklist()[idxevt][l].addNextState(elt3);
                        }
                    }
                }
            }
        }
        for (A_Element prev : getconcurents(event, tsk)) {
            tsk = getTasklist()[getEventlist().indexOf(event)][getStatuslist().indexOf(prev)];
            if (tsk.getNextStates().contains(elt3) == false) {
                tsk.addNextState(elt3);
            }
            if (tsk.getPrevState() == null) {
                tsk.setPrevState(elt1);
            }
            getTasklist()[getEventlist().indexOf(event)][getStatuslist().indexOf(prev)] = tsk;
        }
    }

    public void setTask(A_Element elt1, A_Element elt2) {
        A_Element tmp;
        if (elt1.isState() == elt2.isState()) {
            if (elt1.isState()) {
                tmp = createElement(false, false, null);
                addElement(tmp);
                setTask(elt1, tmp, elt2);
            }
            return;
        } else if (elt1.isState()) {
            tmp = elt2;
            elt2 = elt1;
            elt1 = tmp;
        }
        A_Task tsk = getTasklist()[getEventlist().indexOf(elt1)][getStatuslist().indexOf(elt2)];
        if (tsk == null) {
            tsk = new A_Task();
        }
        tsk.addNextState(elt2);
        if (tsk.getPrevState() == null) {
            tsk.setPrevState(elt2);
        }
        if (tsk.getActions().isEmpty()) {
            // WHY ??
            tsk.getActions().add(new com.jdcm.common.C_Actiondef(""));
        }
        getTasklist()[getEventlist().indexOf(elt1)][getStatuslist().indexOf(elt2)] = tsk;
    }

    public void setStatusAsFields(Object fieldsrc) {
        Field[] fields = fieldsrc.getClass().getFields();
        for (A_Element st : States) {
            String txt = st.getcontent().toString();
            for (Field field : fields) {
                if (field.getName().endsWith(txt)) {
                    st.setcontent(field);
                }
            }
        }
    }

    /**
     * complete Tasks with states (from strings) & add previous states
     *
     */
    private void check() {
        Color[] cols = new Color[]{Color.BLACK, Color.RED, Color.BLUE, Color.CYAN.darker(), Color.GREEN.darker(), Color.YELLOW.darker(), Color.MAGENTA, Color.PINK, Color.ORANGE, new Color(255, 0, 255)};
        int colptr = 1;
        // checks that Tasks match this Events et states
        if (Tasks == null || Tasks.length == 0
                || Tasks.length != Events.size()
                || Tasks[0].length != States.size()) {
            return;
        }
        //fill with empty tasks & check/modify next states with matchin States 
        for (int i = 0; i < Events.size(); i++) {
            for (int j = 0; j < States.size(); j++) {
                //   res[i][j] = table[i][j];
                if (Tasks[i][j] == null) {
                    Tasks[i][j] = emptytask;
                } else {
                    java.util.ArrayList<A_Element> nextlist = Tasks[i][j].getNextStates();
                    if (Tasks[i][j].getColor() == null) {
                        if (++colptr == cols.length) {
                            colptr = 0;
                        }
                        Tasks[i][j].setColor(cols[colptr]);
                    }

                    // TO WORK AROUND => Pb if several equal State should be fobiden
                    // differeces beetween saved & read...
                    if (nextlist.isEmpty() == false) {
                        for (int idx = 0; idx < nextlist.size(); idx++) {
                            A_Element nxtst = nextlist.get(idx);
                            boolean found = false;
                            for (A_Element st : States) {
                                Object entrie = st.getcontent();
                                if (entrie instanceof Field && nxtst.getcontent() instanceof Field == false) {
                                    try {
                                        entrie = ((Field) entrie).get(entrie);
                                    } catch (IllegalArgumentException | IllegalAccessException ex) {
                                    }
                                }
                                if (entrie != null && entrie.equals(nxtst.getcontent())) {
                                    found = true;
                                    nextlist.set(idx, st);
                                    break;
                                }
                            }
                            if (found == false) {
                                Tasks[i][j] = emptytask;
                            }
                        }
                    }
                }
            }
        }
        // fill previous states in tasks
        for (int i = 0; i < Events.size(); i++) {
            for (int j = 0; j < States.size(); j++) {
                A_Task init = Tasks[i][j];
                if (init.isEmpty() == false) {
                    init.setPrevState(null);
                    init.setPrevState(States.get(j));
                }
            }
        }
        if (TDeb != null) {
            TDeb.wri("normal end of check");
        }
    }

    public Object[] getStates() {
        return States.toArray();
    }

    /**
     * to create typical matrix (old for debuging)
     *
     * @param nbstatus
     * @param nbevtin
     * @param nbevtout
     */
    public final void createEVT_matrice(int nbstatus, int nbevtin, int nbevtout) {
        A_Events evt = null;
        if (evt != null) {
            return;
        }
        if (TDeb != null) {
            TDeb.wri("create full matrix nb status :" + nbstatus + " nb events :" + nbevtin + nbevtout);
        }
        java.util.List<A_Element> cbst = new java.util.ArrayList<>();
        java.util.List<Object> evtin = new java.util.ArrayList<>();
        java.util.List<Object> evtout = new java.util.ArrayList<>();
        A_Task[][] table = new A_Task[nbevtin + nbevtout][nbstatus];

        for (int i = 0; i < nbstatus; i++) {
            cbst.add(new A_Element("stat_" + (i + 1)));
            int next = i;
            if (i == 0) {
                next = nbstatus;
            }
            if (i < nbevtin) {
                evtin.add(("" + i).getBytes());
            }
            for (int j = 0; j < nbevtin; j++) {
                table[j][i] = new A_Task("stat_" + next, j);
            }
            for (int j = nbevtin; j < nbevtin + nbevtout; j++) {
                table[j][i] = new A_Task("stat_" + next, j);
            }
            if (i < nbevtout) {
                evtout.add(("" + (i + 5)).getBytes());
            }
        }
        cbst.get(0).setInitial(true);
        States = cbst;
        Events = evt2obj(evtin);
        Events.addAll(evt2obj(evtout));
        Tasks = table;
        if (TDeb != null) {
            TDeb.wri("table created, will check");
        }
        check();
        this.fireTableDataChanged();
    }

    /**
     * load this evtmatrix to file
     *
     * @return the treeModel of the Events
     */
    public DefaultTreeModel toTree() {
        DefaultMutableTreeNode tr = new DefaultMutableTreeNode(new XMLhdr("1.0", "UTF-8"));
        DefaultMutableTreeNode pdml = new DefaultMutableTreeNode(new XMLres("pdml"));
        tr.add(pdml);
        XMLres h = new XMLres("EventsMatrice");
        h.addAttribut("title", title);
        h.addAttribut("ishisto", ishisto);
        h.setAttrVal("zoom", zoom);
        h.setAttrVal("description", description);
        DefaultMutableTreeNode h1 = new DefaultMutableTreeNode(h);
        pdml.add(h1);
        DefaultMutableTreeNode t1 = new DefaultMutableTreeNode(new XMLres("states"));
        h1.add(t1);
        for (A_Element st : States) {
            XMLres xmlst = st.getXMLformat();
            xmlst.addAttrVal("index");
            xmlst.setAttrVal("index", States.indexOf(st));
            t1.add(new DefaultMutableTreeNode(xmlst));
        }
        DefaultMutableTreeNode t2 = new DefaultMutableTreeNode(new XMLres("events"));

        for (A_Element evt : Events) {
            XMLres xmlev = evt.getXMLformat();
            xmlev.addAttrVal("index");
            xmlev.setAttrVal("index", Events.indexOf(evt));
            DefaultMutableTreeNode tev = new DefaultMutableTreeNode(xmlev);
            if (evt.getcontent() instanceof C_Eventdef) {
                XMLres xmlevt = ((C_Eventdef) evt.getcontent()).getXMLformat();
                tev.add(new DefaultMutableTreeNode(xmlevt));
            }
            t2.add(tev);
        }
        h1.add(t2);
        XMLres r = new XMLres("tasks");
        r.addAttrVal("nbevts");
        r.setAttrVal("nbevts", Tasks.length);
        r.addAttrVal("nbstats");
        r.setAttrVal("nbstats", Tasks[0].length);
        DefaultMutableTreeNode t3 = new DefaultMutableTreeNode(r);
        for (int i = 0; i < Tasks.length; i++) {
            for (int j = 0; j < Tasks[0].length; j++) {
                if (Tasks[i][j] != null && Tasks[i][j].isEmpty() == false) {
                    XMLres tsk = Tasks[i][j].getXMLformat();
                    int[] nxtsts = new int[Tasks[i][j].getNextStates().size()];
                    int k = 0;
                    for (A_Element nxtst : Tasks[i][j].getNextStates()) {
                        nxtsts[k++] = States.indexOf(nxtst);
                    }
                    tsk.addAttribut("nxtstat", nxtsts);
                    tsk.addAttribut("event", i);
                    tsk.addAttribut("state", j);
                    DefaultMutableTreeNode t4 = new DefaultMutableTreeNode(tsk);
                    for (C_Actiondef a : Tasks[i][j].getActions()) {
                        t4.add(new DefaultMutableTreeNode(a.getXMLformat()));
                    }
                    t3.add(t4);
                }
            }
        }
        h1.add(t3);
        return new DefaultTreeModel(tr);
    }

    /**
     * load this evtmatrix to file
     *
     * @param filename
     */
    public void toFile(String filename) {
        if (Tasks.length == 0) {
            //need at least 1 evt (Task size...)
            return;
        }
        if (filename == null || filename.length() == 0) {
            filename = title + ".xml";
        } else if (filename.endsWith(".xml") == false) {
            filename = filename + ".xml";
        }
        if (TDeb != null) {
            TDeb.wri("startd to file to : " + filename);
        }
        java.io.FileOutputStream o;
        try {
            o = new java.io.FileOutputStream(filename);
            XML2tree t = new XML2tree();
            t.savetree2XML(toTree(), o);
        } catch (FileNotFoundException ex) {
            return;
        }
        if (TDeb != null) {
            TDeb.wri("file created");
        }
        // byte[] Obser = p.getSerial();
    }

    public final void createfromFile(String filename, String dir) {
        createfromStream(D_Utils.getFileStream(filename, dir, getClass().getClassLoader()));
    }

    /**
     * !!!don't WORK with JAR, use createfromStream()
     *
     * @param file
     */
    public final void createfromFile(java.io.File file) {
        try {
            createfromStream(new java.io.FileInputStream(file));
        } catch (FileNotFoundException ex) {
            TDeb.wri("could not creta from : " + file + "=>" + ex.getMessage());
        }
    }

    /**
     * load evtmatrix from tree (.. from XML file)
     *
     * @param tree the source tree.
     */
    public void createfromTree(TreeModel tree) {
        States = new java.util.ArrayList<>();
        Events = new java.util.ArrayList<>();
        Tasks = new A_Task[0][0];
        if (tree == null) {
            return;
        }
        if (tree.getChildCount(tree.getRoot()) > 0) {
            for (int childnb = 0; childnb < tree.getChildCount(tree.getRoot()); childnb++) {
                DefaultMutableTreeNode hr = (DefaultMutableTreeNode) tree.getChild(tree.getRoot(), childnb);
                if (hr.getUserObject() instanceof XMLres && ((XMLres) hr.getUserObject()).getEltName().startsWith("pdml")) {
                    int schild = 0;
                    while (schild < hr.getChildCount()) {
                        DefaultMutableTreeNode h1 = (DefaultMutableTreeNode) hr.getChildAt(schild++);
                        if (h1.getUserObject() instanceof XMLres && ((XMLres) h1.getUserObject()).getEltName().startsWith("EventsMatrice")) {
                            title = "No Title";
                            description = "No Description";
                            if (((XMLres) h1.getUserObject()).getAttrVal("title") != null) {
                                title = ((XMLres) h1.getUserObject()).getAttrVal("title").toString();
                            }
                            if (((XMLres) h1.getUserObject()).getAttrVal("description") != null) {
                                description = ((XMLres) h1.getUserObject()).getAttrVal("description").toString();
                            }
                            setHisto((Boolean) ((XMLres) h1.getUserObject()).getAttrVal("ishisto"));
                            if (((XMLres) h1.getUserObject()).getAttrVal("zoom") != null) {
                                zoom = (int) ((XMLres) h1.getUserObject()).getAttrVal("zoom");
                            }
                            for (int i = 0; i < h1.getChildCount(); i++) {
                                DefaultMutableTreeNode t1 = (DefaultMutableTreeNode) tree.getChild(h1, i);
                                if (t1.getUserObject() instanceof XMLres && h1.getChildCount() > 0) {
                                    XMLres typeelt = (XMLres) t1.getUserObject();
                                    String type = typeelt.getEltName();
                                    if (type != null && type.startsWith("tasks")) {
                                        Tasks = new A_Task[Events.size()][States.size()];
                                    }
                                    for (int j = 0; j < t1.getChildCount(); j++) {
                                        DefaultMutableTreeNode e1 = (DefaultMutableTreeNode) tree.getChild(t1, j);
                                        if (e1.getUserObject() instanceof XMLres) {
                                            java.util.ArrayList<XMLres> list = new java.util.ArrayList<>();
                                            for (int k = 0; k < e1.getChildCount(); k++) {
                                                list.add((XMLres) ((DefaultMutableTreeNode) e1.getChildAt(k)).getUserObject());
                                            }
                                            add2matrice((XMLres) e1.getUserObject(), list);
                                        }
                                    }
                                }
                            }
                            check();
                            this.fireTableChanged(null);
                        }
                    }
                }
            }
        }
    }

    public final void createfromStream(java.io.InputStream filestr) {
        States.clear();
        Events.clear();
        Tasks = null;
        title = "";

        if (filestr == null) {
            return;
        }
        if (TDeb != null) {
            TDeb.wri("file reading, starting");
        }
        XML2tree t = new XML2tree();
        createfromTree(t.XMLBase(filestr));

        if (TDeb != null) {
            TDeb.wri("file readed");
        }
    }

    public C_Actiondef getActionFor(XMLres act) {
        if (act.getAttrVal("dest") == null) {
            return new C_Actiondef(act.getAttrVal("parametres"));
        } else {
            String meth = null;
            if (act.getAttrVal("method") != null) {
                meth = act.getAttrVal("method").toString();
            }
            if (act.getAttrVal("parametres") == null) {
                return new C_Actiondef(act.getAttrVal("dest").toString(), meth);
            } else {
                return new C_Actiondef(act.getAttrVal("dest").toString(), meth, act.getAttrVal("parametres"));
            }
        }
    }

    /**
     * for loading etmatrix from XML
     *
     * @param type states/events/tasks type of elment
     * @param res XMLres Object coming from file
     */
    private void add2matrice(XMLres res, java.util.ArrayList<XMLres> list) {
        if (TDeb != null) {
            TDeb.wri("adding :" + res.getEltName() + " " + res.toString());
        }
        final A_Element elt;
        if (res.getEltName().compareTo("state") == 0) {
            elt = new A_Element(res.getAttrVal("value"));
            elt.setActive((Boolean) res.getAttrVal("isactive"));
            elt.recposition = (java.awt.Rectangle) res.getAttrVal("bounds");
            elt.setWeight((Integer) res.getAttrVal("weight"));
            elt.setInitial((Boolean) res.getAttrVal("isinitial"));
            States.add(elt);
            elt.addChangeStateListner(new A_Element.changeStateListner() {

                @Override
                public boolean beforeNameChange(Object newContent) {
                    for (A_Element a : States) {
                        if (a != null
                                && a.getcontent().equals(newContent)
                                && States.indexOf(elt) < States.size() - 1) {
                            return false;
                        }
                    }
                    return true;
                }
            });
        } else if (res.getEltName().compareTo("event") == 0) {
            int conc = (Integer) res.getAttrVal("concurent");
            Object ob = null;
            for (XMLres act : list) {
                if (act.getAttrVal("src") == null) {
                    ob = new C_Eventdef(act.getAttrVal("parametres"));
                } else {
                    ob = new C_Eventdef(act.getAttrVal("src").toString(), act.getAttrVal("method").toString(), act.getAttrVal("parametres"));
                }
            }
            if (ob == null) {
                //for migration
                ob = res.getAttrVal("value");
            }
            if (ob instanceof C_Eventdef) {
                elt = new A_Element(ob);
            } else {
                elt = new A_Element(new C_Eventdef(ob));
            }
            elt.setConcurent(conc);
            elt.recposition = (java.awt.Rectangle) res.getAttrVal("bounds");

            Events.add(elt);
        } else if (res.getEltName().compareTo("task") == 0) {
            A_Task tsk = new A_Task();
            tsk.setColor((Color) res.getAttrVal("color"));
            int[] nxt = (int[]) res.getAttrVal("nxtstat");
            for (int nb : nxt) {
                tsk.addNextState(States.get(nb));
            }
            for (XMLres act : list) {
                tsk.getActions().add(getActionFor(act));
            }
            Tasks[(Integer) res.getAttrVal("event")][(Integer) res.getAttrVal("state")] = tsk;
        }

    }

    public void removeStatus(A_Element st) {
        // need to manage Task impacted...
        if (States.indexOf(st) == -1 || States.isEmpty()) {
            return;
        } /*
         * else if (st.isActive()) { return; }
         */ else {
            if (TDeb != null) {
                TDeb.wri("asking remove : " + st.toString());
            }
            int column = States.indexOf(st);
            States.remove(st);
            if (Tasks != null && Tasks.length > 0) {
                A_Task[][] temp = new A_Task[Tasks.length][States.size()];
                for (int i = 0; i < Tasks.length; i++) {
                    A_Task[] newos = new A_Task[States.size()];
                    System.arraycopy(Tasks[i], 0, newos, 0, column);
                    System.arraycopy(Tasks[i], column + 1, newos, column, Tasks[i].length - column - 1);
                    temp[i] = newos;
                }
                Tasks = temp;
            }
        }
        this.fireTableStructureChanged();
    }

    public void addStatus(A_Element obj) {
        if (States.contains(obj)) {
            return;
        }
        if (TDeb != null) {
            TDeb.wri("asking add : " + obj.toString());
        }
        for (A_Element st : States) {
            if (st.getcontent() == null && TDeb != null) {
                TDeb.wri("content is null in A_Event(621)!!!");
            }
            if (st.getcontent().equals(obj.getcontent())) {
                return;
            }
        }
        States.add(obj);
        if (Tasks != null) {
            A_Task[][] temp = new A_Task[Tasks.length][States.size()];
            for (int i = 0; i < temp.length; i++) {
                A_Task[] newos = new A_Task[States.size()];
                System.arraycopy(Tasks[i], 0, newos, 0, Tasks[i].length);
                newos[newos.length - 1] = new A_Task();
                temp[i] = newos;
            }
            Tasks = temp;
        }
        this.fireTableStructureChanged();
    }

    public void addEvent(A_Element evt) {
        if (TDeb != null) {
            TDeb.wri("asking add : " + evt.toString());
        }
        Events.add(evt);

        A_Task[][] res = new A_Task[Events.size()][States.size()];
        for (int i = 0; i < Events.size() - 1; i++) {
            System.arraycopy(Tasks[i], 0, res[i], 0, States.size());
        }
        for (int i = 0; i < States.size(); i++) {
            res[Events.indexOf(evt)][i] = new A_Task();
        }
        Tasks = res;
        fireTableDataChanged();
    }

    public void removeEvt(A_Element evt) {
        // manage impacted State
        if (evt == null || Events.indexOf(evt) == -1) {
            return;
        }
        if (TDeb != null) {
            TDeb.wri("asking remove : " + evt.toString());
        }
        int row = Events.indexOf(evt);
        Events.remove(evt);
        A_Task[][] res = new A_Task[Tasks.length - 1][Tasks[0].length];
        System.arraycopy(Tasks, 0, res, 0, row);
        System.arraycopy(Tasks, row + 1, res, row, Tasks.length - row - 1);
        Tasks = res;
        fireTableDataChanged();
    }

    /**
     * CARE : return ONLY ONE status IF Actie status nomber is ONE!! Should be
     * deleted (but used...telnet...)
     *
     * @return
     */
    public A_Element getActivStatus() {
        java.util.ArrayList<A_Element> tmp = this.getActivStatusList();
        if (tmp != null && tmp.isEmpty() == false && tmp.size() == 1) {
            return tmp.get(0);
        }
        return null;
    }

    public void setStatus(java.util.List<A_Element> prev, A_Element next) {
        //  for (A_Element st : this.getStatuslist()) {
        if (States.contains(next)) {
            next.setActive(true);
            if (prev != null) {
                for (A_Element pst : prev) {
                    if (pst.equals(next) == false) {
                        pst.setActive(false);
                    }
                }
            }

        } else {
            TDeb.wri("State not found:" + next);
        }
        //   }
    }

    public java.util.ArrayList<A_Element> getActivStatusList() {
        java.util.ArrayList<A_Element> res = new java.util.ArrayList<>();
        for (A_Element st : this.getStatuslist()) {
            if (st.isActive()) {
                res.add(st);
            }
        }
        return res;
    }

    public Object getlastrvd() {
        return lastRcvd;
    }

    /**
     * retun list of actions for receveid event according to States / actions
     * table
     *
     * @param rcvd the receved event
     * @return a list of C_Actions
     */
    public java.util.ArrayList<C_Actiondef> getTask(Object rcvd) {
        java.util.ArrayList<C_Actiondef> res = null;
        // ENTREE : FORM APPLI / FROM MEDIA / ...
        //SORTIE : STATUS / BUFFER / TO APPLI / FROM MEDIA
        A_Task action;
        if (getActivStatusList().isEmpty()) {
            return null;
        } else {
            for (A_Element event : Events) {
                if (event.getcontent() instanceof C_Eventdef) {
                    C_Eventdef evt = (C_Eventdef) event.getcontent();
                    boolean evtisok = evt.equals(rcvd);
                    if (evtisok) {
                        lastRcvd = event;
                        for (A_Element st : getActivStatusList()) {
                            int statusidx = States.indexOf(st);
                            if (statusidx >= 0) {
                                action = Tasks[Events.indexOf(event)][statusidx]; /// 2 events should be sames (for differents states
                                if (action != null && action.isEmpty() == false) {
                                    java.util.List<A_Element> prevs = getconcurents(event, action);
                                    if (prevs.isEmpty() == false) {

                                        boolean isPrevActivated = action.getPrevState() != null;
                                        //if concurent mgt, check previous state are active                              
                                        for (A_Element prv : prevs) {
                                            isPrevActivated = prv.isActive() && isPrevActivated;
                                        }

                                        if (isPrevActivated || event.getConcurent() != A_Element.CONCUR_FULL) {
                                            if (isHisto()) {
                                                histotabl.addhisto(new java.sql.Timestamp(System.currentTimeMillis()), rcvd, event, st, action);
                                            }
                                            action.setActionsSource(rcvd);
                                            res = new java.util.ArrayList<>();
                                            res.addAll(action.getActions());
                                            if (event.getConcurent() == A_Element.CONCUR_FULL) {
                                                for (A_Element nst : action.getNextStates()) {
                                                    setStatus(prevs, nst);
                                                }
                                            } else if (event.getConcurent() == A_Element.CONCUR_WEIGHT) {
                                                setStatus(prevs, action.getMoreWeightNextState());
                                            } else if (event.getConcurent() == A_Element.CONCUR_RAND) {
                                                int aleatoire = (int) Math.round(Math.random() * (action.getNextStates().size() - 1));
                                                setStatus(prevs, action.getNextStates().get(aleatoire));
                                            }
                                            return res;
                                        }
                                    }
                                }
                            }
                        }
                        //event ok but no actie task
                        res = null;
                    }
                }
            }
        }
//        this.fireTableStructureChanged();
        this.fireTableDataChanged();
        // IF nothing 
        return res;
    }

    /**
     * Return the previous list state , - if not concurent : simple previous one
     * - if concurent & if first concerned task : list of all previous states -
     * if concurent & not fist : empty list
     *
     * @param Evt the concerned event
     * @param tsk the task to complete
     * @return the list of previous events.
     */
    public java.util.List<A_Element> getconcurents(A_Element Evt, A_Task tsk) {
        java.util.List<A_Element> res = new java.util.ArrayList<>();
        res.add(tsk.getPrevState());
        if (Evt.getConcurent() != A_Element.CONCUR_FULL) {
            return res;
        }
        java.util.List<A_Task> store = new java.util.ArrayList<>();
        // add all other previous states for this event with go to NextState
        // store matching tasks
        int idxevt = Events.indexOf(Evt);
        for (int l = 0; l < States.size(); l++) {
            if (tsk.equals(Tasks[idxevt][l]) == false) {
                for (A_Element nxtl : Tasks[idxevt][l].getNextStates()) {
                    if (tsk.getNextStates().contains(nxtl)) {
                        res.add(States.get(l));
                        if (store.contains(Tasks[idxevt][l]) == false) {
                            store.add(Tasks[idxevt][l]);
                        }
                    }
                }
            }
        }
        // return all previous states only for the first matching state
        int idxst = States.indexOf(tsk.getPrevState());
        for (int l = 0; l < idxst; l++) {
            if (store.contains(Tasks[idxevt][l])) {
                res.clear();
            }
        }
        return res;
    }

    @Override
    public int getRowCount() {
        int res;
        if (States == null) {
            res = 0;
        } else if (Tasks == null) {
            res = 1;
        } else {
            res = Events.size();
        }
        return res;
    }

    @Override
    public int getColumnCount() {
        return States.size() + 1;
    }

    @Override
    public String getColumnName(int columnIndex) {
        String res = "/";
        if (columnIndex == 0) {
            return res;
        } else {
            Object stat = States.get(columnIndex - 1);
            if (stat instanceof Field) {
                return ((Field) stat).getName();
            } else {
                return stat.toString();
            }
        }
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
        /*
         * if (rowIndex == 0 && columnIndex == 0) { return true; } else if
         * (rowIndex == 0 && ((A_Element) getValueAt(0, columnIndex)).isActive())
         * { return true; } return true;
         */
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        /**
         * else if(rowIndex == 0) { if (columnIndex == 0) { return label; } else
         * { return States.get(columnIndex - 1); } }
         */
        if (rowIndex < 0) {
            return "null";
        } else if (rowIndex < Events.size()) {
            if (columnIndex == 0) {
                return Events.get(rowIndex);
            } else {
                try {
                    return Tasks[rowIndex][columnIndex - 1];
                } catch (Exception ex) {
                    return "nothing";
                }
            }
        }
        return "bug";
    }
// shoulf not be edittable;;;=< return true

    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        if (rowIndex == 0) {
            if (columnIndex == 0) {
                title = aValue.toString();
            }
        }
        if (true) {
            return;
        }
        if (rowIndex == 0) {
            if (columnIndex != 0) {
                if (aValue instanceof A_Element) {
                    States.set(columnIndex - 1, (A_Element) aValue);
                }
            }
        } else if (columnIndex == 0) {
            if (aValue instanceof A_Element) {
                Events.set(rowIndex - 1, (A_Element) aValue);
            }
        } else if (aValue instanceof A_Task) {
            if (rowIndex < Tasks.length + 1) {
                Tasks[rowIndex - 1][columnIndex] = (A_Task) aValue;
            }
        }
    }

    @Override
    public String toString() {
        return "EvtMatrix[" + States.size() + "/" + Events.size() + "]";
    }

    class histoModel extends javax.swing.table.AbstractTableModel {

        private static final long serialVersionUID = 1L;
        java.util.List<java.sql.Timestamp> timestamps = new java.util.ArrayList<>();
        java.util.List<A_Element> states = new java.util.ArrayList<>();
        java.util.List<A_Element> events = new java.util.ArrayList<>();
        java.util.List<String> tasks = new java.util.ArrayList<>();
        java.util.List<Object> received = new java.util.ArrayList<>();
        String[] columnnames = new String[]{"N°", "Time", "Rcv", "Event", "State", "Task"};

        @Override
        public int getRowCount() {
            return timestamps.size();
        }

        @Override
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return columnIndex != 0;
        }

        @Override
        public String getColumnName(int columnIndex) {
            return columnnames[columnIndex];
        }

        @Override
        public int getColumnCount() {
            return 6;
        }

        public void addhisto(java.sql.Timestamp d, Object o, A_Element evt, A_Element st, A_Task action) {
            timestamps.add(d);
            received.add(o);
            states.add(st);
            events.add(evt);
            tasks.add(action.toString());
            this.fireTableDataChanged();
        }

        @Override
        public Object getValueAt(int rowIndex, int columnIndex) {
            if (columnIndex == 0) {
                return rowIndex;
            } else if (columnIndex == 1) {
                return timestamps.get(rowIndex);
            } else if (columnIndex == 2) {
                return received.get(rowIndex);
            } else if (columnIndex == 3) {
                return events.get(rowIndex);
            } else if (columnIndex == 4) {
                return states.get(rowIndex);
            } else if (columnIndex == 5) {
                return tasks.get(rowIndex);
            } else {
                return "bug";
            }
        }
    }
}
