package com.jdcm.diagram;

import java.awt.Color;
import com.jdcm.common.*;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 *
 * @author M.Marc GIBOIN
 */
public class A_Task {

    private java.util.ArrayList<A_Element> next_states = new java.util.ArrayList<>();
    private A_Element prev_state = null;
    private java.util.List<C_Actiondef> actions = new java.util.ArrayList<>();
    private Color color = null;
    private boolean selected=false;

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }
    
    public A_Task(Object next, java.util.List<C_Actiondef> a) {
        next_states.add(new A_Element(next));
        actions = a;
    }

    public A_Task(Object next, C_Actiondef a) {
        next_states.add(new A_Element(next));
        actions.add(a);
    }
    //should be removed

    public A_Task(Object next, Object obj) {
        next_states.add(new A_Element(next));
        actions.add(new C_Actiondef(obj));
    }

    public A_Task(Object next) {
        next_states.add(new A_Element(next));
        actions = new java.util.ArrayList<>();
    }

    public A_Task() {
        next_states.clear();
        actions = new java.util.ArrayList<>();
    }

    public Options_Model getOptionModel() {
        Options_Model option = new Options_Model(this);
        option.addOption("Color", color, "Color", "0/1", true, true, "Task color");
        option.addOption("Actions", actions, "Object", "everything", true, true, "Action");
        option.addOption("nxtState", next_states, "List", "states", true, true, "states to follow");
        option.addOption("prvState", prev_state, "List", "states", true, true, "states to follow");
        option.setTitle("properties");
        return option;

    }

    public DefaultMutableTreeNode getTreeNode() {
        java.awt.Label lbl = new java.awt.Label("Task");
        lbl.setForeground(color);
        DefaultMutableTreeNode ttsk = new DefaultMutableTreeNode(this);
         for(C_Actiondef act:actions){
             ttsk.add(new DefaultMutableTreeNode(act));
         }
        return ttsk;
    }

    public java.util.ArrayList<A_Element> getNextStates() {
        return next_states;
    }

    public void addNextState(A_Element st) {
        next_states.add(st);
    }

    public void setNextState(java.util.ArrayList<A_Element> st) {
        next_states = st;
    }


    public Color getColor() {
        return color;
    }

    public void setColor(Color col) {
        color = col;
    }

    public A_Element getPrevState() {
        return prev_state;
    }

    public void setPrevState(A_Element st) {
        prev_state = st;
    }

    /**
     * An object to transmit at transition
     *  send is proceed by port/machine managing the evtmatrix
     * @return 
     */
    public java.util.List<C_Actiondef> getActions() {
        return actions;

    }
    public void setActionsSource(Object src) {
        for (C_Actiondef action:actions){
            action.setSource(src);
        }

    }
    public void setActions(java.util.List<C_Actiondef> list) {
        actions = list;
    }
    /**
     * An empty Tasks are used to complete matrix..
     * if to next state then empty ( && prev? why not)
     * @return 
     */
    public boolean isEmpty() {
        return next_states.isEmpty() && prev_state == null;

    }

    @Override
    public boolean equals(Object ob) {
        if (ob instanceof A_Task) {
            A_Task task = (A_Task) ob;
            return (task.hashCode() == this.hashCode());
        }

        return false;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 19 * hash + (this.next_states != null ? this.next_states.hashCode() : 0);
        hash = 19 * hash + (this.prev_state != null ? this.prev_state.hashCode() : 0);
        hash = 19 * hash + (this.actions != null ? this.actions.hashCode() : 0);
        return hash;
    }

    public XMLres getXMLformat() {
        XMLres res = new XMLres("task");
        res.addAttribut("color", color);
        return res;
    }

    /**
     * find the choosed task if several ways and no concurent management 
     * @return the choosed State.
     */
    public A_Element getMoreWeightNextState() {
        A_Element choosednxt = null;
        for (A_Element sta : next_states) {
            if (choosednxt == null || sta.getWeight() > choosednxt.getWeight()) {
                choosednxt = sta;
            }
        }
        return choosednxt;
    }

    @Override
    public String toString() {
        String st = "", se = "", act = "", res = "";
        if (this.isEmpty()){
            return "Empty Task";
        }
        if (next_states != null) {
            for (A_Element sta : next_states) {
                String content = "" + sta.getcontent();
                if (sta.getcontent() instanceof java.lang.reflect.Field) {
                    content = ((java.lang.reflect.Field) sta.getcontent()).getName();
                }
                if (next_states.size() > 1 && sta.equals(getMoreWeightNextState())) {
                    res += "Task[" + content + "*]";
                } else {
                    res += "Task[" + content + "]";
                }
            }
        }
        if (actions != null) {
            res += ";'" + actions + "'";
        }
        return res;
    }
}
