package com.jdcm.diagram;

import com.jdcm.common.C_Actiondef;
import com.jdcm.common.D_Utils;
import java.util.List;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.JTree;
import javax.swing.event.TableModelEvent;
import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JFileChooser;

/**
 * P_Diagram.java Created on 13 janv. 2011, 22:37:37
 *
 * Only a graphic representation (and editor) of A_Events
 *
 * @author Marc GIBOIN
 */
public class P_Diagram extends javax.swing.JPanel {

    private static final long serialVersionUID = 1L;
    private final D_Utils TUtils = new D_Utils();
    private final List<A_Element> selected = new java.util.ArrayList<>();
    // Could be better with Stroke maniulation BUT ... howto do ???
    private final Stroke basicpen = new BasicStroke(1.0F);
    private final Stroke bbasicpen = new BasicStroke(2.0F);
    private final Stroke dashpen1 = new BasicStroke(1.0F, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 7.0F, new float[]{3.0F, 6.0F}, 0.F);
    private final Stroke bdashpen1 = new BasicStroke(2.0F, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 7.0F, new float[]{3.0F, 6.0F}, 0.F);
    private final Stroke dashpen2 = new BasicStroke(1.0F, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 7.0F, new float[]{9F, 4F, 1F, 4F}, 0.F);
    private final Stroke bdashpen2 = new BasicStroke(2.0F, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 7.0F, new float[]{9F, 4F, 1F, 4F}, 0.F);

    private A_Events evtmx = null;
    private java.awt.Point dragpt = null;
    private java.awt.Rectangle selrect = null;
    private F_DiagHisto FHisto = null;

    /**
     * reates new form P_Diagram
     *
     * @param mx the matrix supported by this Diagram
     */
    public P_Diagram(A_Events mx) {
        if (mx != null) {
            evtmx = mx;
        }
        initComponents();
        init();
    }

    public P_Diagram() {
        evtmx = new A_Events();
        initComponents();
        init();
    }

    /**
     * Change the Event/state Matrix for this diagram
     *
     * @param mx the matrix to set
     */
    public void setEvents(A_Events mx) {
        clear();
        evtmx = mx;
        if (evtmx != null) {
            evtmx.setPanelButton(false);
            if (evtmx.getEventlist().size() > 0) {
                A_Element firstevt = evtmx.getEventlist().get(0);
                jRollingButton1.setValue(firstevt.getConcurent());
                jRollingButton1.setToolTipText(firstevt.getConcurentLabel());
            }
            jCB_ObjectEditor1.setValue(evtmx);
            evtmx.addTableModelListener(new javax.swing.event.TableModelListener() {
                @Override
                public void tableChanged(TableModelEvent e) {
                    eventsModelDataChange();
                }
            });
        }
        /*
         for (javax.swing.JComponent c:evtmx.getStatuslist()){
         c.setToolTipText(""+evtmx.getStatuslist().indexOf(c));
         }
         for (javax.swing.JComponent c:evtmx.getEventlist()){
         c.setToolTipText(""+evtmx.getEventlist().indexOf(c));
         }*/
        majtree();
        if (FHisto != null && FHisto.isVisible() && evtmx != null) {
            FHisto.setEvents(evtmx.getHistoModel());
        }
        getConfObjects();
        setDiag_names(getDiag_name());
        paintDiagram();
    }

    /**
     * provide the State/Event Matrix used by this Diagram
     *
     * @return
     */
    public A_Events getEvents() {
        return evtmx;
    }

    /**
     * generic init() GUI after initComponent() for this care that a lot of
     * Mouse/key actionsListner are added
     *
     */
    private void init() {
        jButton1.setIcon(TUtils.getIcon("Empty.gif"));
        jButton2.setIcon(TUtils.getIcon("Add.gif"));
        jButton3.setIcon(TUtils.getIcon("Remove.gif"));
        jButton7.setIcon(TUtils.getIcon("Events-reset.gif"));
        jButton8.setIcon(TUtils.getIcon("Xedit.gif"));
        jButton4.setIcon(TUtils.getIcon("Save.gif"));
        jButton5.setIcon(TUtils.getIcon("Open.gif"));
        jRollingButton1.addValuedItem(TUtils.getIcon("Concur_off.gif"), 0);
        jRollingButton1.addValuedItem(TUtils.getIcon("Concur_weight.gif"), 1);
        jRollingButton1.addValuedItem(TUtils.getIcon("Concur_on.gif"), 2);
        jToggleButton2.setIcon(TUtils.getIcon("Viewoff.gif"));
        jToggleButton2.setSelectedIcon(TUtils.getIcon("View.gif"));
        jToggleButton1.setIcon(TUtils.getIcon("Events-buid_off.gif"));
        jToggleButton1.setSelectedIcon(TUtils.getIcon("Events-buid_on.gif"));
        jTree1.setModel(null);
        jTree1.setEditable(true);
        jTree1.setCellRenderer(new DiagramRenderer());
        jTree1.setCellEditor(new DiagramEditor());
        jTree1.setInvokesStopCellEditing(true);
        jSlider1.getModel().addChangeListener(new javax.swing.event.ChangeListener() {
            int oldvalue = jSlider1.getModel().getValue();

            @Override
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                int newValue = jSlider1.getModel().getValue();
                if (newValue != oldvalue) {
                    paintDiagram();
                    oldvalue = newValue;
                    if (evtmx != null) {
                        evtmx.setZoom(newValue);
                    }
                }
            }
        });
        jCB_ObjectEditor1.setEditable(true);
        jCB_ObjectEditor1.setEditing(true);
        jPanel1.removeAll();
        jPanel1.addKeyListener(new java.awt.event.KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                super.keyPressed(e);
                if (e.getKeyCode() == java.awt.event.KeyEvent.VK_ESCAPE) {
                    setToolBar(!isToolBar());
                }
            }
        });
        jPanel1.addMouseListener(new java.awt.event.MouseAdapter() {
            @Override
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                for (Component c : jPanel1.getComponents()) {
                    if (c.getBounds().contains(evt.getPoint())) {
                        if (c instanceof A_Element) {
                            if (evt.isAltDown()) {
                                send2mx((A_Element) c);
                            } else if (evt.isControlDown() == false) {
                                selectAll(false);
                                selected.add((A_Element) c);
                                ((A_Element) c).setSelected(true);
                                majtree();
                            }
                            return;
                        }
                    }
                }

            }

            @Override
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                dragpt = null;
                if (selrect != null) {
                    for (Component c : jPanel1.getComponents()) {
                        if (selrect.contains(c.getBounds())) {
                            if (c instanceof A_Element) {
                                A_Element elt = (A_Element) c;
                                if (evt.isControlDown()) {
                                    if (elt.isSelected()) {
                                        selected.remove(elt);
                                        elt.setSelected(false);
                                    } else {
                                        selected.add(elt);
                                        elt.setSelected(true);
                                    }
                                } else {
                                    selected.add(elt);
                                    elt.setSelected(true);
                                }
                            }
                        }
                    }
                    // store positions
                    for (A_Element elt : selected) {
                        elt.recposition = elt.getBounds();
                    }
                    majtree();
                    checkPanelSize();
                    selrect = null;
                    repaint();
                }
            }

            @Override
            public void mousePressed(java.awt.event.MouseEvent evt) {
                if (evt.isAltDown()) {
                    return;
                }
                boolean found = false;
                for (Component c : jPanel1.getComponents()) {
                    if (c.getBounds().contains(evt.getPoint()) && c instanceof A_Element) {
                        A_Element elt = (A_Element) c;
                        found = true;
                        if (evt.isControlDown()) {
                            if (selected.contains(elt) == false) {
                                selected.add(elt);
                                elt.setSelected(true);
                            } else {
                                selected.remove(elt);
                                elt.setSelected(false);
                            }
                            majtree();
                            break;
                        } else if (selected.isEmpty()) {
                            // selectAll(false);
                            selected.add(elt);
                            elt.setSelected(true);
                            selrect = null;
                            majtree();
                            repaint();
                        }
                    }
                }
                if (found == false && evt.isControlDown() == false) {
                    selectAll(false);
                }
                repaint();
            }
        });

        jPanel1.addMouseMotionListener(
                new java.awt.event.MouseMotionAdapter() {
                    @Override
                    public void mouseDragged(java.awt.event.MouseEvent evt) {
                        if (dragpt == null && selrect == null) {
                            boolean boolsel = false;
                            A_Element unsel = null;
                            for (Component c : jPanel1.getComponents()) {
                                if (c.getBounds().contains(evt.getPoint())) {
                                    if (c instanceof A_Element) {
                                        A_Element a = (A_Element) c;
                                        if (a.isSelected()) {
                                            boolsel = true;
                                        } else {
                                            unsel = a;
                                        }
                                    }
                                }
                            }
                            if (boolsel == false) {
                                selectAll(false);
                                if (unsel != null) {
                                    unsel.setSelected(true);
                                    selected.add(unsel);
                                }
                            }
                            for (Component c : jPanel1.getComponents()) {
                                if (c.getBounds().contains(evt.getPoint())) {
                                    if (c instanceof A_Element) {
                                        dragpt = evt.getPoint();
                                        return;
                                    }
                                }
                            }
                        } else if (dragpt != null && selrect == null) {
                            for (A_Element c : selected) {
                                java.awt.Point p = c.getLocation();
                                p.translate(evt.getPoint().x - dragpt.x, evt.getPoint().y - dragpt.y);
                                c.setLocation(p);
                                float magn = (jSlider1.getModel().getValue() - 5) * 0.04f;
                                c.recposition.x = Math.round(c.getBounds().x / magn);//+ beforeRect.x;
                                c.recposition.y = Math.round(c.getBounds().y / magn);//+ beforeRect.y;

                            }
                            dragpt = evt.getPoint();

                            repaint();
                            checkPanelSize();
                            jScrollPane1.repaint();
                            return;
                        }
                        if (selrect == null) {
                            selrect = new java.awt.Rectangle(evt.getPoint(), new java.awt.Dimension());
                            dragpt = evt.getPoint();
                        } else if (dragpt != null) {
                            int newX = dragpt.x;
                            int newY = dragpt.y;
                            if (dragpt.x > evt.getPoint().getX()) {
                                newX = evt.getPoint().x;
                            }
                            if (dragpt.y > evt.getPoint().getY()) {
                                newY = evt.getPoint().y;
                            }
                            selrect = new java.awt.Rectangle(newX, newY, (int) Math.abs(dragpt.x - evt.getPoint().getX()), (int) Math.abs(dragpt.y - evt.getPoint().getY()));
                            for (Component c : jPanel1.getComponents()) {
                                if (c instanceof A_Element) {
                                    ((A_Element) c).setSelected(selrect.contains(c.getBounds()));
                                }
                            }
                            repaint();
                        } else {
                            System.out.println(" dragPt should be NULL???");
                        }
                    }
                });
        setEvents(evtmx);
        setEditTree(false);
        jPanel1.repaint();
    }

    public void setEditTable(boolean bool) {
        jCB_ObjectEditor1.setEnabled(bool);
    }

    /**
     * ONLY for Main configuration future facilities (find config element
     * &methodes) => Not used => nothing to do here => should be in T_CONF?
     */
    public void getConfObjects() {
        java.util.List<String> jcbnames = new java.util.ArrayList<>();
        java.util.List<Class<?>> jcbObj = new java.util.ArrayList<>();
        jcbObj.add(com.jdcm.eltrx.C_Main.class);
        jcbnames.add("Main");
        // TUtils.getIcon(ui)
        if (evtmx != null && evtmx.getTasklist() != null && evtmx.getTasklist().length > 0) {
            for (A_Task[] tasklist : evtmx.getTasklist()) {
                for (int j = 0; j < evtmx.getTasklist()[0].length; j++) {
                    for (C_Actiondef act : tasklist[j].getActions()) {
                        if (act.getMethodName() != null
                                && act.getMethodName().compareTo("addCX") == 0
                                && act.getParamList() != null
                                && act.getParamList().size() == 3) {
                            Object type = act.getParamList().get(0);
                            Object name = act.getParamList().get(1);
                            if (type instanceof String && name instanceof String) {
                                if (type.toString().compareTo("Port") == 0) {
                                    jcbObj.add(com.jdcm.eltrx.C_Port.class);
                                } else if (type.toString().compareTo("Machine") == 0) {
                                    jcbObj.add(com.jdcm.eltrx.C_Machine.class);
                                }
                                jcbnames.add(name.toString());

                            }
                        }
                    }
                }
            }
        }
        com.jdcm.common.Options_Model options = new com.jdcm.common.Options_Model(this);
        javax.swing.DefaultComboBoxModel<String> cm = new javax.swing.DefaultComboBoxModel<>(jcbnames.toArray(new String[]{}));
        options.addOption("Diag_names", cm, "ObjectsasString", "", true, true, "List of created Object in config");
        options.addOption("Diag_type", new javax.swing.DefaultComboBoxModel<>(jcbObj.toArray(new Class<?>[]{})), "Classe", "", true, true, "List of created classes");
        options.addOption("Diag_meth", "", "methods", "", true, true, "List method for selected Class");
        options.addOption("Diag_txt", "", "string", "commad", true, true, "text translation of event");
        jCB_ObjectEditor2.setValue(options);

    }

    public String getDiag_name() {
        Object opt = jCB_ObjectEditor2.getValue();
        if (opt != null && opt instanceof com.jdcm.common.Options_Model) {
            com.jdcm.common.Options_Model option = (com.jdcm.common.Options_Model) opt;
            javax.swing.DefaultComboBoxModel<String> cm = option.getValuefor("Diag_names", new javax.swing.DefaultComboBoxModel<String>());
            return cm.getSelectedItem().toString();
        }
        return null;
    }

    /**
     * Diag nameswas an idea that simple configuratio need names & methode of
     * implmented element => NOT USED
     *
     * @param name
     */
    public void setDiag_names(String name) {
        if (name != null) {
            com.jdcm.common.Options_Model option = (com.jdcm.common.Options_Model) jCB_ObjectEditor2.getValue();
            javax.swing.DefaultComboBoxModel<String> cm = option.getValuefor("Diag_names", new javax.swing.DefaultComboBoxModel<String>());
            if (cm != null) {
                int idx = cm.getIndexOf(name);
                if (idx >= 0) {
                    javax.swing.DefaultComboBoxModel<Class<?>> ct = option.getValuefor("Diag_type", new javax.swing.DefaultComboBoxModel<Class<?>>());
                    Class<?> cl = ct.getElementAt(idx);
                    ct.setSelectedItem(cl);
                    java.lang.reflect.Method[] ms = cl.getMethods();
                    if (ms != null && ms.length > 0) {
                        option.setValuefor("Diag_meth", new javax.swing.DefaultComboBoxModel<>(ms));
                        setDiag_meth(ms[0]);
                    }
                }
            }
        }
    }

    public void setDiag_meth(java.lang.reflect.Method meth) {
        if (meth != null) {
            com.jdcm.common.Options_Model option = (com.jdcm.common.Options_Model) jCB_ObjectEditor2.getValue();
            option.setValuefor("Diag_txt", getDiag_name() + "." + meth.getName());
        }
    }

    /**
     * used to force event/status on the event table NB : do no perform
     * action(config/proto/machine/..)
     *
     * @param elt the element to be received
     */
    private void send2mx(A_Element elt) {
        if (elt.isState()) {
            elt.setActive(!elt.isActive());
            // TODO : for headers of evtmx to be repaint()
            evtmx.fireTableStructureChanged();
        } else {
            // if (elt.getcontent() instanceof C_Eventdef){
            //     evtmx.getTask(((C_Eventdef)elt.getcontent()).);
            //  }else{
            evtmx.getTask(elt.getcontent());
            //   }
        }

    }

    /**
     * hide/show the toolbas
     *
     * @param bool true for show
     */
    public void setToolBar(boolean bool) {
        jToolBar1.setVisible(bool);
    }

    /**
     * return true if toolbar is shown
     *
     * @return
     */
    public boolean isToolBar() {
        return jToolBar1.isVisible();
    }

    /**
     * show/hide the Editing pannel for Action/state/Task
     *
     * @param bool true for show panel
     */
    public void setEditTree(boolean bool) {
        if (bool) {
            if (jSplitPane1.getDividerLocation() > jSplitPane1.getHeight() - 20) {//20 is an estimation of bordre, dividerSize,..??..
                jSplitPane1.setDividerLocation(0.8);
            }
            jSplitPane1.setDividerSize(5);
        } else {
            jSplitPane1.setDividerLocation(1.0);
            jSplitPane1.setDividerSize(0);
        }
        jSplitPane1.setEnabled(bool);
    }

    /**
     * return the show/hide status of the Editing Panel
     *
     * @return true if EditPanel is showed
     */
    public boolean isEditTree() {
        return jToggleButton1.isSelected();
    }

    /**
     * show/hide the histo (log) panel of the Diagram
     *
     * @param bool true to show logs
     */
    public void setHisto(boolean bool) {
        if (bool && evtmx == null) {
            setHisto(false);
            return;
        } else if (bool == false) {
            if (FHisto != null) {
                FHisto.setVisible(false);
            }
            FHisto = null;
        } else if (evtmx != null && bool) {
            FHisto = new F_DiagHisto(this);
            FHisto.setVisible(true);
            FHisto.addWindowListener(new java.awt.event.WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent we) {
                    setHisto(false);
                }
            });
        }
        if (jToggleButton2.isSelected() != bool) {
            jToggleButton2.setSelected(bool);
        }
        if (evtmx.isHisto() != bool) {
            evtmx.setHisto(bool);
        }

    }

    /**
     * reset the diagram and manage GUI
     */
    public void clear() {
        java.util.ArrayList<A_Element> empty = new java.util.ArrayList<>();
        evtmx = new A_Events(empty, new java.util.ArrayList<>(), new A_Task[0][0]);
        jCB_ObjectEditor1.setValue(evtmx);
        jPanel1.removeAll();
        jPanel1.repaint();
        selected.clear();
        majtree();
        paintDiagram();
        repaint();
    }

    /**
     * return the Rectange of a List of component to compute Zoom (Uper Left
     * Corner / Buttom Right) could be an Util function BUT no simple conversion
     * beetween List<A_Element> et List<componment>
     */
    private java.awt.Rectangle getCorners(java.awt.Component[] comps) {
        java.awt.Rectangle rect = null;
        for (java.awt.Component cmp : comps) {
            if (cmp.getBounds() != null) {
                if (rect == null) {
                    rect = cmp.getBounds();
                } else {
                    rect = rect.union(cmp.getBounds());
                }
            }
        }
        return rect;
    }

    private java.awt.Rectangle getCorners(List<A_Element> elts) {
        java.awt.Component[] comps = new java.awt.Component[elts.size()];
        int i = 0;
        for (A_Element elt : elts) {
            comps[i] = elt;
            if (elt.recposition != null) {
                comps[i].setBounds(elt.recposition);
            }
            i++;
        }
        return getCorners(comps);
    }

    /**
     * Paint the Diagram with Action/state/Tasks
     */
    private void paintDiagram() {
        float magn = (jSlider1.getModel().getValue() - 5) * 0.04f;
        int size = jSlider1.getModel().getValue();
        int marg = size / 5;
        List<A_Element> temp = new java.util.ArrayList<>();
        temp.addAll(evtmx.getStatuslist());
        temp.addAll(evtmx.getEventlist());
        // find upperleft corner
        java.awt.Rectangle beforeRect = getCorners(temp);
        if (beforeRect == null) {
            beforeRect = new java.awt.Rectangle(0, 0, 10, 10);
        }
        for (A_Element st : temp) {
            if (st.recposition == null) {
                if (st.isState()) {
                    st.recposition = new Rectangle(5, 5, size, size);
                    st.recposition.translate(0, size + marg);
                } else {
                    st.recposition = new Rectangle(5, 5, size, size / 2);
                }
                while (!isFreePosition(st.recposition.getLocation())) {
                    if (jPanel1.getWidth() <= st.recposition.y + size + marg) {
                        jPanel1.setSize(st.recposition.y + size + marg, jPanel1.getHeight());
                    }
                    st.recposition.translate(size + marg, 0);
                }
            } else {
                if (st.isState()) {
                    st.recposition = new Rectangle(st.recposition.x, st.recposition.y, size, size);
                } else {
                    st.recposition = new Rectangle(st.recposition.x, st.recposition.y, size, size / 2);
                }

            }
            if (st.getParent() == null) {
                jPanel1.add(st);
                st.setVisible(true);
            } else {
                int shiftedx = Math.round(magn * (st.recposition.x - beforeRect.x));
                int shiftedy = Math.round(magn * (st.recposition.y - beforeRect.y));
                st.setBounds(beforeRect.x + shiftedx, beforeRect.y + shiftedy, st.recposition.width, st.recposition.height);
            }
        }

        checkPanelSize();
        jPanel1.repaint();
    }

    private void checkPanelSize() {
        java.awt.Rectangle afterRect = getCorners(jPanel1.getComponents());
        int marg = 10;
        if (afterRect != null) {
            // add marg for Task lines
            afterRect.setSize(afterRect.width + marg, afterRect.height + marg);
            //   jPanel1.setSize(afterRect.getSize());
            jPanel1.setPreferredSize(afterRect.getSize());
            jScrollPane1.doLayout();
        }
    }

    /**
     * check if a location is free of events/states on the diagram
     *
     * @param pt the point to check
     * @return true if not element on this place
     */
    private boolean isFreePosition(java.awt.Point pt) {
        boolean res = true;
        for (Component c : jPanel1.getComponents()) {
            if (c instanceof A_Element) {
                if (((A_Element) c).recposition.contains(pt)) {
                    return false;
                }
            } else {
                if (c.getBounds().contains(pt)) {
                    return false;
                }
            }
        }
        return res;
    }

    /**
     * select all Actions/state of this diagram
     *
     * @param bool
     */
    private void selectAll(boolean bool) {
        for (A_Element a : selected) {
            a.setSelected(false);
        }
        if (bool) {
            for (Component c : jPanel1.getComponents()) {
                if (c instanceof A_Element) {
                    selected.add((A_Element) c);
                }
            }
        } else {
            selected.clear();
        }
        majtree();
        repaint();
    }

    /**
     * Update the tree to match with selected element (Action/state)
     */
    private void majtree() {
        DefaultMutableTreeNode tr = new DefaultMutableTreeNode("selection");
        DefaultTreeModel treeModel = new javax.swing.tree.DefaultTreeModel(tr);

        for (A_Element elt : selected) {
            DefaultMutableTreeNode ty = new DefaultMutableTreeNode(elt);//elt.gettreenode();
            int idx = evtmx.getEventlist().indexOf(elt);
            if (idx >= 0) {
                A_Task[] tsks = evtmx.getTasklist()[idx];
                if (tsks != null && tsks.length > 0) {
                    for (A_Task tsk : tsks) {
                        if (tsk.isEmpty() == false) {
                            java.util.List<A_Element> prevs = evtmx.getconcurents(elt, tsk);
                            if (prevs.isEmpty() == false) {
                                tsk.setSelected(true);
                                DefaultMutableTreeNode tty = tsk.getTreeNode();
                                ty.add(tty);
                            }
                        }
                    }
                }
            }
            tr.add(ty);
        }
        jTree1.setModel(treeModel);
        for (int i = 0; i < jTree1.getRowCount(); i++) {
            jTree1.expandRow(i);
        }

    }

    /**
     * Performed by changing Matrix table content set in this.setEvent()<br> for
     * diagram to match with the new Matrix
     */
    private void eventsModelDataChange() {
        for (int i = 0; i < evtmx.getEventlist().size(); i++) {
            if (i > 0) {
                if (evtmx.getEventlist().get(i).getConcurent() != evtmx.getEventlist().get(i - 1).getConcurent()) {
                    //NOK undefined
                    break;
                }
                if (i == evtmx.getEventlist().size() - 1 && evtmx.getEventlist().get(i).getConcurent() == evtmx.getEventlist().get(i - 1).getConcurent()) {
                    jRollingButton1.setEnabled(true);
                    jRollingButton1.setValue(evtmx.getEventlist().get(i).getConcurent());
                }
            }
        }
        paintDiagram();
        jPanel1.repaint();
    }

    /**
     * provide a frame that include this Diagram
     *
     * @return a Daigram Frame
     */
    static public javax.swing.JFrame getasFrame() {
        javax.swing.JFrame f = new javax.swing.JFrame();
        if (javax.swing.JFrame.getOwnerlessWindows().length == 1) {
            f.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
        }
        f.setBounds(10, 10, 300, 400);
        f.add(new P_Diagram());
        f.setIconImage(new D_Utils().getImage("Events.gif"));
        f.setTitle("Tasks Diagram");
        return f;
    }

    /**
     * used by running frame
     *
     * @param args no arguments used
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                javax.swing.JFrame f = getasFrame();
                f.setVisible(true);
            }
        });

    }

    /**
     * a Panel with a large paintComponent
     */
    class DiagramPanel extends javax.swing.JPanel {

        private static final long serialVersionUID = 1L;

        @Override
        public void paintComponent(Graphics g) {
            if (g == null) {
                return;
            }
            super.paintComponent(g);
            Graphics2D g2 = (Graphics2D) g;
            Stroke str = new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 10.0f);
            g2.setStroke(str);
            //--------------for selection of elements-------------------------
            if (selrect != null) {
                g2.setColor(new java.awt.Color(0, 255, 0, 20));
                g2.fillRect(selrect.x, selrect.y, selrect.width, selrect.height);
                g2.setColor(new java.awt.Color(0, 255, 0, 200));
                g2.drawRect(selrect.x, selrect.y, selrect.width, selrect.height);
                g2.setColor(Color.black);
            }
            //------------------for Diagram --------------------------------
            for (int i = 0; i < evtmx.getEventlist().size(); i++) {
                A_Element event = evtmx.getEventlist().get(i);
                for (int j = 0; j < evtmx.getStatuslist().size(); j++) {
                    A_Task task = evtmx.getTasklist()[i][j];
                    if (task.isEmpty() == false) {
                        Point2D p_tr = event.getLocation();
                        int shift = event.getWidth() / 2;
                        double trupy = p_tr.getY();
                        String eltdir = "up";
                        int sizearrow = 3;//* zoom
                        java.util.List<A_Element> prevs = evtmx.getconcurents(event, task);
                        if (prevs.isEmpty() == false) {
                            for (A_Element prev : prevs) {
                                task.setSelected(event.isSelected());
                                g2.setPaint(task.getColor());
                                if (task.isSelected()) {
                                    g2.setStroke(bbasicpen);
                                }
                                int shiftctrl = event.getWidth();
                                Point2D p_prv = prev.getLocation();
                                double prvupy = p_prv.getY() + prev.getHeight();
                                //--------------------check if this element is down previous one-------------------------
                                if (p_prv.getY() > p_tr.getY() - 2 * shift) {
                                    // go to buttom of p_tr && add an UP arrow
                                    eltdir = "down";
                                    Shape arrow1 = new Line2D.Double(p_tr.getX() + shift, trupy, p_tr.getX() + sizearrow + shift, trupy + sizearrow);
                                    g2.draw(arrow1);
                                    Shape arrow2 = new Line2D.Double(p_tr.getX() + shift, trupy, p_tr.getX() - sizearrow + shift, trupy + sizearrow);
                                    g2.draw(arrow2);
                                    trupy = p_tr.getY() + event.getHeight();
                                    shiftctrl = -shiftctrl;
                                }
                                int shift2 = prev.getWidth() / 2;
                                //--------------------draw curb from previous element-------------------------
                                Shape from = new CubicCurve2D.Double(p_tr.getX() + shift, trupy, p_tr.getX() + shift, trupy - shiftctrl, p_prv.getX() + shift2, prvupy + Math.abs(shiftctrl), p_prv.getX() + shift2, prvupy);
                                g2.draw(from);
                            }

                            for (A_Element nxtst : task.getNextStates()) {
                                if (event.getConcurent() == A_Element.CONCUR_WEIGHT
                                        && task.getMoreWeightNextState().equals(nxtst) == false
                                        && task.getNextStates().size() > 1) {
                                    if (task.isSelected()) {
                                        g2.setStroke(bdashpen1);
                                    } else {
                                        g2.setStroke(dashpen1);
                                    }
                                } else if (event.getConcurent() == A_Element.CONCUR_RAND
                                        && task.getNextStates().size() > 1) {
                                    if (task.isSelected()) {
                                        g2.setStroke(bdashpen2);
                                    } else {
                                        g2.setStroke(dashpen2);
                                    }
                                } else {
                                    if (task.isSelected()) {
                                        g2.setStroke(bbasicpen);
                                    } else {
                                        g2.setStroke(basicpen);
                                    }
                                }
                                Point2D p_nxt = nxtst.getLocation();
                                int shiftctrl = nxtst.getHeight();
                                double nxty = p_nxt.getY();
                                double trdwn = p_tr.getY() + event.getHeight();
                                if (eltdir.compareTo("down") == 0) {
                                    // go to buttom of p_tr && add an UP arrow
                                    trdwn = p_tr.getY();
                                    shiftctrl = -shiftctrl;
                                }
                                int shift2 = nxtst.getWidth() / 2;
                                //--------------------draw curb to next element-------------------------
                                Shape to = new CubicCurve2D.Double(p_tr.getX() + shift, trdwn, p_tr.getX() + shift, trdwn + shiftctrl, p_nxt.getX() + shift2, nxty - Math.abs(shiftctrl), p_nxt.getX() + shift2, nxty);
                                g2.draw(to);
                            }
                        }
                        g2.setStroke(basicpen);
                        g2.setPaint(Color.black);
                    }
                }
            }
        }
    }

    /**
     * Editor used only for Actions of Tasks that have to be stored. A_Elements
     * & Task are modified as an other Object by current ACE_ObjEditor
     *
     */
    class DiagramEditor extends com.jdcm.editors.ACE_ObjEditor<Object> {

        private static final long serialVersionUID = 1L;
        A_Task elt = null;
        int idx = -1;

        public DiagramEditor() {
            this.setEditing(true);
        }

        @Override
        public Component getTreeCellEditorComponent(JTree tree, Object value, boolean isSelected, boolean expanded, boolean leaf, int row) {
            if (value instanceof javax.swing.tree.DefaultMutableTreeNode) {
                javax.swing.tree.DefaultMutableTreeNode node = (javax.swing.tree.DefaultMutableTreeNode) value;
                if (node.getParent() != null && node.getParent() instanceof javax.swing.tree.DefaultMutableTreeNode) {
                    if (((javax.swing.tree.DefaultMutableTreeNode) node.getParent()).getUserObject() instanceof A_Task) {
                        elt = (A_Task) ((javax.swing.tree.DefaultMutableTreeNode) node.getParent()).getUserObject();
                    }
                    idx = ((javax.swing.tree.DefaultMutableTreeNode) node.getParent()).getIndex(node);
                }
            }
            return super.getTreeCellEditorComponent(tree, value, isSelected, expanded, leaf, row);
        }

        @Override
        public Object getCellEditorValue() {
            Object nobj = super.getCellEditorValue();
            if (idx >= 0 && elt != null && nobj instanceof C_Actiondef) {
                elt.getActions().set(idx, (C_Actiondef) nobj);
                idx = -1;
            }
            return nobj;
        }
    }

    /**
     * A CellRender for the Editor to show MutableTreNode.UserObject() & Label
     */
    class DiagramRenderer extends javax.swing.CellRendererPane implements javax.swing.tree.TreeCellRenderer {

        private static final long serialVersionUID = 1L;
        com.jdcm.editors.ACE_ObjRenderer<Object> a = new com.jdcm.editors.ACE_ObjRenderer<>();

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            Object val = null;
            if (value instanceof DefaultMutableTreeNode) {
                val = ((DefaultMutableTreeNode) value).getUserObject();
            }
            if (val instanceof java.awt.Label) {
                javax.swing.JLabel box = new javax.swing.JLabel(((java.awt.Label) val).getText());
                box.setForeground(((java.awt.Label) val).getForeground());
                box.setBorder(new javax.swing.border.EmptyBorder(0, 0, 0, 0));
                return box;
            } else {
                return a.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
            }

        }
    }

    private void removeElement() {
        //REMOVE
        if (selected.size() == 2
                && selected.get(0).isState()
                && selected.get(1).isState() == false) {
            A_Element event = selected.get(1);
            evtmx.getTasklist()[evtmx.getEventlist().indexOf(event)][evtmx.getStatuslist().indexOf(selected.get(0))] = new A_Task();
        } else if (selected.size() == 3
                && selected.get(0).isState()
                && selected.get(1).isState() == false
                && selected.get(2).isState()) {
            A_Element event = selected.get(1);
            A_Task a = evtmx.getTasklist()[evtmx.getEventlist().indexOf(event)][evtmx.getStatuslist().indexOf(selected.get(0))];
            // if only one => set a new task OR if several NextStates => only remove selected next State
            if (a.getNextStates() != null && a.getNextStates().isEmpty() == false) {
                a.getNextStates().remove(selected.get(2));
            }
            if (a.getNextStates().isEmpty()) {
                evtmx.getTasklist()[evtmx.getEventlist().indexOf(event)][evtmx.getStatuslist().indexOf(selected.get(0))] = new A_Task();
            }
        } else {
            for (A_Element elt : selected) {
                if (elt.isState()) {
                    // need to detele relating tasks
                    evtmx.removeStatus(elt);
                } else {
                    evtmx.removeEvt(elt);
                }
                if (elt.getParent() == jPanel1) {
                    jPanel1.remove(elt);
                }
            }
        }
        selectAll(false);
        paintDiagram();
        repaint();
    }

    private void addElement() {
        if (selected.isEmpty()) {
            if (evtmx.getEventlist().isEmpty()) {
                evtmx.addElement(false, false, "st");
            } else {
                // addElement(isState,isInit,Contents)
                evtmx.addElement(true, evtmx.getStatuslist().isEmpty(), "st");
            }
        }
        if (selected.size() == 1) {
            evtmx.addElement(selected.get(0).isState(), selected.get(0).isInitial(), selected.get(0).getcontent());
            if (selected.get(0).isState()) {
                // copy State Weight
                evtmx.getStatuslist().get(evtmx.getStatuslist().size() - 1).setWeight(selected.get(0).getWeight());
            } else {
                // copy Concurency Type (FULL/RANDOM/WEIGHT)
                evtmx.getEventlist().get(evtmx.getEventlist().size() - 1).setConcurent(selected.get(0).getConcurent());
            }
        } else if (selected.size() == 2) {
            evtmx.setTask(selected.get(0), selected.get(1));
        } else if (selected.size() == 3) {
            evtmx.setTask(selected.get(0), selected.get(1), selected.get(2));
        }
        selectAll(false);
        paintDiagram();
        repaint();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToolBar1 = new javax.swing.JToolBar();
        jButton5 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jButton8 = new javax.swing.JButton();
        jSlider1 = new javax.swing.JSlider();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jButton7 = new javax.swing.JButton();
        jRollingButton1 = new com.jdcm.common.G_JRollingButton();
        jToggleButton2 = new javax.swing.JToggleButton();
        jToggleButton1 = new javax.swing.JToggleButton();
        jCB_ObjectEditor2 = new com.jdcm.editors.JCB_ObjectEditor();
        jCB_ObjectEditor1 = new com.jdcm.editors.JCB_ObjectEditor();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jPanel1 = new DiagramPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        jTree1 = new javax.swing.JTree();

        setLayout(new java.awt.BorderLayout());

        jToolBar1.setRollover(true);

        jButton5.setToolTipText("open");
        jButton5.setBorderPainted(false);
        jButton5.setFocusable(false);
        jButton5.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton5.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton5);

        jButton4.setToolTipText("save");
        jButton4.setBorderPainted(false);
        jButton4.setFocusable(false);
        jButton4.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton4.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton4);

        jButton8.setToolTipText("Edit");
        jButton8.setBorderPainted(false);
        jButton8.setFocusable(false);
        jButton8.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton8.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton8ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton8);

        jSlider1.setMinimum(10);
        jSlider1.setValue(30);
        jToolBar1.add(jSlider1);

        jButton2.setToolTipText("add");
        jButton2.setBorderPainted(false);
        jButton2.setFocusable(false);
        jButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton2);

        jButton3.setToolTipText("remove");
        jButton3.setBorderPainted(false);
        jButton3.setFocusable(false);
        jButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton3);

        jButton1.setToolTipText("Clear");
        jButton1.setBorderPainted(false);
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton1);

        jButton7.setToolTipText("Reset");
        jButton7.setBorderPainted(false);
        jButton7.setFocusable(false);
        jButton7.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton7.setMaximumSize(new java.awt.Dimension(25, 17));
        jButton7.setMinimumSize(new java.awt.Dimension(25, 17));
        jButton7.setPreferredSize(new java.awt.Dimension(25, 17));
        jButton7.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton7ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton7);

        jRollingButton1.setToolTipText("CONCURENCY");
        jRollingButton1.setBorderPainted(false);
        jRollingButton1.setFocusable(false);
        jRollingButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jRollingButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jRollingButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRollingButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jRollingButton1);

        jToggleButton2.setToolTipText("Histo");
        jToggleButton2.setBorderPainted(false);
        jToggleButton2.setFocusable(false);
        jToggleButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToggleButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(jToggleButton2);

        jToggleButton1.setToolTipText("Edit Elements");
        jToggleButton1.setBorderPainted(false);
        jToggleButton1.setFocusable(false);
        jToggleButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToggleButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jToggleButton1);

        jCB_ObjectEditor2.setAsButton(true);
        jToolBar1.add(jCB_ObjectEditor2);

        jCB_ObjectEditor1.setEditable(true);
        jCB_ObjectEditor1.setEditing(false);
        jToolBar1.add(jCB_ObjectEditor1);

        add(jToolBar1, java.awt.BorderLayout.NORTH);

        jSplitPane1.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        jSplitPane1.setDividerLocation(150);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                jSplitPane1ComponentResized(evt);
            }
        });

        jScrollPane1.setBorder(null);

        jPanel1.setAutoscrolls(true);
        jPanel1.addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                jPanel1MouseWheelMoved(evt);
            }
        });
        jPanel1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jPanel1MouseClicked(evt);
            }
        });
        jPanel1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                jPanel1KeyPressed(evt);
            }
        });
        jPanel1.setLayout(null);
        jScrollPane1.setViewportView(jPanel1);

        jSplitPane1.setTopComponent(jScrollPane1);

        jScrollPane3.setViewportView(jTree1);

        jSplitPane1.setRightComponent(jScrollPane3);

        add(jSplitPane1, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        clear();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        addElement();
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        removeElement();
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        String filename = TUtils.getUIsaveFile(this, true, null);
        evtmx.toFile(filename);
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
        //OPEN
        clear();
        java.io.File xmlfile = TUtils.getUIreadFile(this, JFileChooser.FILES_AND_DIRECTORIES, null, true);
        evtmx.createfromFile(xmlfile);
        paintDiagram();
        jSlider1.setValue(evtmx.getZoom());
        // repaint();
        if (evtmx.getEventlist().size() > 0) {
            A_Element firstevt = evtmx.getEventlist().get(0);
            jRollingButton1.setValue(firstevt.getConcurent());
            jRollingButton1.setToolTipText(firstevt.getConcurentLabel());
        }
    }//GEN-LAST:event_jButton5ActionPerformed

    private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton7ActionPerformed
        evtmx.reset();

    }//GEN-LAST:event_jButton7ActionPerformed

    private void jToggleButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton2ActionPerformed
        setHisto(jToggleButton2.isSelected());
    }//GEN-LAST:event_jToggleButton2ActionPerformed

    private void jRollingButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRollingButton1ActionPerformed
        //CONCURENCY 
        int val = (Integer) jRollingButton1.getValue();
        if (selected.isEmpty()) {
            for (A_Element event : evtmx.getEventlist()) {
                event.setConcurent(val);
            }
        } else {
            for (A_Element event : selected) {
                if (event.isState() == false) {
                    event.setConcurent(val);
                } else {
                    event.setWeight(val);
                }
            }
        }
        if (evtmx.getEventlist().size() > 0) {
            jRollingButton1.setToolTipText(evtmx.getEventlist().get(0).getConcurentLabel());
        }
        this.repaint();
    }//GEN-LAST:event_jRollingButton1ActionPerformed

    private void jButton8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton8ActionPerformed
        //EDIT
        if (evtmx != null) {
            final com.jdcm.editors.P_XML a = new com.jdcm.editors.P_XML(false);
            a.displayTree(evtmx.toTree());
            a.openFrame();
            a.getFrame().addWindowListener(new java.awt.event.WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent e) {
                    if (e.getSource() == a.getFrame()) {
                        //  evtmx = new A_Events();
                        clear();
                        evtmx.createfromTree(a.getTreeModel());
                        paintDiagram();
                        repaint();
                    }
                    super.windowClosing(e);
                }
            });
        }
    }//GEN-LAST:event_jButton8ActionPerformed

    private void jToggleButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton1ActionPerformed
        setEditTree(isEditTree());
    }//GEN-LAST:event_jToggleButton1ActionPerformed

    private void jSplitPane1ComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_jSplitPane1ComponentResized
        setEditTree(this.isEditTree());
    }//GEN-LAST:event_jSplitPane1ComponentResized

    private void jPanel1KeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jPanel1KeyPressed
        removeElement();
    }//GEN-LAST:event_jPanel1KeyPressed

    private void jPanel1MouseWheelMoved(java.awt.event.MouseWheelEvent evt) {//GEN-FIRST:event_jPanel1MouseWheelMoved
        jSlider1.setValue(jSlider1.getValue() + evt.getWheelRotation());
    }//GEN-LAST:event_jPanel1MouseWheelMoved

    private void jPanel1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jPanel1MouseClicked

        // try to select Tasks...
        if (evtmx == null) {
            return;
        }
        Color pointColor = TUtils.getNearColored(jPanel1, evt.getPoint(),4,null);
        if (pointColor != null) {
            // Search colored matching task and select Matching Event
            int row = -1;
            for (A_Task[] a : evtmx.getTasklist()) {
                row++;
                for (A_Task b : a) {
                    if (b.isEmpty() == false
                            && b.getColor().equals(pointColor)
                            && b.isSelected() == false) {
                        A_Element elt = evtmx.getEventlist().get(row);
                        elt.setSelected(true);
                        selected.add(elt);
                    }
                }
            }
            // Update Selection Graphics & tree
            majtree();
            repaint();
        }
    }//GEN-LAST:event_jPanel1MouseClicked

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton7;
    private javax.swing.JButton jButton8;
    private com.jdcm.editors.JCB_ObjectEditor jCB_ObjectEditor1;
    private com.jdcm.editors.JCB_ObjectEditor jCB_ObjectEditor2;
    private javax.swing.JPanel jPanel1;
    private com.jdcm.common.G_JRollingButton jRollingButton1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JSlider jSlider1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JToggleButton jToggleButton1;
    private javax.swing.JToggleButton jToggleButton2;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JTree jTree1;
    // End of variables declaration//GEN-END:variables
}
