package com.jdcm.editors;

import java.awt.Color;
import java.awt.Component;
import javax.swing.JList;
import javax.swing.JTree;
import javax.swing.table.TableCellRenderer;
import javax.swing.DefaultComboBoxModel;
import javax.swing.AbstractSpinnerModel;
import com.jdcm.common.C_Packet;
import java.sql.SQLException;
import java.util.Enumeration;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * ObjectRender.java Created on 1 nov. 2007, 16:11:13
 * 
 * Cell Render to display used Classes not manager by default editor<br>
 *
 * @author Marc GIBOIN
 */
public class ACE_ObjRenderer<E extends Object> extends javax.swing.CellRendererPane implements TableCellRenderer, TreeCellRenderer, javax.swing.ListCellRenderer<E> {
    // for serializable interface
    // !!! renders are building a new Render each repaint()... is it possible to reuse previous one??

    private static final long serialVersionUID = 1L;
    private final int source = -1;

    public ACE_ObjRenderer() {
        super();
    }
    public Component getRendererComponent(Object obj){
        return getTableCellRendererComponent(null,obj,false,false,0,0);
    }
    @Override
    public Component getTableCellRendererComponent(
            javax.swing.JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col) {
        Component Compnt = null;

        if (value == null) {
            javax.swing.JLabel box = new javax.swing.JLabel();
            box.setText("null");
            box.setEnabled(false);
            box.setOpaque(true);
            Compnt = box;
        }else{
            try {
                if (value.getClass().getName().startsWith("com.jdcm.common.Options_Model$OM_Label")) {
                    Compnt = (javax.swing.JLabel) value;
                } else if (value.getClass().getName().startsWith("jpcap.NetworkInterface") || value.getClass().getName().startsWith("usb.")) {
                    JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                    Compnt = editor;
                } else if (value.getClass().getName().startsWith("gnu.io.CommPortIdentifier")) {
                    javax.swing.JLabel box = new javax.swing.JLabel();
                    box.setOpaque(true);
                    box.setText(((gnu.io.CommPortIdentifier) value).getName());
                    String typ = "type";
                    switch (((gnu.io.CommPortIdentifier) value).getPortType()) {
                        case gnu.io.CommPortIdentifier.PORT_I2C:
                            typ = "I2C";
                            break;
                        case gnu.io.CommPortIdentifier.PORT_PARALLEL:
                            typ = "PORT_PARALLEL";
                            break;
                        case gnu.io.CommPortIdentifier.PORT_RAW:
                            typ = "PORT_RAW";
                            break;
                        case gnu.io.CommPortIdentifier.PORT_RS485:
                            typ = "PORT_RS485";
                            break;
                        case gnu.io.CommPortIdentifier.PORT_SERIAL:
                            typ = "PORT_SERIAL";
                            break;
                    }
                    box.setToolTipText(((gnu.io.CommPortIdentifier) value).getName() + ":" + typ);
                    if (((gnu.io.CommPortIdentifier) value).isCurrentlyOwned()) {
                        box.setForeground(Color.red);
                        box.setToolTipText(box.getToolTipText() + ":" + ((gnu.io.CommPortIdentifier) value).getCurrentOwner());
                    }
                    Compnt = box;
                }
            } catch (java.lang.NoClassDefFoundError er) {
                //  System.out.println("usb or jpcap seems not managed: " + er.getMessage());
            } catch (RuntimeException ex) {
                //    System.out.println("usb or jpcap Pb with code source " + ex.getMessage());
            } catch (java.lang.Throwable t) {
                //   System.out.println("usb or jpcap Pb  other " + t.getMessage());
            }
            if (Compnt != null) {
            } else if (value.getClass().getName().startsWith("com.jdcm.charts")) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                editor.displayBorder(false);
                Compnt = editor;
            } else if (value instanceof Number || value instanceof String) {
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setOpaque(true);
                box.setText(value.toString());
                box.setToolTipText(value.getClass().getSimpleName());
                Compnt = box;
            } else if (value instanceof com.jdcm.diagram.A_Element) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                editor.setBorder(new javax.swing.border.EmptyBorder(0, 0, 0, 0));
                Compnt = editor;
            } else if (value instanceof java.awt.Font) {
                java.awt.Font ft = (java.awt.Font) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setOpaque(true);
                java.awt.FontMetrics fm = box.getFontMetrics(ft);
                String txt = ft.getFamily() + ":" + ft.getSize();
                while (fm.stringWidth(txt) > 100) {
                    txt = txt.substring(0, txt.length() - 4) + "...";
                }
                box.setText(txt);
                box.setFont(ft);
                String sty = "PLAIN";
                if (ft.getStyle() == java.awt.Font.BOLD) {
                    sty = "BOLD";
                } else if (ft.getStyle() == java.awt.Font.ITALIC) {
                    sty = "ITALIC";
                }
                box.setToolTipText("Font: " + ft.getFamily() + ", Size: " + ft.getSize() + " Style: " + sty);
                Compnt = box;
            } else if (value instanceof com.jdcm.diagram.A_Task) {
                com.jdcm.diagram.A_Task task = ((com.jdcm.diagram.A_Task) value);
                String txt = "";
                if (task.isEmpty() == false || table != null) {
                    txt = task.toString();
                }
                javax.swing.JLabel label = new javax.swing.JLabel(txt);
                label.setForeground((Color) task.getOptionModel().getValuefor("Color"));
                Compnt = label;
            } else if (value instanceof Character) {
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setOpaque(true);
                box.setText("'" + value.toString() + "'");
                box.setToolTipText(value.getClass().getSimpleName());
                Compnt = box;
            } else if (value instanceof com.jdcm.common.XMLres
                    || value instanceof java.lang.reflect.AnnotatedElement
                    || value instanceof com.jdcm.common.C_textElement
                    || value instanceof java.lang.annotation.Annotation
                    || value instanceof com.jdcm.common.C_Actiondef) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                editor.setToolTipText(value.getClass().getSimpleName());
                editor.displayBorder(false);
                Compnt = editor;
            } else if (value instanceof com.jdcm.common.C_textPackage) {
                javax.swing.JLabel box = new javax.swing.JLabel(value.toString());
                box.setOpaque(true);
                box.setIcon((new com.jdcm.common.D_Utils()).getIcon("Package.gif"));
                Compnt = box;
            } else if (value instanceof byte[]) {
                byte[] b=(byte[])value;
                String txt="byte[] { ";
                String oct;
                for (int i=0;i<16&&i<b.length;i++){
                    oct =Integer.toHexString(b[i]);
                    if (oct.length()==1){
                        oct="0"+oct;
                    }else if(oct.startsWith("ffffff")) {
                        oct=oct.substring(6);
                    }
                    txt=txt+oct+", ";
                }
                if (b.length==0){
                    txt=txt.substring(0,txt.length()-2);
                }else if (b.length<=16){
                    txt=txt.substring(0,txt.length()-2) +" }";
                }else{
                     txt=txt+"... }";
                }
                javax.swing.JLabel box = new javax.swing.JLabel(txt);
                box.setOpaque(true);
                Compnt = box;
            }else if (value instanceof com.jdcm.diagram.A_Events
                    || value instanceof java.net.InetAddress) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                editor.setToolTipText(value.getClass().getSimpleName());
                editor.displayBorder(false);
                Compnt = editor;
            } else if (value instanceof com.jdcm.eltrx.I_Command) {
                com.jdcm.eltrx.I_Command cmd = (com.jdcm.eltrx.I_Command) value;
                final javax.swing.JLabel label = new javax.swing.JLabel(cmd.getCMDOptions().getTitle(), cmd.getCMDIcon(), 0);
                Compnt = label;
            } else if (value instanceof C_Packet) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                Compnt = editor;
            } else if (value instanceof java.io.File) {
                java.io.File f = (java.io.File) value;
                javax.swing.ImageIcon ico = null;
                String name = f.getName();
                if (name == null || name.length() == 0) {
                    // C:\\;D:\\
                    name = f.toString();
                }
                if (f.isDirectory()) {
                    ico = (new com.jdcm.common.D_Utils()).getIcon("Directory.gif");
                } else if (name!=null && name.indexOf(".") == -1 && f.toString().contains("jar!")) {
                    ico = (new com.jdcm.common.D_Utils()).getIcon("DirectoryJar.gif");
                } else if (f.getName().indexOf(".") > 0) {
                    String ext = f.getName().substring(f.getName().lastIndexOf(".") + 1);
                    ico = (new com.jdcm.common.D_Utils()).getIcon("File_" + ext + ".gif");
                    // name=f.getName().substring(0,f.getName().lastIndexOf("."));
                }
                if (ico == null) {
                    ico = (new com.jdcm.common.D_Utils()).getIcon("File.gif");
                }
                javax.swing.JLabel box = new javax.swing.JLabel(name);
                box.setToolTipText(f.getAbsolutePath());
                box.setOpaque(true);
                box.setIcon(ico);
                Compnt = box;
            } else if (value instanceof javax.swing.Icon) {
                javax.swing.Icon ico = (javax.swing.Icon) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setIcon(ico);
                box.setToolTipText(ico.toString());
                if (ico instanceof javax.swing.ImageIcon) {
                    box.setText(((javax.swing.ImageIcon) ico).getDescription());
                }
                Compnt = box;
            } else if (value instanceof java.awt.Image) {
                java.awt.Image ico = (java.awt.Image) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setText(ico.getClass().getSimpleName() + "[" + ico.getWidth(null) + "*" + ico.getHeight(null) + "]");
                box.setToolTipText(ico.toString());
                Compnt = box;
            } else if (value instanceof javax.sound.sampled.Line) {
                javax.sound.sampled.Line line = (javax.sound.sampled.Line) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                if (value instanceof javax.sound.sampled.Mixer) {
                    box.setText(((javax.sound.sampled.Mixer) line).getMixerInfo().getName());
                } else {
                    box.setText(line.getLineInfo().toString());
                    box.setToolTipText(line.getLineInfo().toString());
                }
                Compnt = box;
            } else if (value instanceof javax.sound.sampled.AudioFormat) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                Compnt = editor;
            } else if (value instanceof java.awt.geom.Line2D
                    || value instanceof java.awt.geom.Rectangle2D
                    || value instanceof java.awt.geom.Ellipse2D
                    || value instanceof java.awt.geom.Arc2D
                    || value instanceof java.awt.geom.Point2D
                    || value instanceof java.awt.Point
                    || value instanceof java.awt.Rectangle
                    || value instanceof java.awt.Dimension) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                editor.setToolTipText(value.getClass().getSimpleName());
                editor.displayBorder(false);
                Compnt = editor;
            } else if (value instanceof javax.security.cert.X509Certificate) {
                javax.security.cert.X509Certificate cert = (javax.security.cert.X509Certificate) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setIcon((new com.jdcm.common.D_Utils()).getIcon("cert_X509.gif"));
                box.setText(cert.getSubjectDN().getName());
                Compnt = box;
            } else if (value instanceof sun.security.x509.X509CertImpl) {
                //!l is Sun proprietary API!!grand mystere implement sun....(extends X509Cerf..) BUT NOT  X509Certificat
                sun.security.x509.X509CertImpl cert = (sun.security.x509.X509CertImpl) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setIcon((new com.jdcm.common.D_Utils()).getIcon("cert_X509.gif"));
                box.setText(cert.getName());
                Compnt = box;
            } else if (value instanceof javax.security.cert.Certificate) {
                javax.security.cert.Certificate cert = (javax.security.cert.Certificate) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setToolTipText(cert.toString());
                box.setIcon((new com.jdcm.common.D_Utils()).getIcon("cert_simple.gif"));
                Compnt = box;
            } /* else if (value instanceof javax.swing.JToggleButton){
             ((javax.swing.JToggleButton)value).setEnabled(false);
             Compnt = (javax.swing.JToggleButton)value;
             } */ else if (value instanceof java.awt.Component) {
                JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
                editor.displayBorder(false);
                editor.setToolTipText(value.getClass().getSimpleName());
                if (((Component) value).isEnabled() == false) {
                    //   label.setForeground(Color.gray);
                }
                Compnt = editor;
            } else if (value instanceof DefaultComboBoxModel<?>) {
                DefaultComboBoxModel<?> cbm = (DefaultComboBoxModel<?>) value;
                java.awt.Component box ;
                if (cbm.getSize() > 0) {
                    box = getTableCellRendererComponent(null, cbm.getSelectedItem(), false, false, 0, 0);
                    //  box.setIcon((new com.jdcm.common.D_Utils()).getIcon("arrowgrey.gif"));
                } else {
                    box = new javax.swing.JLabel();
                    ((javax.swing.JLabel) box).setText("-Empty-");
                }
                Compnt = box;
            } else if (value instanceof AbstractSpinnerModel) {
                AbstractSpinnerModel model = (AbstractSpinnerModel) value;
                javax.swing.JSpinner spbox = new javax.swing.JSpinner(model);
                Compnt = spbox;
            } else if (value instanceof javax.swing.tree.TreeModel) {
                javax.swing.JTree box = new javax.swing.JTree((javax.swing.tree.TreeModel) value);
                if (hasFocus) {
                    for (int i = 0; i < box.getRowCount(); i++) {
                        box.expandRow(i);
                    }
                } else {
                    box.collapseRow(0);
                }
                Compnt = box;
            } else if (value instanceof Boolean) {
                Boolean bool = (Boolean) value;
                javax.swing.JCheckBox box = new javax.swing.JCheckBox();
                box.setSelected(bool);
                box.setMargin(new java.awt.Insets(0, 0, 0, 0));
                box.setBorder(new javax.swing.border.EmptyBorder(0, 0, 0, 0));
                Compnt = box;
            } else if (value instanceof java.util.Collection<?> || value instanceof Object[]) {
                javax.swing.JLabel box = new javax.swing.JLabel();
                if (value.getClass().isArray()) {
                    int size = java.lang.reflect.Array.getLength(value);
                    String txt = value.getClass().getComponentType().getSimpleName();
                    box.setText(txt + "[" + size + "]");
                    /*if (size == 0) {
                     box.setText("Array[0]");
                     } else {
                     box.setText(java.lang.reflect.Array.get(value, 0).getClass().getSimpleName() + "[" + size + "]");
                     }*/
                } else {
                    java.util.Collection<?> v = (java.util.Collection<?>) value;
                    String stype = "<?>";
                    if (v.isEmpty() == false) {
                        stype = "<" + v.toArray()[0].getClass().getSimpleName() + ">";
                    }
                    box.setText(v.getClass().getSimpleName() + stype + "/" + v.size());
                }
                Compnt = box;
            }
            /*  */
            /*     if (value instanceof Modifier_Ext){
             String res="";
             Modifier_Ext mod=(Modifier_Ext)value;
             javax.swing.JLabel box= new javax.swing.JLabel();
             box.setOpaque(true);
             box.setText(mod.toString());
             Compnt=box;
             }*/
            if (value instanceof java.sql.Date || value instanceof java.sql.Time || value instanceof java.sql.Timestamp) {
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setToolTipText(value.getClass().getSimpleName());
                box.setOpaque(true);
                //box.setText(value.toString());

                if (value instanceof java.sql.Timestamp) {
                    java.text.SimpleDateFormat format = new java.text.SimpleDateFormat("yy-MM-dd HH:mm:ss.SSS");
                    box.setText(format.format((java.sql.Timestamp) value));
                } else if (value instanceof java.sql.Time) {
                    box.setText(java.text.DateFormat.getTimeInstance(java.text.DateFormat.MEDIUM).format(value));
                } else if (value instanceof java.sql.Date) {
                    box.setText(java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT).format(value));
                } else {
                    java.text.DateFormat format =java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT, java.util.Locale.getDefault());  
                    box.setText(format.format(value));
                }

                Compnt = box;
            } else if (value instanceof java.sql.ResultSet) {
                java.sql.ResultSet rset = (java.sql.ResultSet) value;
                String txt = "SQL ResulSet";
                try {
                    if (rset.isClosed()) {
                        txt = " closed " + txt;
                    } else if (rset.isAfterLast()) {
                        txt = " afterLast " + txt;
                    }
                } catch (SQLException ex) {
                     System.err.println("ACE_ObjectRender : ResultSet=>"+ex.getMessage());
                }
                try {
                    String conc = "unknown";
                    String type = conc;
                    if (rset.getConcurrency() == java.sql.ResultSet.CONCUR_UPDATABLE) {
                        conc = "UPDATABLE";
                    } else if (rset.getConcurrency() == java.sql.ResultSet.CONCUR_READ_ONLY) {
                        conc = "READ_ONLY";
                    }
                    if (rset.getType() == java.sql.ResultSet.TYPE_FORWARD_ONLY) {
                        type = "FORWARD_ONLY";
                    } else if (rset.getType() == java.sql.ResultSet.TYPE_SCROLL_INSENSITIVE) {
                        type = "SCROLL_INSENSITIVE";
                    } else if (rset.getType() == java.sql.ResultSet.TYPE_SCROLL_SENSITIVE) {
                        type = "SCROLL_SENSITIVE";
                    }
                    txt = txt + "[" + rset.getFetchSize() + "*" + rset.getMetaData().getColumnCount() + "," + conc + "," + type + "]";
                } catch (SQLException ex) {
                       System.err.println("ACE_ObjectRender : ResultSet=>"+ex.getMessage());
                }
                javax.swing.JLabel box = new javax.swing.JLabel(txt);
                box.setBackground(Color.white);
                Compnt = box;
            } else if (value instanceof java.awt.Color) {
                java.awt.Color c = (java.awt.Color) value;
                javax.swing.JLabel box = new javax.swing.JLabel();
                box.setForeground(c);
                box.setText(c.getClass().getSimpleName() + "[r=" + c.getRed() + " g=" + c.getGreen() + " b=" + c.getBlue() + " a= " + c.getAlpha() + "]");
                if (c.getBlue() > 200 && c.getRed() > 200 && c.getGreen() > 200) {
                    box.setOpaque(true);
                    box.setBackground(Color.DARK_GRAY);
                }
                Compnt = box;
            } else if (value instanceof Enumeration<?>) {
                Enumeration<?> v = (Enumeration<?>) value;
                javax.swing.JList<Object> box = new javax.swing.JList<>();
                javax.swing.DefaultListModel<Object> listModel = new javax.swing.DefaultListModel<>();
                while (v.hasMoreElements()) {
                    listModel.addElement(v.nextElement());
                }
                box.setModel(listModel);
                Compnt = box;
            } else if (value instanceof P_ArrayHeader) {
                String name = ((P_ArrayHeader) value).toString();
                javax.swing.JTextArea lbl = new javax.swing.JTextArea(name);
                lbl.setForeground(java.awt.Color.blue);
                lbl.setEditable(false);
                Compnt = lbl;
            }
        }
        if (value != null) {
            if (Compnt == null) {
                if (value instanceof com.jdcm.common.O_Amount) {
                    com.jdcm.common.O_Amount mon = (com.jdcm.common.O_Amount) value;
                    javax.swing.JLabel box = new javax.swing.JLabel(mon.toString());
                    box.setToolTipText("" + mon.getFrechName() +" "+mon.getSymbol()+ " [" +mon.getISO4217NumCode() + "]");
                    if (mon.AMOUNT<0){
                        box.setForeground(Color.red);
                    }
                    box.setOpaque(true);
                    box.getText();
                    Compnt = box;
                } else if (value instanceof java.util.Date) {
                    java.util.Date dt = (java.util.Date) value;
                    javax.swing.JLabel box = new javax.swing.JLabel();
                    box.setOpaque(true);
                    box.setText(java.text.DateFormat.getDateTimeInstance(java.text.DateFormat.SHORT, java.text.DateFormat.MEDIUM).format(dt));
                    box.getText();
                    Compnt = box;
                }/* else if (value instanceof java.awt.Component) {
                 java.awt.Component c = (java.awt.Component) value;
                 if (c.getParent() != null) {
                 c.getParent().remove(c);
                 }
                 java.awt.Panel p = new java.awt.Panel();
                 p.setLayout(new java.awt.FlowLayout(0, 0, 0));
                 p.add(c);
                 JCB_ObjectEditor edi=(JCB_ObjectEditor)Compnt;
                 p.add(edi);
                 Compnt = p;
                 } else if (value.getClass().isArray()) {
                 int lg = java.lang.reflect.Array.getLength(value);
                 String name = value.getClass().getSimpleName();
                 name = name.substring(0, name.length() - 2) + "[" + lg + "]";
                 javax.swing.tree.DefaultMutableTreeNode root = new javax.swing.tree.DefaultMutableTreeNode(name);
                 for (int i = 0; i < lg; i++) {
                 Object o = java.lang.reflect.Array.get(value, i);
                 root.add(new DefaultMutableTreeNode(o));
                 }
                 javax.swing.JTree tree = new javax.swing.JTree(root);
                 tree.setCellRenderer(new ACE_ObjRenderer());
                 Compnt = tree;
                 }*/
            }
        }

        if (Compnt == null) {
            JCB_ObjectEditor editor = new JCB_ObjectEditor(value, false);
            Compnt = editor;
        }
        if (table != null) {
            if (Compnt instanceof javax.swing.JTree) {
                javax.swing.JTree tree = (javax.swing.JTree) Compnt;
                tree.setCellEditor(new ACE_ObjEditor<>());
                tree.setCellRenderer(new ACE_ObjRenderer<>());
                tree.setEditable(true);
            }
            Compnt.setEnabled(table.getModel().isCellEditable(row, col) || source != -1);
            if (Compnt instanceof JCB_ObjectEditor) {
                ((JCB_ObjectEditor) Compnt).setSelectable(true);
            }
            if (isSelected) {
                Compnt.setBackground(table.getSelectionBackground());
            } else {
                if (value instanceof java.awt.Color == false) {
                    // for white color on white back, keep black background
                    Compnt.setBackground(table.getBackground());
                }
            }
            //for DBTable, not funny !!NOT FOR RENDER , OK FOR EDITOR
            if (table.getRowHeight(row) > table.getRowHeight()) {
                table.setRowHeight(row, table.getRowHeight());
            }

        }

        return Compnt;
    }

    private void ComponentResized(java.awt.event.ComponentEvent evt, int row) {

        if (evt.getComponent().getParent() instanceof javax.swing.JTable) {
            javax.swing.JTable t = (javax.swing.JTable) evt.getComponent().getParent();
            int h = 0;
            if (evt.getComponent() instanceof java.awt.Panel) {
                java.awt.Panel p = (java.awt.Panel) evt.getComponent();
                for (java.awt.Component c : p.getComponents()) {
                    h = Math.max(h, c.getHeight());
                }
            } else {
                h = evt.getComponent().getPreferredSize().height;
            }
            h = Math.max(h, evt.getComponent().getMinimumSize().height);
            t.setRowHeight(row, h);
        }
    }

    /**
     * implement the same Object Render than table for tree <br> Exept for JTree
     * with is inserted in the tree<br>
     *
     * @param tree
     * @param initvalue
     * @param selected
     * @param expanded
     * @param leaf
     * @param row
     * @param hasFocus
     * @return
     */
    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object initvalue, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        // setValue for tree do not overwrite UserObject ... TO DO!!
        Object value = initvalue;
        if (value instanceof DefaultMutableTreeNode) {
            value = ((DefaultMutableTreeNode) value).getUserObject();
        }
        Component comp = getTableCellRendererComponent(null, value, selected, hasFocus, row, 0);
        java.awt.Color c = new java.awt.Color(255, 255, 255, 100);
        if (selected) {
            // should be something.selected()
            c = new java.awt.Color(230, 230, 255);
        }
        comp.setBackground(c);
        if (comp instanceof JTree) {
            DefaultMutableTreeNode valroot;
            if (((JTree) comp).getModel().getRoot() instanceof DefaultMutableTreeNode) {
                valroot = (DefaultMutableTreeNode) ((JTree) comp).getModel().getRoot();
            } else {
                valroot = new DefaultMutableTreeNode(((JTree) comp).getModel().getRoot());
            }
            return new javax.swing.JLabel(valroot.toString());
        }
        return comp;
    }

    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        Component comp = this.getTableCellRendererComponent(null, value, isSelected, cellHasFocus, index, 0);
        java.awt.Color c = new java.awt.Color(255, 255, 255, 100);
        if (isSelected) {
            c = list.getSelectionBackground();
        }
        comp.setBackground(c);
        return comp;
    }
};
