package com.jdcm.editors;

import java.awt.event.MouseEvent;
import javax.swing.event.CaretEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JTextPane;
import javax.swing.JPanel;
import com.jdcm.common.C_textClass;
import javax.swing.event.AncestorEvent;
import javax.swing.text.BadLocationException;

/**
 * F_Completion.java Created on 29 oct. 2011, 09:16:03
 * 
 * propose a popup Text completion Panel
 *
 * @author Marc GIBOIN
 */
public class F_Completion extends JPanel {

    private static final long serialVersionUID = 1L;
    private DefaultMutableTreeNode treeCompletion = null;
    private JTextPane Parent = null;
    private com.jdcm.common.D_Utils TUtils = new com.jdcm.common.D_Utils();
    private java.awt.event.KeyAdapter keyEvent = null;
    private com.jdcm.editors.P_LoaderEditor loader = null;
    private java.awt.Point caretLocation = null;
    private String Type = "";
    private F_PopUp popup;

    public F_Completion(String type, JTextPane p) {
        initComponents();
        Parent = p;
        Type = type;
        loader = new com.jdcm.editors.P_LoaderEditor(new java.net.URLClassLoader(new java.net.URL[0]));
        Parent.addCaretListener(new javax.swing.event.CaretListener() {

            @Override
            public void caretUpdate(CaretEvent e) {
                updatePosition();
            }
        });
        keyEvent = new java.awt.event.KeyAdapter() {

            @Override
            public void keyReleased(java.awt.event.KeyEvent evt) {
                if (evt.getKeyCode()==27){
                    //escape
                    return;
                }
                int caretPosition = Parent.getCaretPosition();
                try {
                    String ctxt = Parent.getStyledDocument().getText(0, Parent.getStyledDocument().getLength());
                    int startpos = Math.max(ctxt.lastIndexOf(" ", caretPosition - 1), ctxt.lastIndexOf("\n", caretPosition - 1));
                    if (startpos >= 0) {
                        String txt = ctxt.substring(startpos, caretPosition);
                        if (isVisible() || (txt.length() > 0)) {//&& txt.indexOf(".") >= 0)
                            CodeCompletion(caretLocation, txt.trim());
                        }
                    }
                } catch (BadLocationException ex) {
                    System.err.println("F_Completion : keyReleased() =>"+ex.getMessage());
                }
            }
        };
        Parent.addKeyListener(keyEvent);
        Parent.addMouseListener(new java.awt.event.MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent e) {
                if (isVisible() && popup.isAsFrame() == false) {
                    setVisible(false);
                }
            }
        });
        Parent.addAncestorListener(new javax.swing.event.AncestorListener() {

            @Override
            public void ancestorAdded(AncestorEvent event) {
                //  nothing to do
            }

            @Override
            public void ancestorRemoved(AncestorEvent event) {
                setVisible(false);
            }

            @Override
            public void ancestorMoved(AncestorEvent event) {
                updatePosition();
                if (caretLocation != null) {
                    //   setLocation(caretLocation);
                }
            }
        });
        jList1.setModel(new javax.swing.AbstractListModel<Object>() {

            private static final long serialVersionUID = 1L;
            String[] strings = "com,java,javax".split(",");

            @Override
            public int getSize() {
                return strings.length;
            }

            @Override
            public Object getElementAt(int i) {
                return strings[i];
            }
        });
        jList1.setCellRenderer(new com.jdcm.editors.ACE_ObjRenderer<>());
        if (Type.compareTo("JAVA") == 0) {
            initJavaCodeCompletion();
        }
        CodeCompletion(null, "");
        popup = new F_PopUp(this, "Completion");
    }

    public void close() {
        if (Parent != null) {
            Parent.removeKeyListener(keyEvent);
        }
        setVisible(false);
        if (popup != null) {
            popup.setVisible(false);
            popup = null;
        }
    }

    public void updatePosition() {
        java.awt.Point p = Parent.getCaret().getMagicCaretPosition();
        if (p != null) {
            try {
                p.translate(Parent.getLocationOnScreen().x, Parent.getLocationOnScreen().y);
            } catch (java.awt.IllegalComponentStateException ex) {
                //if screern is not visible
                close();
            }
            p.translate(0, Parent.getFontMetrics(Parent.getFont()).getHeight());
            caretLocation = p;
        }
    }

    private void initJavaCodeCompletion() {

        if (treeCompletion == null) {
            treeCompletion = TUtils.loader2tree(getURLLoader());
        }
    }

    private java.net.URLClassLoader getURLLoader() {
        return (java.net.URLClassLoader) loader.getValue();
    }

    private boolean isPossibleChoice(java.lang.reflect.Member member, String start) {
        boolean res = false;
        if (start.length() == 0 || member.getName().toLowerCase().startsWith(start.toLowerCase())) {
            if (java.lang.reflect.Modifier.isAbstract(member.getModifiers())) {
                return false;
            } else if (java.lang.reflect.Modifier.isPrivate(member.getModifiers())) {
                return false;
            }
            return true;
        }
        return res;
    }

    final public void CodeCompletion(java.awt.Point p, String txt) {
        if (treeCompletion == null) {
            return;
        }
        javax.swing.DefaultListModel<Object> model = null;
        if (Type.compareTo("JAVA") == 0) {
            model = javaCompletion(txt);
        }
        if (model == null || model.isEmpty()) {
            setVisible(false);
        } else {
            setVisible(true);
            jList1.setModel(model);
            int h = Math.min(10 + model.size() * jList1.getCellBounds(0, 0).height, 200);
            int w = Math.min(10 + jList1.getPreferredSize().width, 400);
            popup.setBounds(p.x, p.y, w, h);
            jList1.setSelectedIndex(0);
        }
    }

    public javax.swing.DefaultListModel<Object> javaCompletion(String txt) {

        if (txt == null || txt.length() == 0) {
            return null;
        }
        // '\\.' should not be a 'good' regular expression
        if (txt.endsWith(".") && txt.length() == 1) {
            txt = "";
        }
        if (txt.endsWith(".")) {
            txt = txt + " ";
        }
        String packpath = "";
        if (txt.lastIndexOf(".") > 0) {
            packpath = txt.substring(0, txt.lastIndexOf("."));
        }
        javax.swing.DefaultListModel<Object> model = new javax.swing.DefaultListModel<>();
        String[] labels = txt.split("\\.");
        DefaultMutableTreeNode node = treeCompletion;
        int idx = 0;
        boolean found = true;
        while (idx >= 0 && idx < labels.length - 1 && found) {
            found = false;
            for (int i = 0; i < node.getChildCount() && !found; i++) {
                DefaultMutableTreeNode child = (DefaultMutableTreeNode) node.getChildAt(i);
                if (child.getUserObject().toString().compareTo(labels[idx]) == 0) {
                    node = child;
                    idx++;
                    found = true;
                }
            }
        }
        String start = labels[labels.length - 1].trim();
        //------------------------LOOK FOR Methodes-------------------------------
        Class<?> cl = null;
        if (txt.lastIndexOf(".") > 0) {
            String clname = txt.substring(0, txt.lastIndexOf("."));
            cl = TUtils.getClassbyName(getURLLoader(), clname);
            if (cl == null && clname.indexOf(".") < 0) {
                cl = TUtils.getClassbyName(getURLLoader(), "java.lang." + clname);
            }
        }

        if (cl != null) {
            for (java.lang.reflect.Constructor<?> c : cl.getConstructors()) {
                String name = c.getName();
               /* if (name.indexOf(".") > 0) {
                    name = name.substring(name.lastIndexOf(".") + 1);
                }*/
                if (isPossibleChoice(c, start)) {
                    model.addElement(c);
                }
            }
            for (java.lang.reflect.Method m : cl.getMethods()) {
                if (isPossibleChoice(m, start)) {
                    model.addElement(m);
                }
            }
            for (java.lang.reflect.Field f : cl.getFields()) {
                if (isPossibleChoice(f, start)) {
                    model.addElement(f);
                }
            }

        } else {
            //------------------------LOOK FOR OTHER PACKAGES-------------------------------
            for (int i = 0; i < node.getChildCount(); i++) {
                DefaultMutableTreeNode child = (DefaultMutableTreeNode) node.getChildAt(i);
                if (start.length() == 0 || child.getUserObject().toString().toLowerCase().startsWith(start.toLowerCase())) {
                    model.addElement(child.getUserObject());
                }
            }
            //------------------------LOOK FOR Classes-------------------------------
            java.util.ArrayList<C_textClass> cls = TUtils.getClasses(getURLLoader(), packpath);
            if (txt.indexOf(".") < 0) {
                cls.addAll(0, TUtils.getClasses(getURLLoader(), "java.lang"));
            }
            for (C_textClass classe : cls) {
                if (start.length() == 0 || classe.getSimpleName().toLowerCase().startsWith(start.toLowerCase())) {
                    model.addElement(classe);
                }
            }
        }
        return model;
    }

    private String getSelectedCode() {
        String res;
        Object selection = jList1.getSelectedValue();
        if (selection instanceof com.jdcm.common.C_textElement) {
            com.jdcm.common.C_textElement clfm = (com.jdcm.common.C_textElement) selection;
            res = clfm.getSimpleName() + ".";
        } else if (selection instanceof com.jdcm.common.C_textPackage) {
            res = ((com.jdcm.common.C_textPackage) selection).toString() + ".";
        } else if (selection instanceof java.lang.reflect.Method) {
            res = ((java.lang.reflect.Method) selection).getName() + "()";
            if (((java.lang.reflect.Method) selection).getReturnType() == null) {
                res = res + ";";
            }
        } else if (selection instanceof java.lang.reflect.Field) {
            res = ((java.lang.reflect.Field) selection).getName();
        } else if (selection instanceof java.lang.reflect.Constructor) {
            res = ((java.lang.reflect.Constructor) selection).getName() + "()";
            if (res.indexOf(".") > 0) {
                res = res.substring(res.lastIndexOf(".") + 1);
            }
        } else {
            res = selection.toString();
        }
        return res;
    }

    private void setSelectedItem() {
        try {
            String ctxt = Parent.getStyledDocument().getText(0, Parent.getStyledDocument().getLength());
            String complement = this.getSelectedCode();
            int caretPosition = Parent.getCaretPosition() - 1;
            int startpos = Math.max(ctxt.lastIndexOf(" ", caretPosition), ctxt.lastIndexOf("\n", caretPosition));
            startpos = Math.max(startpos, ctxt.lastIndexOf(".", caretPosition));
            Parent.getStyledDocument().remove(startpos + 1, caretPosition - startpos);
            Parent.getStyledDocument().insertString(startpos + 1, complement, null);
            java.awt.event.KeyEvent evt2 = new java.awt.event.KeyEvent(Parent,
                    java.awt.event.KeyEvent.KEY_RELEASED, System.currentTimeMillis(), 0,
                    java.awt.event.KeyEvent.VK_PERIOD, ".".charAt(0));
            keyEvent.keyReleased(evt2);
        } catch (BadLocationException ex) {
                System.err.println("F_Completion : setSelectedItem() =>"+ex.getMessage());
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList<>();

        setLayout(new java.awt.BorderLayout());

        jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jList1MouseClicked(evt);
            }
        });
        jList1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jList1KeyReleased(evt);
            }
        });
        jScrollPane1.setViewportView(jList1);

        add(jScrollPane1, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    private void jList1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MouseClicked
        if (evt.getClickCount() == 2) {
            setSelectedItem();
        }
    }//GEN-LAST:event_jList1MouseClicked

    private void jList1KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jList1KeyReleased
        if (evt.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER) {
            setSelectedItem();
        } else if (evt.getKeyCode() == java.awt.event.KeyEvent.VK_ESCAPE) {
            this.setVisible(false);
        }
    }//GEN-LAST:event_jList1KeyReleased
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList<Object> jList1;
    private javax.swing.JScrollPane jScrollPane1;
    // End of variables declaration//GEN-END:variables
}
