package com.jdcm.editors;

import com.jdcm.common.O_X25address;
import com.jdcm.common.O_MACaddress;

/**
 * P_adresses.java Created on 11 novembre 2007, 18:17 
 * 
 * Use to manage IPV4/IPV6/X25/Mac adresses
 *
 * @author Marc GIBOIN
 */
public class P_AdresseEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
        private static final  java.awt.Color cko = java.awt.Color.lightGray;
    private static final java.awt.Color cok = new java.awt.Color(150, 220, 150);
    // ipv4 & v6 are separed because edition is different
    private java.net.Inet4Address addripv4;
    private java.net.Inet6Address addripv6;
    private O_X25address X25addr;
    private O_MACaddress macaddr;
    private boolean editing = false;
    private javax.swing.text.JTextComponent ParentText;


    /**
     *create a panel <br>
     * is used by JCB_ObjectEditor<br>
     *
     * @author  Marc GIBOIN
     */
    public P_AdresseEditor() {
        initComponents();
        jSpinner1.setValue(100);
        jCheckBox11.setEnabled(false);
        jComboBox1.setModel(new javax.swing.DefaultComboBoxModel<>("none,X121,O_MACaddress,Inet4Address,Inet6Address,O_X25address".split(",")));
        jComboBox1.setSelectedItem("none");
    }
            /**
     * Provide an Icon if needed (asked by frames)
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
             return (new com.jdcm.common.D_Utils()).getImage("Inet4Address.gif");
        }catch(Exception ex){
            return null;
        }
    }
    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        editing = bool;
        majinfo();
    }
/**
 * Ajust GUI for type of adresses
 * @param bool is an IP 
 */
    private void setIPelements(boolean bool) {
        jButton1.setVisible(bool);
        jLabel6.setVisible(bool);
        jComboBox3.setVisible(bool);
        jButton2.setVisible(bool);
        jCheckBox11.setVisible(bool);
        jSpinner1.setVisible(bool);
        jComboBox4.setVisible(bool);

    }
    /**
     * Return the Label to display on 'simple' text as Editor 
     * @param  jtxt the initial txtComponent display by the GUI
     * @return the textComponent to display for this Editor
     */
    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent jtxt) {
        if (ParentText == null) {
            ParentText = jtxt;
        }
        if (addripv4 != null) {
            jtxt.setText(addripv4.getHostAddress());
        } else if (addripv6 != null) {
            jtxt.setText(addripv6.getHostAddress());
        } else if (X25addr != null) {
            jtxt.setText(X25addr.toString());
        } else if (macaddr != null) {
            jtxt.setText(macaddr.toString());
        } else {
            jtxt.setText("");
        }
        jtxt.setBackground(checkAddrColor(jtxt.getText()));
        return jtxt;
    }
/**
 * set the Value (address) for this Editor
 * @param val the addresse to display
 */
    @Override
    public void setValue(Object val) {
        initinfo();
        teachValue(val);
        majinfo();
    }
/*
 * change the stored value (and set null for other addresses type)
 */
    private void teachValue(Object val) {
        if (val instanceof java.net.Inet4Address || val instanceof java.net.Inet6Address
                || val instanceof O_MACaddress || val instanceof O_X25address) {
            addripv4 = null;
            addripv6 = null;
            macaddr = null;
            X25addr = null;
        } else {
            return;
        }
        setIPelements(val instanceof java.net.InetAddress);
        if (val instanceof java.net.InetAddress) {
            if (val instanceof java.net.Inet4Address) {
                addripv4 = (java.net.Inet4Address) val;
            } else if (val instanceof java.net.Inet6Address) {
                addripv6 = (java.net.Inet6Address) val;
            }
        } else {
            if (val instanceof O_MACaddress) {
                macaddr = (O_MACaddress) val;
            } else if (val instanceof O_X25address) {
                X25addr = (O_X25address) val;
            }
        }
        if (val != null) {
            jComboBox1.setSelectedItem(val.getClass().getSimpleName());
        }
    }
/*
 * return the store/displayed adresse as Object (mandatory by I_CBObjectEditor)
 */
    @Override
    public Object getValue() {
        if (addripv4 != null) {
            return addripv4;
        } else if (addripv6 != null) {
            return addripv6;
        } else if (X25addr != null) {
            return X25addr;
        } else if (macaddr != null) {
            return macaddr;
        }
        return null;
    }
/*
 * set blanks to Addresses field.
 */
    private void initinfo() {
        // jTextField2.setText("");
        jTextField1.setText("");
        jComboBox3.setModel(new javax.swing.DefaultComboBoxModel<java.net.InetAddress>());
        jComboBox4.removeAllItems();
    }
/*
 * display the stored address according to address type
 */
    private void majinfo() {
        if (addripv4 != null || addripv6 != null || macaddr != null || X25addr != null) {
            //is a mask??
        }
        if (jComboBox3.getItemCount() != 0) {
            teachValue(jComboBox3.getSelectedItem());
        }
        java.net.InetAddress addrip = null;
        if (addripv4 != null) {
            addrip = addripv4;
            jTextField1.setText(addripv4.toString().split("/")[0]);
            jTextField2.setText(addripv4.toString().split("/")[1]);
            // jTextField2.setBackground(checkAddrColor(jTextField2.getText()));
            jTextField1.setEditable(editing);
            jTextField1.setEnabled(editing);
            jTextField2.setEditable(editing);
            jTextField2.setEnabled(editing);
            jTextField1.setVisible(true);
            jComboBox3.setVisible(true);
        } else if (addripv6 != null) {
            addrip = addripv6;
            if (addripv6.isIPv4CompatibleAddress()) {
                jComboBox4.addItem("IPv4Compatible");
            }
            jTextField1.setText(addripv6.toString().split("/")[0]);
            jTextField2.setText(addripv6.toString().split("/")[1]);
        } else if (macaddr != null) {
            jTextField2.setText(macaddr.toString());
            jTextField2.setEditable(editing);
            jTextField2.setEnabled(editing);
            jComboBox3.setVisible(false);
            jTextField1.setVisible(false);
        } else if (X25addr != null) {
            jTextField2.setText(X25addr.toString());
            jTextField2.setEditable(editing);
            jTextField2.setEnabled(editing);
            jComboBox3.setVisible(false);
            jTextField1.setVisible(false);
        }
        jComboBox4.removeAllItems();
        if (addrip != null) {
            if (ParentText != null && ParentText.getText().compareTo(jTextField2.getText()) != 0) {
                ParentText.setText(jTextField2.getText());
                //   ParentText.setBackground(checkAddrColor(jTextField2.getText()));
            }
            if (addrip.isAnyLocalAddress()) {
                jComboBox4.addItem("AnyLocalAddress");
            }
            if (addrip.isLinkLocalAddress()) {
                jComboBox4.addItem("LinkLocalAddress");
            }
            if (addrip.isLoopbackAddress()) {
                jComboBox4.addItem("LoopbackAddress");
            }
            if (addrip.isMCGlobal()) {
                jComboBox4.addItem("MCGlobal");
            }
            if (addrip.isMCLinkLocal()) {
                jComboBox4.addItem("MCLinkLocal");
            }
            if (addrip.isMCNodeLocal()) {
                jComboBox4.addItem("MCNodeLocal");
            }
            if (addrip.isMCOrgLocal()) {
                jComboBox4.addItem("MCOrgLocal");
            }
            if (addrip.isMCSiteLocal()) {
                jComboBox4.addItem("MCSiteLocal");
            }
            if (addrip.isMulticastAddress()) {
                jComboBox4.addItem("MulticastAddress");
            }
            if (addrip.isSiteLocalAddress()) {
                jComboBox4.addItem("SiteLocalAddress");
            }

        }
    }

    /**
     * Ask DNS (via getallbyname())
     * 
     */
    private void getDNSAddrs(String txt) {
        // DNS search=> previous IP@ makes no sens
        //jTextField2.setText("");
        if (txt == null || txt.length() == 0) {
            return;
        }
        try {
            java.net.InetAddress[] ips = java.net.InetAddress.getAllByName(txt);
            jComboBox3.setModel(new javax.swing.DefaultComboBoxModel<>(ips));

        } catch (java.net.UnknownHostException ex) {
            initinfo();
        } catch (Throwable ex) {
            initinfo();
        }
        int i = 0;
        while (i >= 0) {
            if (i >= jComboBox3.getItemCount()) {
                jComboBox3.setEnabled(false);
                i = -1;
            } else if (jComboBox3.getItemAt(i).getClass().getSimpleName().startsWith(jComboBox1.getSelectedItem().toString())) {
                jComboBox3.setEnabled(true);
                jComboBox3.setSelectedIndex(i);
                jTextField1.setText(((java.net.InetAddress) jComboBox3.getSelectedItem()).getCanonicalHostName());
                i = -1;
            } else {
                i++;
            }
        }
        majinfo();
    }
    /**
     * Analyse color and return color according to analyse
     * @param txt the addresse as String
     * @return the color (cok/cko) found after the check
     */
    public java.awt.Color checkAddrColor(String txt) {
        initinfo();
        if (txt.length() == 0) {
            return cko;
        } else if (X25addr != null) {
            try {
                X25addr = new O_X25address(txt);
            } catch (Exception ex) {
                return cko;
            }
            majinfo();
            return cok;
        } else if (macaddr != null) {
            try {
                macaddr = new O_MACaddress(txt);
            } catch (Exception ex) {
                return cko;
            }
            majinfo();
            return cok;
        } else {
            try {
                byte[] b ;
                if (addripv4 != null) {
                    String[] bst = txt.split("\\.");
                    b = new byte[bst.length];
                    for (int i = 0; i < b.length; i++) {
                        try {
                            b[i] = Byte.parseByte(bst[i]);
                        } catch (Exception ex) {
                            b[i] = (byte) (Integer.parseInt(bst[i]) - 256);
                        }
                    }
                } else if (addripv6 != null) {
                    try {
                        java.net.InetAddress ip = java.net.InetAddress.getByName(txt);
                        if (ip instanceof java.net.Inet6Address) {
                            return cok;
                        }
                    } catch (Exception ex) {
                    }
                    return cko;
                } else {
                    b = null;
                }
                if (b == null) {
                    return cko;
                }
                try {
                    if (addripv4 != null) {
                        addripv4 = (java.net.Inet4Address) java.net.Inet4Address.getByAddress(b);
                    } else if (addripv6 != null) {
                    }
                } catch (java.net.UnknownHostException ex) {
                    return cko;
                }
                majinfo();
                return cok;
            } catch (Exception ex) {
                return cko;
            }
        }
    }
    /*
     * called after a Text change in the 'simple TextComponent ' modification
     */
    @Override
    public void TextManager(javax.swing.text.JTextComponent jtxt) {
        String txt = jtxt.getText();
        jtxt.setBackground(checkAddrColor(txt));

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jComboBox1 = new javax.swing.JComboBox<>();
        jPanel2 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        jTextField2 = new javax.swing.JTextField();
        jPanel3 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jPanel4 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jComboBox3 = new javax.swing.JComboBox<>();
        jPanel5 = new javax.swing.JPanel();
        jButton2 = new javax.swing.JButton();
        jLabel6 = new javax.swing.JLabel();
        jCheckBox11 = new javax.swing.JCheckBox();
        jSpinner1 = new javax.swing.JSpinner();
        jComboBox4 = new javax.swing.JComboBox<>();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.LINE_AXIS));

        jLabel2.setText("Type : ");
        jPanel1.add(jLabel2);

        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });
        jPanel1.add(jComboBox1);

        add(jPanel1);

        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));

        jLabel5.setText("Num : ");
        jPanel2.add(jLabel5);

        jTextField2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextField2ActionPerformed(evt);
            }
        });
        jPanel2.add(jTextField2);

        add(jPanel2);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        jLabel3.setText("Name : ");
        jPanel3.add(jLabel3);

        jTextField1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextField1ActionPerformed(evt);
            }
        });
        jTextField1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextField1KeyReleased(evt);
            }
        });
        jPanel3.add(jTextField1);

        add(jPanel3);

        jPanel4.setLayout(new javax.swing.BoxLayout(jPanel4, javax.swing.BoxLayout.LINE_AXIS));

        jButton1.setFont(new java.awt.Font("Tahoma", 0, 10)); // NOI18N
        jButton1.setText("DNS");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton1);

        jComboBox3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox3ActionPerformed(evt);
            }
        });
        jPanel4.add(jComboBox3);

        add(jPanel4);

        jPanel5.setLayout(new javax.swing.BoxLayout(jPanel5, javax.swing.BoxLayout.LINE_AXIS));

        jButton2.setFont(new java.awt.Font("Tahoma", 0, 10)); // NOI18N
        jButton2.setText("Reach");
        jButton2.setMargin(new java.awt.Insets(2, 2, 2, 2));
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel5.add(jButton2);

        jLabel6.setText("TO(ms) :");
        jPanel5.add(jLabel6);

        jCheckBox11.setToolTipText("seems to be a bug or tcp6?");
        jPanel5.add(jCheckBox11);

        jSpinner1.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(300), Integer.valueOf(1), null, Integer.valueOf(1)));
        jSpinner1.setToolTipText("reach tests TO in ms");
        jPanel5.add(jSpinner1);

        add(jPanel5);
        add(jComboBox4);
    }// </editor-fold>//GEN-END:initComponents
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        String txt = jTextField1.getText(); //www.google.com
        if (txt.length() == 0) {
            txt = jTextField2.getText(); //192.168.10.1
        }
        getDNSAddrs(txt);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jTextField1KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextField1KeyReleased
        if (evt.getKeyCode() != 10) {
            jTextField2.setText("");
            jComboBox3.setModel(new javax.swing.DefaultComboBoxModel<java.net.InetAddress>());
            jComboBox3.removeAllItems();
            jTextField2.setBackground(jTextField1.getBackground());
        }
    }//GEN-LAST:event_jTextField1KeyReleased

    private void jComboBox3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox3ActionPerformed
        // jComboBox3.getSelectedItem()
        majinfo();
        /*
         * jTextField2.setText(""); if (ob == null) { return; }
         * jTextField2.setText(((java.net.InetAddress) ob).getHostAddress()); if
         * (ob instanceof java.net.InetAddress) { addrip = (java.net.InetAddress)
         * ob; }
         */
}//GEN-LAST:event_jComboBox3ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        try {
            jCheckBox11.setSelected(addripv4.isReachable((Integer) jSpinner1.getValue()));
        } catch (Exception ex) {
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jTextField2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextField2ActionPerformed
        jTextField1.setText("");
        jTextField2.setBackground(checkAddrColor(jTextField2.getText()));
        if (jTextField2.getBackground() == cok) {
            ParentText.setText(jTextField2.getText());
        }

    }//GEN-LAST:event_jTextField2ActionPerformed

    private void jTextField1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextField1ActionPerformed
        getDNSAddrs(jTextField1.getText());
    }//GEN-LAST:event_jTextField1ActionPerformed

    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        if (jComboBox1.getSelectedItem().toString().startsWith("Inet")) {
            // getDNSAddrs("localhost");
        }
    }//GEN-LAST:event_jComboBox1ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JCheckBox jCheckBox11;
    private javax.swing.JComboBox<String> jComboBox1;
    private javax.swing.JComboBox<java.net.InetAddress> jComboBox3;
    private javax.swing.JComboBox<String> jComboBox4;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JSpinner jSpinner1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    // End of variables declaration//GEN-END:variables
}
