package com.jdcm.editors;

import javax.sound.sampled.AudioFormat;

/**
 * P_adresses.java  Created on 11 novembre 2007, 18:17
 * Use to manage IP adresse
 *
 * @author Marc GIBOIN
 */
public class P_AudioFormatEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private AudioFormat format = null;
    private boolean editing = false;

    /**
     *create a panel <br>
     * is used by JCB_ObjectEditor<br>
     *
     * @author  Marc GIBOIN
     */
    public P_AudioFormatEditor() {
        initComponents();
        jComboBox1.removeAllItems();
        jComboBox3.setModel(new javax.swing.DefaultComboBoxModel<>(new String[]{"mono", "stereo"}));
     //   jComboBox1.setModel(new javax.swing.DefaultComboBoxModel(new String[]{"ALaw", "ULaw", "PCM-Signed", "PCM-Unsigned"}));
        for (java.lang.reflect.Field f : javax.sound.sampled.AudioFormat.Encoding.class.getFields()) {
            jComboBox1.addItem(f);
        }
        jComboBox1.setRenderer(new ACE_ObjRenderer<java.lang.reflect.Field>());
    }
        /**
     * Provide an Icon if needed (asked by frames)
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
             return (new com.jdcm.common.D_Utils()).getImage("sound.gif");
        }catch(Exception ex){
            return null;
        }
    }
    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        editing = bool;
        for (java.awt.Component c : this.getComponents()) {
            c.setEnabled(bool);
        }
    }

    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent txt) {
        if (format != null) {
            txt.setText(format.toString());
        }
        return txt;
    }

    @Override
    public void setValue(Object val) {
        if (val instanceof AudioFormat) {
            format = (AudioFormat) val;
            jTextField1.setText("" + format.getFrameRate());
            if (format.getChannels() == 1) {
                jComboBox3.setSelectedItem("mono");
            } else {
                jComboBox3.setSelectedItem("stereo");
            }
            jCheckBox1.setSelected(format.isBigEndian());

            buttonGroup1.setSelected(jRadioButton1.getModel(), format.getSampleSizeInBits() == 8);
        }
    }

    @Override
    public AudioFormat getValue() {
        boolean isbigendian = jCheckBox1.isSelected();
        int samplesize = 16;
        if (jRadioButton1.isSelected()) {
            samplesize = 8;
        }
        int chanels = 2;
        if (jComboBox3.getSelectedItem().toString().startsWith("mono")) {
            chanels = 1;
        }
        Float rate = Float.valueOf(jTextField1.getText()).floatValue();

        try {
            AudioFormat.Encoding enc = (AudioFormat.Encoding) jComboBox3.getSelectedItem();
            format = new AudioFormat(enc, rate, samplesize, chanels, chanels * (samplesize / 8), rate, isbigendian);
        } catch (Exception ex) {
            System.out.println("P_AudioFormatEditor.getValue(): " + "rate:" + rate + " channels:" + chanels + " samplesize:" + samplesize + " BigEndian:" + isbigendian);
            setValue(format);
        }
        return format;
    }

    @Override
    public void TextManager(javax.swing.text.JTextComponent Jtxt) {
        String txt = Jtxt.getText();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jComboBox1 =  new javax.swing.JComboBox<>();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jSpinner1 = new javax.swing.JSpinner();
        jTextField1 = new javax.swing.JTextField();
        jPanel2 = new javax.swing.JPanel();
        jComboBox3 =  new javax.swing.JComboBox<>();
        jCheckBox1 = new javax.swing.JCheckBox();
        jPanel3 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton2 = new javax.swing.JRadioButton();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));

        jComboBox1.setToolTipText("Encoding");
        add(jComboBox1);

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.LINE_AXIS));

        jLabel1.setText("Rate : ");
        jPanel1.add(jLabel1);

        jSpinner1.setModel(new javax.swing.SpinnerListModel(new String[] {"8000", "11025", "16000", "22050", "44100"}));
        jSpinner1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jSpinner1MouseClicked(evt);
            }
        });
        jSpinner1.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                jSpinner1InputMethodTextChanged(evt);
            }
        });
        jSpinner1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                jSpinner1KeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jSpinner1KeyReleased(evt);
            }
        });
        jPanel1.add(jSpinner1);
        jPanel1.add(jTextField1);

        add(jPanel1);

        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));

        jComboBox3.setToolTipText("Chanels");
        jPanel2.add(jComboBox3);

        jCheckBox1.setText("BigEndians");
        jPanel2.add(jCheckBox1);

        add(jPanel2);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        jLabel5.setText("SampleSize : ");
        jPanel3.add(jLabel5);

        buttonGroup1.add(jRadioButton1);
        jRadioButton1.setSelected(true);
        jRadioButton1.setText("8");
        jPanel3.add(jRadioButton1);

        buttonGroup1.add(jRadioButton2);
        jRadioButton2.setText("16");
        jPanel3.add(jRadioButton2);

        add(jPanel3);
    }// </editor-fold>//GEN-END:initComponents

    private void jSpinner1InputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_jSpinner1InputMethodTextChanged
        jTextField1.setText("" + jSpinner1.getValue().toString());
    }//GEN-LAST:event_jSpinner1InputMethodTextChanged

    private void jSpinner1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSpinner1MouseClicked
        jTextField1.setText("" + jSpinner1.getValue().toString());
    }//GEN-LAST:event_jSpinner1MouseClicked

    private void jSpinner1KeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jSpinner1KeyPressed
    }//GEN-LAST:event_jSpinner1KeyPressed

    private void jSpinner1KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jSpinner1KeyReleased
        jTextField1.setText("" + jSpinner1.getValue().toString());
    }//GEN-LAST:event_jSpinner1KeyReleased
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JComboBox<java.lang.reflect.Field> jComboBox1;
    private javax.swing.JComboBox<String> jComboBox3;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JSpinner jSpinner1;
    private javax.swing.JTextField jTextField1;
    // End of variables declaration//GEN-END:variables
}
