package com.jdcm.editors;

import java.text.*;

/**
 *  P_DateEditor.java  Created on 31 octobre 2007, 21:26
 * 
 * Used to Edit/display Date & Times
 *
 * @author Marc GIBOIN
 */
public class P_DateEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private java.util.Date dt = null;
    private java.lang.Class<?> dtclass = null;
    private java.text.DateFormat dateform = null;
    private java.text.DateFormat timeform = null;
    private boolean editing = true;

    /**
 *create a panel <br>
 * is used by JCB_ObjectEditor<br>
 * 
 */
    public P_DateEditor() {
        initComponents();
        //  how to set MILISECOND field??? => spinner
        dateform = java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT, java.util.Locale.getDefault());
        timeform = java.text.DateFormat.getTimeInstance(java.text.DateFormat.MEDIUM, java.util.Locale.getDefault());
        jComboBox1.setModel(new javax.swing.DefaultComboBoxModel<>(Ctrylist()));
        jComboBox1.setSelectedItem(java.util.Locale.getDefault().getDisplayCountry());
        jComboBox2.setModel(new javax.swing.DefaultComboBoxModel<>(java.util.TimeZone.getAvailableIDs()));
        jComboBox2.setSelectedItem(java.util.TimeZone.getDefault().getID());
    }
        /**
     * Provide an Icon if needed (asked by frames)
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
             return (new com.jdcm.common.D_Utils()).getImage("time_hour.gif");
        }catch(Exception ex){
            return null;
        }
    }
    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        editing = bool;
        jTextField1.setEditable(bool);
        jTextField2.setEditable(bool);
        jSpinner1.setEnabled(bool);
    }

    @Override
    public void TextManager(javax.swing.text.JTextComponent txt) {
        java.util.Date myDate = new java.util.Date(0);
        java.text.DateFormat txtform = java.text.DateFormat.getDateTimeInstance(java.text.DateFormat.SHORT, java.text.DateFormat.MEDIUM, java.util.Locale.getDefault());
        if (txt.getText().length() == 0) {
            return;
        }

        try {
            if (dtclass == java.sql.Time.class) {
                myDate = timeform.parse(txt.getText());
                setValue(new java.sql.Time(myDate.getTime()));
                return;
            }else if (dtclass == java.sql.Date.class) {
                myDate = dateform.parse(txt.getText());
                setValue(new java.sql.Date(myDate.getTime()));
            }
            if (myDate.getTime()==0 && dtclass == java.util.Date.class || dtclass == java.sql.Timestamp.class) {

                myDate = txtform.parse(txt.getText());
                //ms not parsed > how to use txtform.MILLISECOND_FIELD?????
                int pos = txt.getText().lastIndexOf(" ");
                int ms = Integer.parseInt(txt.getText().substring(pos).trim());
                myDate.setTime(myDate.getTime() + ms);

            } else if (dtclass == java.sql.Timestamp.class) {
                setValue(new java.sql.Timestamp(myDate.getTime()));
            } else {
                setValue(myDate);
            }
        } catch (NumberFormatException | ParseException ex) {
            System.err.println("=>"+ex);
        }
    }

    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent txt) {
        String res = "";
        if (jTextField1.isEnabled()) {
            res += jTextField1.getText();
        }
        if (jTextField2.isEnabled()) {
            res = res + " " + jTextField2.getText();
        }
        if (jSpinner1.isEnabled()) {
            res = res + " " + jSpinner1.getValue();
        }
        txt.setText(res);
        return txt;
    }

    @Override
    public Object getValue() {
        java.util.Date myDate,myTime ;
        try {
            myDate = dateform.parse(jTextField1.getText());
            java.util.TimeZone.getDefault().getOffset(myDate.getTime());
            myTime = timeform.parse(jTextField2.getText());
            //parsing is made as CET and not default timezone=> adding offset
            int tz1 = java.util.TimeZone.getDefault().getOffset(myTime.getTime());
            myDate.setTime(myDate.getTime() + myTime.getTime() + tz1);
            long ms = new Long("" + jSpinner1.getValue());
            myDate.setTime(myDate.getTime() + ms);
            // Time Zone for day management (not time)
            String TZID = jComboBox2.getSelectedItem().toString();
            int tz2 = java.util.TimeZone.getTimeZone(TZID).getOffset(myDate.getTime()) - java.util.TimeZone.getDefault().getOffset(myDate.getTime());
            myDate.setTime(myDate.getTime() + tz2);
        } catch (ParseException | NumberFormatException ex) {
            return "dterr. : " + jTextField1.getText() + "/" + jTextField2.getText() + "/" + ex.getMessage();
        }
        if (dtclass == myDate.getClass()) {
            return myDate;
        }
        if (dtclass == java.sql.Time.class) {
            return new java.sql.Time(myDate.getTime());
        }
        if (dtclass == java.sql.Date.class) {
            return new java.sql.Date(myDate.getTime());
        }
        if (dtclass == java.sql.Timestamp.class) {
            return new java.sql.Timestamp(myDate.getTime());
        }

        return null;
    }

    @Override
    public void setValue(Object val) {
        if (val instanceof java.util.Date) {
            dt = (java.util.Date) val;
        } else {
            jTextField1.setText(val.getClass() + " is not a date");
            return;
        }
        dtclass = val.getClass();
        jTextField1.setText(dateform.format(dt));
        jTextField2.setText(timeform.format(dt));
        long msec = dt.getTime() - 1000 * ((dt.getTime() / 1000));
        jSpinner1.setValue(msec);
        if (val instanceof java.sql.Date) {
            showTime(false);
        }
        if (val instanceof java.sql.Time) {
            showDate(false);
        }
        setEditing(isEditing());
    }

    private void showDate(boolean bool) {
        // jTextField1.setText("");
        jTextField1.setEnabled(bool);
        jTextField1.setVisible(bool);
        jLabel1.setVisible(bool);
        jLabel3.setVisible(bool);
        jComboBox2.setSelectedItem(java.util.TimeZone.getDefault().getID());
        jComboBox2.setEnabled(bool);
        jComboBox2.setVisible(bool);
        jSpinner1.setValue(0);
        jSpinner1.setEnabled(bool);
        jSpinner1.setVisible(bool);
    }

    private void showTime(boolean bool) {
        //jTextField2.setText("");
        jTextField2.setEnabled(bool);
        jTextField2.setVisible(bool);
        jLabel2.setVisible(bool);
        jLabel2.setVisible(bool);
        jSpinner1.setValue(0);
        jSpinner1.setEnabled(bool);
        jSpinner1.setVisible(bool);
        jComboBox2.setSelectedItem(java.util.TimeZone.getDefault().getID());
        jComboBox2.setEnabled(bool);
        jComboBox2.setVisible(bool);
    }

    private String[] Ctrylist() {
        java.util.Locale[] loc = java.util.Locale.getAvailableLocales();
        String[] res = new String[loc.length];
        for (int i = 0; i < loc.length; i++) {
            res[i] = loc[i].getDisplayCountry();
        }
        java.util.Arrays.sort(res);
        int start = 0;
        while (res[start].length() == 0) {
            start++;
        }
        // clear blanks du to ??? in the begin of list
        String[] res2 = new String[res.length - start];
        for (int i = 0; i < res2.length; i++) {
            res2[i] = res[i + start];
        }
        return res2;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jPanel2 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jTextField2 = new javax.swing.JTextField();
        jSpinner1 = new javax.swing.JSpinner();
        jPanel3 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jComboBox2 = new javax.swing.JComboBox<>();
        jComboBox1 = new javax.swing.JComboBox<>();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.LINE_AXIS));

        jLabel1.setText("Date : ");
        jPanel1.add(jLabel1);

        jTextField1.setText("Date");
        jPanel1.add(jTextField1);

        add(jPanel1);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jLabel2.setText("Time :");
        jPanel2.add(jLabel2, java.awt.BorderLayout.WEST);

        jTextField2.setText("Time");
        jPanel2.add(jTextField2, java.awt.BorderLayout.CENTER);

        jSpinner1.setModel(new javax.swing.SpinnerNumberModel(0, 0, 999, 1));
        jPanel2.add(jSpinner1, java.awt.BorderLayout.EAST);

        add(jPanel2);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        jLabel3.setText("GMT :");
        jPanel3.add(jLabel3);

        jComboBox2.setFont(new java.awt.Font("Tahoma", 2, 10)); // NOI18N
        jComboBox2.setBorder(null);
        jPanel3.add(jComboBox2);

        jComboBox1.setFont(new java.awt.Font("Tahoma", 2, 10)); // NOI18N
        jComboBox1.setBorder(null);
        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });
        jPanel3.add(jComboBox1);

        add(jPanel3);
    }// </editor-fold>//GEN-END:initComponents
    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        if (dt != null) {
            String tgctry = (String) jComboBox1.getSelectedItem();
            java.util.Locale[] loc = java.util.Locale.getAvailableLocales();
            int pos = 0;
            while (loc[pos].getDisplayCountry().compareTo(tgctry) != 0) {
                pos++;
            }
            dateform = java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT, loc[pos]);
            timeform = java.text.DateFormat.getTimeInstance(java.text.DateFormat.MEDIUM, loc[pos]);
            setValue(dt);
        }
    }//GEN-LAST:event_jComboBox1ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox<String> jComboBox1;
    private javax.swing.JComboBox<String> jComboBox2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JSpinner jSpinner1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    // End of variables declaration//GEN-END:variables
}
