package com.jdcm.editors;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Ellipse2D;

/**
 *P_GeoEditor.java Created on 6 août 2009, 19:50:59
 * 
 * Edit Geometric Objects (ex : Point2D) as a table of values
 * 
 * @author Marc Giboin
 */
public class P_GeomEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private boolean editing = false;
    private Class<?> cl = null;
    private String clname = null;

    /**
     * Creates new form P_GeoEditor
     */
    public P_GeomEditor() {
        initComponents();
    }
    /**
     * Provide an Icon if needed (asked by frames)
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
             return (new com.jdcm.common.D_Utils()).getImage("Line2D.gif");
        }catch(Exception ex){
            return null;
        }
    }
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();

        setPreferredSize(new java.awt.Dimension(246, 70));
        setLayout(new java.awt.BorderLayout());

        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(jTable1);

        add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel1.setLayout(new java.awt.GridLayout(1, 0));

        jLabel1.setText("[Empty]");
        jPanel1.add(jLabel1);

        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        jLabel2.setText("plus");
        jPanel1.add(jLabel2);

        add(jPanel1, java.awt.BorderLayout.SOUTH);
    }// </editor-fold>//GEN-END:initComponents

    /**
     * return the basic label to be displayed as editor
     *
     * @param txt the initial Text
     * @return the label to be displayed for this editor
     */
    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent txt) {
        String dim = "";
        for (int i = 0; i < jTable1.getColumnCount(); i++) {
            dim = dim + jTable1.getColumnName(i) + ":" + jTable1.getValueAt(0, i) + ",";
        }

        txt.setText(clname + " [" + dim.substring(0, dim.length() - 1) + "]");
        return txt;
    }

    @Override
    public void setValue(Object val) {
        cl = null;
        if (val.getClass() == Point.class
                || val.getClass() == java.awt.Dimension.class) {
            Point pt;
            cl = val.getClass();
            String[] hdr;
            if (val.getClass() == Point.class) {
                pt = (Point) val;
                hdr = new String[]{
                    "x", "y",};
            } else {
                pt = new Point(((java.awt.Dimension) val).width, ((java.awt.Dimension) val).height);
                hdr = new String[]{
                    "w", "h",};
            }
            jTable1.setModel(new javax.swing.table.DefaultTableModel(
                    new Object[][]{
                        {pt.x, pt.y}
                    },
                    hdr));
        }
        if (val.getClass() == Rectangle.class) {
            Rectangle pt = (Rectangle) val;
            // jLabel2.setText("Center : "+pt.getCenterX()+","+pt.getCenterY());
            cl = val.getClass();
            jTable1.setModel(new javax.swing.table.DefaultTableModel(
                    new Object[][]{
                        {pt.x, pt.y, pt.width, pt.height}
                    },
                    new String[]{
                        "x", "y", "w", "h"
                    }));
        }
        if (val.getClass() == Rectangle2D.Double.class
                || val.getClass() == Rectangle2D.Float.class
                || val.getClass() == Ellipse2D.Double.class
                || val.getClass() == Ellipse2D.Float.class) {
            Object[][] o = null;
            if (val instanceof Rectangle2D) {
                Rectangle2D pt = (Rectangle2D) val;
                o = new Object[][]{
                    {pt.getX(), pt.getY(), pt.getWidth(), pt.getHeight()}};
            } else if (val instanceof Ellipse2D) {
                Ellipse2D pt = (Ellipse2D) val;
                o = new Object[][]{
                    {pt.getX(), pt.getY(), pt.getWidth(), pt.getHeight()}};
            }
            // jLabel2.setText("Center : "+pt.getCenterX()+","+pt.getCenterY());
            cl = val.getClass();
            jTable1.setModel(new javax.swing.table.DefaultTableModel(
                    o,
                    new String[]{
                        "x", "y", "w", "h"
                    }));
        }
        if (val.getClass() == Line2D.Double.class || val.getClass() == Line2D.Float.class) {
            Line2D pt = (Line2D) val;
            cl = val.getClass();
            jTable1.setModel(new javax.swing.table.DefaultTableModel(
                    new Object[][]{
                        {pt.getX1(), pt.getY1(), pt.getX2(), pt.getY2()}
                    },
                    new String[]{
                        "xa", "ya", "xb", "yb"
                    }));
        }
        if (val.getClass() == Point2D.Double.class || val.getClass() == Point2D.Float.class) {
            cl = val.getClass();
            Point2D pt = (Point2D) val;
            String[] hdr = new String[]{"x", "y",};
            jTable1.setModel(new javax.swing.table.DefaultTableModel(
                    new Object[][]{
                        {pt.getX(), pt.getY()}
                    },
                    hdr));
        }
        if (cl == null) {
            jTable1.setModel(null);
            clname = null;
            return;
        } else if (cl == Rectangle2D.Double.class) {
            clname = "Rectangle2D.d";
        } else if (cl == Rectangle2D.Float.class) {
            clname = "Rectangle2D.f";
        } else if (cl == Ellipse2D.Double.class) {
            clname = "Ellipse2D.d";
        } else if (cl == Ellipse2D.Float.class) {
            clname = "Ellipse2D.f";
        } else if (cl == Line2D.Double.class) {
            clname = "Line2D.d";
        } else if (cl == Line2D.Float.class) {
            clname = "Line2D.f";
        } else if (cl == Point2D.Double.class) {
            clname = "Point2D.d";
        } else if (cl == Point2D.Float.class) {
            clname = "Point2D.d";
        } else {
            clname = cl.getSimpleName();
        }
        jLabel1.setText("[" + clname + "]");
    }

    @Override
    public Object getValue() {
        if (jTable1.getModel() == null || jTable1.getRowCount() != 1) {
            return null;
        }
        if (cl == java.awt.Point.class || cl == java.awt.Dimension.class) {
            int x = new Integer(jTable1.getValueAt(0, 0).toString());
            int y = new Integer(jTable1.getValueAt(0, 1).toString());
            if (cl == java.awt.Point.class) {
                return new Point(x, y);
            } else {
                return new java.awt.Dimension(x, y);
            }
        }
        if (cl == java.awt.Rectangle.class) {
            int x = new Integer(jTable1.getValueAt(0, 0).toString());
            int y = new Integer(jTable1.getValueAt(0, 1).toString());
            int w = new Integer(jTable1.getValueAt(0, 2).toString());
            int h = new Integer(jTable1.getValueAt(0, 3).toString());
            return new Rectangle(x, y, w, h);
        }
        if (cl == java.awt.geom.Point2D.Double.class) {
            double x = new Double(jTable1.getValueAt(0, 0).toString());
            double y = new Double(jTable1.getValueAt(0, 1).toString());
            return new Point2D.Double(x, y);
        }
        if (cl == java.awt.geom.Rectangle2D.Double.class
                || cl == java.awt.geom.Line2D.Double.class
                || cl == java.awt.geom.Ellipse2D.Double.class) {
            double x = new Double(jTable1.getValueAt(0, 0).toString());
            double y = new Double(jTable1.getValueAt(0, 1).toString());
            double w = new Double(jTable1.getValueAt(0, 2).toString());
            double h = new Double(jTable1.getValueAt(0, 3).toString());
            if (cl == java.awt.geom.Rectangle2D.Double.class) {
                return new Rectangle2D.Double(x, y, w, h);
            } else if (cl == java.awt.geom.Line2D.Double.class) {
                return new Line2D.Double(x, y, w, h);
            } else if (cl == java.awt.geom.Ellipse2D.Double.class) {
                return new Ellipse2D.Double(x, y, w, h);
            }
        }
        if (cl == java.awt.geom.Point2D.Float.class) {
            float x = new Float(jTable1.getValueAt(0, 0).toString());
            float y = new Float(jTable1.getValueAt(0, 1).toString());
            return new Point2D.Float(x, y);
        }
        if (cl == java.awt.geom.Rectangle2D.Float.class
                || cl == java.awt.geom.Line2D.Float.class
                || cl == java.awt.geom.Ellipse2D.Float.class) {
            float x = new Float(jTable1.getValueAt(0, 0).toString());
            float y = new Float(jTable1.getValueAt(0, 1).toString());
            float w = new Float(jTable1.getValueAt(0, 2).toString());
            float h = new Float(jTable1.getValueAt(0, 3).toString());
            if (cl == java.awt.geom.Rectangle2D.Float.class) {
                return new Rectangle2D.Float(x, y, w, h);
            } else if (cl == java.awt.geom.Line2D.Float.class) {
                return new Line2D.Float(x, y, w, h);
            } else if (cl == java.awt.geom.Ellipse2D.Float.class) {
                return new Ellipse2D.Double(x, y, w, h);
            }
        }
        return null;
    }

    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        for (int i = 0; i < jTable1.getColumnCount(); i++) {
            jTable1.setEnabled(bool);
        }
        editing = bool;

    }

    @Override
    public void TextManager(javax.swing.text.JTextComponent jtxt) {
        String txt = jtxt.getText();
        txt = txt.replaceAll("]", ",");
        try {
            String[] txtval = new String[jTable1.getColumnCount()];
            for (int i = 0; i < jTable1.getColumnCount(); i++) {
                int pos = txt.indexOf(jTable1.getColumnName(i) + ":");
                if (pos != -1) {
                    pos = pos + jTable1.getColumnName(i).length() + 1;
                    txtval[i] = txt.substring(pos, txt.indexOf(",", pos));
                    if (txtval[i] != null) {
                        if (cl.getName().endsWith("Double")) {
                            jTable1.setValueAt(new Double(txtval[i]), 0, i);
                        } else if (cl.getName().endsWith("Float")) {
                            jTable1.setValueAt(new Float(txtval[i]), 0, i);
                        } else {
                            jTable1.setValueAt(new Integer(txtval[i]), 0, i);
                        }
                    }
                }
            }
            jtxt.setBackground(java.awt.Color.green);
        } catch (java.lang.IndexOutOfBoundsException ex) {
            jtxt.setBackground(java.awt.Color.red);
        } catch (NumberFormatException ex) {
            jtxt.setBackground(java.awt.Color.red);
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable jTable1;
    // End of variables declaration//GEN-END:variables
}
