package com.jdcm.editors;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.event.AdjustmentEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 *
 * Used to display Images - could edit, redraw,....
 *
 *
 * @author Marc GIBOIN
 */
public final class P_ImageEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private final com.jdcm.common.D_Utils TUtils = new com.jdcm.common.D_Utils();
    private final com.jdcm.common.Options_Model options = new com.jdcm.common.Options_Model(this);
    private final Color imageBorderColor = new java.awt.Color(200, 200, 200);
    private final Color imageChangeColor = new java.awt.Color(200, 51, 52);
    private final Color gridColor = new java.awt.Color(75, 75, 75);
    private final Color activGridColor = new java.awt.Color(250, 150, 150, 100);
    private java.awt.Point resizePt = null;
    private java.awt.Point movePt = null;
    private java.awt.Point movePt2 = null;
    private java.awt.Point margPt = null;
    private java.awt.Point margPt2 = null;
    private float zoom = 1;
    private Image image = null;
    private Image imageRef = null;
    private final java.util.List<Image> imgsRedo = new java.util.LinkedList<>();
    private boolean editing = true;

    /**
     * create a panel <br> is used by JCB_ObjectEditor<br>
     */
    public P_ImageEditor() {
        initComponents();
        jButton4.setIcon(TUtils.getIcon("Save.gif"));
        jButton3.setIcon(TUtils.getIcon("Open.gif"));
        options.addOption("img_src", new java.awt.Color(0), "color", "", true, true, "base color");
        options.addOption("img_trgt", new java.awt.Color(255, 255, 255, 0), "color", "", true, true, "target color");
        options.addOption("img_marg", 0.3f, "float", "marg %", true, true, "marge definition");
        options.addOption("img_size", new java.awt.Rectangle(0, 0), "color", "", true, true, "base color");
        options.addOption("img_lbl", "no image", "String", "", true, true, "Image string");
        jCB_ObjectEditor1.setValue(options);
        jCB_ObjectEditor1.setEditable(true);
        jScrollPane1.setViewportView(new LoadImagePanel());
        jScrollPane1.getVerticalScrollBar().addAdjustmentListener(new java.awt.event.AdjustmentListener() {

            @Override
            public void adjustmentValueChanged(AdjustmentEvent ae) {
                repaint();
            }
        });
        jScrollPane1.getVerticalScrollBar().addAdjustmentListener(new java.awt.event.AdjustmentListener() {

            @Override
            public void adjustmentValueChanged(AdjustmentEvent ae) {
                repaint();
            }
        });
        java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(100, 100, java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
        setValue(image4);
    }

    /**
     * Provide an Icon if needed (asked by frames)
     *
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
            return (new com.jdcm.common.D_Utils()).getImage("home.gif");
        } catch (Exception ex) {
            return null;
        }
    }

    public boolean isChangeColor() {
        return jToggleButton1.isSelected();
    }

    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        editing = bool;
    }

    @Override
    public void TextManager(javax.swing.text.JTextComponent txt) {
    }

    /**
     * return the basic label to be displayed as editor
     *
     * @param txt the initial Text
     * @return the label to be displayed for this editor
     */
    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent txt) {
        if (image != null) {
            txt.setText(image.getClass().getSimpleName() + "[" + image.getWidth(null) + "*" + image.getHeight(null) + "]");
        } else {
            txt.setText("no image");
        }
        return txt;
    }

    private void setReScaleRef(Image im) {
        imgsRedo.add(imageRef);
        float coefx = (float) im.getWidth(this) / imageRef.getWidth(this);
        float coefy = (float) im.getHeight(this) / imageRef.getHeight(this);
        imageRef = im.getScaledInstance(Math.round(coefx * imageRef.getWidth(this)), Math.round(coefy * imageRef.getHeight(this)), 1);
        options.setValuefor("img_size", new java.awt.Rectangle(imageRef.getWidth(null), imageRef.getHeight(null)));
    }

    private void setUpdateRef(Image im) {
        imgsRedo.add(imageRef);
        float coefx = (float) imageRef.getWidth(this) / image.getWidth(this);
        float coefy = (float) imageRef.getHeight(this) / image.getHeight(this);
        imageRef = im.getScaledInstance(Math.round(coefx * im.getWidth(this)), Math.round(coefy * im.getHeight(this)), 1);
        java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(imageRef.getWidth(this), imageRef.getHeight(this), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
        java.awt.Graphics2D g2 = image4.createGraphics();
        g2.drawImage(imageRef, null, this);
        imageRef = image4;
        options.setValuefor("img_size", new java.awt.Rectangle(imageRef.getWidth(null), imageRef.getHeight(null)));
    }

    private void setZoom() {
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        int viewSizeX = Math.round(imageRef.getWidth(this) * zoom);
        int viewSizeY = Math.round(imageRef.getHeight(this) * zoom);
        java.awt.image.BufferedImage image4;
        try {
            image4 = new java.awt.image.BufferedImage(viewSizeX, viewSizeY, java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
        } catch (Throwable ex) {
            System.err.println("P_ImageEditor : setZoom()=>"+ex);
            return;
        }
        image = imageRef.getScaledInstance(viewSizeX, viewSizeY, 1);
        java.awt.Graphics2D g2 = image4.createGraphics();
        g2.drawImage(image, null, this);
        image = image4;
        jScrollPane1.repaint();
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    private java.awt.Image changeColor() {
        if (margPt2 == null || margPt == null) {
            return imageRef;
        } else {

            float colmarge = (float) (margPt2.x - margPt.x) / 4;// a little bit short but screens (nb pixels) are growning
            if (colmarge <= 0) {
                return imageRef;
            } else if (colmarge > 255) {
                colmarge = 255;
            }
            java.awt.Point frPt = margPt.getLocation();
            frPt.translate(-jScrollPane1.getLocationOnScreen().x, -jScrollPane1.getLocationOnScreen().y);
            Color source = new Color(((BufferedImage) imageRef).getRGB(frPt.x, frPt.y));
            image = imageRef.getScaledInstance(Math.round(imageRef.getWidth(this) * zoom), Math.round(imageRef.getHeight(this) * zoom), 1);
            Color target = options.getValuefor("img_trgt", new Color(0));
            java.awt.image.BufferedImage bImage = new java.awt.image.BufferedImage(image.getWidth(this), image.getHeight(this), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
            java.awt.Graphics2D g2 = bImage.createGraphics();
            g2.drawImage(image, null, this);
            int distMarge = Math.abs(margPt.y - margPt2.y);
            for (int x = Math.max(0, frPt.x - distMarge); x < Math.min(bImage.getWidth(), frPt.x + distMarge); x++) {
                for (int y = Math.max(0, frPt.y - distMarge); y < Math.min(bImage.getHeight(), frPt.y + distMarge); y++) {
                    Color pointColor = new Color(bImage.getRGB(x, y));
                    if (Math.abs(pointColor.getRed() - source.getRed()) < colmarge
                            && Math.abs(pointColor.getBlue() - source.getBlue()) < colmarge
                            && Math.abs(pointColor.getGreen() - source.getGreen()) < colmarge
                            && Math.abs(pointColor.getAlpha() - source.getAlpha()) < colmarge) {
                        bImage.setRGB(x, y, target.getRGB());
                    }
                }
            }
            image = bImage;
            return bImage;
        }
    }

    private void drawPoint(java.awt.Point pos) {
        BufferedImage bImage;
        Color target = options.getValuefor("img_trgt", new Color(0));
        bImage = (BufferedImage) image;
        if (pos.x > bImage.getWidth()
                || pos.y > bImage.getHeight()
                || new Color(bImage.getRGB(pos.x, pos.y)).equals(target)) {
            return;
        }
        int pixSize = image.getWidth(this) / imageRef.getWidth(this);
        int relpixx = Math.round((float) Math.floor(pos.getX() / pixSize));
        int relpixy = Math.round((float) Math.floor(pos.getY() / pixSize));
        int[] RBGarr = new int[pixSize * pixSize];
        java.util.Arrays.fill(RBGarr, target.getRGB());
        bImage.setRGB(relpixx * pixSize, relpixy * pixSize, pixSize, pixSize, RBGarr, 0, 0);
        setUpdateRef(bImage);
        // setValue(bImage);
        jScrollPane1.repaint();
    }

    @Override
    public Image getValue() {
        // should be modified ImageRef
        return imageRef;
    }

    @Override
    public void setValue(Object val) {
        if (val instanceof Image) {
            image = (Image) val;
            if (imageRef == null) {
                imageRef = image;
                options.setValuefor("img_size", new java.awt.Rectangle(image.getWidth(null), image.getHeight(null)));
            }
            jButton7.setEnabled(!imgsRedo.isEmpty());
            //
            jScrollPane1.repaint();
            options.setValuefor("img_lbl", image.toString());
            repaint();
        }
    }

    class LoadImagePanel extends JPanel {

        private static final long serialVersionUID = 1L;

        public LoadImagePanel() {

        }

        @Override
        public Dimension getPreferredSize() {
            if (image == null) {
                return new Dimension(100, 100);
            } else {
                return new Dimension(image.getWidth(this), image.getHeight(this));
            }
        }

        @Override
        public void paint(Graphics g) {
            java.awt.Color defaultColor = g.getColor();
            g.clearRect(0, 0, getWidth(), getHeight());
            if (isChangeColor()) {
                image = changeColor();
            }
            int pixel = 10;
            int pixSize = image.getWidth(this) / imageRef.getWidth(this);
            if (pixSize > 5) {
                g.setColor(activGridColor);
                pixel = pixSize;
                g.drawImage(image, 0, 0, this);
                // draw the Grid
                for (int x = 0; x < this.getWidth(); x = x + pixel) {
                    g.drawLine(x, 0, x, this.getHeight());
                }
                for (int y = 0; y < this.getHeight(); y = y + pixel) {
                    g.drawLine(0, y, getWidth(), y);
                }
            } else {
                g.setColor(gridColor);
                for (int x = 0; x < this.getWidth(); x = x + pixel) {
                    for (int y = 0; y < this.getHeight(); y = y + pixel) {
                        g.drawLine(x, y, x + pixel, y + pixel);
                        g.drawLine(x, y + pixel, x + pixel, y);
                    }
                }

                g.drawImage(image, 0, 0, this);

                //   System.out.println("=> redraw");
            }
            if (resizePt == null) {
                g.setColor(imageBorderColor);
                g.drawRect(0, 0, image.getWidth(null), image.getHeight(null));
                g.setColor(imageChangeColor);
                if (image.getWidth(null) > 10 && image.getHeight(null) > 10) {
                    int wi = image.getWidth(null);
                    int he = image.getHeight(null);
                    g.drawLine(0, 0, 0, 10);
                    g.drawLine(0, 0, 10, 0);
                    g.drawRect(wi - 10, he + 1, 12, 1);
                    g.drawRect(wi + 1, he - 10, 1, 12);
                    g.drawRect(wi / 2 - 4, he + 1, 8, 1);
                    g.drawRect(wi + 1, he / 2 - 4, 1, 8);
                }
            } else {
                g.setColor(imageChangeColor);
                g.drawRect(0, 0, resizePt.x, resizePt.y);
            }
            if (isChangeColor() && margPt != null) {
                g.setColor(imageChangeColor);
                java.awt.Point frPt = margPt.getLocation();
                frPt.translate(-jScrollPane1.getLocationOnScreen().x, -jScrollPane1.getLocationOnScreen().y);
                g.drawRect(frPt.x - 1, frPt.y - 1, 2, 2);
                if (margPt2 != null) {
                    int radius = Math.abs(margPt2.y - margPt.y);
                    g.drawRect(frPt.x - radius, frPt.y - radius, 2 * radius, 2 * radius);
                }
                 g.setColor(defaultColor);
                 g.drawRect(0, image.getHeight(this), image.getWidth(this), 10);
            }
            g.setColor(defaultColor);
            if (movePt != null && movePt2 != null) {
                java.awt.Graphics2D g2 = (java.awt.Graphics2D) g;
                jScrollPane1.getViewport().repaint();
                Composite stored = g2.getComposite();
                g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
                g2.drawImage(image, movePt2.x - movePt.x, movePt2.y - movePt.y, null);
                g2.setComposite(stored);
            }
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jToolBar1 = new javax.swing.JToolBar();
        jButton2 = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jButton7 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();
        jCB_ObjectEditor1 = new com.jdcm.editors.JCB_ObjectEditor();
        jButton8 = new javax.swing.JButton();
        jButton9 = new javax.swing.JButton();
        jButton10 = new javax.swing.JButton();
        jToggleButton1 = new javax.swing.JToggleButton();

        setLayout(new java.awt.BorderLayout());

        jScrollPane1.setOpaque(false);
        jScrollPane1.addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                jScrollPane1MouseWheelMoved(evt);
            }
        });
        jScrollPane1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jScrollPane1MouseClicked(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jScrollPane1MousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jScrollPane1MouseReleased(evt);
            }
        });
        jScrollPane1.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                jScrollPane1MouseDragged(evt);
            }
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                jScrollPane1MouseMoved(evt);
            }
        });
        jScrollPane1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                jScrollPane1KeyPressed(evt);
            }
        });
        add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jToolBar1.setRollover(true);
        jToolBar1.setMinimumSize(new java.awt.Dimension(39, 20));
        jToolBar1.setPreferredSize(new java.awt.Dimension(39, 20));

        jButton2.setText("-");
        jButton2.setBorderPainted(false);
        jButton2.setFocusable(false);
        jButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton2);

        jButton1.setText("+");
        jButton1.setBorderPainted(false);
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton1);

        jButton7.setText("<");
        jButton7.setToolTipText("redo");
        jButton7.setBorderPainted(false);
        jButton7.setEnabled(false);
        jButton7.setFocusable(false);
        jButton7.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton7.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton7ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton7);

        jButton3.setToolTipText("Open");
        jButton3.setBorderPainted(false);
        jButton3.setFocusable(false);
        jButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton3);

        jButton4.setToolTipText("Write");
        jButton4.setBorderPainted(false);
        jButton4.setFocusable(false);
        jButton4.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton4.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton4);

        jButton6.setText("4");
        jButton6.setToolTipText("4 byte colors");
        jButton6.setBorderPainted(false);
        jButton6.setFocusable(false);
        jButton6.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton6.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton6);

        jCB_ObjectEditor1.setAsButton(true);
        jToolBar1.add(jCB_ObjectEditor1);

        jButton8.setText("Z-");
        jButton8.setToolTipText("Zoom -");
        jButton8.setBorderPainted(false);
        jButton8.setFocusable(false);
        jButton8.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton8.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton8ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton8);

        jButton9.setText("Z+");
        jButton9.setToolTipText("Zoom +");
        jButton9.setBorderPainted(false);
        jButton9.setFocusable(false);
        jButton9.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton9.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton9.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton9ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton9);

        jButton10.setText("Z0");
        jButton10.setToolTipText("stored image");
        jButton10.setBorderPainted(false);
        jButton10.setFocusable(false);
        jButton10.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton10.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton10.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton10ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton10);

        jToggleButton1.setText("=>");
        jToggleButton1.setBorderPainted(false);
        jToggleButton1.setFocusable(false);
        jToggleButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToggleButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jToggleButton1);

        add(jToolBar1, java.awt.BorderLayout.NORTH);
    }// </editor-fold>//GEN-END:initComponents

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        if (image != null) {
            Image image2 = image.getScaledInstance(image.getWidth(this) / 2, image.getHeight(this) / 2, 1);
            java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(image2.getWidth(this), image2.getHeight(this), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
            java.awt.Graphics2D g2 = image4.createGraphics();
            g2.drawImage(image2, null, this);
            image2 = image4;
            setReScaleRef(image2);
            setValue(image2);
            repaint();
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        if (image != null) {
            Image image2 = image.getScaledInstance(image.getWidth(this) * 2, image.getHeight(this) * 2, 1);
            java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(image2.getWidth(this), image2.getHeight(this), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
            java.awt.Graphics2D g2 = image4.createGraphics();
            g2.drawImage(image2, null, this);
            image2 = image4;
            setReScaleRef(image2);
            setValue(image2);
            repaint();
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        //READ (OPEN)   choose to set As 4 BYTE on Loadding...
        java.io.File f = TUtils.getUIreadFile(this, javax.swing.JFileChooser.FILES_ONLY, null, true);
        try {
            BufferedImage bi = javax.imageio.ImageIO.read(f);
            java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(bi.getWidth(this), bi.getHeight(this), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
            java.awt.Graphics2D g2 = image4.createGraphics();
            g2.drawImage(bi, null, this);
            imageRef = null;
            setValue(bi);
            repaint();
        } catch (IOException ex) {
            System.err.println("P_ImageEditor : jButton3ActionPerformed() =>" + ex.getMessage());
        }

    }//GEN-LAST:event_jButton3ActionPerformed

private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
    //WRITE TO File ; whats about tiff, bmp,...
    try {
        // retrieve image
        java.awt.image.BufferedImage bi = (java.awt.image.BufferedImage) image;
        String outputfilename = TUtils.getUIsaveFile(this, false, null);
        if (outputfilename != null) {
            String filetype = "gif";
            if (outputfilename.toLowerCase().endsWith(".png")) {
                filetype = "png";
            } else if (outputfilename.toLowerCase().endsWith(".jpg")) {
                filetype = "jpg";
            } else {
                outputfilename = outputfilename + ".gif";
            }
            java.io.File outputfile = new java.io.File(outputfilename);
            javax.imageio.ImageIO.write(bi, filetype, outputfile);
        }
    } catch (IOException ex) {
        System.err.println("P_ImageEditor : jButton4ActionPerformed() =>" + ex.getMessage());
    }
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jScrollPane1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jScrollPane1MouseClicked

    }//GEN-LAST:event_jScrollPane1MouseClicked

    private void jScrollPane1MouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jScrollPane1MouseMoved
        if (resizePt == null) {
            int borderSize = 15;
            // detect change color mouseOver  
            // CARE ABOUT cursor CHANGED BY WINDOWS
            Color col = TUtils.getNearColored(jScrollPane1, evt.getPoint(), 5, imageChangeColor);
            if (col == null) {
                if (getCursor().getType() != Cursor.MOVE_CURSOR
                        && getCursor().getType() != Cursor.CROSSHAIR_CURSOR) {
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
            } else if (col.equals(imageChangeColor)) {
                if (evt.getX() < borderSize && evt.getY() < borderSize) {
                    if (getCursor().getType() == Cursor.HAND_CURSOR) {
                        // setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                    } else if (getCursor().getType() != Cursor.CROSSHAIR_CURSOR
                            && getCursor().getType() != Cursor.MOVE_CURSOR) {
                        setCursor(new Cursor(Cursor.HAND_CURSOR));
                    }
                } else if (Math.abs(evt.getX() - image.getWidth(null)) < borderSize
                        && Math.abs(evt.getY() - image.getHeight(null)) < borderSize) {
                    setCursor(new Cursor(Cursor.NW_RESIZE_CURSOR));
                } else if (Math.abs(evt.getX() - image.getWidth(null)) < borderSize
                        && evt.getY() < image.getHeight(null)) {
                    setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
                } else if (Math.abs(evt.getY() - image.getHeight(null)) < borderSize
                        && evt.getX() < image.getWidth(null)) {
                    setCursor(new Cursor(Cursor.S_RESIZE_CURSOR));
                }
            }
            //   System.err.println("=>"+getCursor().getName()+ " : "+col);
        }
    }//GEN-LAST:event_jScrollPane1MouseMoved

    private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
        java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(image.getWidth(null), image.getHeight(null), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
        java.awt.Graphics2D g2 = image4.createGraphics();
        g2.drawImage(image, null, null);
        setUpdateRef(image4);
        setValue(image4);
    }//GEN-LAST:event_jButton6ActionPerformed

    private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton7ActionPerformed
        if (imgsRedo.isEmpty() == false) {
            Image lastimg = imgsRedo.remove(imgsRedo.size() - 1);
            image = null;
            imageRef = null;
            setValue(lastimg);
        }
        jButton7.setEnabled(!imgsRedo.isEmpty());
    }//GEN-LAST:event_jButton7ActionPerformed

    private void jScrollPane1MouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jScrollPane1MouseDragged
        if (isChangeColor() && margPt != null) {
            margPt2 = evt.getLocationOnScreen();
            jScrollPane1.getViewport().repaint();
        } else if (movePt != null) {
            if (getCursor().getType() == Cursor.MOVE_CURSOR) {
                movePt2 = evt.getPoint();
                jScrollPane1.getViewport().repaint();
            }
        } else if (resizePt != null) {
            if (getCursor().getType() == Cursor.E_RESIZE_CURSOR) {
                resizePt = new java.awt.Point(evt.getPoint().x, resizePt.y);
            } else if (getCursor().getType() == Cursor.S_RESIZE_CURSOR) {
                resizePt = new java.awt.Point(resizePt.x, evt.getPoint().y);
            } else if (getCursor().getType() == Cursor.NW_RESIZE_CURSOR) {
                resizePt = evt.getPoint();
            }
            repaint();
        } else if (getCursor().getType() == Cursor.CROSSHAIR_CURSOR) {
            java.awt.Point p = new java.awt.Point(evt.getX() + jScrollPane1.getHorizontalScrollBar().getValue(),
                    evt.getY() + jScrollPane1.getVerticalScrollBar().getValue());
            drawPoint(p);
        }
    }//GEN-LAST:event_jScrollPane1MouseDragged

    private void jScrollPane1MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jScrollPane1MouseReleased
        if (isChangeColor()) {
            setUpdateRef(image);
            setValue(image);
            jToggleButton1.setSelected(false);
            margPt = null;
            margPt2 = null;
        } else if (resizePt != null) {
            if (SwingUtilities.isLeftMouseButton(evt)) {
                java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(resizePt.x, resizePt.y, java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
                java.awt.Graphics2D g2 = image4.createGraphics();
                g2.drawImage(image, null, this);
                setUpdateRef(image4);
                setValue(image4);
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            resizePt = null;
        } else if (movePt != null && getCursor().getType() == Cursor.MOVE_CURSOR) {
            if (SwingUtilities.isLeftMouseButton(evt)) {
                AffineTransform toCenterAt = new AffineTransform();
                int dx = evt.getPoint().x - movePt.x;
                int dy = evt.getPoint().y - movePt.y;
                toCenterAt.translate(dx, dy);
                java.awt.image.BufferedImage image4 = new java.awt.image.BufferedImage(image.getWidth(this), image.getHeight(this), java.awt.image.BufferedImage.TYPE_4BYTE_ABGR);
                java.awt.Graphics2D g2 = image4.createGraphics();
                g2.drawImage(image, toCenterAt, null);
                setUpdateRef(image4);
                setValue(image4);
            }
            movePt = null;
            jScrollPane1.getViewport().repaint();
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        } else if (SwingUtilities.isLeftMouseButton(evt) && getCursor().getType() == Cursor.CROSSHAIR_CURSOR) {
            java.awt.Point p = new java.awt.Point(evt.getX() + jScrollPane1.getHorizontalScrollBar().getValue(),
                    evt.getY() + jScrollPane1.getVerticalScrollBar().getValue());
            drawPoint(p);
        }
    }//GEN-LAST:event_jScrollPane1MouseReleased

    private void jScrollPane1MousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jScrollPane1MousePressed
        if (isChangeColor()) {
            if (evt.getX() < image.getWidth(null) && evt.getY() < image.getHeight(null)) {
                options.setValuefor("img_marg", 0f);
                margPt = evt.getLocationOnScreen();
            }
        } else if (getCursor().getType() != Cursor.DEFAULT_CURSOR) {
            if (getCursor().getType() == Cursor.HAND_CURSOR) {
                if (SwingUtilities.isLeftMouseButton(evt)) {
                    setCursor(new Cursor(Cursor.MOVE_CURSOR));
                } else if (SwingUtilities.isRightMouseButton(evt)) {
                    setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
                } else {
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
            } else if (getCursor().getType() == Cursor.E_RESIZE_CURSOR) {
                resizePt = new java.awt.Point(evt.getPoint().x, image.getHeight(null));
            } else if (getCursor().getType() == Cursor.S_RESIZE_CURSOR) {
                resizePt = new java.awt.Point(image.getWidth(null), evt.getPoint().y);
            } else if (getCursor().getType() == Cursor.NW_RESIZE_CURSOR) {
                resizePt = evt.getPoint();
            } else if (getCursor().getType() == Cursor.CROSSHAIR_CURSOR
                    || getCursor().getType() == Cursor.MOVE_CURSOR) {
                if (getCursor().getType() == Cursor.MOVE_CURSOR
                        && SwingUtilities.isLeftMouseButton(evt)) {
                    movePt = evt.getPoint();
                    movePt2 = evt.getPoint();
                }
                if (SwingUtilities.isRightMouseButton(evt)) {
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                    movePt = null;
                    resizePt = null;
                }
            }
        }
    }//GEN-LAST:event_jScrollPane1MousePressed

    private void jScrollPane1KeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jScrollPane1KeyPressed
        if (evt.getKeyCode() == java.awt.event.KeyEvent.VK_ESCAPE) {
            resizePt = null;
            movePt = null;
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            repaint();
        }
    }//GEN-LAST:event_jScrollPane1KeyPressed

    private void jButton8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton8ActionPerformed
        if (image != null) {
            if (imageRef == null) {
                imageRef = image;
            }
            zoom = zoom / 2;
            setZoom();
        }
    }//GEN-LAST:event_jButton8ActionPerformed

    private void jButton9ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton9ActionPerformed
        if (image != null) {
            if (imageRef == null) {
                imageRef = image;
            }
            zoom = zoom * 2;
            setZoom();
        }
    }//GEN-LAST:event_jButton9ActionPerformed

    private void jButton10ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton10ActionPerformed
        setValue(imageRef);
    }//GEN-LAST:event_jButton10ActionPerformed

    private void jScrollPane1MouseWheelMoved(java.awt.event.MouseWheelEvent evt) {//GEN-FIRST:event_jScrollPane1MouseWheelMoved
        if (evt.getWheelRotation() > 0) {
            zoom = zoom * 2;
        } else {
            zoom = zoom / 2;
        }
        if (zoom < 0) {
            zoom = 0;
        }
        setZoom();
    }//GEN-LAST:event_jScrollPane1MouseWheelMoved

    private void jToggleButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton1ActionPerformed
        if (getCursor().getType() != Cursor.DEFAULT_CURSOR) {
            jToggleButton1.setSelected(false);
        }
    }//GEN-LAST:event_jToggleButton1ActionPerformed
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                javax.swing.JFrame f = new javax.swing.JFrame();
                f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                P_ImageEditor p = new P_ImageEditor();
                f.add(p);
                f.setBounds(new java.awt.Rectangle(0, 0, 400, 400));
                f.setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton10;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton6;
    private javax.swing.JButton jButton7;
    private javax.swing.JButton jButton8;
    private javax.swing.JButton jButton9;
    private com.jdcm.editors.JCB_ObjectEditor jCB_ObjectEditor1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToggleButton jToggleButton1;
    private javax.swing.JToolBar jToolBar1;
    // End of variables declaration//GEN-END:variables
}
