package com.jdcm.editors;

import java.net.MalformedURLException;
import java.net.URLClassLoader;

/**
 * P_Classloader.java Created on 16 janvier 2008, 22:50
 * 
 * Use to display/edit URLClassLoader
 *
 * @author Marc GIBOIN
 */
public class P_LoaderEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private java.lang.ClassLoader loader;
    private boolean editing = false;
    private URLmodel mod = null;
    private Object otherobj = null;

    /**
     *Editing ClassLoader <br>
     * is used by JCB_ObjectEditor<br>
     *
     */
    public P_LoaderEditor() {
        initComponents();
        initialisation();
        setMode("Properties");
    }

    /** Creates new form P_View_Packet 
     *
     *
     * @param p a loader to edit
     */
    public P_LoaderEditor(java.lang.ClassLoader p) {
        initComponents();
        initialisation();
        setValue(p);
    }

    private void initialisation() {
        try {
            com.jdcm.common.D_Utils TUtils = new com.jdcm.common.D_Utils();
            jButton3.setIcon(TUtils.getIcon("Remove.gif"));
            jButton2.setIcon(TUtils.getIcon("File.gif"));
            jButton1.setIcon(TUtils.getIcon("Add.gif"));
        } catch (Exception ex) {
            jButton3.setText("Remove");
            jButton2.setText("File");
            jButton1.setText("Add");
        }
        jComboBox1.setModel(new javax.swing.DefaultComboBoxModel<>("Custum,Properties,System,thisLoader,WEB".split(",")));

    }
 /**
     * Provide an Icon if needed (asked by frames)
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
             return (new com.jdcm.common.D_Utils()).getImage("loader.gif");
        }catch(Exception ex){
            return null;
        }
    }
    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        if (loader instanceof java.net.URLClassLoader) {
            editing = bool;
        } else {
            editing = false;
        }
    }
/**
 * Manage the changes of the text if editied directly
 * @param txt the TextComponent modified to treate
 */
    @Override
    public void TextManager(javax.swing.text.JTextComponent txt) {
    }
    /**
     * return the basic label to be displayed as editor
     * @param txt the initial Text
     * @return the label to be displayed for this editor
     */
    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent txt) {
        if (mod != null) {
            java.net.URLClassLoader l = (java.net.URLClassLoader) getValue();
            txt.setText(l.getClass().getSimpleName() + "[" + l.getURLs().length + "]");
        } else if (loader != null) {
            if (loader == ClassLoader.getSystemClassLoader()) {
                txt.setText("SystemClassLoader");
            } else {
                txt.setText(loader.toString());
            }
        } else {
            txt.setText("Unknown loader : " + otherobj.toString());
        }
        return txt;
    }

    @Override
    public final void setValue(Object val) {
        if (val instanceof URLClassLoader) {
            loader = null;
            URLClassLoader l = (URLClassLoader) val;
            if (l.getURLs().length > 0) {
                mod = new URLmodel(l.getURLs());
                jList1.setModel(mod);
               // jComboBox1.setSelectedItem("Custum");
            } else {
                // mod = new URLmodel(getdefaultURLs());
                setMode("Properties");
            }
            setEdition(true);
        } else if (val instanceof java.lang.ClassLoader) {
            mod = null;
            if (val.equals(ClassLoader.getSystemClassLoader())) {
                jComboBox1.setSelectedItem("System");
            } else {
                loader = (java.lang.ClassLoader) val;
            }
            setEdition(false);
        } else {
            otherobj = val;
            setEdition(false);
        }

    }

    private void setEdition(boolean bool) {
        jList1.setVisible(bool);
        jButton3.setEnabled(bool);
        jButton1.setEnabled(bool);
        jButton2.setEnabled(bool);
    }

    @Override
    public java.lang.ClassLoader getValue() {
        if (loader != null) {
            return loader;
        } else if (mod != null) {
            return new java.net.URLClassLoader(mod.getURLs());
        } else {
            return null;
        }
    }

    private String getMode() {
        return jComboBox1.getSelectedItem().toString();
    }

    private void setMode(String mode) {
        if (mode.compareTo("System") == 0) {
            setValue(ClassLoader.getSystemClassLoader());
        } else if (mode.compareTo("Properties") == 0) {
            setValue(new java.net.URLClassLoader(getPropertyURLs()));
        } else if (mode.compareTo("thisLoader") == 0) {
            setValue(this.getClass().getClassLoader());
        }else if (mode.compareTo("WEB") == 0) {
            setValue(new java.net.URLClassLoader(getWebURLs()));
        }
    }
     private java.net.URL[] getWebURLs() {
                 java.net.URL[] urls;
        try {
            urls = new java.net.URL[]{
                new java.net.URL("http://sgiboin.free.fr/"),
                 new java.net.URL("http://sgiboin.free.fr/DirTest/")
             };
        } catch (MalformedURLException ex) {
            return new java.net.URL[]{};
        }

        return urls;
        
     }
    private java.net.URL[] getPropertyURLs() {
        try {
            com.jdcm.common.D_Utils TUtils = new com.jdcm.common.D_Utils();
            String slash = System.getProperty("file.separator");
            String pathsep = System.getProperty("path.separator");
            String[] rep = {System.getProperty("java.home") + slash + "lib", System.getProperty("java.home") + slash + "lib" + slash + "ext"};
            rep = TUtils.addtab(System.getProperty("java.class.path").split(pathsep), rep);
            java.net.URL chemins[] = null;
            for (int i = 0; i < rep.length; i++) {
                try {
                    if (rep[i].startsWith(slash)) {
                        rep[i] = "file:" + rep[i];
                    } else {
                        rep[i] = "file:" + slash + rep[i];
                    }
                    java.net.URL path[] = {new java.net.URL(rep[i])};
                    chemins = TUtils.addtab(chemins, path);
                } catch (MalformedURLException ex) {
                    System.out.println("P_LoadeEditor : getPropertyURLs() =>" + ex.getMessage());
                }
            }
            //add Application class loader
            java.net.URLClassLoader u = (java.net.URLClassLoader) this.getClass().getClassLoader();
            java.net.URL[] urls = u.getURLs();
            chemins = TUtils.addtab(chemins, urls);
            return chemins;
        } catch (Exception ex) {
            return null;
        }
    }

    public void addURL(String txt) {
        if (mod == null) {
            return;
        }
        jTextField1.setText(txt);
        try {
            java.net.URL a = new java.net.URL(jTextField1.getText());
            jTextField1.setBackground(java.awt.Color.white);
            jTextField1.setText("");
                    mod.addURL(a);
        } catch (java.net.MalformedURLException ex) {
            jTextField1.setBackground(java.awt.Color.RED);
        }

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList<>();
        jPanel1 = new javax.swing.JPanel();
        jButton3 = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jTextField1 = new javax.swing.JTextField();
        jButton2 = new javax.swing.JButton();
        jComboBox1 = new javax.swing.JComboBox<>();

        setLayout(new java.awt.BorderLayout());

        jList1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jList1KeyReleased(evt);
            }
        });
        jScrollPane1.setViewportView(jList1);

        add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.LINE_AXIS));

        jButton3.setToolTipText("remove");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton3);

        jButton1.setToolTipText("add");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton1);
        jPanel1.add(jTextField1);

        jButton2.setToolTipText("search");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton2);

        jComboBox1.setToolTipText("typical URLs group");
        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });
        jPanel1.add(jComboBox1);

        add(jPanel1, java.awt.BorderLayout.SOUTH);
    }// </editor-fold>//GEN-END:initComponents
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        if (jTextField1.getText().length() == 0) {
            return;
        }
        addURL(jTextField1.getText());
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        javax.swing.JFileChooser ch = new javax.swing.JFileChooser();
        if (mod == null) {
            return;
        }
        ch.setFileSelectionMode(javax.swing.JFileChooser.FILES_AND_DIRECTORIES);
        ch.addChoosableFileFilter(new javax.swing.filechooser.FileFilter() {
            //Accept all directories and jar files.

            @Override
            public boolean accept(java.io.File f) {
                if (f.isDirectory()) {
                    return true;
                } else {
                    return f.getAbsolutePath().endsWith(".jar");
                }
            }

            @Override
            public String getDescription() {
                return "Dir & Jar";
            }
        });
        if (ch.showDialog(this, "Open") == javax.swing.JFileChooser.APPROVE_OPTION) {
            try {
                mod.addURL(ch.getSelectedFile().toURI().toURL());
            } catch (java.net.MalformedURLException ex) {
                jTextField1.setText("");
            }
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        if (jList1.getSelectedIndex() == -1 || mod == null) {
            return;
        }
        mod.removeitem(jList1.getSelectedIndex());
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        setMode(getMode());
    }//GEN-LAST:event_jComboBox1ActionPerformed

    private void jList1KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jList1KeyReleased
       if (evt.getKeyCode()==java.awt.event.KeyEvent.VK_DELETE){
           jButton3ActionPerformed(null);
       }
    }//GEN-LAST:event_jList1KeyReleased

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JComboBox<String> jComboBox1;
    private javax.swing.JList<java.net.URL> jList1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextField jTextField1;
    // End of variables declaration//GEN-END:variables
}

class URLmodel extends javax.swing.AbstractListModel<java.net.URL> {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    java.net.URL[] URLS = null;

    public URLmodel(java.net.URL[] u) {
        setURLs(u);
    }

    @Override
    public int getSize() {
        if (URLS == null) {
            return 0;
        }
        return URLS.length;
    }

    @Override
    public java.net.URL getElementAt(int index) {
        return URLS[index];
    }

    public final void setURLs(java.net.URL[] u) {
        URLS = u;
    }

    public java.net.URL[] getURLs() {
        return URLS;
    }

    public void addURL(java.net.URL u) {
        for (int i = 0; i < getSize(); i++) { 
            if ( URLS[i].equals(u)){
                return;
            }
        }
        java.net.URL[] res = java.util.Arrays.copyOf(URLS, getSize() + 1);
        res[getSize()] = u;
        URLS = res;
        fireIntervalAdded(this, 0, this.getSize());
    }

    public void removeitem(int idx) {
        java.net.URL[] res = new java.net.URL[getSize() - 1];
        int pos = 0;
        for (int i = 0; i < getSize(); i++) {
            if (i != idx) {
                res[pos] = getElementAt(i);
                pos++;
            }
        }
        URLS = res;
        this.fireIntervalRemoved(this, 0, this.getSize());
    }
}
