package com.jdcm.editors;

import usb.core.*;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Locale;
import usb.util.LangCode;
import javax.swing.tree.*;

/**
 *  D_USBInt.java Created on 9 juin 2008, 12:51
 * 
 *Used to Display/edit USB Interfaces<br>
 * @author  Marc GIBOIN 
 */
public class P_USBEditor extends javax.swing.JPanel implements I_CBObjectEditor {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private usb.core.Host host = null;
    private usb.core.Interface intf = null;
    private boolean editing = false;
    private usb.core.Device dev = null;
    private usb.core.Endpoint ept = null;
    private String portid = null;
    private usb.core.Bus bus = null;
    private Object otherusbobj = null;
    private ControllerUSBListener ctrl = null;
    private DefaultTreeModel usbtm;

    /** Creates new form D_USBInt */
    public P_USBEditor() {
        initComponents();
    }
 /**
     * Provide an Icon if needed (asked by frames)
     * @return an Icon of this Panel
     */
    @Override
    public java.awt.Image getIcon() {
        try {
             return (new com.jdcm.common.D_Utils()).getImage("USB.gif");
        }catch(Exception ex){
            return null;
        }
    }
    /**
     * prints all devices on this bus inclusive the root hub
     */
    public javax.swing.tree.DefaultTreeModel seeUSBStack() {
        Bus[] busses = null;
        try {
            busses = host.getBusses();
        } catch (IOException ex) {
        }
        DefaultMutableTreeNode top = new DefaultMutableTreeNode(host.toString());
        try {
            Device device;
            for (Bus b : busses) {
                DefaultMutableTreeNode busnode = new DefaultMutableTreeNode(b);
                //loop until getDevice(i)==null?
                int i = 0;
                //<127??
                try {
                    while (i < 126) {
                        i++;
                        //active devices could be 1,3,4,5..
                        //  while ((Device) busses[k].getDevice(++i) != null) {
                        device = b.getDevice(i);
                        if (device != null) {
                            // TreeNode devnode = new com.jdcm.editors.USBNode(dev, busnode);
                            DefaultMutableTreeNode mn = new DefaultMutableTreeNode(device);
                            busnode.add(mn);
                            insertInterfaces(mn);
                        }
                    }
                } catch (IOException e) {
                    // TDeb.wri(e.getMessage());
                }
                top.add(busnode);
            }
        } catch (Exception e) {
            //   TDeb.wri(e.getMessage());
        }
        usbtm = new javax.swing.tree.DefaultTreeModel(top);
        return usbtm;
    }

    public void insertInterfaces(DefaultMutableTreeNode devicenode) {
        Device device = (Device) devicenode.getUserObject();
        try {
            if (device.getConfiguration() != null) {
                for (int l = 0; l < device.getConfiguration().getNumInterfaces(); l++) {
                    usb.core.Interface interf = null;
                    try {
                        interf = device.getConfiguration().getInterface(l, 0);
                    } catch (Exception ex) {
                        //     TDeb.wri("=>" + ex.getMessage());
                    }
                    if (interf != null) {
                        DefaultMutableTreeNode in = new DefaultMutableTreeNode(interf);
                        devicenode.add(in);
                        for (int i = 0; i < interf.getNumEndpoints(); i++) {
                            in.add(new DefaultMutableTreeNode(interf.getEndpoint(i)));
                        }
                    }
                }
            }

        } catch (IOException ex) {
        }
    }

    public void USBevent() {
        jTree1.setModel(usbtm);
    }
    /**
     * return the basic label to be displayed as editor
     * @param txt the initial Text
     * @return the label to be displayed for this editor
     */
    @Override
    public javax.swing.text.JTextComponent getLabel(javax.swing.text.JTextComponent txt) {

        if (intf != null) {
            txt.setText(" Interface-" + intf.getNumber() + " ");
        } else if (bus != null) {
            bus.getClass().getSimpleName();
            try {
                txt.setText(" bus-" + bus.getBusId() + " ");
            } catch (Exception ex) {
                txt.setText("bus- Unknown busID : " + ex.getMessage());
            }
        } else if (dev != null) {

            usb.core.Descriptor info = dev.getDeviceDescriptor();//getDeviceDescriptor();
            if (info != null) {
                txt.setText(info.getDescriptorTypeName() + "-" + dev.getAddress() + " ");
            } else {
                try {
                    Class<?> cl = Class.forName("usb.windows.DeviceImpl");
                    //if (dev instanceof usb.windows.DeviceImpl) {
                    Object o = cl.cast(dev);
                    String m = cl.getMethod("getDeviceFriendlyName").invoke(o).toString();
                    txt.setText(m + "-" + dev.getAddress() + " ");
                } catch (ClassNotFoundException ex) {
                    txt.setText(" device-" + dev.getAddress() + " ");
                } catch (Throwable ex) {
                    txt.setText(" device-" + dev.getAddress() + " ");
                }
            }
        } else if (ept != null) {
            if (ept.isInput()) {
                txt.setText("IN EndPoint");
            } else {
                txt.setText("OUT EndPoint");
            }
        } else if (host != null) {
            txt.setText("-USB Host-");
        } else {
            txt.setText("USB : Unknown : " + otherusbobj.toString());
        }
        return txt;
    }

    private void seeTreePanel(boolean bool) {
        jTextArea1.setVisible(bool == false);
        jTree1.setVisible(bool);
        jButton1.setVisible(bool);
        jButton2.setVisible(bool==false);
        // jScrollPane2.setViewport(null);
        if (bool) {
            // jScrollPane2.remove(jTextArea1);
            // jScrollPane2.add(jTree1);
            jScrollPane2.setViewportView(jTree1);
        } else {
            //  jScrollPane2.remove(jTree1);
            // jScrollPane2.add(jTextArea1);
            jScrollPane2.setViewportView(jTextArea1);
        }
        repaint();
    }

    @Override
    public void setValue(Object val) {
        if (val.getClass().getCanonicalName().startsWith("usb.")) {
            dev = null;
            bus = null;
            intf = null;
            ept = null;
        } else {
            return;
        }
        if (val instanceof usb.core.Host) {
            host = (usb.core.Host) val;
            try {
                ctrl = new ControllerUSBListener(this);
                host.addUSBListener(ctrl);
                jTree1.setModel(seeUSBStack());
                jTree1.setCellRenderer(new ACE_ObjRenderer<>());
                jTree1.setCellEditor(new ACE_ObjEditor<>());
                jTree1.setEditable(true);
                seeTreePanel(true);
            } catch (SecurityException e) {
                //     TDeb.wri("USB permissions problem:"+e.getMessage());
            } catch (USBException e) {
                //   TDeb.wri(e.getMessage());
            } catch (IOException e) {
                //  TDeb.wri(e.getMessage());
            }
        } else {
            seeTreePanel(false);
            if (val instanceof usb.core.Interface) {
                intf = (usb.core.Interface) val;
                jTextArea1.setText(getIfInfo(intf));
            } else if (val instanceof usb.core.Bus) {
                bus = (usb.core.Bus) val;
                String res = "";
                try {
                    res = res + "Bus ID :" + bus.getBusId() + "\r\n";
                    res = res + "Bus root hub :" + bus.getRootHub().toString() + "\r\n";
                    res = res + "Host : " + bus.getHost() + "\r\n";
                } catch (IOException ex) {
                    res = "Pb with bus : " + bus.toString();
                }
                jTextArea1.setText(res);
            } else if (val instanceof usb.core.Device) {
                dev = (usb.core.Device) val;
                portid = dev.getPortIdentifier();
                String usbtxt = "";
                usbtxt += showDevice();
                usbtxt += showDeviceDescriptors();
                usbtxt += showConfigurations();
                jTextArea1.setText(usbtxt);
            } else if (val instanceof usb.core.Endpoint) {
                ept = (usb.core.Endpoint) val;
                jTextArea1.setText(showEndPoint(ept));
            } else {
                otherusbobj = val;
            }
        }
    }

    @Override
    public Object getValue() {
        if (intf != null) {
            return intf;
        } else if (ept != null) {
            return ept;
        } else if (bus != null) {
            return bus;
        } else if (dev != null) {
            return dev;
        } else if (host != null) {
            return host;
        } else {
            return otherusbobj;
        }
    }

    @Override
    public boolean isEditing() {
        return editing;
    }

    @Override
    public void setEditing(boolean bool) {
        editing = bool;
    }
/**
 * Manage the changes of the text if editied directly
 * @param txt the TextComponent modified to treate
 */
    @Override
    public void TextManager(javax.swing.text.JTextComponent txt) {
    }

    public String getdetails() {
        return jTextArea1.getText();
    }
    // this doesn't handle alternates right yet ...

    private String getIfInfo(usb.core.Interface intface) {
        String res = "";
        res += "Interface\r\n";
        if (intface == null) {
            return res + " is null";
        }
        res = "-" + res + intface.toString() + "\r\n";
        try {
            // some devices don't expose these when configured

            String temp;
            int value;

            res = res + "-Interface " + intface.getNumber() + "\r\n";
            value = intface.getInterfaceClass();
            if (value != 0) {
                res = res + "- (" + intface.getInterfaceClassName() + ")\r\n";
            }
            value = intface.getAlternateSetting();
            if (value != 0) {
                res = res + ", alt " + value + "\r\n";
            }
            temp = intface.getInterface(0);
            if (temp == null) {
                temp = "N/A";
            }
            res = res + "-Description: " + temp + "\r\n";
            res = res + "-String(): " + intface.toString() + "\r\n";
            try {
                temp = intface.getClaimer();
            } catch (IOException ex) {
            }
            if (temp == null) {
                temp = "N/A";
            }
            res = res + "-Claimer: " + temp + "\r\n";
            res = res + "-class: " + intface.getInterfaceClass() + "-" + Interface.getClassName(intface.getInterfaceClass(), "0");
            res = res + "-subclass: " + intface.getInterfaceSubClass() + "-" + Interface.getClassName(intface.getInterfaceSubClass(), "0");
            //buf.append (", subclass "+intf.getInterfaceSubclass ());
            res = res + ", protocol " + intface.getInterfaceProtocol() + "\r\n";
            // NOTE:  can't try for interface status (zeroes)
            // or endpoint status (HALT) unless we claimed the
            // interface, which we don't want at this time.

        } catch (Exception e) {
            res += "Interface data unavailable. \r\n";
            res += "Diagnostic:\r\n";
            res = res + e.getMessage() + "\r\n";
        }
        return res;
    }

    protected String showEndPoint(usb.core.Endpoint endp) {
        String res = "";
        res += "Desc. Type: " + endp.getDescriptorTypeName() + "\r\n";
        res += "Type: " + endp.getType() + "\r\n";
        res += "isInput: ";
        if (endp.isInput()) {
            res += " IN \r\n";
        } else {
            res += " OUT \r\n";
        }
        //  if ("interrupt".equals(endp.getType())) {
        res += "Interval: " + endp.getInterval() + " ms. \r\n";
        //   }
        res += "MaxPacketSize: " + endp.getMaxPacketSize() + "\r\n";
        res += "Attributs: " + endp.getAttributes() + "\r\n";
        res += "getEndpointAddress: " + endp.getEndpointAddress();
        return res;
    }

    /**
     * This method is a hook to provide class-specific data (in HTML)
     * when this node is selected.  Subclasses using the default HTML
     * display should append markup to the string buffer, ending in
     * an empty line (use <em>&lt;br&gt;;&lt;br&gt;</em>).  Note that
     * this can be used both to provide standardized USB class data
     * (e.g. for hubs and printers) as well as "class-per-interface"
     * style data, for devices with less standard behaviours.
     * 
     * @return the devices
     */
    protected String showDevice() {
        String res = "";
        String temp = dev.toString();
        if (temp == null) {
            temp = "N/A";
        }
        res = res + dev.toString() + "\r\n";
        res += "Device: \r\n";
        res = res + "-Port Identifier: " + portid + "\r\n";
        res = res + "-device speed  (!.getspeed?):" + temp + "\r\n";
        res = res + "-address: " + dev.getAddress() + "\r\n";
        res = res + "-HubPortNum : " + dev.getHubPortNum() + "\r\n";
        res = res + "-NumPorts : " + dev.getNumPorts() + "\r\n";
        try {
            Class<?> cl = Class.forName("usb.windows.DeviceImpl");
            Object o = cl.cast(dev);
            String m = cl.getMethod("getDeviceFriendlyName").invoke(o).toString();
            res += "USB Windows :\r\n";
            res = res + "-USB FriendlyDeviceName: " + cl.getMethod("getDeviceFriendlyName").invoke(o).toString() + "\r\n";
            res = res + "-USB DevicePath: " + cl.getMethod("getDevicePath").invoke(o).toString() + "\r\n";
            res = res + "-USB UniqueDeviceID: " + cl.getMethod("getUniqueDeviceID").invoke(o).toString() + "\r\n";
            res = res + "-USB DriverKeyName: " + cl.getMethod("getDriverKeyName()").invoke(o).toString() + "\r\n";
        } catch (ClassNotFoundException ex) {
        } catch( NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
        }
        return res;
    }

    private String showDeviceDescriptors() {
        String res = "";
        res += "Device Descriptor \r\n";
        usb.core.DeviceDescriptor info = dev.getDeviceDescriptor();
        if (info == null) {
            return "no Descriptor\r\n";
        }
        res += showDeviceDescriptor(info);
        return res;
    }

    private String showDeviceDescriptor(usb.core.DeviceDescriptor dd) {
        String res = "";
        res = res + "-bcdUSB: " + dd.getUSBVersion() + "\r\n";
        res = res + "-type: " + dd.getDescriptorTypeName() + "\r\n";
        res = res + "-Device Class: " + dd.getDeviceClass() + " ( " + dd.getDeviceClassName() + " )\r\n";
        res = res + "-Device Subclass: " + dd.getDeviceSubClass() + "\r\n";
        res = res + "-Device Protocol: " + dd.getDeviceProtocol() + "\r\n";
        res = res + "-Max Packet Size: " + dd.getMaxPacketSize0() + "\r\n";
        res = res + "-idVendor: " + dd.getVendorId() + "      0x" + Integer.toHexString(dd.getVendorId()) + "\r\n";
        res = res + "-idProduct: " + dd.getProductId() + "      0x" + Integer.toHexString(dd.getProductId()) + "\r\n";
        res = res + "-bcdDevice release number: " + dd.getDeviceId() + "\r\n";
        res = res + "-iManufacturer: " + dd.getManufacturerStringId() + "\r\n";
        res = res + "-iProduct: " + dd.getProductStringId() + "\r\n";
        res = res + "-iSerialNumber: " + dd.getSerialStringId() + "\r\n";
        res = res + "-NumConfiguration: " + dd.getNumConfigurations() + "\r\n";

        //temp = info.getManufacturer(0);
        //temp = info.getProduct(0);
        // temp = info.getSerial(0);
        int languages[] = null;
        try {
            languages = ControlMessage.getLanguages(dev);
        } catch (IOException ex) {
        }
        if (languages != null) {
            res += "-Languages: ";
            for (int langidx : languages) {
                Locale locale = LangCode.getLocale(langidx);
                if (locale != null) {
                    res += locale.toString();
                } else {
                    res = res + "0x" + Integer.toHexString(langidx);
                }
            }
            res += "\r\n";
        }
        try {
            byte status[] = ControlMessage.getStatus(dev,
                    ControlMessage.RECIPIENT_DEVICE, 0, 0, 2);

            if (ControlMessage.getBit(0, status, Device.DEVICE_SELFPOWERED)) {
                res += "-Currently self-powered\r\n";
            }
            if (ControlMessage.getBit(1, status, Device.DEVICE_REMOTE_WAKEUP)) {
                res += "-Remote Wakeup is enabled\r\n";
            }

        } catch (IOException e) {
            // should never happen !!
            res += "USB Exception on getDeviceStatus:\r\n";
            res = res + e.getMessage() + "\r\n";
        }
        return res;
    }

    private String showConfigurations() {
        String res = "";
        usb.core.Configuration cd = null;
        try {
            res += "Configuration :\r\n";
            res += showConfiguration(dev.getConfiguration());
        } catch (IOException ex) {
            return "no Configuration \r\n";
        }
        try {
            int i = 0;
            while (dev.getConfiguration(i) != null) {
                res = res + "Configuration-" + i + ": \r\n";
                res += showConfiguration(dev.getConfiguration(i));
            }
        } catch (IOException ex) {
        }
        return res;
    }

    private String showConfiguration(usb.core.Configuration cd) {
        String res = "";
        res = res + "-wTotalLength: " + cd.getTotalLength() + "\r\n";
        res = res + "-bNumInterfaces: " + cd.getNumInterfaces() + "\r\n";
        res = res + "-bConfigurationValue: " + cd.getConfigurationValue() + "\r\n";
        res = res + "-bmAttributes: " + cd.getAttributes() + "\r\n";
        if ((Configuration.ATTR_REMOTE_WAKEUP & cd.getAttributes()) == Configuration.ATTR_REMOTE_WAKEUP) {
            res += "-Device supports Remote Wakeup!\r\n";
        }
        if ((Configuration.ATTR_SELF_POWERED & cd.getAttributes()) == Configuration.ATTR_SELF_POWERED) {
            res += "-Device is Self Powered!\r\n";
        }
        res = res + "-bMaxPower: " + cd.getMaxPower();
        res = res + "  ( " + (cd.getMaxPower() * 2) + "mA)" + "\r\n";
        return res;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jButton2 = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        jTree1 = new javax.swing.JTree();

        setLayout(new java.awt.BorderLayout());

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.LINE_AXIS));

        jButton2.setText("Host");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton2);

        jButton1.setText("Select");
        jButton1.setEnabled(false);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton1);

        add(jPanel1, java.awt.BorderLayout.NORTH);

        jTextArea1.setEditable(false);
        jTextArea1.setColumns(20);
        jTextArea1.setRows(5);
        jTextArea1.setPreferredSize(new java.awt.Dimension(50, 94));
        jScrollPane2.setViewportView(jTextArea1);

        jTree1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jTree1MouseClicked(evt);
            }
        });
        jScrollPane2.setViewportView(jTree1);

        add(jScrollPane2, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    private void jTree1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTree1MouseClicked
        jButton1.setEnabled(jTree1.getSelectionPath() != null || jTree1.isEditing());
    }//GEN-LAST:event_jTree1MouseClicked

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        if (jTree1.getSelectionPath().getLastPathComponent() instanceof DefaultMutableTreeNode) {
            Object res = ((DefaultMutableTreeNode) jTree1.getSelectionPath().getLastPathComponent()).getUserObject();
            setValue(res);
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        setValue(host);
    }//GEN-LAST:event_jButton2ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTree jTree1;
    // End of variables declaration//GEN-END:variables
}

// @java.lang.SuppressWarnings("");
class ControllerUSBListener extends usb.core.USBListenerAdapter {

    //private Host host;
    private final P_USBEditor sl;
    private Bus bus;
    

    /** Creates a new instance of MyUSBListener */
    public ControllerUSBListener(P_USBEditor mainwin) throws java.io.IOException {
        sl = mainwin;
    }

    @Override
    public void busAdded(Bus bus) throws java.io.IOException {
        //  System.out.println("\n>>> USB L I S T E N E R <<<  Bus to listener added! BusID: " + bus.getBusId() + "\n");
        sl.seeUSBStack();
        sl.USBevent();
    }

    @Override
    public void busRemoved(Bus bus) throws java.io.IOException {
        // System.out.println("\n>>> USB L I S T E N E R <<<  Bus from listener removed! BusID: " + bus.getBusId() + "\n");
        sl.seeUSBStack();
        sl.USBevent();
    }

    @Override
    public void deviceAdded(Device dev) throws java.io.IOException {
        // System.out.println("\n>>> USB L I S T E N E R <<<  Device to BusID[" + dev.getBus().getBusId() + "] added! Port Identifier: " + dev.getPortIdentifier() + "\n ");
        sl.seeUSBStack();
        sl.USBevent();
    }

    @Override
    public void deviceRemoved(Device dev) throws java.io.IOException {
        //  System.out.println("\n>>> USB L I S T E N E R <<<  Device from BusID[" + dev.getBus().getBusId() + "] removed!  Port Identifier: " + dev.getPortIdentifier() + "\n ");
        sl.seeUSBStack();
        sl.USBevent();
    }
}
/**
 * prints all devices on this bus inclusive the root hub

private void printUSBStack(Bus bus) {
try {
usb.windows.DeviceImpl dev;
usb.linux.HID.

for (int i = 0; i < 7; i++) {
dev = (usb.windows.DeviceImpl) bus.getDevice(i);
System.out.print(" [ " + i + " ] :  ");
if (dev != null) {
if (dev.getAddress() == 0) {
System.out.println(" [ROOT]          numOfPort:" + dev.getNumPorts() + "  Address:" + dev.getAddress());
} else {
if (dev.getNumPorts() > 0) {
System.out.println(" [EXTERNAL HUB]  numOfPort:" + dev.getNumPorts() + "  Address:" + dev.getAddress());
} else {
System.out.println(" [USB DEVICE]    on Port " + dev.getHubPortNum() + "  Address           : " + dev.getAddress());
}
System.out.println("                                      uniqueID          : " + dev.getUniqueDeviceID());
System.out.println("                                      driverKeyName     : " + dev.getDriverKeyName());
System.out.println("                                      friendlyDeviceName: " + dev.getFriendlyDeviceName());
if (dev instanceof Device) {
System.out.print("                                      Object Type       : Device");
}
if (dev instanceof usb.windows.DeviceImpl) {
System.out.print(", DeviceImpl");
}
if (dev instanceof usb.windows.JUSB) {
System.out.println(", JUSB");
}
if (dev instanceof usb.windows.NonJUSB) {
System.out.println(", NonJUSB");
}
}
} else {
System.out.println(" [NULL] ");
}
}
} catch (Exception e) {
e.printStackTrace();
}
}*/
