package com.jdcm.eltrx;

import com.jdcm.analyse.I_Code_Parent;
import com.jdcm.common.C_Packet;
import com.jdcm.common.D_Utils;
import com.jdcm.common.A_Commande;
import com.jdcm.common.Options_Model;
import com.jdcm.common.XML2tree;
import com.jdcm.editors.P_TableEditor;
import com.jdcm.charts.P_Stat_P;
import com.jdcm.charts.P_Stat_RS;
import gnu.io.PortInUseException;
import java.io.*;
import java.awt.*;
import java.lang.reflect.InvocationTargetException;
import java.net.UnknownHostException;
import java.util.*;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * C_Port.java Created on 6 aout 2006, 16:05
 *
 * Port is use to send/receive infoprmations<br>
 *
 * outsid_in_=>write_in=> Main<br> outside '<= write_out <= readd_in <=Main'<br>
 *
 *
 * @author Marc GIBOIN
 */
public class C_Port implements I_Port, I_Command, I_Code_Parent {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private final D_Debug TDeb;
    private final long[][] accounting = new long[2][4];
    private final DefaultComboBoxModel<String> CBstrModel = new DefaultComboBoxModel<>();

    private D_Utils TUtils = new D_Utils();
    public C_PScan scan = null;
    public I_Slave slave = null;
    private I_Main Parent;
    private P_Stat_RS stat_RS = null;
    private P_Stat_P stat_P = null;
    private OutputStream out = null;
    private InputStream in = null;
    private C_Port listenport;           // SHould be in slave but needed out of slave
    private boolean slreading;
    private boolean slrunning;
    private boolean isreading = false;
    // option & F_option are model & table for options (DCX/CX/DESC/Par/SSL/...)
    private Options_Model option;
    private P_TableEditor F_option;
    //status of the port(see label1)
    private java.awt.Color sparecolor = java.awt.Color.BLACK;
    private java.util.List<String> modenames = null;
    // the Componement displayed to see this port (typical IF_Port)
    private java.awt.Component portSkin = null;
    // for system ports (capture reading/shell/...) identification
    private int system_index = -1;

    /**
     * Creates new form C_Port
     */
    public C_Port() {
        init();
        system_index = 100;
        //  setIndex(-1);
        TDeb = new D_Debug("Port: [-1]");
        TUtils = new D_Utils();
        init_Port();
        listenport = null;
        constructors();
    }

    public C_Port(I_Main P, I_Console c) {
        init();
        Parent = P;
        TDeb = new D_Debug("Port: [new]");
        TDeb.setconsole(c);
        init_Port();
        listenport = null;
        option.setValuefor("TCP_Lmax", -1);
        constructors();
    }

    public C_Port(I_Main P, I_Console c, int idx) {
        init();
        system_index = idx;
        Parent = P;
        TDeb = new D_Debug("SysPort: [" + system_index + "]");
        TDeb.setconsole(c);
        init_Port();
        listenport = null;
        option.setValuefor("TCP_Lmax", -1);
        constructors();
    }

    private void init() {
        //must respect cabs in IF_Port
        String[] labels = {"IPL4", "Serial", "File", "Term", "DB", "ETH", "WIN", "SYS", "USB", "SND"}; //"LPT", "MEM","MOM","JNA" };
        modenames = java.util.Arrays.asList(labels);
        initOptions();
    }

    private void constructors() {
        if (this.getConsole() != null) {
            setColor(this.getConsole().getColor(getIndex()));
        } else {
            setColor(java.awt.Color.BLACK);
        }
        this.setboxenable(true);
        setcxstatus("new");
        setTerm_Type("classic");
    }

    public java.util.List<String> getModesNames() {
        return modenames;
    }

    @Override
    public void setSkin(java.awt.Component obj) {
        if (portSkin != null) {
            portSkin.setVisible(false);
            if (portSkin.getParent() instanceof java.awt.Container) {
                portSkin.getParent().remove(portSkin);
            }
            portSkin = null;
        }
        portSkin = obj;
    }

    @Override
    public java.awt.Component getSkin() {
        return portSkin;
    }

    public void displayPort() {
        if (Parent instanceof C_Main) {
            ((C_Main) Parent).displayPort(this);
        }
    }

    // should be managed in each hook (setTCP../setRS../...)
    public void optionChanged(String label, Object obj) {
        if (obj == null || portSkin == null) {
            return;
        }
        java.lang.reflect.Method meth = TUtils.getMethod(portSkin.getClass(), label, obj.getClass());
        if (meth != null) {
            try {
                meth.invoke(portSkin, obj);
            } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException exe) {
                try {
                    meth.invoke(portSkin, "" + obj);
                } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
                }
            }
        }
    }

    @Override
    public final void init_Port() {
        setCX_autocx(true);
        setWIN_kbd(java.util.Locale.getDefault().getDisplayCountry());
        initFILEbox();
        init_DBbox();
    }

    @Override
    public String toString() {
        return getPortName();
    }

    public String getTitle() {
        String res = getPortName() + "[" + getModeName() + "]";
        return res;
    }

    private void initOptions() {
        // create & feel defaults NAME/VAL (&DEFAULT)/TYPE/DESC/USED/NULLONEMPTY/COMMENT
        option = new Options_Model(this);
        option.addOption("Acc_initsize", 0, "int", ">0", false, false, "accounting size waited");
        option.addOption("Acc_position", 0, "int", ">0", false, false, "accounting position");
        option.addOption("Acc_startdate", new java.util.Date(0), "date", ">0", false, false, "last start timestamp");
        option.addOption("Acc_stopdate", new java.util.Date(0), "date", ">0", false, false, "last stop timestamp");
        option.addOption("CX_Banner", new byte[0], "Byte[]", "", true, true, "transmited on connection");
        option.addOption("CX_strin", new byte[0], "Byte[]", "", true, true, "waited to connect");
        option.addOption("CX_TOval", 0, "msec", ">0", true, false, "wait before connect");
        option.addOption("CX_autocx", true, "bool", "0/1", true, false, "automatic connect after openned");
        option.addOption("CX_autoopen", true, "bool", "0/1", true, false, "automatic open after started");
        option.addOption("CX_openbyslave", true, "bool", "0/1", true, false, "szt if slave open connection else main open it");
        option.addOption("CX_tagsend", false, "bool", "0/1", true, false, "send TAG after connect");
        option.addOption("CX_tagwait", false, "bool", "0/1", true, false, "wait TAG to connect");
        option.addOption("CX_tagval", new byte[0], "Byte[]", " used string...", true, true, "Define TAG value");
        option.addOption("CX_Transmit", false, "bool", "0/1", true, false, "Transmit STRIN after connect");
        option.addOption("DCX_Banner", new byte[0], "Byte[]", "", true, true, "Transmitted before disconnect");
        option.addOption("DCX_strin", new byte[0], "Byte[]", "", true, true, "Waited to disconnect");
        option.addOption("DCX_Transmit", true, "bool", "0/1", true, false, "Transmit STRIN before disconnect");
        option.addOption("DCX_TOcom", false, "bool", "0/1", true, false, "Enable comm time without comm before disconnect");
        option.addOption("DCX_TOcomval", 1000, "msec", ">0", false, false, "Define time without comm before disconnect");
        option.addOption("DCX_TOabs", false, "bool", "0/1", true, false, "Enable absolute time before disconnect");
        option.addOption("DCX_TOabsval", 7000, "msec", ">0", false, false, "define absolute time before disconnect");
        option.addOption("DCX_tostop", false, "bool", "0/1", true, false, "ask stop the port instead of disconnection");
        option.addOption("Par_serial", false, "bool", "0/1", true, false, "Enable bytes=>object and obj => bytes on port");
        option.addOption("Par_readtoend", true, "bool", "0/1", true, false, "read the stream until waited data are red from stream");
        option.addOption("Par_readat", false, "bool", "0/1", true, false, "Enable reading only a limited size from the stream");
        option.addOption("Par_readingat", true, "bool", "0/1", false, false, "allow readinig the stream after limited size reading (readat)");
        option.addOption("Par_readatval", 5000, "bytes", ">0", true, false, "define the maximum size to read in the stream");
        option.addOption("Par_readatloop", true, "bool", "0/1", false, false, "loop a limited size reading of the stream");
        option.addOption("Par_mxgrp", "", "List<String>", "AAZZEE", true, false, "selectors for group matrix");
        option.addOption("Par_PadtStr", false, "bool", "0/1", true, false, "Enable Pading message received within time interval On Stream");
        option.addOption("Par_PadtStrval", 40, "msec", ">0", true, false, "define PAD time interval On Stream ");
        option.addOption("Term_Type", new DefaultComboBoxModel<String>(), "type name List", "", true, true, "Terminal Class Name (empty is classic)");
        option.addOption("Term_outMode", new DefaultComboBoxModel<>(F_Terminal.outModes), "output List", "", true, true, "Terminal output Mode");
        option.addOption("Term_capReader", false, "bool", "", true, true, "Terminal receve models to set as frames");
        option.addOption("DB_interfaces", new DefaultComboBoxModel<String>(), "DB interface list", "MySQL/ODBC/..", true, true, "DB interfaces");
        option.addOption("DB_name", "", "String", "", true, true, "the JDBC name of the DB");
        option.addOption("DB_user", "root", "String", "", true, true, "the JDBC user name of the DB access");
        option.addOption("DB_pwd", "admin", "String", "", true, true, "the JDBC user password of the DB access");
        option.addOption("DB_host", "localhost", "String", "", true, true, "the host addresse of the DB host");
        option.addOption("DB_port", 3386, "int", "1-65580", true, true, "tcp port of the JDBC drivers service");
        option.addOption("DB_driver", String.class, "Class", "", true, true, "the JDBC driver class");
        option.addOption("DB_URL", "", "txt", "", true, true, "URL string of the DB");
        //java.nio.charset.Charset.defaultCharset().displayName();
        java.util.List<java.nio.charset.Charset> l = new java.util.ArrayList<>(java.nio.charset.Charset.availableCharsets().values());
        DefaultComboBoxModel<java.nio.charset.Charset> co = new DefaultComboBoxModel<>(l.toArray(new java.nio.charset.Charset[]{}));
        co.setSelectedItem(java.nio.charset.Charset.defaultCharset());
        option.addOption("DB_charSet", co, "charSet", "", true, true, "CharsetPropertie for DB");
        option.addOption("DB_maxrows", 300, "nb", "", true, true, "Nb of resultset maximum rows (0=>infinite)");
        option.addOption("DB_TO", 3, "sec", "", true, true, "TimeOut of an SQL request(0=>infinite)");
        option.addOption("DB_fetchnb", 100, "rows", "", true, true, " the number of rows to fetch(0=>ignored)");
        String[] DBout = {"ResultSet", "Tree", "Table"};
        option.addOption("DB_resultType", new DefaultComboBoxModel<>(DBout), "data models", "ResultSet/Tree/Table", true, true, "DB result type no implemented");
        option.addOption("DB_pack2DB", false, "bool", "0/1", true, true, "feel 'shema.cxPacket' with Packets id is Hash");
        option.addOption("DB_errors", true, "bool", "0/1", true, true, "return errors, if false return nothing");
        option.addOption("DB_mapfile", new DefaultComboBoxModel<File>(), "file", "", true, true, "mapping file for DB to XML");
        option.addOption("SSL_Enabled", false, "bool", "0/1", true, false, "Enable SSL");
        option.addOption("SSL_Kfile", new DefaultComboBoxModel<File>(), "certificats", "", false, false, "Define the Key file");
        option.addOption("SSL_Tfile", new DefaultComboBoxModel<File>(), "certificats", "", false, false, "Define the Trusted file");
        option.addOption("SSL_Kpwd", "123456", "txt", "", false, false, "Define the Key file password");
        option.addOption("SSL_Tpwd", "123456", "txt", "", false, false, "Define the Trusted file password");
        option.addOption("SSL_SrvAuth", true, "bool", "0/1", false, false, "Enable SrvAuth");
        option.addOption("SSL_CltAuth", false, "bool", "0/1", false, false, "Enable ClientAuth");
        String[] alg = {"SunX509"};
        option.addOption("SSL_KAlgo", new DefaultComboBoxModel<>(alg), "SSLAlgo", "", false, false, "Define Key Algo");
        option.addOption("SSL_TAlgo", new DefaultComboBoxModel<>(alg), "SSLAlgo", "", false, false, "Define Trust Algo");
        String[] typ = {"JKS", "PKCS12"};
        option.addOption("SSL_KType", new DefaultComboBoxModel<>(typ), "certType", "", false, false, "Define Key file type");
        option.addOption("SSL_TType", new DefaultComboBoxModel<>(typ), "certType", "", false, false, "Define Trusted file type");
        String[] proto = {"TLS", "TLSv1", "SSL", "SSLv2", "SSLv3"};
        option.addOption("SSL_Proto", new DefaultComboBoxModel<>(proto), "SSLproto", "", false, false, "Specify SSL protocol");
        DefaultComboBoxModel<Integer> c = new DefaultComboBoxModel<>(new Integer[]{300, 1200, 2400, 4800, 9600, 14400, 19200, 28800, 38400, 57600, 115200});
        c.setSelectedItem(9600);
        option.addOption("RS_Rate", c, "int list", "1200;9600;14400;...", false, false, " normalized speed rates of the port");
        option.addOption("RS_Databit", 8, "int", "5-8", false, false, "specify number of data bits in asynch bytes");
        option.addOption("RS_FlowCtrl", "none", "String", "none/X/..", false, false, "type of flow ctrl on asynch port");
        option.addOption("RS_Parity", "none", "String", "none/even/pair/..", false, false, "specify parity on the asynch port");
        option.addOption("RS_Stopbit", "1", "String", "1/1.5/2", false, false, "specify number of stop bit");
        option.addOption("RS_OpenTO", 1000, "int", "1-inf?", false, false, "??? to check");
        option.addOption("RS_SysLab", "", "String", "xxx", false, false, "System Label for serial process");
        option.addOption("RS_RTS", false, "bool", "0/1", false, false, "specify if RTS is UP");
        option.addOption("RS_DTR", false, "bool", "0/1", false, false, "specify if DTR is UP");
        option.addOption("RS_Lights", false, "bool", "0/1", false, false, "specify if asynch signamls lights are runing");
        option.addOption("RS_interfaces", new DefaultComboBoxModel<String>(), "Serial interface list", "", true, true, "Serial interfaces");
        option.addOption("LPT_interfaces", new DefaultComboBoxModel<String>(), "LPT interface list", "", true, true, "LPT interfaces");
        option.addOption("LPT_SysLab", "Jdatacom", "String", "", true, true, "Label of the LPT user program enty");
        String[] lptmode = {"EPP", "ECP", "NIBBLE", "PS2", "SPP"};
        option.addOption("LPT_Mode", new DefaultComboBoxModel<>(lptmode), "liste", "", true, true, "LPT mode used EPP/ECP/NIBBLE/PS2/SPP");
        option.addOption("LPT_OpenTO", 0, "int", "", true, true, "LPT open TineOut");
        option.addOption("MEM_wire", 0, "int", "int", false, false, "nb of the virtual wire to use");
        option.addOption("SYS_CheckSM", false, "bool", "0/1", false, false, "true if Security Manager checked before reuning");
        option.addOption("SYS_lib", null, "String", "fileName", false, false, "library file name to use with system execution");
        option.addOption("SYS_exitasbyte", false, "bool", "0/1", false, false, "if exit value is sended as a byte array");
        option.addOption("SYS_intoproc", false, "bool", "0/1", false, false, "send input to running process or keep it as new commandes");
        option.addOption("SYS_nbmup", -1, "int", "", false, false, "nb of microprocessor used for this task");
        option.addOption("SYS_status", "", "String", "", false, false, "???");
        option.addOption("SYS_sendExit", false, "bool", "0/1", false, false, "if process exit value is the only received value");
        option.addOption("SYS_sendOut", true, "bool", "0/1", false, false, "if process output stream is received");
        option.addOption("SYS_DCXatend", false, "bool", "0/1", false, false, " will stop on the process kill or wait for next cmde");
        option.addOption("SYS_stdinout", false, "bool", "0/1", false, false, "if port is used with standard in and out");
        option.addOption("WIN_sendmousmv", true, "bool", "0/1", true, false, "Enable transmit mouse mouve");
        option.addOption("WIN_sendmousmvTO", 10, "int", "ms", true, false, "period in ms to check if mouse mouved");
        option.addOption("WIN_rcvmousmv", true, "bool", "0/1", true, false, "Enable apply mouse mouve");
        option.addOption("WIN_sendkeys", true, "bool", "0/1", true, false, "Enable key tobe send");
        option.addOption("WIN_rcvkeys", false, "bool", "0/1", true, false, "Enable key to be received");
        option.addOption("WIN_sendscreen", false, "bool", "0/1", true, false, "Enable key to be received");
        option.addOption("WIN_sendscreenTO", 1000, "int", "ms", true, false, "period in ms to send a screen copy");
        String[] kbd = {"Basic", "France"};
        option.addOption("WIN_kbd", new DefaultComboBoxModel<>(kbd), "Keyboard", "", false, false, "Keyboard used to send keys");
        option.addOption("TCP_Mode", "TCP_C", "String", "TCP_C/TCP_L/UDP", false, false, "set IPL4 port Mode");
        java.net.InetAddress addr = null;
        try {
            addr = java.net.InetAddress.getLocalHost();
        } catch (UnknownHostException e) {
            try {
                addr = java.net.InetAddress.getByName("10.0.0.1");
            } catch (UnknownHostException ex) {
                System.out.println("C_Port : initOptions() Pb addrIP=>" + ex.getMessage());
            }
        }
        option.addOption("TCP_Addr", addr, "InetAddr", "all IPs", false, false, "set destination IP adress");
        option.addOption("TCP_Port", 2000, "int", "0-65580", false, false, "if destination port");
        option.addOption("TCP_ReuseAddr", false, "bool", "0/1", true, false, "before binding, allow TO befor close");
        option.addOption("TCP_linger", false, "bool", "0/1", true, false, "closing socket specificaton,rcv discard");
        option.addOption("TCP_OOBinline", false, "bool", "0/1", true, false, "enable urgent data");
        option.addOption("TCP_lingerTO", 1, "int", "0-inf.", true, false, "timeout (sec) before exception");
        option.addOption("TCP_SoTO", 100, "int", "0-inf.", false, false, "in ms the timeout of the socket blocks  period");
        String[] clas_ip = {"IPTOS_NONE", "IPTOS_LOWCOST", "IPTOS_RELIABILITY", "IPTOS_THROUGHPUT", "IPTOS_LOWDELAY"};
        option.addOption("TCP_class", new DefaultComboBoxModel<>(clas_ip), "IPTOS classes list", "0-255", true, false, "the class of the traffic (!) to work");
        option.addOption("TCP_keepalive", false, "bool", "0/1", false, false, "Set TCP KEEPALIVE option");
        option.addOption("TCP_nodelay", false, "bool", "0/1", false, false, "set the NODELAY option");
        option.addOption("TCP_Lmax", -1, "int", "0-inf.", false, false, "the maximum of connected port to a listner 0 is inf.");
        option.addOption("TCP_ioBuffers", new Point(-1, -1), "int[]", "0-inf.", true, false, "the rcv and send buffers");
        option.addOption("USB_Element", null, "USB Element", "", true, true, "USB element deice,interface,host,..");
        // see javax.sound....filetypes
        option.addOption("File_traces", new DefaultComboBoxModel<File>(), "file liste", "", true, true, "traces file listing");
        option.addOption("File_ClearatStart", false, "bool", "0/1", true, true, "else keep file data and follow writting");
        option.addOption("File_EOFstop", true, "bool", "0/1", true, true, "stop the port on EOF else continue to read (on fly)");
        option.addOption("File_read", true, "bool", "0/1", true, true, "set if file access as input (read)");
        option.addOption("File_write", true, "bool", "0/1", true, true, "set if file access as output (write)");
        option.addOption("File_addExt", false, "bool", "0/1", true, true, "set a writing fimename extention with timestamp on starting");
        option.addOption("File_addr", new java.io.File("test"), "File", "", true, true, "file for input/output");
        option.addOption("File_XML2tree", false, "bool", "0/1", true, true, "XML files are tree object input");
        String[] eth_pack = {"BUFFER", "PACKET", "TRANSPORT"};
        option.addOption("ETH_Pack", new DefaultComboBoxModel<>(eth_pack), "bool", "BUFFER|PACKET|TRANSPORT", true, true, "transmit input packet as jpcap packet, byte array or tcp/udp payload ");
        option.addOption("ETH_filter", "", "String", "", true, true, "set the ETH capture fliter");
        option.addOption("ETH_interfaces", new DefaultComboBoxModel<String>(), "Ethernet interface list", "", true, true, "Ethernet capture interfaces");
        option.addOption("ETH_promiscuous", true, "bool", "0/1", true, true, "set captuire as promisc.");
        option.addOption("SND_linesin", new DefaultComboBoxModel<javax.sound.sampled.Mixer>(), "list", "micro/apli/...", true, true, "Source audio line sampled");
        option.addOption("SND_linesout", new DefaultComboBoxModel<javax.sound.sampled.Mixer>(), "list", "spkr/line/headph...", true, true, "Target audio line sampled");
        option.addOption("SND_mixers", new DefaultComboBoxModel<javax.sound.sampled.Mixer>(), "list", "mixers...", true, true, "audio mixer to usesampled");
        option.addOption("SND_formatin", new javax.sound.sampled.AudioFormat(0.0f, 8, 1, false, true), "AudioFormat", "format", true, true, "audio format to usesampled");
        option.addOption("SND_formatout", new javax.sound.sampled.AudioFormat(0.0f, 8, 1, false, true), "AudioFormat", "format", true, true, "audio format to usesampled");
        // should not be set as option else Pb with optionModel invoke...
        option.addOption("cxstatus", "status", "String", "stopped/connected/..", true, true, "set the status of the port");
        option.addOption("locked", false, "bool", "0/1", true, true, "locks in and out");
        option.addOption("Echo", false, "bool", "0/1", true, true, "Echo the output on input");
        option.addOption("lockin", false, "bool", "0/1", true, true, "locks in");
        option.addOption("lockout", false, "bool", "0/1", true, true, "locks out");
        option.addOption("loopedin", false, "bool", "0/1", true, true, "loop in the port (out on input)");
        option.addOption("loopedout", false, "bool", "0/1", true, true, "loop out the port ");
        option.addOption("log", false, "bool", "0/1", true, true, "active this port as log receiver");
        option.addOption("PortName", "Port", "String", "", true, true, "name of the port");
        option.addOption("Modeidx", 0, "int", "0/1/2/3/4", true, true, "fixe the type of port");
        option.addOption("stop2end", false, "bool", "0/1", true, true, "end the port after stoped ");
        option.setTitle(getTitle());
    }

    @Override
    public I_Main getMain() {
        return Parent;
    }

    @Override
    public I_Console getConsole() {
        if (this.getMain() instanceof I_Console) {
            return ((I_Console) this.getMain());
        } else {
            return TDeb.getconsole();
        }
    }

    public I_Slave getSlave() {
        return slave;
    }

    public boolean isreading() {
        return isreading;
    }

    public int available() {
        if (getin() != null) {
            // si port serie : attente DATA AVALABLE avant de lire le port.
            try {
                if (slave instanceof T_RS) {
                    if (((T_RS) slave).isavailable() == false) {
                        return 0;
                    }
                }
            } catch (NoClassDefFoundError er) {
            }
            int a = 0;
            //NB for serial port => comevt=> data avalable=> pas de besoin de getin.available()
            //getin.available=> 1 access a Win32.DLL...
            try {
                a = getin().available();
            } catch (IOException ex) {
                TDeb.wri("available() error =>Clossing Port [" + ex.getMessage() + "]");
                CXstop();
            }
            if (a == -1 && slave instanceof T_RS && isreading == false) {
                CXstop();
            }
            setAcc_bytercvd(a);
            return a;
        }
        if (slave instanceof I_Terminal) {
            return ((I_Terminal) slave).getinsize();
        }
        return -1;
    }

    /**
     * Manage the name/status of th
     *
     * @param c target Port Color
     */
    @Override
    public void setColor(Color c) {
        int idx = getIndex();
        if (getConsole() != null && idx >= 0 && getConsole().getColor(idx) != c) {
            getConsole().setColor(getIndex(), c);
        } else {
            sparecolor = c;
        }
        //NOT managed by options model
        if (getSkin() instanceof IF_Port) {
            ((IF_Port) getSkin()).setColor(c);
        }
    }

    @Override
    public Color getColor() {
        if (this.getIndex() < 0) {
            return sparecolor;
        } else if (getConsole() != null) {
            return getConsole().getColor(this.getIndex());
        } else {
            return java.awt.Color.BLACK;
        }

    }

    @A_Commande(help = "set the port to end after stopped")
    @Override
    public void setstop2end(boolean bool) {
        option.setValuefor("stop2end", bool);
    }

    @Override
    public boolean getstop2end() {
        return option.getValuefor("stop2end", true);
    }

    @A_Commande(help = "set the port label used for port identification")
    @Override
    public void setPortName(String name) {
        if (name == null || name.compareTo(getPortName()) == 0) {
            return;
        }
        option.setValuefor("PortName", name);
        option.setTitle(getTitle());
        if (slave instanceof F_Terminal) {
            ((F_Terminal) slave).setextTitle(name);
        }
        Parent.Arcv(this, "PortName", "Type");
    }

    @Override
    public String getPortName() {
        return (String) option.getValuefor("PortName");
    }

    @Override
    public String getcxstatus() {
        return option.getValuefor("cxstatus").toString();
        // return cxstatus;
    }

    public void setcxstatus(String status) {
        if (getcxstatus().compareTo(status) == 0) {
            // already set to this status !!! => pb in Option_Model setValue=> reflect checking
            TDeb.wri("no new status : " + status);
            return;
        }
        if (getcxstatus().startsWith("creating") && status.startsWith("started")) {
            TDeb.wri("dezdz");
        }
        TDeb.wri("status : " + getcxstatus() + " => " + status);
        option.setValuefor("cxstatus", status);
        if (stat_P != null) {
            stat_P.store(status);
        }
        // spread status for other port actions && check other actions
        //before seting to avoid Pb un received frames while connecting
        if (Parent != null) {
            Parent.Arcv(this, status, "Status");
        }
        if (status.compareTo("opened") == 0) {
            setCX_autocx(isCX_autocx());
        }
//        optionChanged("setcxstatus", status);

    }

    public I_Slave getcxslave() {
        return slave;
    }

    /**
     * Manage status of the thread
     *
     * @return true if slave slrunning is up
     */
    public boolean isslrunning() {
        return slrunning;
    }

    public boolean isslreading() {
        return slreading;
    }

    public void setslrunning(boolean bool) {
        slrunning = bool;
    }

    @Override
    public void setslreading(boolean bool) {
        TDeb.wri("setslreading :" + bool);
        slreading = bool;
    }

    /**
     * Manage accouting part cabs
     *
     * @return the out stream of this port
     */
    public OutputStream getout() {
        return out;
    }

    public InputStream getin() {
        return in;
    }

    public void setout(OutputStream o) {
        out = o;
    }

    public void setin(InputStream i) {
        in = i;
    }

    @Override
    public int getAcc_initsize() {
        return option.getValuefor("Acc_initsize", 1);
    }

    @Override
    public void setAcc_initsize(int x) {
        option.setValuefor("Acc_initsize", x);
    }

    @Override
    public void setAcc_position(int x) {
        option.getValuefor("Acc_position", 1);
    }

    public long getAcc_bytein() {
        return accounting[0][1];
    }

    public void setAcc_bytein(long nbbyte) {
        accounting[0][1] = nbbyte;
    }

    public long getAcc_objin() {
        return accounting[0][2];
    }

    public void setAcc_objin(long nbbyte) {
        accounting[0][2] = nbbyte;
    }

    public void setAcc_bytercvd(long nbbyte) {
        accounting[0][0] = nbbyte;
    }

    public long getAcc_bytercvd() {
        return accounting[0][0];
    }

    public long getAcc_byteout() {
        return accounting[1][1];
    }

    public void setAcc_byteout(long nbbyte) {
        accounting[1][1] = nbbyte;
    }

    public void setAcc_objout(long nbbyte) {
        accounting[1][2] = nbbyte;
    }

    public long getAcc_objout() {
        return accounting[1][2];
    }

    public void setAcc_packin(long nbbyte) {
        accounting[0][3] = nbbyte;
    }

    public long getAcc_packin() {
        return accounting[0][3];
    }

    public void setAcc_packout(long nbbyte) {
        accounting[1][3] = nbbyte;
    }

    public long getAcc_packout() {
        return accounting[1][3];
    }

    public void setAcc_startdate(Date time) {
        option.setValuefor("Acc_startdate", time);
    }

    public void setAcc_stopdate(Date time) {
        option.setValuefor("Acc_stopdate", time);
    }

    @Override
    public boolean isPort_Stat() {
        return stat_P != null;
    }

    @Override
    public void setPort_Stat(boolean bool) {
        if (isPort_Stat() == bool) {
            return;
        }
        if (bool) {
            if (getListener() == null) {
                stat_P = new P_Stat_P(this);
                stat_P.setVisible(true);
                //!! port Name should be updated if changed
                Parent.addTab((javax.swing.JPanel) stat_P, getPortName(), TUtils.getIcon("traffic_on.gif"));
            } else {
                stat_P = getListener().getPort_Stat();
            }
            if (this.getTCP_Mode().compareTo("IPL4") == 0 && this.getTCP_Mode().compareTo("TCP_L") == 0) {
                for (I_Port p : Parent.getPorts()) {
                    if (p instanceof C_Port) {
                        if (((C_Port) p).getListener() == this) {
                            ((C_Port) p).setPort_Stat(bool);
                        }
                    }
                }
            }
        } else {
            if (getListener() == null) {
                Parent.removeTab(stat_P);
            }
            stat_P = null;
        }
    }

    public P_Stat_P getPort_Stat() {
        return stat_P;
    }

    /**
     * return the index of the port in Main
     *
     * @return the index of this C_Port
     */
    @Override
    public int getIndex() {
        //Object Obj = option.getValuefor("Index");
        if (Parent != null && isSystem() == false) {
            return Parent.getPorts().indexOf(this);
        } else if (system_index < -1) {
            return system_index;
        }
        return -1;
    }

    @Override
    public boolean isSystem() {
        return system_index != -1;
    }

    public void setListener(C_Port i) {
        // ?? for memory, changed by real port
        //-1=>  created by button (manually).
        //-2=> created by XML
        //>=0 => Number of the lisner 
        listenport = i;
    }

    /**
     * return the listner port bind with this port
     *
     * @return if connect reveived TCPPort, the return listning poprt
     */
    public C_Port getListener() {
        return listenport;
    }

    /**
     * Manage Type cabs (IPL4,Serial,..)
     *
     * @return value match with type (0:IPL4/term/...)
     */
    public int getModeidx() {
        return option.getValuefor("Modeidx", 1);
    }

    @A_Commande(help = "return the funy label on the port mode")
    @Override
    public String getModeName() {
        return modenames.get(getModeidx());
    }

    @A_Commande(help = "set the Mode of the port (IPL4,Serial,File,Terminal,DB,ETH,SYS,USB,SND,LPT,MEM)")
    @Override
    public void setModeName(String txt) {
        if (txt == null || txt.compareTo(getModeName()) == 0) {
            return;
        }
        TDeb.wri("Mode setting : " + txt);
        int idx = modenames.indexOf(txt);

        if (idx == -1) {
            TDeb.wri("Setmode : Unknown tab:" + txt + "=> stop program");
            return;
        }
        setModeidx(idx);
        if (Parent != null) {
            Parent.Arcv(this, getModeName(), "Type");
        }
    }

    private void setModeidx(int i) {
        // inti => 0 => IPL4=>initIPL4=> Nothing to do!!!
        if (TDeb != null) {
            TDeb.wri("Mode setting cab : " + i);
        }
        if (getModeidx() != i) {
            option.setValuefor("Modeidx", i);
        }
        option.setTitle(getTitle());
        String mode = modenames.get(i);
        if (mode.startsWith("Serial")) {
            initRSbox();

        } else if (mode.startsWith("ETH")) {
            initETHbox(-2);
        } else if (mode.startsWith("LPT")) {
            initLPTbox();
        } else if (mode.startsWith("USB")) {
            initUSBbox();
        } else if (mode.startsWith("SND")) {
            initSNDbox();
        }
    }

    @Override
    public int getOsiLevel() {
        int portlevel = 1;
        if (getcxstatus().startsWith("closed") || getcxstatus().startsWith("new")) {
            return -1;
        }
        if (getModeName().compareTo("IPL4") == 0) {
            /**
             * ** removed because Stack copy on the TCP client **** if
             * (port.getTCP_Mode().startsWith("TCP_L")) { return -1; }
             */
            portlevel = 4;
            if (isSSL_Enabled()) {
                portlevel = 5;
            }
        } else if (getModeName().compareTo("SYS") == 0) {
            portlevel = 7;
        } else if (getModeName().compareTo("MEM") == 0) {
            portlevel = 0;
        } else if (getModeName().compareTo("JNA") == 0) {
            portlevel = 0;
        } else if (getModeName().compareTo("Term") == 0) {
            portlevel = 0;
        } else if (getModeName().compareTo("DB") == 0) {
            portlevel = 6;
        }
        return portlevel;
    }

    /**
     * Manage the top part of the port.
     *
     * @return true if port return input received
     */
    @A_Commande(help = "return true if Echo is activated")
    @Override
    public boolean isEcho() {
        return option.getValuefor("Echo", true);
    }

    @A_Commande(help = "if set, incomming byte are send on the output")
    public void setEcho(boolean bool) {
        option.setValuefor("Echo", bool);
        if (slave instanceof F_Terminal) {
            if (((F_Terminal) slave).isTecho() != bool) {
                ((F_Terminal) slave).setTecho(bool);
            }
        }

    }

    @A_Commande(help = "return true if port receive logs")
    public boolean islog() {
        return option.getValuefor("log", true);
    }

    @A_Commande(help = "if set, port receive logs")
    public void setlog(boolean bool) {
        option.setValuefor("log", bool);
        if (Parent instanceof I_Console) {
            if (bool) {
                ((I_Console) Parent).addconsole(this);
            } else {
                ((I_Console) Parent).removeconsole(this);
            }
        }
        if (slave instanceof F_Terminal) {
            if (((F_Terminal) slave).islog() != bool) {
                ((F_Terminal) slave).setlog(bool);
            }
        }
    }

    @A_Commande(help = "return true if Loop in is activated")
    public boolean isloopedin() {
        return option.getValuefor("loopedin", true);
    }

    @A_Commande(help = "return true if Loop out is activated")
    public boolean isloopedout() {
        return option.getValuefor("loopedout", true);
    }

    @A_Commande(help = "if set, incomming byte are only send on the output")
    public void setloopedin(boolean bool) {
        option.setValuefor("loopedin", bool);
    }

    @A_Commande(help = "if set, incomming byte are only send on the output")
    public void setloopedout(boolean bool) {
        option.setValuefor("loopedout", bool);
    }

    @A_Commande(help = "return true if lockedout is activated")
    public boolean islockout() {
        return option.getValuefor("lockout", true);
    }

    @A_Commande(help = "if set, no byte are send the port")
    public void setlockout(boolean bool) {
        if (islockout() != bool) {
            option.setValuefor("lockout", bool);
            if (this.getTCP_Mode().startsWith("TCP_L") == false) {
                TDeb.wri("Changing Locked_out to :" + bool);
            }
        }
    }

    @A_Commande(help = "return true if lockedin is activated")
    public boolean islockin() {
        return option.getValuefor("lockin", true);
    }

    @A_Commande(help = "if set, no byte are received from the port")
    public void setlockin(boolean bool) {
        if (islockin() != bool) {
            option.setValuefor("lockin", bool);
            if (getTCP_Mode().startsWith("TCP_L") == false) {
                TDeb.wri("Changing Locked_in to :" + bool);
            } else {
                //need to lock clients ???
            }
        }
        if (slave instanceof F_Terminal) {
            if (((F_Terminal) slave).islockedin() != bool) {
                ((F_Terminal) slave).setlockedin(bool);
            }
        }
    }

    @A_Commande(help = "if set, no byte are received from/to the port")
    public void setlocked(boolean bool) {
        setlockin(bool);
        setlockout(bool);
    }

    /**
     * Manage Connection/Disconnection of the port.
     *
     * @return if connexion is asked after openning
     */
    @A_Commande(help = "return true if CX_autocx is activated")
    public boolean isCX_autocx() {
        return option.getValuefor("CX_autocx", true);
    }

    @A_Commande(help = "if set, the state open => connect is automatic")
    public void setCX_autocx(boolean bool) {
        if (isCX_autocx() != bool) {
            option.setValuefor("CX_autocx", bool);
        }
        setboxCXenable(bool == false);
        // si rien ds boite=> autocx sinon attente getcxstrin
        if (isCX_autocx() && (getcxstatus().compareTo("opened") == 0 || getcxstatus().compareTo("connecting") == 0)) {
            // connecting if aleady tryed to connect
            TDeb.wri("state : try to autoconnect ...");
            if (this.getCX_TOval() == 0) {
                this.CXconnect(null);
            } else {
                //see swing.Timer...
                TDeb.wri("state : try to autoconnect after :" + this.getCX_TOval() + " ms");
                java.util.Timer timercx = new java.util.Timer();
                timercx.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        CXconnect(null);
                    }
                }, getCX_TOval());
            }
        }
    }

    @A_Commande(help = "return true if CX_autoopen is activated")
    public boolean isCX_autoopen() {
        return option.getValuefor("CX_autoopen", true);
    }

    @A_Commande(help = "if set, the state started => open is automatic")
    public void setCX_autoopen(boolean bool) {
        if (isCX_autoopen() != bool) {
            option.setValuefor("CX_autoopen", bool);
        }
        if (isCX_autocx() && getcxstatus().compareTo("started") == 0) {
            if ((getModeName().startsWith("IPL4") && getTCP_Mode().startsWith("TCP_L")) == false) {
                CXopen();
            }
        }
    }

    /**
     * if false, the Main manage openning, block Main and wait <br> for oppened
     * connection before continue.<br> implemented for ETH/FILE/IPL4/DB/SYS
     * (typical TCP)
     *
     * @return
     */
    @A_Commande(help = "return true port is open by the slave thread")
    public boolean isCX_openbyslave() {
        return option.getValuefor("CX_openbyslave", true);
    }

    @A_Commande(help = "if set, port will be open by slave thread")
    public void setCX_openbyslave(boolean bool) {
        if (isCX_openbyslave() != bool) {
            option.setValuefor("CX_openbyslave", bool);
        }
    }

    @A_Commande(help = "text (or bytes) will be send on port output while connecting ")
    public void setCX_Banner(String hxtxt) {
        byte[] by = null;
        try {
            by = TUtils.string2byte(hxtxt);
        } catch (Exception exe) {
        }
        if (by == null && hxtxt != null) {
            by = hxtxt.getBytes();
        }
        if (by != null) {
            setCX_Banner(by);
        }
    }

    public void setCX_Banner(byte[] by) {
        option.setValuefor("CX_Banner", by);
    }

    @A_Commande(help = "return CX_Banner value")
    public byte[] getCX_Banner() {
        //result is banner txtx as byte "31 32 33" (=>"123")
        Object Obj = option.getValuefor("CX_Banner");
        return (byte[]) Obj;
    }

    /**
     * set time to wait befor port connection
     *
     * @param val TO in ms 0=> immediate
     */
    @A_Commande(help = "set TO waited after what port will connect")
    public void setCX_TOval(String val) {
        int i;
        try {
            i = Integer.parseInt(val);
        } catch (NumberFormatException ex) {
            return;
        }
        option.setValuefor("CX_TOval", i);
        // this.jTextField7.setText(""+val);
    }

    /**
     * get time to wait befor port connection
     *
     * @return TO in ms 0=> immediate
     */
    @A_Commande(help = "return CX_TOval value")
    public int getCX_TOval() {
        Object Obj = option.getValuefor("CX_TOval");
        try {
            return Integer.parseInt(Obj.toString());
        } catch (NumberFormatException ex) {
            TDeb.wri("'" + Obj.toString() + "'=> 0 as DB_TO (=>immediate) : " + ex.getMessage());
            return 0;
        }
    }

    @A_Commande(help = "return CX_tagsend value")
    public boolean isCX_tagsend() {
        return option.getValuefor("CX_tagsend", true);
    }

    @A_Commande(help = "allow tag sending on connect")
    public void setCX_tagsend(boolean bool) {
        if (isCX_tagsend() != bool) {
            option.setValuefor("CX_tagsend", bool);
        }
    }

    @A_Commande(help = "return CX_tagwait value")
    public boolean isCX_tagwait() {
        return option.getValuefor("CX_tagwait", true);
    }

    @A_Commande(help = "set the tag value")
    public void setCX_tagval(String cxstr) {
        try {
            option.setValuefor("CX_tagval", TUtils.string2byte(cxstr));
        } catch (Exception ex) {
        }
    }

    public void setCX_tagval(byte[] by) {
        option.setValuefor("CX_tagval", by);
    }

    @A_Commande(help = "return CX_tagval value")
    public byte[] getCX_tagval() {
        Object Obj = option.getValuefor("CX_tagval");
        return (byte[]) Obj;
    }

    @A_Commande(help = "set the text(or bytes) waited on input to connect the port")
    public void setCX_strin(String hxtxt) {
        byte[] by;
        if (hxtxt == null) {
            return;
        }
        try {
            by = TUtils.string2byte(hxtxt);
        } catch (Exception ex) {
            by = hxtxt.getBytes();
        }
        if (by != null) {
            setCX_strin(by);
        }
    }

    public void setCX_strin(byte[] cxstr) {
        option.setValuefor("CX_strin", cxstr);
    }

    @A_Commande(help = "return CX_strin value")
    public byte[] getCX_strin() {
        Object Obj = option.getValuefor("CX_strin");
        return (byte[]) Obj;

    }

    @A_Commande(help = "allow CX_Tranmit of CXstrin byte")
    public void setCX_Transmit(boolean bool) {
        option.setValuefor("CX_Transmit", bool);
    }

    @A_Commande(help = "return CX_Transmit value")
    public boolean isCX_Transmit() {
        return option.getValuefor("CX_Transmit", true);
    }

    @A_Commande(help = "set the text (or byte) sendn before port Disconnexion")
    public void setDCX_Banner(String hxtxt) {
        byte[] by = null;
        try {
            by = TUtils.string2byte(hxtxt);
        } catch (Exception ex) {
        }
        if (by == null && hxtxt != null) {
            by = hxtxt.getBytes();
        }
        if (by != null) {
            setDCX_Banner(by);
        }
    }

    public void setDCX_Banner(byte[] by) {
        option.setValuefor("DCX_Banner", by);
    }

    @A_Commande(help = "return DCX_Banner value")
    public byte[] getDCX_Banner() {
        //result is banner txtx as byte "31 32 33" (=>"123")
        Object Obj = option.getValuefor("DCX_Banner");
        return (byte[]) Obj;
    }

    @A_Commande(help = "set the text (or byte) waited for port disconnexion")
    public void setDCX_strin(String hxtxt) {
        byte[] by = null;
        try {
            by = TUtils.string2byte(hxtxt);
        } catch (Exception exe) {
        }
        if (by == null && hxtxt != null) {
            by = hxtxt.getBytes();
        }
        if (by != null) {
            setDCX_strin(by);
        }
    }

    public void setDCX_strin(byte[] cxstr) {
        option.setValuefor("DCX_strin", cxstr);
    }

    @A_Commande(help = "return DCX_strin value")
    public byte[] getDCX_strin() {
        Object Obj = option.getValuefor("DCX_strin");
        return (byte[]) Obj;
    }

    @A_Commande(help = "allow text used for disconnection to be transmit")
    public void setDCX_Transmit(boolean bool) {
        option.setValuefor("DCX_Transmit", bool);
    }

    @A_Commande(help = "return DCX_Transmit value")
    public boolean isDCX_Transmit() {
        return option.getValuefor("DCX_Transmit", true);
    }

    @A_Commande(help = "return DCX_TOcom value")
    public boolean isDCX_TOcom() {
        return option.getValuefor("DCX_TOcom", true);
    }

    @A_Commande(help = "allow disconnection after TO without communications on port")
    public void setDCX_TOcom(boolean bool) {
        if (isDCX_TOcom() != bool) {
            option.setValuefor("DCX_TOcom", bool);
        }
        option.setused("DCX_TOcomval", isDCX_TOcom());
        setscaned();
    }

    @A_Commande(help = "set the millisecond without communications on port before disconnection")
    public void setDCX_TOcomval(int val) {
        option.setValuefor("DCX_TOcomval", val);
    }

    @A_Commande(help = "return TOcomval value")
    public int getDCX_TOcomval() {
        Object Obj = option.getValuefor("DCX_TOcomval");
        return (Integer) Obj;
    }

    @A_Commande(help = "return DCX_TOabs value")
    public boolean isDCX_TOabs() {
        return option.getValuefor("DCX_TOabs", true);
    }

    @A_Commande(help = "allow maximum timeout before disconnection")
    public void setDCX_TOabs(boolean bool) {
        if (isDCX_TOabs() != bool) {
            option.setValuefor("DCX_TOabs", bool);
        }
        option.setused("DCX_TOabsval", isDCX_TOabs());
        setscaned();
    }

    @A_Commande(help = "set maximum timeout (ms) before disconnection")
    public void setDCX_TOabsval(int val) {
        option.setValuefor("DCX_TOabsval", val);

    }

    @A_Commande(help = "return DCX_TOabsval value")
    public int getDCX_TOabsval() {
        Object Obj = option.getValuefor("DCX_TOabsval");
        return (Integer) Obj;
    }

    @A_Commande(help = "true if the port should close on disconnection")
    public boolean isDCX_tostop() {
        return option.getValuefor("DCX_tostop", true);
    }

    @A_Commande(help = "ask to close the port on dsi-connection")
    public void setDCX_tostop(boolean bool) {
        if (isDCX_tostop() != bool) {
            option.setValuefor("DCX_tostop", bool);
        }
    }

    /**
     * Manage Function (Param) part cabs
     *
     * @return true if scan is lanched for this port (for latencie/...)
     */
    public boolean isscaned() {
        return scan != null;
    }

    public void setscaned() {
        boolean bool;
        bool = isDCX_TOcom() || isDCX_TOabs() || isPar_PadtStr();
        if (bool && scan == null) {
            TDeb.wri("Scan port : Starting...");
            scan = new C_PScan(this);
        }
        if (bool) {
            scan.scanning();
        }
        if (bool == false) {
            if (scan != null) {
                TDeb.wri("Scan port : Stopping..");
                scan.close();
                // !! if packet are stored what append??
                scan = null;
            }
        }
    }

    @A_Commande(help = "remove a protocol stack for this Port")
    public void removeStack_proto(String protoName) {
        if (Parent instanceof C_Main) {
            ((C_Main) Parent).removeStack(protoName, getIndex());
        }
    }

    /**
     * @return Padt value
     */
    public boolean isPar_PadtStr() {
        return option.getValuefor("Par_PadtStr", true);
    }

    /**
     * @param bool active/disactiv PadStr and launch scan
     */
    public void setPar_PadtStr(boolean bool) {
        if (isPar_PadtStr() != bool) {
            option.setValuefor("Par_PadtStr", bool);
        }
        setscaned();
    }

    /**
     * @return Padtval value
     */
    public int getPar_PadtStrval() {
        Object Obj = option.getValuefor("Par_PadtStrval");
        return (Integer) Obj;
    }

    /**
     * @param txtval mixumum time (ms) waiting for datas befor sendin packet
     */
    public void setPar_PadtStrval(String txtval) {
        option.setValuefor("Par_PadtStrval", Integer.parseInt(txtval));
    }

    @A_Commande(help = "return Par_serial value")
    public boolean isPar_serial() {
        return option.getValuefor("Par_serial", true);
    }

    @A_Commande(help = "allow Object to be send/received with there serialized form")
    public void setPar_serial(boolean bool) {
        option.setValuefor("Par_serial", bool);
    }

    @A_Commande(help = "return Par_readat value")
    public boolean isPar_readat() {
        return option.getValuefor("Par_readat", true);
    }

    @A_Commande(help = "allow reading flow to a maximum nuber of byte")
    public void setPar_readat(boolean bool) {
        option.setValuefor("Par_readat", bool);
        option.setused("Par_readingat", bool && isPar_readatloop() == false);
        option.setused("Par_readatloop", bool);
    }

    @A_Commande(help = "return Par_readatloop value")
    public boolean isPar_readatloop() {
        return option.getValuefor("Par_readatloop", true);
    }

    @A_Commande(help = "allow reading at (xbyte) continuously")
    public void setPar_readatloop(boolean bool) {
        option.setValuefor("Par_readatloop", bool);
        option.setused("Par_readingat", bool == false);
    }

    @A_Commande(help = "return Par_readtoend value")
    public boolean isPar_readtoend() {
        return option.getValuefor("Par_readtoend", true);
    }

    @A_Commande(help = "allow reading to the end of the flow")
    public void setPar_readtoend(boolean bool) {
        option.setValuefor("Par_readtoend", bool);
        setscaned();
    }

    @A_Commande(help = "return Par_readingat value")
    public boolean isPar_readingat() {
        return option.getValuefor("Par_readingat", true);
    }

    @A_Commande(help = "ask for a reading of a limited (readatval) on flow")
    public void setPar_readingat(boolean bool) {
        option.setValuefor("Par_readingat", bool);
        setscaned();
    }

    @A_Commande(help = "set redat value")
    public void setPar_readatval(String txtval) {
        option.setValuefor("Par_readatval", Integer.parseInt(txtval));
    }

    @A_Commande(help = "return Par_readatval value")
    public int getPar_readatval() {
        Object Obj = option.getValuefor("Par_readatval");
        return (Integer) Obj;
    }

    @A_Commande(help = "set route selector for group_matrix")
    @Override
    public void setPar_mxgrp(String txtval) {
        option.setValuefor("Par_mxgrp", txtval);
        // Update boolean matrix table with this value.
        if (Parent instanceof C_Main) {
            ((C_Main) Parent).setMat_Type(((C_Main) Parent).getMat_Type());
        }
    }

    @A_Commande(help = "return route selector for group matrix")
    @Override
    public String getPar_mxgrp() {
        return option.getValuefor("Par_mxgrp").toString();
    }

    @A_Commande(help = "return SSL_Enabled value")
    public boolean isSSL_Enabled() {
        return option.getValuefor("SSL_Enabled", true);
    }

    @A_Commande(help = "enable SSL")
    public void setSSL_Enabled(boolean bool) {
        TDeb.wri("TCP ask to change SSL to [" + bool + "]");
        option.setValuefor("SSL_Enabled", bool);
        option.setused("SSL_Kfile", bool);
        option.setused("SSL_Tfile", bool);
        option.setused("SSL_Kpwd", bool);
        option.setused("SSL_Tpwd", bool);
        option.setused("SSL_SrvAuth", bool);
        option.setused("SSL_CltAuth", bool);
        option.setused("SSL_KAlgo", bool);
        option.setused("SSL_TAlgo", bool);
        option.setused("SSL_KType", bool);
        option.setused("SSL_TType", bool);
        option.setused("SSL_Proto", bool);
    }

    @A_Commande(help = "return name of the Key file")
    public File getSSL_Kfile() {
        javax.swing.DefaultComboBoxModel<File> cbox
                = option.getValuefor("SSL_Kfile", new javax.swing.DefaultComboBoxModel<File>());
        if (cbox.getSelectedItem() instanceof File) {
            return (File) cbox.getSelectedItem();
        }
        return null;
    }

    @A_Commande(help = "set the Keys file (.jks)")
    public void setSSL_Kfile(File file) {
        javax.swing.DefaultComboBoxModel<File> cbox = option.getValuefor("SSL_Kfile", new javax.swing.DefaultComboBoxModel<File>());
        if (cbox.getIndexOf(file) < 0) {
            cbox.addElement(file);
        }
        cbox.setSelectedItem(file);
    }

    @A_Commande(help = "set the Keys file (.jks)")
    public void setSSL_Kfile(String txt) {
        javax.swing.DefaultComboBoxModel<File> cbox = option.getValuefor("SSL_Kfile", new javax.swing.DefaultComboBoxModel<File>());
        for (int i = 0; i < cbox.getSize(); i++) {
            if (cbox.getElementAt(i).getName().compareTo(txt) == 0) {
                cbox.setSelectedItem(cbox.getElementAt(i));
            }
        }
    }

    @A_Commande(help = "return name of the Trusted file")
    public File getSSL_Tfile() {
        javax.swing.DefaultComboBoxModel<File> cbox
                = option.getValuefor("SSL_Tfile", new javax.swing.DefaultComboBoxModel<File>());
        if (cbox.getSelectedItem() instanceof File) {
            return (File) cbox.getSelectedItem();
        }
        return null;
    }

    @A_Commande(help = "set the Trusts file (.jks)")
    public void setSSL_Tfile(File file) {
        javax.swing.DefaultComboBoxModel<File> cbox = option.getValuefor("SSL_Tfile", new javax.swing.DefaultComboBoxModel<File>());
        if (cbox.getIndexOf(file) < 0) {
            cbox.addElement(file);
        }
        cbox.setSelectedItem(file);
    }

    @A_Commande(help = "set the Trusts file (.jks)")
    public void setSSL_Tfile(String txt) {
        javax.swing.DefaultComboBoxModel<File> cbox = option.getValuefor("SSL_Tfile", new javax.swing.DefaultComboBoxModel<File>());
        for (int i = 0; i < cbox.getSize(); i++) {
            if (cbox.getElementAt(i).getName().compareTo(txt) == 0) {
                cbox.setSelectedItem(cbox.getElementAt(i));
            }
        }
    }

    @A_Commande(help = "return the Key file Password")
    public String getSSL_Kpwd() {
        return option.getValuefor("SSL_Kpwd", new String());

    }

    @A_Commande(help = "set the Key file password")
    public void setSSL_Kpwd(String txt) {
        option.setValuefor("SSL_Kpwd", txt);
    }

    @A_Commande(help = "return the Trusted file password")
    public String getSSL_Tpwd() {
        return option.getValuefor("SSL_Tpwd", new String());

    }

    @A_Commande(help = "set the Trust file password")
    public void setSSL_Tpwd(String txt) {
        option.setValuefor("SSL_Tpwd", txt);
    }

    @A_Commande(help = "allow server authentication")
    public void setSSL_SrvAuth(boolean bool) {
        option.setValuefor("SSL_SrvAuth", bool);
    }

    @A_Commande(help = "return true if SSL_SrvAuth is activated")
    public boolean isSSL_SrvAuth() {
        return option.getValuefor("SSL_SrvAuth", true);
    }

    @A_Commande(help = "allow client authentication")
    public void setSSL_CltAuth(boolean bool) {
        option.setValuefor("SSL_CltAuth", bool);
    }

    @A_Commande(help = "return true if SSL_CltAuth is activated")
    public boolean isSSL_CltAuth() {
        return option.getValuefor("SSL_CltAuth", true);
    }

    @A_Commande(help = "return the SSL_Key file Algo value")
    public String getSSL_KAlgo() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_KAlgo", CBstrModel);
        if (cbox.getSelectedItem() == null) {
            return null;
        }
        return cbox.getSelectedItem().toString();
    }

    @A_Commande(help = "set SSL type for Keys (SunX509)")
    public void setSSL_KAlgo(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_KAlgo", CBstrModel);
        cbox.setSelectedItem(txt);
    }

    @A_Commande(help = "return the SSL_Trusted file Algo value")
    public String getSSL_TAlgo() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_TAlgo", CBstrModel);
        if (cbox.getSelectedItem() == null) {
            return null;
        }
        return cbox.getSelectedItem().toString();
    }

    @A_Commande(help = "set SSL type for Trust (SunX509)")
    public void setSSL_TAlgo(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_TAlgo", CBstrModel);
        cbox.setSelectedItem(txt);
    }

    @A_Commande(help = "return the SSL_Key file Type")
    public String getSSL_KType() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_KType", CBstrModel);
        if (cbox.getSelectedItem() == null) {
            return null;
        }
        return cbox.getSelectedItem().toString();
    }

    @A_Commande(help = "set type for Keys (JKS,PKCS12)")
    public void setSSL_KType(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_KType", CBstrModel);
        cbox.setSelectedItem(txt);
    }

    @A_Commande(help = "return the SSL_Trusted file type")
    public String getSSL_TType() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_TType", CBstrModel);
        if (cbox.getSelectedItem() == null) {
            return null;
        }
        return cbox.getSelectedItem().toString();
    }

    @A_Commande(help = "set type for Trust (JKS,PKCS12)")
    public void setSSL_TType(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_TType", new DefaultComboBoxModel<String>());
        cbox.setSelectedItem(txt);
    }

    @A_Commande(help = "return the SSL_Protocol")
    public String getSSL_Proto() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_Proto", CBstrModel);
        if (cbox.getSelectedItem() == null) {
            return null;
        }
        return cbox.getSelectedItem().toString();
    }

    @A_Commande(help = "set SSL protocol Used? v1,v2,TLS")
    public void setSSL_Proto(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("SSL_Proto", CBstrModel);
        cbox.setSelectedItem(txt);
    }

    /**
     * Manage IPL4 part of the port
     *
     * @return the socket of this port (if TCP)
     */
    public java.net.Socket getsocket() {
        if (slave instanceof T_IP) {
            return ((T_IP) slave).getTCPsocket();
        }
        return null;
    }

    @A_Commande(help = "return the IPL4 port mode ")
    public String getTCP_Mode() {
        Object Obj = option.getValuefor("TCP_Mode");
        if (Obj != null) {
            return Obj.toString();
        } else {
            return "N/A";
        }
    }

    @A_Commande(help = "set the IPL4 port mode TCP_C/TCP_L/UDP")
    public void setTCP_Mode(String txt) {
        if (getcxstatus().compareTo("stopped") != 0 && this.getcxstatus().compareTo("creating") != 0) {
            TDeb.wri("Pb to TCP mode : " + txt + "during port is : " + getcxstatus());
            return;
        }
        option.setValuefor("TCP_Mode", txt);
        TDeb.wri("set to TCP mode : " + txt);
        if (txt.compareTo("UDP") == 0) {
            setboxtcpenable(true);
        } else if (txt.compareTo("TCP_C") == 0) {
            if (getTCP_Addr() == null) {
                setTCP_Addr("localhost");
            }
        } else if (txt.compareTo("TCP_L") == 0) {
            setTCP_Addr("0.0.0.0");
        } else {
            TDeb.wri("this mode is not acceptd : " + txt);
        }
    }

    public void setTCP_Listner(boolean bool) {
        if (bool) {
            setTCP_Mode("TCP_L");
        } else {
            setTCP_Mode("TCP_C");
        }
    }

    @A_Commande(help = "return the TCP/UDP port number")
    public int getTCP_Port() {
        return (Integer) option.getValuefor("TCP_Port");
    }

    @A_Commande(help = "set the TCP/UDP destination port ")
    public void setTCP_Port(String port) {
        int tcpport = 1;
        if (port == null) {
            return;
        }
        try {
            tcpport = new Integer(port);
        } catch (NumberFormatException ex) {
            TDeb.wri("Pb : setTCP_Port(" + port + ") set 1 =>" + ex.getMessage());
        }
        setTCP_Port(tcpport);
    }

    public void setTCP_Port(int port) {
        TDeb.wri("TCP ask to set port  : [" + port + "]");
        option.setValuefor("TCP_Port", port);
    }

    @A_Commande(help = "return the TCP/UDP remote host address")
    public java.net.InetAddress getTCP_Addr() {
        return (java.net.InetAddress) option.getValuefor("TCP_Addr");
    }

    public void setTCP_Addr(java.net.InetAddress addr) {
        option.setValuefor("TCP_Addr", addr);
    }

    @A_Commande(help = "set the TCP/UDP destination address ")
    public void setTCP_Addr(String txt) {
        try {
            setTCP_Addr(java.net.Inet4Address.getByName(txt));
        } catch (UnknownHostException ex) {
        }
    }

    public Point getTCP_ioBuffers() {
        return (Point) option.getValuefor("TCP_ioBuffers");
    }

    public void setTCP_ioBuffers(Point pt) {
        option.setValuefor("TCP_ioBuffers", pt);
    }

    @A_Commande(help = "is urgent data enabled")
    public boolean isTCP_OOBinline() {
        return option.getValuefor("TCP_OOBinline", true);
    }

    @A_Commande(help = "active/disactive urgent data")
    public void setTCP_OOBinline(boolean bool) {
        option.setValuefor("TCP_OOBinline", bool);
    }

    @A_Commande(help = "is linger active")
    public boolean isTCP_linger() {
        return option.getValuefor("TCP_linger", true);
    }

    @A_Commande(help = "active/disactive linger")
    public void setTCP_linger(boolean bool) {
        option.setValuefor("TCP_linger", bool);
    }

    @A_Commande(help = "return the TCP Linger timeout")
    public int getTCP_lingerTO() {
        return option.getValuefor("TCP_lingerTO", 1);
    }

    @A_Commande(help = "set the TCP linger timeout")
    public void setTCP_lingerTO(String txt) {
        setTCP_lingerTO(Integer.parseInt(txt));
    }

    public void setTCP_lingerTO(int sotimeout) {
        option.setValuefor("TCP_lingerTO", sotimeout);
        if (slave instanceof T_IP) {
            ((T_IP) slave).updateTCPsocket();
        }
    }

    @A_Commande(help = "return the TCP So Timeout")
    public int getTCP_SoTO() {
        return option.getValuefor("TCP_SoTO", 1);
    }

    @A_Commande(help = "set the TCP SoTimeout ")
    public void setTCP_SoTO(String txt) {
        setTCP_SoTO(Integer.parseInt(txt));
    }

    public void setTCP_SoTO(int sotimeout) {
        option.setValuefor("TCP_SoTO", sotimeout);
        if (slave instanceof T_IP) {
            ((T_IP) slave).updateTCPsocket();
        }
    }

    @A_Commande(help = "active/disactive TCP soicket ReuseAddr")
    public void setTCP_ReuseAddr(boolean bool) {
        option.setValuefor("TCP_ReuseAddr", bool);
    }

    @A_Commande(help = "return the TCP socket ReuseAddr")
    public boolean isTCP_ReuseAddr() {
        return option.getValuefor("TCP_ReuseAddr", true);
    }

    @A_Commande(help = "return the TCP keepalive status")
    public boolean isTCP_keepalive() {
        return option.getValuefor("TCP_keepalive", true);
    }

    @A_Commande(help = "set the TCP Keepalive")
    public void setTCP_keepalive(boolean bool) {
        option.setValuefor("TCP_keepalive", bool);
        if (slave instanceof T_IP) {
            ((T_IP) slave).updateTCPsocket();
        }
    }

    @A_Commande(help = "return the TCP no delay status")
    public boolean isTCP_nodelay() {
        return option.getValuefor("TCP_nodelay", true);
    }

    @A_Commande(help = "set the TCP nodelay")
    public void setTCP_nodelay(boolean bool) {
        option.setValuefor("TCP_nodelay", bool);
        if (slave instanceof T_IP) {
            ((T_IP) slave).updateTCPsocket();
        }
    }

    public int getTCP_class() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("TCP_class", CBstrModel);
        String txt = cbox.getSelectedItem().toString();
        if (txt.startsWith("IPTOS_LOWCOST")) {
            return 2;
        } else if (txt.startsWith("IPTOS_RELIABILITY")) {
            return 4;
        } else if (txt.startsWith("IPTOS_THROUGHPUT")) {
            return 8;
        } else if (txt.startsWith("IPTOS_LOWDELAY")) {
            return 10;
        } else {
            return 0;
        }
    }

    @A_Commande(help = "set the TCP class")
    public void setTCP_class(int nb) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("TCP_class", CBstrModel);
        if (nb == 2) {
            cbox.setSelectedItem("IPTOS_LOWCOST");
        } else if (nb == 4) {
            cbox.setSelectedItem("IPTOS_RELIABILITY");
        } else if (nb == 8) {
            cbox.setSelectedItem("IPTOS_THROUGHPUT");
        } else if (nb == 10) {
            cbox.setSelectedItem("IPTOS_LOWDELAY");
        } else {
            cbox.setSelectedItem("IPTOS_NONE");
        }
    }

    @A_Commande(help = "return the TCP Listner Maximum client number")
    public int getTCP_Lmax() {
        return (Integer) option.getValuefor("TCP_Lmax");
    }

    @A_Commande(help = "set the TCP server maximum client allowed")
    public void setTCP_Lmax(int nbcxmax) {

        option.setValuefor("TCP_Lmax", nbcxmax);
    }

    /**
     * Ask to remove the TCP client from listner port list
     *
     * @param closedclient number of the TCP connected port
     */
    public void stopLclient(int closedclient) {
        if (slave instanceof T_IP) {
            ((T_IP) slave).removeTCPclient(closedclient);
        }
    }

    @Override
    public int getSIndex() {
        if (listenport == null || getModeName().startsWith("IPL4") == false || getTCP_Mode().startsWith("TCP_C") == false) {
            return -1;
        } else {
            return ((T_IP) listenport.getSlave()).getTCPclients().indexOf(this);
        }
    }

    /**
     * Ask to print the number of tcp client connected
     */
    public void DisplayLclientNumber() {
        int res;
        if (slave instanceof T_IP && this.getTCP_Mode().startsWith("TCP_L")) {
            if (((T_IP) slave).getTCPclients() != null) {
                res = ((T_IP) slave).getTCPclients().size();
                if (portSkin instanceof IF_Port) {
                    ((IF_Port) portSkin).DisplayLclientNumber(res);
                }
                // for UIupdate...
                Parent.Arcv(this, "Slave", "Type");
            }
        }
    }

    /**
     * Manage Serial Async part of the port RS port is not include in standard
     * java=> Object
     *
     * @return the RS Port (COMX/sd0/...)
     */
    @A_Commande(help = "return the Serial Port to Use")
    public Object getRS_Port() {
        DefaultComboBoxModel<Object> list = option.getValuefor("RS_interfaces", new DefaultComboBoxModel<>());
        if (list != null && list.getSize() > 0) {
            return list.getSelectedItem();
        }
        return null;
    }

    @A_Commande(help = "set the RS port to use (COM1)")
    public void setRS_Port(Object port) {
        TDeb.wri("RS ask to set port  : [" + port + "]");
        DefaultComboBoxModel<Object> list = option.getValuefor("RS_interfaces", new DefaultComboBoxModel<>());
        if (port != null && list.getIndexOf(port) >= 0) {
            list.setSelectedItem(port);
        }
        if (getSkin() instanceof IF_Port) {
            ((IF_Port) getSkin()).setboxenableRS(true);
        }
    }

    @A_Commande(help = "set the RS port to use (COM1)")
    public void setRS_Port(String portName) {
        TDeb.wri("RS ask to set port  : [" + portName + "]");
        if (slave != null && slave instanceof T_RS) {
            Object port = ((T_RS) slave).getPort(portName);
            if (port != null) {
                setRS_Port(port);
            }
        }

    }

    @A_Commande(help = "return the Serial Port rate (1200/2400/9600/19200/115200/..")
    public int getRS_Rate() {
        return (Integer) ((DefaultComboBoxModel) option.getValuefor("RS_Rate")).getSelectedItem();
    }

    @A_Commande(help = "set the RS port speed to used (default 9600b/s)")
    public void setRS_Rate(String bandtxt) {
        setRS_Rate(new Integer(bandtxt));
    }

    public void setRS_Rate(int rate) {
        TDeb.wri("RS ask to set rates  : [" + rate + "]");
        DefaultComboBoxModel<Integer> c = option.getValuefor("RS_Rate", new DefaultComboBoxModel<Integer>());
        if (c.getIndexOf(rate) < 0) {
            TDeb.wri("RS ask to set rates  : [" + rate + "] seems not normalized");
            return;
        }
        c.setSelectedItem(rate);
    }

    @A_Commande(help = "return the Serial Port flow control")
    public String getRS_FlowCtrl() {
        return (String) option.getValuefor("RS_FlowCtrl");
    }

    @A_Commande(help = "set the RS port flowcontrol (none/CTSRTS/XonXoff)")
    public void setRS_FlowCtrl(String txt) {
        TDeb.wri("RS ask to set FlowCtrl_in  : [" + txt + "]");
        option.setValuefor("RS_FlowCtrl", txt);
    }

    @A_Commande(help = "return the Serial Port databit")
    public int getRS_Databit() {
        return (Integer) option.getValuefor("RS_Databit");
    }

    @A_Commande(help = "set the RS port databit (8/7/6/5)")
    public void setRS_Databit(String nbtxt) {
        setRS_Databit(new Integer(nbtxt));
    }

    public void setRS_Databit(int nb) {
        TDeb.wri("RS ask to set Databit  : [" + nb + "]");
        option.setValuefor("RS_Databit", nb);
    }

    @A_Commande(help = "return the Serial Port parity")
    public String getRS_Parity() {
        return (String) option.getValuefor("RS_Parity");
    }

    @A_Commande(help = "set the RS port parity (Even/Odd/None/Mark/Space)")
    public void setRS_Parity(String txt) {
        TDeb.wri("RS ask to set Parity  : [" + txt + "]");
        option.setValuefor("RS_Parity", txt);
    }

    @A_Commande(help = "return the Serial Port Stop bit")
    public String getRS_Stopbit() {
        return (String) option.getValuefor("RS_Stopbit");
    }

    @A_Commande(help = "set the RS port stop bit (1/2/1.5)")
    public void setRS_Stopbit(String txt) {
        TDeb.wri("RS ask to set Stopbit  : [" + txt + "]");
        option.setValuefor("RS_Stopbit", txt);
    }

    @A_Commande(help = "return the Serial Port Open Timeout")
    public int getRS_OpenTO() {
        return (Integer) option.getValuefor("RS_OpenTO");
    }

    @A_Commande(help = "return the Serial Port System label")
    public String getRS_SysLab() {
        return (String) option.getValuefor("RS_SysLab");
    }

    @A_Commande(help = "set the DTR signal")
    public void setRS_DTR(boolean bool) {
        TDeb.wri("RS ask to set DTR  : [" + bool + "]");
        if (bool != isRS_DTR()) {
            option.setValuefor("RS_DTR", bool);
            if (stat_RS != null) {
                stat_RS.setDTR(bool);
            }
            if (slave instanceof T_RS) {
                ((T_RS) slave).setDTR_RTS();
            }
        }
    }

    @A_Commande(help = "return the Serial Port DTR status")
    public boolean isRS_DTR() {
        return option.getValuefor("RS_DTR", true);
    }

    @A_Commande(help = "set the RTS signal")
    public void setRS_RTS(boolean bool) {
        TDeb.wri("RS ask to set RTS  : [" + bool + "]");
        if (bool != isRS_RTS()) {
            option.setValuefor("RS_RTS", bool);
            if (stat_RS != null) {
                stat_RS.setRTS(bool);
            }
            if (slave instanceof T_RS) {
                ((T_RS) slave).setDTR_RTS();
            }
        }
    }

    @A_Commande(help = "return the Serial Port RTS status")
    public boolean isRS_RTS() {
        return option.getValuefor("RS_RTS", true);

    }

    public boolean isRS_lights() {
        return option.getValuefor("RS_Lights", true);
    }

    public void setRS_lights(boolean bool) {
        option.setValuefor("RS_Lights", bool);
        if (bool && stat_RS == null) {
            stat_RS = new P_Stat_RS(this);
            stat_RS.initcourbs();
            stat_RS.setVisible(true);

            Parent.addTab((javax.swing.JPanel) stat_RS, "Serial port: " + this.getIndex(), TUtils.getIcon("stat.gif"));
        }
        // if (bool && stat_RS==null ){stat_RS=new F_Stat_RS(this);stat_RS.initcourbs();stat_RS.setVisible(true);}
        if (stat_RS != null) {
            stat_RS.setLightActive(isslrunning());
        }
        if (bool == false && stat_RS != null) {
            Parent.removeTab(stat_RS);
            stat_RS = null;
        }
        //notify on RS signals &  init lights
        if (slave instanceof T_RS) {
            ((T_RS) slave).setlights(isRS_lights());
            if (stat_RS != null) {
                stat_RS.setUARTType(((T_RS) slave).getRXTXinfo());
            }
        }
    }

    public void setRS_lights(String lightname, boolean ison, String comment) {
        if (isRS_lights() == false || stat_RS == null) {
            return;
        }
        // Frame error; parity error,...=> RED other GREEN
        TDeb.wri("SerialPort exception received on : " + lightname + " => " + ison);
        stat_RS.setlight(lightname, ison, comment);
    }

    /**
     * Manage File part of the port
     *
     * @return the file for a File Port
     */
    @A_Commande(help = "return File for output")
    public java.io.File getFile_addr() {
        java.io.File fi = (java.io.File) option.getValuefor("File_addr");
        return fi;
    }

    @A_Commande(help = "true if file access is input")
    public boolean isFile_read() {
        return option.getValuefor("File_read", true);
    }

    @A_Commande(help = "true if file acess is output")
    public boolean isFile_write() {
        return option.getValuefor("File_write", true);
    }

    @A_Commande(help = "set File name from initial list as a in flow")
    public void setFile_addr(String finame) {
        javax.swing.DefaultComboBoxModel<java.io.File> cbm = option.getValuefor("File_traces", new javax.swing.DefaultComboBoxModel<java.io.File>());
        for (int i = 0; i < cbm.getSize(); i++) {
            if ((cbm.getElementAt(i)).getName().compareTo(finame) == 0) {
                setFile_addr(cbm.getElementAt(i));
                return;
            }
        }
        try {
            java.io.File fi = new java.io.File(finame);
            setFile_addr(fi);
        } catch (Exception ex) {
            TDeb.wri("not able to make file with " + finame);
        }
    }

    public void setFile_addr(java.io.File fi) {
        if (fi == null) {//|| fi.isFile()==false){
            TDeb.wri("file: fil is NULL=> is not a 'normal' file (should be directory");
            return;
        }
        javax.swing.DefaultComboBoxModel<java.io.File> cbm = option.getValuefor("File_traces", new javax.swing.DefaultComboBoxModel<java.io.File>());
        if (cbm.getIndexOf(fi) == -1) {
            cbm.addElement(fi);
        }
        cbm.setSelectedItem(fi);
        setFile_read(TUtils.canRead(fi));
        setFile_write(false);//fi.exists() == false || TUtils.caWrite(fi)
        TDeb.wri("File set name  : " + fi.getName());
        option.setValuefor("File_addr", fi);
    }

    @A_Commande(help = "set File access as input file")
    public void setFile_read(boolean bool) {
        if (isFile_read() != bool) {
            option.setValuefor("File_read", bool);
        }
    }

    @A_Commande(help = "set File acces as out file")
    public void setFile_write(boolean bool) {
        if (isFile_write() != bool) {
            option.setValuefor("File_write", bool);
        }
    }

    @A_Commande(help = "set a writing fimename extention with timestamp on starting")
    public void setFile_addExt(boolean bool) {
        option.setValuefor("File_addExt", bool);
    }

    @A_Commande(help = "get true for writing fimename extention with timestamp on starting")
    public boolean istFile_addExt() {
        return option.getValuefor("File_addExt", true);
    }

    @A_Commande(help = "for input, stop the port on file EOF")
    public void setFile_EOFstop(boolean bool) {
        option.setValuefor("File_EOFstop", bool);
    }

    @A_Commande(help = "return File EOFstop status")
    public boolean isFile_EOFstop() {
        return option.getValuefor("File_EOFstop", true);
    }

    @A_Commande(help = "for output, clear (or not) file before start")
    public void setFile_ClearatStart(boolean bool) {
        option.setValuefor("File_ClearatStart", bool);
    }

    @A_Commande(help = "return File Clear at start status")
    public boolean isFile_ClearatStart() {
        return option.getValuefor("File_ClearatStart", true);
    }

    public boolean isFile_xml2tree() {
        return option.getValuefor("File_XML2tree", true);
    }

    @A_Commande(help = "return XML transfmor to tree")
    public void setFile_xml2tree(boolean bool) {
        option.setValuefor("File_XML2tree", bool);
    }

    public void setTerm_outMode(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("Term_outMode", CBstrModel);
        cbox.setSelectedItem(txt);
        if (slave instanceof F_Terminal) {
            if (((F_Terminal) slave).getTerm_outMode().compareTo(txt) != 0) {
                ((F_Terminal) slave).setTerm_outMode(txt);
            }
        }
    }

    public String getTerm_outMode() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("Term_outMode", CBstrModel);
        return cbox.getSelectedItem().toString();
    }

    public void setTerm_Type(String txt) {
        DefaultComboBoxModel<String> cbox = option.getValuefor("Term_Type", CBstrModel);
        if (txt == null || txt.length() == 0) {
            cbox.setSelectedItem(cbox.getElementAt(0));
        } else {
            cbox.setSelectedItem(txt);
        }
    }

    public String getTerm_Type() {
        DefaultComboBoxModel<String> cbox = option.getValuefor("Term_Type", CBstrModel);
        return cbox.getSelectedItem().toString();
    }

    public void setTERM_capReader(boolean bool) {
        option.setValuefor("Term_capReader", bool);
    }

    public boolean isTERM_capReader() {
        return option.getValuefor("Term_capReader", true);
    }

    /**
     * Manage DB part of the port
     *
     * @return the type of DB (MysQL...) for this Port
     */
    @A_Commande(help = "return DB type")
    public String getDB_type() {
        DefaultComboBoxModel<String> res = option.getValuefor("DB_interfaces", CBstrModel);
        if (res != null && res.getSelectedItem() != null) {
            return res.getSelectedItem().toString();
        } else {
            return "";
        }
    }

    @A_Commande(help = "set the type of DB (MySQL/SQLServer/Orathin/derby/ODBC..)")
    public void setDB_type(String txt) {
        if (txt == null) {
            return;
        }
        DefaultComboBoxModel<String> combo = option.getValuefor("DB_interfaces", CBstrModel);
        if (combo.getIndexOf(txt) < 0) {
            return;
        }
        if (txt.compareTo("ODBC_mdb") == 0) {
            setDB_charSet("ISO-8859-1");
        }
        if (txt.compareTo("Oracle_thin") == 0) {
            setDB_port(1521);
        }
        if (txt.compareTo("Derby") == 0) {
            setDB_port(1527);
        }
        if (txt.compareTo("Sybase") == 0) {
            setDB_port(1455);
        }
        if (txt.compareTo("SQLServer") == 0) {
            setDB_port(1433);
        }
        if (txt.compareTo("MySQL") == 0) {
            setDB_port(3306);
        }
        if (txt.compareTo("HSQL_odb") == 0) {
            setDB_user("sa");
        }
        combo.setSelectedItem(txt);
        setDB_driver();
        setDB_url();
    }

    @A_Commande(help = "return DB host")
    public String getDB_host() {
        return (String) option.getValuefor("DB_host");
    }

    @A_Commande(help = "set the adress/name of the DB host")
    public void setDB_host(String txt) {
        option.setValuefor("DB_host", txt);
        setDB_url();
    }

    @A_Commande(help = "return DB port")
    public int getDB_port() {
        return (Integer) option.getValuefor("DB_port");
    }

    @A_Commande(help = "set the port of the DB listner")
    public void setDB_port(int nb) {
        option.setValuefor("DB_port", nb);
        setDB_url();
    }

    public String getDB_name() {
        return (String) option.getValuefor("DB_name");
    }

    /**
     * set The name of the used Database (or shéma,..)
     *
     * @param name
     */
    @A_Commande(help = "set the name of the DB (default schema)")
    public void setDB_name(String name) {
        String txt = name;
        if (txt == null) {
            txt = "";
        }
        option.setValuefor("DB_name", txt);
        setDB_url();
    }

    @A_Commande(help = "return DB user (login)")
    public String getDB_user() {
        return (String) option.getValuefor("DB_user");
    }

    @A_Commande(help = "set the the DB user(login)")
    public void setDB_user(String txt) {
        option.setValuefor("DB_user", txt);
        setDB_url();
    }

    @A_Commande(help = "return DB CharSet")
    public java.nio.charset.Charset getDB_charSet() {
        return (java.nio.charset.Charset) option.getValuefor("DB_charSet", new DefaultComboBoxModel<java.nio.charset.Charset>()).getSelectedItem();
    }

    @A_Commande(help = "set the the DB CharSet")
    public void setDB_charSet(java.nio.charset.Charset cs) {
        option.getValuefor("DB_charSet", new DefaultComboBoxModel<java.nio.charset.Charset>()).setSelectedItem(cs);
    }

    @A_Commande(help = "set the the DB CharSet")
    public void setDB_charSet(String cstxt) {
        //  "ISO-8859-1"/UTF
        java.nio.charset.Charset cs = java.nio.charset.Charset.forName(cstxt);
        setDB_charSet(cs);
    }

    @A_Commande(help = "return DB password")
    public String getDB_pwd() {
        return (String) option.getValuefor("DB_pwd");
    }

    @A_Commande(help = "set the the DB password")
    public void setDB_pwd(String txt) {
        option.setValuefor("DB_pwd", txt);
        setDB_url();
    }
    // class<?> could be managed rearer than DBdrivers

    @A_Commande(help = "return DB drivers")
    public Class<?> getDB_driver() {
        Object Obj = option.getValuefor("DB_driver");
        if (Obj != null) {
            return (Class<?>) Obj;
        }
        return null;
    }

    public void setDB_driver() {
        String res = getConsole().getDB_drivername(getDB_type());
        Class<?> driver = String.class;
        try {
            driver = Class.forName(res);
            TDeb.wri("SQL Driver : " + res + "loaded");
        } catch (ClassNotFoundException e) {
            try {
                driver = Class.forName(res, true, TUtils.getloader("system"));
                TDeb.wri("SQL Driver : " + res + "loaded by System");
            } catch (ClassNotFoundException ex) {
                TDeb.wri("SQL Driver : " + res + "NOT loaded : " + ex.getMessage());
            }
        }
        option.setValuefor("DB_driver", driver);

    }

    @A_Commande(help = "return DB URL")
    public String getDB_url() {
        Object Obj = option.getValuefor("DB_URL");
        if (Obj != null) {
            return Obj.toString();
        }
        return "";
    }

    public void setDB_url() {
        String res = null;
        String instance = getDB_name();
        if (instance == null) {
            instance = "";
        }
        if (getDB_type().compareTo("MySQL") == 0) {
            res = "jdbc:mysql://" + getDB_host() + ":" + getDB_port() + "/" + instance + "?zeroDateTimeBehavior=convertToNull";
        }
        if (getDB_type().compareTo("Derby") == 0) {
            res = "jdbc:derby://" + getDB_host() + ":" + getDB_port() + "/" + instance;
        }
        if (getDB_type().compareTo("SQLServer") == 0) {
            // voir ;user= et ;password=
            if (instance.length() != 0) {
                instance = "\\" + instance;
            }
            res = "jdbc:sqlserver://" + getDB_host() + instance + ":" + getDB_port();//+";user="+getDB_user()+";password="+getDB_pwd();
        }
        if (getDB_type().compareTo("Oracle_thin") == 0) {
            // default port 1521
            res = "jdbc:oracle:thin:@" + getDB_host() + ":" + getDB_port() + ":" + instance;
        }
        if (getDB_type().compareTo("Oracle_tns") == 0) {
            // name => TNSNAME.ORA name.
            res = "jdbc:oracle:oci:@" + instance;
        }
        if (getDB_type().compareTo("ODBC_sys") == 0) {
            //here host=ODBC DSN entry
            res = "jdbc:odbc:" + instance;
        }
        if (getDB_type().compareTo("ODBC_mdb") == 0) {
            //   String url= "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};DBQ=db1.mdb;DriverID=22;READONLY=false;DefaultDir=C:\;";
            //characterEncoding='ISO-8859-1'
            String type = "(*.mdb)";
            if (System.getProperty("sun.arch.data.model").compareTo("64") == 0
                    && System.getProperty("os.name").compareTo("Windows 7") == 0) {
                type = "(*.mdb, .accdb)";
            }
            res = "jdbc:odbc:Driver={Microsoft Access Driver " + type + "};DBQ=" + instance + ";DriverID=25;READONLY=false;charSet='ISO-8859-1'";
        }
        if (getDB_type().compareTo("ODBC_xls") == 0) {
            //   String url= "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};DBQ=db1.mdb;DriverID=22;READONLY=false;DefaultDir=C:\;";
            res = "jdbc:odbc:Driver={Driver do Microsoft Excel(*.xls)};DBQ=" + instance + ";DriverID=277;READONLY=false;";
        }
        if (getDB_type().compareTo("HSQL_odb") == 0) {
            //LOCALHOST(network? port?) DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "sa", "");
            res = "jdbc:hsqldb:file:" + instance + ";default_schema=true";
        }
        if (getDB_type().compareTo("HSQL_mem") == 0) {

            res = "jdbc:hsqldb:mem:" + instance;
        }
        if (getDB_type().compareTo("AS400") == 0) {
            //http://publib.boulder.ibm.com/infocenter/iadthelp/v6r0/index.jsp?topic=/com.ibm.etools.iseries.toolbox.doc/jdbcdbmd.htm
            res = "jdbc:as400://" + getDB_host() + "/" + instance;
        }
        if (getDB_type().compareTo("Sybase") == 0) {
            //default port:1455
            res = "jdbc:sybase://" + getDB_host() + ":" + getDB_port() + "/" + instance;
        }
        TDeb.wri("SQL URL : " + res);
        option.setValuefor("DB_URL", res);
    }

    @A_Commande(help = "if true return result as ResultSet/Tree/Table ")
    public void setDB_resultType(String txt) {
        DefaultComboBoxModel<String> comb = option.getValuefor("DB_resultType", new DefaultComboBoxModel<String>());
        if (comb.getIndexOf(txt) >= 0 && getDB_resultType().compareTo(txt) != 0) {
            comb.setSelectedItem(txt);
        }
    }

    @A_Commande(help = "return tree mode for DB")
    public String getDB_resultType() {
        DefaultComboBoxModel<String> comb = option.getValuefor("DB_resultType", new DefaultComboBoxModel<String>());
        return comb.getSelectedItem().toString();
    }

    @A_Commande(help = "if true return result as resultset else byte ")
    public void setDB_Packets(boolean bool) {
        option.setValuefor("DB_pack2DB", bool);
    }

    @A_Commande(help = "return Packet mode for DB")
    public boolean isDB_Packets() {
        return option.getValuefor("DB_pack2DB", true);
    }

    @A_Commande(help = "if true return errors like #SQLError...  else null")
    public void setDB_Errors(boolean bool) {
        option.setValuefor("DB_errors", bool);
    }

    @A_Commande(help = "return Error mode for DB")
    public boolean isDB_Errors() {
        return option.getValuefor("DB_errors", true);
    }

    public java.sql.Connection getDB_CX() {
        java.sql.Connection res = null;
        if (slave instanceof T_DB) {
            return ((T_DB) slave).getCX();
        }
        return res;
    }

    /**
     * get the maximum nb of reccord in the resultset 0=> infinite
     *
     * @return nb maximum reccords resultset
     */
    @A_Commande(help = "return DB maximum rows returned on a requet (0=> infinite)")
    public int getDB_maxrows() {
        return (Integer) option.getValuefor("DB_maxrows");
    }

    /**
     * set the maximum nb of reccord in the resultset 0=> infinite
     *
     * @param nbrows the nb of rows to ask (if to large & coscequant reccords =>
     * time
     */
    @A_Commande(help = "set the max nb of rows return 0=> infinite")
    public void setDB_maxrows(String nbrows) {
        String rows = nbrows;
        if (rows == null) {
            rows = "0";
        } //0=> no limit
        try {
            setDB_maxrows(Integer.parseInt(rows));
        } catch (NumberFormatException ex) {
            TDeb.wri("try to set: '" + nbrows + "' as DB_maxrows  => " + ex.getMessage());
        }
    }

    public void setDB_maxrows(int nbrows) {
        option.setValuefor("DB_maxrows", nbrows);
    }

    /**
     * get TO to wait for a resultset 0=>no limite.
     *
     * @return nb of ms to wait for resultset
     */
    public int getDB_TO() {
        return (Integer) option.getValuefor("DB_TO");
    }

    @A_Commande(help = "set the max nb sec waiting for result")
    public void setDB_TO(String sectxt) {
        if (sectxt == null || sectxt.length() == 0) {
            sectxt = "0";
        }
        try {
            setDB_TO(Integer.parseInt(sectxt));
        } catch (NumberFormatException ex) {
            TDeb.wri("try to set: '" + sectxt + "' as DB_TO  => " + ex.getMessage());
        }
    }

    public void setDB_TO(int sec) {
        // 0=> no Timeout;
        option.setValuefor("DB_TO", sec);
    }

    public int getDB_fetchnb() {
        return (Integer) option.getValuefor("DB_fetchnb");
    }

    @A_Commande(help = "set the max nb rows to Fetch")
    public void setDB_fetchnb(String nbtxt) {
        if (nbtxt == null || nbtxt.length() == 0) {
            nbtxt = "0";
        }
        try {
            setDB_fetchnb(Integer.parseInt(nbtxt));
        } catch (NumberFormatException ex) {
            TDeb.wri("try to set: '" + nbtxt + "' as DB_fetchnb  => " + ex.getMessage());
        }
    }

    public void setDB_fetchnb(int nbrows) {
        // 0=> infinit=> default;
        option.setValuefor("DB_fetchnb", nbrows);
    }

    @A_Commande(help = "return the mapping file for DB inout or output")
    public File getDB_mapfile() {
        DefaultComboBoxModel<File> comb = option.getValuefor("DB_mapfile", new DefaultComboBoxModel<File>());
        File f = (File) comb.getSelectedItem();
        if (f.getName().compareTo("none") == 0) {
            return null;
        }
        return f;
    }

    @A_Commande(help = "set the mapping file for DB <=> TREE")
    public void setDB_mapfile(File f) {
        DefaultComboBoxModel<File> comb = option.getValuefor("DB_mapfile", new DefaultComboBoxModel<File>());
        if (comb.getIndexOf(f) < 0) {
            comb.addElement(f);
        }
        comb.setSelectedItem(f);
    }

    @A_Commande(help = "set the mapping file for DB <=> TREE")
    public void setDB_mapfile(String txt) {
        DefaultComboBoxModel<File> comb = option.getValuefor("DB_mapfile", new DefaultComboBoxModel<File>());
        for (int i = 0; i < comb.getSize(); i++) {
            if (comb.getElementAt(i).getName().compareTo(txt) == 0
                    || comb.getElementAt(i).getName().compareTo(txt + ".xml") == 0) {
                setDB_mapfile(comb.getElementAt(i));
                return;
            }
        }
        // to be able to use file in current dir ?? not tested?
        try {
            File f = new File(txt);
            setDB_mapfile(f);
        } catch (Exception ex) {
        }
    }

    @A_Commande(help = "Ask for DB=>XML building or according to mapping file ")
    public void DB2XML() {
        if (slave instanceof T_DB) {
            if (getDB_mapfile() != null) {
                TDeb.wri("ask to DB to XML by mapping: " + getDB_mapfile());
                DefaultTreeModel map_tree = (new com.jdcm.common.XML2tree()).XMLBase(
                        TUtils.getFileStream(getDB_mapfile()));
                if (map_tree != null) {
                    ((T_DB) slave).init_mapping(map_tree, false, null);
                }
            } else {
                TDeb.wri("ask DB 2 XML as auto creation defined in admin");
                DefaultMutableTreeNode res = ((T_DB) slave).DBtoXML(null);
                DefaultTreeModel treemodel = new DefaultTreeModel(res);
                write_in(new C_Packet(treemodel, this));
            }
        }
    }

    @A_Commande(help = "Ask for XML=>DB building or according to mapping file ")
    public void XML2DB() {
        if (slave instanceof T_DB && getDB_mapfile() != null) {
            TDeb.wri("ask to XML 2 DB by mapping: " + getDB_mapfile());
            DefaultTreeModel map_tree = (new com.jdcm.common.XML2tree()).XMLBase(
                    TUtils.getFileStream(getDB_mapfile()));
            if (map_tree != null) {
                ((T_DB) slave).init_mapping(map_tree, true, null);
            }
        }
    }

    /**
     * Manage Jpcap part of the port
     */
    public void setETHcreate() {
        if (slave instanceof T_ETH) {
            ((T_ETH) slave).setdevice();
        }
    }

    @A_Commande(help = "return the number of the selected Ethernet interface")
    public int getETH_interfaceidx() {
        DefaultComboBoxModel<String> comb = option.getValuefor("ETH_interfaces", CBstrModel);
        // is combomodel  a sigleselection...
        return comb.getIndexOf(comb.getSelectedItem());
    }

    @A_Commande(help = "set Ethernet interface as selected")
    public void setETH_interfaceidx(String txt) {
        int val;
        try {
            // if the interface is given by number=> OK else see description.
            val = Integer.parseInt(txt);
            setETH_interfaceidx(val);
        } catch (NumberFormatException ex) {
            javax.swing.ComboBoxModel<String> obj = option.getValuefor("ETH_interfaces", CBstrModel);
            for (int i = 0; i < obj.getSize(); i++) {
                if (obj.getElementAt(i).toString().startsWith(txt)) {
                    setETH_interfaceidx(i);
                    return;
                }
            }
            TDeb.wri("Ethernet Interface not fount : " + txt);
        }
    }

    public void setETH_interfaceidx(int idx) {
        DefaultComboBoxModel<String> obj = option.getValuefor("ETH_interfaces", CBstrModel);
        obj.setSelectedItem(obj.getElementAt(idx));
        this.optionChanged("ETH_interfaceidx", idx);
    }

    public byte[] getETH_MAC() {
        Object obj = option.getValuefor("ETH_interfaces");
        Object obj2 = ((javax.swing.ComboBoxModel) obj).getSelectedItem();
        if (obj2 instanceof jpcap.NetworkInterface) {
            return ((jpcap.NetworkInterface) obj2).mac_address;
        }
        return null;
    }

    public java.net.InetAddress getETH_firstIP() {
        Object obj = option.getValuefor("ETH_interfaces");
        Object obj2 = ((javax.swing.ComboBoxModel) obj).getSelectedItem();
        if (obj2 instanceof jpcap.NetworkInterface) {
            if (((jpcap.NetworkInterface) obj2).addresses.length > 0) {
                return ((jpcap.NetworkInterface) obj2).addresses[0].address;
            }
        }
        return null;
    }

    @A_Commande(help = "return Ethernet sniffer promisciussous mode")
    public boolean isETH_promiscuous() {
        return option.getValuefor("ETH_promiscuous", true);
    }

    @A_Commande(help = "set promiscuisous state for the selected interface")
    public void setETH_promiscuous(boolean bool) {
        option.setValuefor("ETH_promiscuous", bool);

    }
    /*
     * public void setETH_state(boolean bool) { java.awt.Color col =
     * java.awt.Color.RED; if (bool) { col = java.awt.Color.GREEN; }
     * this.jLabel62.setBackground(col); }
     */

    @A_Commande(help = "return Ethernet Packet mode (if false data are send as byte)")
    public String getETH_Pack() {
        DefaultComboBoxModel<String> Obj = option.getValuefor("ETH_Pack", new DefaultComboBoxModel<String>());
        return Obj.getSelectedItem().toString();
    }

    @A_Commande(help = "if true data are sended as packet else as byte")
    public void setETH_Pack(String txt) {
        DefaultComboBoxModel<String> Obj = option.getValuefor("ETH_Pack", new DefaultComboBoxModel<String>());
        Obj.setSelectedItem(txt);
    }

    @A_Commande(help = "return Ethernet JPCAP filter")
    public String getETH_filter() {
        Object Obj = option.getValuefor("ETH_filter");
        return (String) Obj;
    }

    @A_Commande(help = "set the Ethernet JPCAP filter")
    public void setETH_filter(String txt) {
        option.setValuefor("ETH_filter", txt);
    }

    private String display_ETH_msg() {
        String CR = "\r\n";
        String txt = "[src][dst]host 'host'\t\t=> host 10.10.10.10";
        txt = txt + CR + "[src][dst]port 'port'\t\t=> src port 80";
        txt = txt + CR + "[src][dst]net 'net' \t\t=> dst net 192.168";
        txt = txt + CR + "ether proto [\\ip][\\arp][\\rarp ] ";
        txt = txt + CR + "ip proto[\\tcp][\\udp][\\icmp]";
        txt = txt + CR + "Negation:'!' or 'not'";
        txt = txt + CR + "Concatenation:'&&' or 'and'";
        txt = txt + CR + "Alternation:'||' or 'or' ";
        txt = txt + CR + "\t=> host 10.10.10.10 && !net 192.168";
        txt = txt + CR + "ip[8] = 1: Capture all IP packets where the TTL is 1 ";
        txt = txt + CR + "tcp[0:2] = 80: Capture all tcp segments (packets) where 80 is the source port. ";
        return txt;

    }

    @A_Commande(help = "return sound selected line for input")
    public javax.sound.sampled.Line getSND_linein() {
        DefaultComboBoxModel<javax.sound.sampled.Line> Obj = option.getValuefor("SND_linesin", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        if (Obj.getSelectedItem() instanceof javax.sound.sampled.Line) {
            return (javax.sound.sampled.Line) Obj.getSelectedItem();
        } else {
            return null;
        }
    }

    @A_Commande(help = "set sound selected line for input")
    public void setSND_linein(javax.sound.sampled.Line line) {
        DefaultComboBoxModel<javax.sound.sampled.Line> Obj = option.getValuefor("SND_linesin", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        Obj.setSelectedItem(line);
    }

    @A_Commande(help = "return sound selected line for output")
    public javax.sound.sampled.Line getSND_lineout() {
        DefaultComboBoxModel<javax.sound.sampled.Line> Obj = option.getValuefor("SND_linesout", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        if (Obj.getSelectedItem() instanceof javax.sound.sampled.Line) {
            return (javax.sound.sampled.Line) Obj.getSelectedItem();
        } else {
            return null;
        }
    }

    @A_Commande(help = "set sound selected line for output")
    public void setSND_lineout(javax.sound.sampled.Line line) {
        DefaultComboBoxModel<javax.sound.sampled.Line> Obj = option.getValuefor("SND_linesout", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        Obj.setSelectedItem(line);
    }

    @A_Commande(help = "return sound selected format for input")
    public javax.sound.sampled.AudioFormat getSND_formatin() {
        return (javax.sound.sampled.AudioFormat) option.getValuefor("SND_formatin");
    }

    @A_Commande(help = "set sound selected format for input")
    public void setSND_formatin(javax.sound.sampled.AudioFormat format) {
        option.setValuefor("SND_formatin", format);
    }

    @A_Commande(help = "return SOUND selected format for output")
    public javax.sound.sampled.AudioFormat getSND_formatout() {
        return (javax.sound.sampled.AudioFormat) option.getValuefor("SND_formatout");
    }

    @A_Commande(help = "set sound selected format for output")
    public void setSND_formatout(javax.sound.sampled.AudioFormat format) {
        option.setValuefor("SND_formatout", format);
    }

    public String getLPT_Port() {
        DefaultComboBoxModel<String> obj = option.getValuefor("LPT_interfaces", CBstrModel);
        return obj.getSelectedItem().toString();
    }

    public void setLPT_Port(String name) {

        // set mode de mode();
        try {
            gnu.io.CommPortIdentifier portId;
            Enumeration<?> en = gnu.io.CommPortIdentifier.getPortIdentifiers();
            while (en.hasMoreElements()) {
                portId = (gnu.io.CommPortIdentifier) en.nextElement();
                if (portId.getPortType() == gnu.io.CommPortIdentifier.PORT_PARALLEL) {
                    if (this.getLPT_Port().compareTo(portId.getName()) == 0) {
                        gnu.io.ParallelPort p;
                        try {
                            p = (gnu.io.ParallelPort) portId.open("init_CX", 1000);
                        } catch (PortInUseException e) {
                            //gnu.io.PortInUseException
                            TDeb.wri("could init mode because port seems to be already opened");
                            return;
                        }
                        if (p.getMode() == gnu.io.ParallelPort.LPT_MODE_ECP) {
                            setLPT_Mode("ECP");
                        } else if (p.getMode() == gnu.io.ParallelPort.LPT_MODE_EPP) {
                            setLPT_Mode("EPP");
                        } else if (p.getMode() == gnu.io.ParallelPort.LPT_MODE_NIBBLE) {
                            setLPT_Mode("NIBBLE");
                        } else if (p.getMode() == gnu.io.ParallelPort.LPT_MODE_PS2) {
                            setLPT_Mode("PS2");
                        } else if (p.getMode() == gnu.io.ParallelPort.LPT_MODE_SPP) {
                            setLPT_Mode("SPP");
                        }
                        p.close();
                    }
                }

            }
        } catch (Exception ex) {
        }
    }

    public String getLPT_Mode() {
        DefaultComboBoxModel<String> obj = option.getValuefor("LPT_Mode", CBstrModel);
        return obj.getSelectedItem().toString();
    }

    public void setLPT_Mode(String name) {
        DefaultComboBoxModel<String> obj = option.getValuefor("LPT_Mode", CBstrModel);
        if (obj.getSelectedItem().toString().compareTo(name) != 0) {
            obj.setSelectedItem(name);
        }
    }

    public String getLPT_SysLab() {
        return (String) option.getValuefor("LPT_SysLab");
    }

    public int getLPT_OpenTO() {
        return (Integer) option.getValuefor("LPT_OpenTO");
    }

    public void setMEM_wire(int wirenb) {
        option.setValuefor("MEM_wire", wirenb);
    }

    public int getMEM_wire() {
        return (Integer) option.getValuefor("MEM_wire");
    }

    public Object getUSB_Element() {
        Object obj = option.getValuefor("USB_Element");
        return obj;
    }

    public void setUSB_Element(Object intf) {
        if (intf != null && intf.getClass().getCanonicalName().startsWith("usb.")) {
            option.setValuefor("USB_Element", intf);
        }
    }

    @A_Commande(help = "return library file name to load")
    public String getSYS_lib() {
        Object Obj = option.getValuefor("SYS_lib");
        if (Obj != null) {
            return Obj.toString();
        } else {
            return null;
        }
    }

    @A_Commande(help = "set the library file name to load")
    public void setSYS_lib(String txt) {
        option.setValuefor("SYS_lib", txt);
    }

    public int getSYS_nbmup() {
        Object Obj = option.getValuefor("SYS_nbmup");
        return (Integer) Obj;
    }

    public void setSYS_nbmup(int nb) {
        option.setValuefor("SYS_nbmup", nb);
    }

    @A_Commande(help = "return the value return by the system for the last excecution")
    public String getSYS_status() {
        Object Obj = option.getValuefor("SYS_status");
        return (String) Obj;
    }

    public void setSYS_status(String txt) {
        option.setValuefor("SYS_status", txt);
    }

    @A_Commande(help = "true if the command is received by Main")
    public boolean isSYS_DCXatend() {
        return option.getValuefor("SYS_DCXatend", true);
    }

    @A_Commande(help = "set the port allow command by Main")
    public void setSYS_DCXatend(boolean bool) {
        option.setValuefor("SYS_DCXatend", bool);
    }

    @A_Commande(help = "true is Security Manager is checked")
    public boolean isSYS_CheckSM() {
        return option.getValuefor("SYS_CheckSM", true);
    }

    @A_Commande(help = "set Security Manager check before execute")
    public void setSYS_CheckSM(boolean bool) {
        option.setValuefor("SYS_CheckSM", bool);
    }

    @A_Commande(help = "true if input bytes go to runing process else go to buffer")
    public boolean isSYS_intoproc() {
        return option.getValuefor("SYS_intoproc", true);
    }

    @A_Commande(help = "transfert input bytes to runnning process")
    public void setSYS_intoproc(boolean bool) {
        option.setValuefor("SYS_intoproc", bool);
    }

    @A_Commande(help = "true if the process output is returned")
    public boolean isSYS_sendOut() {
        return option.getValuefor("SYS_sendOut", true);
    }

    @A_Commande(help = "set the port to send output of the process")
    public void setSYS_sendOut(boolean bool) {
        option.setValuefor("SYS_sendOut", bool);
    }

    @A_Commande(help = "true if the the exit integer of the process is returned")
    public boolean isSYS_sendExit() {
        return option.getValuefor("SYS_sendExit", true);
    }

    @A_Commande(help = "set the process to send the Exit integer value")
    public void setSYS_sendExit(boolean bool) {
        option.setValuefor("SYS_sendExit", bool);
    }

    @A_Commande(help = "return Exit value as byte")
    public boolean isSYS_exitasbyte() {
        return option.getValuefor("SYS_exitasbyte", true);
    }

    @A_Commande(help = "set returned Exit value as byte")
    public void setSYS_exitasbyte(boolean bool) {
        option.setValuefor("SYS_exitasbyte", bool);
    }

    @A_Commande(help = "if true port used with standartin/out")
    public boolean isSYS_stdinout() {
        return option.getValuefor("SYS_stdinout", true);
    }

    @A_Commande(help = "set port use with in ans out standard")
    public void setSYS_stdinout(boolean bool) {
        option.setValuefor("SYS_stdinout", bool);
    }

    public boolean isWIN_sendmousmv() {
        return option.getValuefor("WIN_sendmousmv", true);
    }

    public void setWIN_sendmousmv(boolean bool) {
        if (isWIN_sendmousmv() != bool) {
            option.setValuefor("WIN_sendmousmv", bool);
        }
    }

    public void setWIN_sendmousmvTO(int nb) {
        if (nb > 0) {
            if (getWIN_sendmousmvTO() != nb) {
                option.setValuefor("WIN_sendmousmvTO", nb);
            }
        }
    }

    public int getWIN_sendmousmvTO() {
        return (Integer) option.getValuefor("WIN_sendmousmvTO");
    }

    public void setWIN_sendscreenTO(int nb) {
        if (nb > 0) {
            if (getWIN_sendscreenTO() != nb) {
                option.setValuefor("WIN_sendscreenTO", nb);
            }
        }
    }

    public int getWIN_sendscreenTO() {
        return (Integer) option.getValuefor("WIN_sendscreenTO");
    }

    public boolean isWIN_rcvmousmv() {
        return option.getValuefor("WIN_rcvmousmv", true);
    }

    public void setWIN_rcvmousmv(boolean bool) {
        if (isWIN_rcvmousmv() != bool) {
            option.setValuefor("WIN_rcmousmv", bool);
        }
    }

    public boolean isWIN_sendkeys() {
        return option.getValuefor("WIN_sendkeys", true);
    }

    public void setWIN_sendkeys(boolean bool) {
        if (isWIN_sendkeys() != bool) {
            option.setValuefor("WIN_sendkeys", bool);
        }
    }

    public boolean isWIN_rcvkeys() {
        return option.getValuefor("WIN_rcvkeys", true);
    }

    public void setWIN_rcvkeys(boolean bool) {
        if (isWIN_rcvkeys() != bool) {
            option.setValuefor("WIN_rcvkeys", bool);
        }
    }

    public boolean isWIN_sendscreen() {
        return option.getValuefor("WIN_sendscreen", true);
    }

    public void setWIN_sendscreen(boolean bool) {
        if (isWIN_sendscreen() != bool) {
            option.setValuefor("WIN_sendscreen", bool);
        }
    }

    public void setWIN_kbd(String txt) {
        DefaultComboBoxModel<String> obj = option.getValuefor("WIN_kbd", CBstrModel);
        if (obj.getIndexOf(txt) != -1) {
            obj.setSelectedItem(txt);
        } else {
            obj.setSelectedItem(obj.getElementAt(0));
        }

    }

    public String getWIN_kbd() {
        return getWIN_kbdModel().getSelectedItem().toString();
    }

    public DefaultComboBoxModel<String> getWIN_kbdModel() {
        DefaultComboBoxModel<String> obj = option.getValuefor("WIN_kbd", CBstrModel);
        return obj;
    }

    /**
     * Receiving packet from Main :<br> - Manage dis/connections.<br> - Check
     * lock,state is "connected". <br> - Manage tag,loop. <br> - Transmit to
     * Scan or write_out.
     *
     * @param packet packet received from Main
     */
    @Override
    public void read_out(C_Packet packet) {
        C_Packet pack = packet;
        // -----------------FROM MAIN--------------------
        if (getcxstatus().compareTo("opened") == 0 || getcxstatus().compareTo("disconnected") == 0) {
            pack = CXconnect(pack);
        } else {
            if (getcxstatus().compareTo("connected") == 0) {
                pack = CXdisconnect(pack, 1);
            }
        }
        if (pack == null || getcxstatus().compareTo("connected") != 0) {
            return;
        }
        if (isloopedin()) {
            if (slave != null) {
                pack.setsource(this);
                pack.setdestination(null);
                write_in(pack);
            }
            return;
        }
        if (isloopedout()) {
            return;
        }
        if (stat_P != null) {
            String txt = pack.getObj().getClass().getSimpleName();
            int size = 1;
            if (pack.getBytes() != null) {
                size = pack.getBytes().length;
                txt = "byte[" + size + "]";
            }
            stat_P.store("main", -size, txt + " comming from main");
        }
        Parent.addpacket("" + pack.hashCode(), System.currentTimeMillis(), "PortRO", pack.toString());
        //end of cx/dcx => dispatch
        if (pack.getBytes() != null) {
            TDeb.wri("Read from CX :" + pack.length() + " bytes");

            // conection frame avoided
            if (isCX_tagwait() && TUtils.equalto(pack.getBytes(), getCX_tagval())) {
                return;
            }
        } else {
            TDeb.wri("Read from CX " + pack.toString());
        }
        if (getcxstatus().compareTo("connected") == 0 && islockout() == false) {
            if (this.isDCX_TOcom()) {
                //shpitchnette pour toper le timer
                scan.From_CX(pack);
            }
            write_out(pack);
        }

    }

    /**
     * Send packet to the media slave : <br> - Check lock,state is
     * "connected".<br> - Manage Packet WOtime.<br> - Tramit to the media. <br>
     * - Manage debug, accounting, tag discard.
     *
     * @param pack packet to transmit to interface
     */
    @A_Commande(help = "send bytes to interface")
    @Override
    public void write_out(C_Packet pack) {
        // -------------TO INTERFACE----------------
        if (pack == null) {
            return;
        }

        if (islockout()) {
            if (pack.getsource() != null) {
                TDeb.wri("Write(out) locked out, packet from[" + pack.getsource() + "]");
            }
            return;
        }
        if (getcxstatus().compareTo("connected") != 0) {
            return;
        }
        // set the OUT time of the packet
        pack.setWOtime(new Date(System.currentTimeMillis()));
        Parent.addpacket("" + pack.hashCode(), System.currentTimeMillis(), "PortWO", pack.toString());
//serialise Object to stream it Else ONLY byte[] are writed
        if (isPar_serial() && pack.isbuffer() == false) {
            pack.setObj(pack.getSerial());
        }
        int accbyte = 0;
        if (slave != null) {
            accbyte = slave.write(pack);
        }
        if (pack.getsource() != null) {
            if (pack.getBytes() != null) {
                TDeb.wri("Writed to stream : " + pack.length() + " bytes comming from :" + pack.getsource());
            } else {
                TDeb.wri("Writed to stream : " + pack.toString() + " comming from :" + pack.getsource());
            }
        }
        //statements are only byte[] (=> strings !!)

//discard tag
        if (pack.getBytes() != null && TUtils.equalto(pack.getBytes(), getCX_tagval())) {
            return;
        }
        // ---Accounting--
        setAcc_byteout(getAcc_byteout() + accbyte);
        if (stat_P != null) {
            String txt = pack.getObj().getClass().getSimpleName();
            int size = 1;
            if (pack.getBytes() != null) {
                size = pack.getBytes().length;
                txt = "byte[" + size + "]";

            }
            stat_P.store("interface", -size, txt + " writed to interface");
            stat_P.store(pack, false);
        }
        if (pack.getObj() != null) {
            setAcc_objout(getAcc_objout() + 1);
        }
        setAcc_packout(getAcc_packout() + 1);
    }

    /**
     * Check readed packet from slave :<br> - Manage dis/connections.<br> -
     * Check state "connected", loop.<br> - Manage echo,accounting. <br> -
     * Tramit to scan or write_in.
     *
     * @param packet packet received from interface
     */
    @A_Commande(help = "received bytes by port")
    @Override
    public void read_in(C_Packet packet) {
        C_Packet pack = packet;
        //------------------ FROM INTERFACE-------------------------
        // try to disconnect=> if disconnected=> d=>null on connect()
        if (isloopedout()) {
            if (slave != null) {
                slave.write(pack);
            }
            return;
        }
        if (isloopedin()) {
            return;
        }
        if (stat_P != null) {
            String txt = pack.getObj().getClass().getSimpleName();
            int size = 1;
            if (pack.getBytes() != null) {
                size = pack.getBytes().length;
                txt = "byte[" + size + "]";

            }
            stat_P.store("interface", size, "readed from interface");
            stat_P.store(pack, true);
        }
        if (getcxstatus().compareTo("opened") == 0 || getcxstatus().compareTo("connecting") == 0 || getcxstatus().compareTo("disconnected") == 0) {
            if (isCX_autocx()) {
                if (getcxstatus().startsWith("open") || getcxstatus().compareTo("connecting") == 0) {
                    //slave is reading (from openned to CX BUT main is not still connected 
                    TDeb.wri("read_in() :port is autoconnect,waiting for connection");
                    while (getcxstatus().compareTo("connected") != 0) {
                        try {
                            Thread.currentThread().join(1);
                        } catch (InterruptedException ex) {

                        }
                    }
                }
            } else {
                pack = CXconnect(pack);
            }
        } else if (getcxstatus().compareTo("connected") == 0) {
            pack = CXdisconnect(pack, 0);
        }
        if (pack == null || getcxstatus().compareTo("connected") != 0) {
            return;
        }
        if (isEcho()) { // && is not a terminal??
            write_out(pack);
        } //try BYTE[]=> Object
        if (isPar_serial() && pack.getBytes() != null) {
            Object fromserial = pack.getSer2Obj();
            if (fromserial != null) {
                pack.setObj(fromserial);
            }
        }
        if (this.getModeName().compareTo("File") == 0 && this.isFile_xml2tree() && pack.getBytes() != null) {
            XML2tree a = new com.jdcm.common.XML2tree();
            pack.setObj(a.XMLBase(pack.getBytes()));
        }
        if (pack.getBytes() != null) {

            TDeb.wri("Read from interface : " + pack.length() + " bytes");
            // "conection frame=TAG " avoided
            if (isCX_tagwait() && TUtils.equalto(pack.getBytes(), getCX_tagval())) {
                return;
            }
            setAcc_bytein(getAcc_bytein() + pack.length());
        } else {
            TDeb.wri("Read from interface " + pack.toString());
            setAcc_objin(getAcc_objin() + 1);
        }
        setAcc_packin(getAcc_packin() + 1);
        if (islockin() == false) {//getcxstatus().compareTo("connected")==0 &&
            if (isDCX_TOcom()) {
                //shpitchnette pour toper le timer
                scan.From_interface(pack);
            }
            write_in(pack);
        }
    }

    /**
     * Transmit the readed Packet to main <br> - Manage EOL, Packet WItime,
     * Packet WItime, debug<br> - Transmit to Main
     *
     * @param pack packet to transmit to Main
     */
    public void write_in(C_Packet pack) {
        // ------------------TO MAIN_CX()---------------------------
        if (stat_P != null) {
            String txt = "null";
            if (pack.getObj() != null) {
                txt = pack.getObj().getClass().getSimpleName();
            }
            int size = 1;
            if (pack.isbuffer()) {
                size = pack.getBytes().length;
                txt = "byte[" + size + "]";
            }
            stat_P.store("main", size, txt + " sended to main");
        } else if (pack != null) {
            Parent.addpacket("" + pack.hashCode(), System.currentTimeMillis(), "PortWI", pack.toString());
            TDeb.wri("Write to Main : " + pack.toString() + " bytes & ");
        }
        Parent.read(pack);
    }

    /**
     * Read bytes asked on event (timeout/data received..)<br> ex : exception
     * DATA AVAILABLE <br> - Check isslreading, <br> - Manage lights,Accounting,
     * create new packet,ask to decode.<BR> <br>note : if IPL4 and Pad
     * unactivated, the managed with blocking read
     *
     * @param waitedbytes nb of byte, if -1 all received data
     */
    public void ReadStream(int waitedbytes) {
        int av;
        int waited = waitedbytes;
        byte donnees[] = null;
        // non blocking read procedure if slave IPL4!!
        if (slave instanceof T_IP && isPar_PadtStr() == false) {
            return;
        }
        if (isslreading() == false) {
            return;
        }
        if (getin() != null) {
            try {
                av = available();
                if (stat_P != null && av != 0) {
                    stat_P.store("avalable", av, "avalable");
                }
                if (av == 0 && slave instanceof T_FS && isFile_EOFstop()) {
                    CXstop();
                    return;
                } else if (av == 0) {
                    return;
                } else if (av < 0) {
                    // for file should be ok ?? bellow a ? quick patch? because connect was not closed on slow PC???
                    if (slave instanceof T_IP) {
                        CXclose();
                    }
                } else {
                    isreading = true;
                    if (waited == -1) {
                        waited = av;
                    }
                    if (isPar_readat()) {
                        if (isPar_readingat() == false && isPar_readatloop() == false) {
                            return;
                        }
                        if (waited > getPar_readatval()) {
                            waited = getPar_readatval();
                            if (isPar_readatloop() == false) {
                                setPar_readingat(false);
                            }
                        }
                    }
                    byte[] dinit = new byte[waited];
                    int res = -1;
                    donnees = new byte[0];
                    while (donnees.length == 0 || (isPar_readtoend() && donnees.length < waited)) {
                        //    System.out.println("size = "+dinit.length+" len="+waited);
                        res = getin().read(dinit, 0, dinit.length);
                        if (res == 0) {
                            //TODO : study this case : need to 
                            // =>wait for 'avalable()??' data 
                            //or => exit and re-reed?(implemented)
                            waited = 0;
                        } else if (res < waited) {
                            byte[] d = java.util.Arrays.copyOfRange(dinit, 0, res);
                            dinit = new byte[waited - res];
                            donnees = TUtils.addtab(donnees, d);
                        } else {
                            donnees = dinit;
                        }
                    }
                    // System.out.println(new String(donnees))
                    setAcc_bytercvd(res);
                }
            } catch (IOException ex) {
                TDeb.wri("Could not get avalable from InputStream =>" + ex.getMessage());
                return;
            }
        }
        if (slave instanceof T_FS) {
            // should be a better management with java.nio...FileChanel read/write
            // BUT no InputStream?...
            if (donnees != null && donnees.length > 0) {
                TDeb.wri("readed : " + donnees[0] + "/" + donnees.length + "/" + available());
            }
            if (((T_FS) slave).isEcho2Clear(donnees)) {
                return;
            }
        }
        try {
            if (slave instanceof T_RS) {
                //!!! check avaiblable==0? or check no other DATA AVAILABLE exception..???.
                ((T_RS) slave).clearavailable();
                setRS_lights("DRX", false, "/" + waited);
            }
        } catch (NoClassDefFoundError er) {
        }
        C_Packet pac = new C_Packet(donnees, this);
        read_in(pac);
        TDeb.wri("packet simply sent");
        isreading = false;
    }

    /**
     * Create Slave for IPL4 connection
     *
     * @param sock if not null assagne this socket to the created T_IP
     */
    public void SLcreate(java.net.Socket sock) {
        if (slave == null) {
            slave = new T_IP(this);
            if (sock != null) {
                ((T_IP) slave).setTCPsocket(sock);
            }   //if listen create client socket, P_Thread follow for blocking reading
        }
    }

    /**
     * Ask new port creationstopLclient for connexion to listener
     *
     * @return the port index of the TCP client port created by Main if errors
     * return -1
     */
    public C_Port addTCPclient() {
        //add client from TCP listning
        if (slave instanceof T_IP) {
            if (((T_IP) slave).getTCPsocket() == null) {
                return null;
            } // if listner receive client socket=> creation of client port
            C_Port a = (C_Port) Parent.addCX(this, "Port");
            return a;
        }
        return null;
    }

    public void addTCPclient(C_Port clport) {
        //add client in the listner TCP client listning
        if (slave instanceof T_IP) {
            if (((T_IP) slave).getTCPsocket() != null) {
                ((T_IP) slave).addTCPclient(clport);
            }
        }
    }

    @Override
    public String[] getcxStatusList() {
        String[] res = {"new", "creating", "created", "starting", "started", "opening",
            "opened", "connected", "closing", "closed", "stopping",
            "stopped", "error"
        };
        return res;
    }

    /*
     * @Override public void addConsoleflow() { if (slave instanceof F_terminal)
     * { F_terminal f = (F_terminal) slave; if (f.getconsolescreen() != null) {
     * ((I_Console) Parent).addTextPaneConsole(f.getconsolescreen()); } } }
     *
     * public void removeConsoleflow() {
     *
     * }
     */
    @Override
    public javax.swing.ImageIcon getCMDIcon() {
        return TUtils.getIcon("Port_" + getcxstatus() + ".gif");
    }

    @Override
    public Options_Model getCMDOptions() {
        return option;
    }

    @Override
    public void setCMDOptions(Options_Model r) {
        java.util.List<java.util.List<Object>> valss = r.getChangedOptions();
        for (java.util.List<Object> vals : valss) {
            String label = vals.get(0).toString();
            if (label != null
                    && label.toString().startsWith("TCP") == false
                    && label.toString().startsWith("SYS") == false
                    && label.toString().startsWith("RS") == false
                    && label.toString().startsWith("ETH") == false
                    && label.toString().startsWith("File") == false
                    && label.toString().startsWith("cxstatus") == false
                    && label.toString().startsWith("Index") == false
                    && label.toString().startsWith("TCP_Mode") == false
                    && label.toString().startsWith("PortName") == false) {
                option.setValuefor(label, vals.get(1));
            }
        }
        option.fireTableDataChanged();
    }

    @Override
    public String getCMDdetails() {
        String res = "";
        res = res + "Name : " + this.getCMDName() + " Mode : " + this.getModeName() + " Status :" + this.getcxstatus() + "\r\n";
        String typ = this.getModeName().toUpperCase() + "_";
        if (typ.startsWith("IPL4")) {
            typ = "TCP_";
        }
        if (typ.startsWith("SERIAL")) {
            typ = "RS_";
        }
        if (typ.startsWith("FILE")) {
            typ = "FS_";
        }
        java.util.List<java.lang.reflect.Method> o = Parent.getmethodes(this);

        for (java.lang.reflect.Method meth : o) {
            if (meth.getParameterTypes().length == 1) {
                if ((meth.getName().startsWith("get") || meth.getName().startsWith("is")) && meth.getName().indexOf(typ) != -1) {
                    try {
                        res = res + "   -" + meth.getName() + " : " + Parent.command(this, meth.getName(), null) + "\r\n";
                    } catch (Exception ex) {
                        res = res + "   !" + meth.getName() + " : " + ex.getMessage() + "\r\n";
                    }
                }
            }
        }
        return res;

    }

    @Override
    public java.util.List<I_Command> getCMDChilds(String child) {
        java.util.List<I_Command> res = new java.util.ArrayList<>();
        java.util.List<I_Command> fulllist = new java.util.ArrayList<>();
        if (slave instanceof I_Command) {
            fulllist.add((I_Command) slave);
        }
        for (I_Command chld : fulllist) {
            if (chld.getCMDName().compareTo(child) == 0) {
                res.add(chld);
            }
        }
        return res;
    }

    @Override
    public I_Command getCMDParent() {
        if (Parent instanceof I_Command) {
            return (I_Command) Parent;
        }
        return null;
    }

    @Override
    public String getCMDdesc() {
        return this.getPortName() + "\t" + this.getClass().getSimpleName() + "\t" + getModeName();
    }

    @Override
    public String getCMDName() {
        return getPortName();
    }

    @Override
    public void CXArcv(I_Port src, String message, String type) {
        // receive Admministrativ messages
        // befor MSG after??? 
    }

    @Override
    public void CXnew() {
        TDeb.setsource("Port: [" + getIndex() + "]");
    }

    /**
     * Modify values of the Port => prefered : MainCX/command but well used in
     * MainCX DEPRECIATED use Main.command(obj,methode,value) or
     * util.getMethod...
     *
     * @param meth the methode of the port to implement (ex : Par_Padt).
     * @param value parameter to assigne to the methode.
     * @return true if successfully created
     *
     */
    @Override
    public boolean CXcreate(String meth, String value) {
        String methode = meth;
        boolean res = true;
        Class<?> mclas = java.lang.String.class;
        Object ob1 = value;
        if (value == null || value.length() == 0) {
            ob1 = null;
            mclas = null;
        }
        if (methode.startsWith("set") || methode.startsWith("add")) {
            methode = methode.substring(3);
        }
        // value is boolean{"true","false"} or String
        if (value != null && ("true".compareTo(value) == 0 || "false".compareTo(value) == 0)) {
            mclas = boolean.class;
            ob1 = Boolean.valueOf(value).booleanValue();
        }

        Object cible = this;
        String fct = methode;
        Object ob2 = null;
        if (cible == null) {
            return false;
        }
        if (methode.startsWith("PORT_")) {
            fct = methode.split("_")[1];
        }
        //if (value.startsWith("dbg")){ fct=composant;}
        if (methode.startsWith("TERM_") && slave instanceof F_Terminal) {
            F_Terminal t = (F_Terminal) slave;
            if (methode.compareTo("TERM_OutputMode") == 0) {
                t.setTerm_outMode(value);
            }
            if (methode.compareTo("TERM_send") == 0) {
                t.setSendbt(value);
            }
        } else {
            java.lang.reflect.Method m = null;
            try {
                if (ob2 == null) {
                    if (mclas != null) {
                        fct = "set" + fct;
                    }
                    try {
                        m = cible.getClass().getDeclaredMethod(fct, mclas);
                    } catch (NoSuchMethodException | SecurityException e) {
                    }
                    if (m == null) {
                        m = cible.getClass().getMethod(fct, mclas);
                    }
                    if (ob1 == null) {
                        m.invoke(cible);
                    } else {
                        m.invoke(cible, ob1);
                    }

                } else {
                    m = cible.getClass().getDeclaredMethod("set" + fct, java.lang.String.class, mclas);
                    m.invoke(cible, ob2, ob1);
                }
            } catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
                TDeb.wri("methode : " + m + "cible : " + cible + "=>" + e.getMessage());
                res = false;
            }
        }
        if (res == false) {
            TDeb.wri("Pb : " + methode + " NOT SET TO " + value);
        }
        return res;
    }

    /**
     * Start the port : <br> - status="creating" =>"starting"=>"started".<br> -
     * slave set to slrunning.<br> - if autoopen=> CXopen().<br>
     */
    @A_Commande(help = "start : slave/admin")
    @Override
    public void CXstart() {
        boolean isaListner = false;
        setAcc_bytein(0);
        setAcc_packout(0);
        setAcc_bytercvd(0);
        setAcc_byteout(0);
        setAcc_packin(0);
        setAcc_objin(0);
        setAcc_objout(0);
        if (getcxstatus().compareTo("creating") != 0 && getcxstatus().compareTo("stopped") != 0 && getcxstatus().compareTo("error") != 0) {
            return;
        }
        if (getcxstatus().compareTo("creating") == 0) {
            setcxstatus("created");
        }
        setcxstatus("starting");
        setslrunning(true);
        setslreading(false);
        setAcc_startdate(new java.util.Date(System.currentTimeMillis()));
        setAcc_stopdate(new java.util.Date(0));
        String mode = getModeName();
        if (mode.startsWith("IPL4")) {
            SLcreate(null);
            if (this.getTCP_Mode().startsWith("TCP_L")) {
                isaListner = true;
            }
        } else if (mode.startsWith("Serial")) {
            slave = new T_RS(this);
        } else if (mode.startsWith("File")) {
            if (isFile_read() == false && isFile_write() == false) {
                CXerror("File is not read not write");
                return;
            }
            slave = new T_FS(this);
        } else if (mode.startsWith("Term")) {
            if (getTerm_Type().compareTo("classic") == 0) {
                slave = new F_Terminal(this);
                ((F_Terminal) slave).setloopedout(isloopedout());
            } else {
                slave = new F_Talc(this);
                ((F_Talc) slave).addPanel(getTerm_Type());
            }
            if (slave instanceof javax.swing.JFrame) {
                ((javax.swing.JFrame) slave).addWindowListener(new java.awt.event.WindowAdapter() {
                    @Override
                    public void windowClosing(java.awt.event.WindowEvent evt) {
                        CXstop();
                    }
                });
            }

        } else if (mode.startsWith("DB")) {
            slave = new T_DB(this);
        } else if (mode.startsWith("ETH")) {
            slave = new T_ETH(this);
        } else if (mode.startsWith("LPT")) {
            slave = new T_LPT(this);
        } else if (mode.startsWith("WIN")) {
            slave = new T_WIN(this);
        } else if (mode.startsWith("USB")) {
            slave = new T_USB(this);
        } else if (mode.startsWith("SYS")) {
            slave = new T_SYS(this);
        } else if (mode.startsWith("MEM")) {
            slave = new T_MEM(this);
        } else if (mode.startsWith("JNA")) {
            slave = new T_MEM(this);
        } else if (mode.startsWith("SND")) {
            slave = new T_SND(this);
        }
        if (slave == null) {
            this.CXerror("try to start a NULL slave");
            return;
        }
        if (slave.sl_start() < 0) {
            return;
        }
        if (slave instanceof java.lang.Thread) {
            // Serial/LPT/tcpdump/filein event listner alredy create Thread
            if (((java.lang.Thread) slave).isAlive()) {
                TDeb.wri("PB : Port thread already Alive at starting");
            } else {
                slave.start();
            }
        } else {
            // if slave is not a Thread...not openned by slave...
            setCX_openbyslave(false);
        }
        setboxenable(false);
        if (getcxstatus().compareTo("starting") == 0) {
            setcxstatus("started");
        } else {
            return;
        } //if Slave IP=> open lauched by slave => else blocking all
        //Pb XML must be managed with autocopen (no data readed) (see RDP)
        if (isCX_autoopen() && isCX_openbyslave() == false && isaListner == false) {
            CXopen();
        }
    }

    /**
     * Open the port :<br> - status="opening" =>"opened".<br> - open streams in
     * / out<br> - slave set to slreading.
     */
    @A_Commande(help = "open : flow/checks")
    public void CXopen() {
        setcxstatus("opening");
        if (slave == null) {
            this.CXerror("try to open a NULL slave");
            return;
        } //!! blocking for DB ko
        if (slave.sl_open() < 0) {
            return;
        }
        if (getcxstatus().compareTo("opening") == 0) {
            setcxstatus("opened");
        } else {
            return;
        } // Ex : error status
        setslreading(true); //read after oppenned to avoid  "datareceived/oppenning"
    }

    public C_Packet CXconnect() {
        return CXconnect(null);
    }

    /**
     * Try to Connect the port :<br> - Check state is "opened" or
     * "disconnected", autocx, if CX_strin receved.<br> - Manage the tag,
     * banner, packet transmition.<br>
     *
     * @param pack the receved packet
     * @return the Packet to transmit to the reading process
     */
    @A_Commande(help = "connect : banner/strin/tag/...")
    public C_Packet CXconnect(C_Packet pack) {
        if (getcxstatus().compareTo("opened") != 0 && getcxstatus().compareTo("disconnected") != 0) {
            return null;
        }
        setcxstatus("connecting");
        boolean bool = false;
        if (isCX_autocx()) {
            bool = true;
        } else {
            if (getCX_strin() != null && pack != null && pack.getBytes() != null) {
                if (TUtils.ismatchto(getCX_strin(), pack.getBytes())) {
                    TDeb.wri("Connect :" + getIndex() + "by recieved string : " + new String(getCX_strin()));
                    bool = true;
                }
            }
            if (getCX_strin() != null) {
                if (getCX_strin().length == 1 && getCX_strin()[0] == 0x25) {
                    bool = true;
                }
            }
        }
        if (bool) {
            setcxstatus("connected");
            if (isCX_tagsend() && getcxstatus().compareTo("connected") == 0 && getAcc_packout() == 0) {
                TDeb.wri("Send 'Connect TAG string' : " + getCX_tagval());
                C_Packet newpack = new C_Packet(getCX_tagval(), this);
                write_out(newpack);
                write_in(newpack);
            }
        }
        if (bool && getCX_Banner() != null) {
            TDeb.wri("Connecting :" + getIndex() + "=> banner (Hex) : " + new String(getCX_Banner()));
            C_Packet newpack = new C_Packet(getCX_Banner(), this);
            write_out(newpack);
        }
        if (scan != null) {
            scan.dcx_TOabs(0);
            scan.dcx_TOcom(0);
        }
        if (isCX_autocx() || this.isCX_Transmit()) {
            return pack;
        } else {
            return null;
        }
    }

    public void CXdisconnect() {
        CXdisconnect(null, 0);
    }

    /**
     * Try to Disconnect the port :<br> - Check state is
     * "connected",DCX_TimeOut, if DCX_strin receved.<br> - Manage the banner,
     * packet transmition. <br> NB : ne need to return packet because port is
     * Disconnected !
     *
     * @param packet the receved packet
     * @param from if 1,DCX from Main DCX_Transmit to media if 0 to Main
     * @return the packet after disconnect process
     *
     */
    @A_Commande(help = "disconnect : TO/banner/strin/...")
    public C_Packet CXdisconnect(C_Packet packet, int from) {
        //from=0=> interface/1=>Main_CX /2=> XML,TOdcx,...

        boolean bool = false;
        C_Packet res = packet;
        if (getcxstatus().compareTo("connected") != 0) {
            return res;
        }
        if (packet == null) {//for (isDCX_TOcom() || isDCX_TOabs()) 
            bool = true;
        }
        if (getDCX_strin() != null && packet != null && packet.getBytes() != null) {
            if (TUtils.ismatchto(packet.getBytes(), getDCX_strin())) {
                TDeb.wri("Disconnect " + getIndex() + "by receve: " + new String(getDCX_strin()));
                bool = true;
            }
        }
        if (bool && getDCX_Banner() != null) {
            TDeb.wri("Disconnecting :" + getIndex() + "=> banner (Hex) : " + new String(getDCX_Banner()));
            C_Packet pack = new C_Packet(getDCX_Banner(), this);
            write_out(pack);
        }
        if (bool & isDCX_Transmit()) {
            if (from == 0) {
                write_in(packet);
            } else {
                write_out(packet);
            }
        }
        if (bool) {
            if (isDCX_tostop()) {
                CXstop();
            } else {
                setcxstatus("disconnected");
            }
        }
        if (scan != null) {
            scan.scanning();
        }
        return res;
    }

    /**
     * Close the port<br> - Set Slave to slreading==false<br> - Close in/out
     * streams, slaves<br> - remove listners Asked by CXstop()=> is private !!.
     */
    private void CXclose() {
        TDeb.wri("Closing streams, sockets, ...");
        setslreading(false);
        if (islog() && slave instanceof F_Terminal) {
            F_Terminal f = (F_Terminal) slave;
            if (f.getconsolescreen() != null) {
                ((I_Console) Parent).removeTextPaneConsole(f.getconsolescreen());
            }
        }
        try {
            if (in != null && in != System.in) {
                in.close();
            }
            if (out != null) {
                out.close();
            }
        } catch (IOException ex) {
            TDeb.wri("try to close slave: " + ex.getMessage());
        }
        if (slave == null) {
            TDeb.wri("try to close a NULL slave");
        }
        if (slave != null) {
            slave.sl_close();
        }
        in = null;
        out = null;
    }

    /**
     * Stop the port :<br> - status "stopping"=>"stopped". <br> - Set Slave to
     * slrunning==false<br> - Stop scan, slaves.<br> - Manage UI, accounting,
     * other TCP port.
     */
    @A_Commande(help = "stop : streams/slave/listners/...")
    @Override
    public void CXstop() {
        if (getcxstatus().compareTo("stopping") == 0
                || getcxstatus().compareTo("stopped") == 0
                || getcxstatus().compareTo("closed") == 0) {
            return;
        }
        setcxstatus("stopping");
        CXclose();
        setslrunning(false);
        // should check (wait) for the slave thread (run) to stop
        //((java.?lang.?Thread)slave).isAlive()==false;
        String mode = this.getModeName();
        if (mode.startsWith("IPL4")) {
            if (getTCP_Mode().startsWith("TCP_L")) {
                TDeb.wri("Asking stop listning to port : " + this.getIndex());
                if (slave instanceof T_IP) {
                    T_IP thr = (T_IP) slave;
                    try {
                        while (thr.getTCPclients().isEmpty() == false) {
                            //for (int i = 0; i < ((T_IP) slave).getTCPclientsize(); i++) {
                            C_Port p = thr.getTCPclients().get(0);
                            p.CXend();
                            thr.removeTCPclient(p);//((T_IP) slave).getTCPclient(i).CXend();
                        }
                    } catch (Exception ex) {
                        TDeb.wri("no TCPClient found for " + ((T_IP) slave).getName());
                    }
                }
            } else {
                if (getTCP_Mode().startsWith("TCP_C") && listenport != null) {
                    if (listenport.getSlave() instanceof T_IP) {
                        ((T_IP) listenport.getSlave()).removeTCPclient(this);
                    }
                }
            }
        } else if (mode.startsWith("Serial")) {
            if (stat_RS != null) {
                stat_RS.setLightActive(false);
            }
            //  initRSbox();
        } else if (mode.startsWith("ETH")) {
            // slave should not be null to select interface
            slave = new T_ETH(this);
        }
        slave = null;
        setboxenable(true);
        setcxstatus("stopped");
        // setlocked(false);
        setAcc_stopdate(new java.util.Date(System.currentTimeMillis()));
        if (getstop2end()) {//getListener() != null) {
            CXend();
        }
    }

    /**
     * Port modify :<br> - force XML config to run.<br> - state
     * "modiying"=>"modified".<br> - Re assign status & XML to old status.
     */
    @A_Commande(help = "modify : check XML??/...")
    public void CXmodify() {
        String etat = getcxstatus();
        boolean precXML = Parent.isConf_Enabled();
        if (precXML == false) {
            Parent.setConf_Enabled(true);
        }
        setcxstatus("modifying");
        setcxstatus(
                "modified");
        // stop XML to set old status without XML actions.
        Parent.setConf_Enabled(false);
        setcxstatus(etat);
        Parent.setConf_Enabled(precXML);
    }

    /**
     * Port error :<br> - Manage debug<br> - Check status (to avoid error loop
     * if closing error).<br> - state "error"=>..."stopping".<
     *
     * @param errtxt the error string to display
     */
    @A_Commande(help = "error : followed by stop")
    public void CXerror(String errtxt) {
        if (errtxt != null) {
            TDeb.wri("Error : " + errtxt);
        }
        if (getcxstatus().compareTo("closed") == 0) {
            return;
        }
        if (getcxstatus().compareTo("error") == 0) {
            return;
        }
        if (getcxstatus().compareTo("stopped") == 0 || getcxstatus().compareTo("stopping") == 0) {
            return;
        }
        setcxstatus("error");
        CXstop();
    }

    /**
     * Port end:<br> - Check status.<br> - Manage UI,debug.<br> - Init as Ended,
     * Status is "closed".
     */
    @A_Commande(help = "end : remove the port")
    @Override
    public void CXend() {
        // close even if connected!!! => stop() !!
        ///  !! should be used ONLY by Main_CX.endCX()this.getIndex());
        TDeb.wri("CXend asked");
        if (getcxstatus().compareTo("closed") == 0) {
            return;
        }
        if (getcxstatus().compareTo("stopped") != 0) {
            CXstop();
        }
        if (F_option != null) {
            F_option.setVisible(false);
        }

        if (portSkin instanceof IF_Port) {
            portSkin.setVisible(false);
            portSkin = null;
        }
        setPort_Stat(false);
        if (stat_RS != null) {
            // could be remove to avoid COM3/COM4 confusion 
            //or matained to store data
            // Parent.removeTab(stat_RS); 
            //  stat_RS = null;
        }
        setlocked(true);
        setModeidx(0);
        //used for connected number accounting
        if (getListener() != null) {
            getListener().stopLclient(getIndex());
            // System.out.println(":"+getListener()+"=>"+getIndex());
        }
        setListener((C_Port) null);
        //port keep his name for event detection after deth
        // setPortName("---");
        setcxstatus("closed");
        setAcc_bytein(0);
        setAcc_packout(0);
        setAcc_byteout(0);
        setAcc_packin(0);
        TDeb.wri("port wil be removed");
        final C_Port c = this;
        java.lang.Thread t = new java.lang.Thread(new java.lang.Runnable() {
            @Override
            public void run() {
                Parent.CXremove(c);
            }
        });
        t.start();

    }

    /**
     * Manage boxes access
     */
    private void setboxCXenable(boolean bool) {
        option.setused("CX_Transmit", bool);
        option.setused("CX_strin", bool);
        option.setused("CX_TOval", bool == false);

    }

    private void setboxtcpenable(boolean bool) {
        if (bool) {
            try {
                setTCP_Addr(java.net.InetAddress.getLocalHost());
            } catch (UnknownHostException e) {
                setTCP_Addr("0.0.0.0");
            }
        }

    }

    private void setboxDescenable(boolean bool) {
        //jSlider1.setEnabled(bool);
        option.setused("DESC_Protolast", bool);
        option.setused("DESC_PadTO", bool);
        option.setused("DESC_in", bool);
        option.setused("DESC_panel", bool);
        //     option.setused("DESC_autostack", bool);
    }

    private void setboxenable(boolean bool) {
        option.setused("SSL_Enabled", bool);
    }

    /**
     * initialise boxes with PC interfaces public because USB listner use it to
     * add/remove devices tate=0 => maj USB BOX at initialisation (COM box is
     * empty) State=1=> Maj asked
     *
     *
     * @return
     */
    public int initUSBbox() {
        T_USB sl = new T_USB(this);
        option.setValuefor("USB_Element", sl.getUSBhost());
        return 0;
    }

    /**
     * initialise boxes with PC interfaces public because SND listner use it to
     * add/remove devices state=0 => maj SND BOX at initialisation State=1=> Maj
     * asked
     *
     * @return
     */
    public int initSNDbox() {

        if (slave == null) {
            slave = new T_SND(this);
        }
        javax.swing.DefaultComboBoxModel<javax.sound.sampled.Line> linesin
                = option.getValuefor("SND_linesin", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        javax.swing.DefaultComboBoxModel<javax.sound.sampled.Line> linesout
                = option.getValuefor("SND_linesout", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        javax.swing.DefaultComboBoxModel<javax.sound.sampled.Line> mixers
                = option.getValuefor("SND_mixers", new DefaultComboBoxModel<javax.sound.sampled.Line>());
        try {
            linesin.removeAllElements();
            linesout.removeAllElements();
            mixers.removeAllElements();
            TDeb.wri("Init SND boxes");
            javax.sound.sampled.Line[] lines = ((T_SND) slave).init_interface("linesin");
            for (javax.sound.sampled.Line line : lines) {
                //  if (line instanceof javax.sound.sampled.Line) {
                linesin.addElement(line);
                //  }
            }
            lines = ((T_SND) slave).init_interface("linesout");
            for (javax.sound.sampled.Line line : lines) {
                //  if (line instanceof javax.sound.sampled.Line) {
                linesout.addElement(line);
                //  }
            }
            lines = ((T_SND) slave).init_interface("mixers");
            for (javax.sound.sampled.Line line : lines) {
                //  if (line instanceof javax.sound.sampled.Mixer) {
                mixers.addElement(line);
                //  }
            }
            javax.sound.sampled.AudioFormat[] af = ((T_SND) slave).initFormat();
            if (af != null && af.length > 0) {
                option.setValuefor("SND_format", af[0]);
            }
        } catch (Exception ex) {
            return -1;
        }
        return 0;
    }

    /**
     * initialise boxes with PC interfaces
     *
     * @return
     */
    public int initRSbox() {
        DefaultComboBoxModel<Object> objrs = option.getValuefor("RS_interfaces", new DefaultComboBoxModel<>());
        int pos = objrs.getIndexOf(objrs.getSelectedItem());
        objrs.removeAllElements();
        try {
            TDeb.wri("Init RS232 or LPT boxes");
            // Serial port are not java standard
            java.util.ArrayList<Object> portslist = new java.util.ArrayList<>();
            //on stop => could rescan with slave...
            portslist.addAll(T_RS.init_interface());
            for (Object port : portslist) {
                if (objrs.getIndexOf(port) < 0) {
                    objrs.addElement(port);
                }
            }
            option.setValuefor("RS_interfaces", objrs);
            //setlptboxenable(lptports != null && lptports.length != 0);
            // setRS_lights(false);
            if (pos < objrs.getSize()) {
                setRS_Port(objrs.getElementAt(pos));
            } else {
                setRS_Port(objrs.getElementAt(0));
            }
        } catch (Exception ex) {
            TDeb.wri("Pb init_RS Boxes: " + ex.getMessage());
            return -1;
        }

        return 0;
    }

    public int initLPTbox() {
        javax.swing.DefaultComboBoxModel<Object> objlpt = option.getValuefor("LPT_interfaces", new DefaultComboBoxModel<>());
        int pos = objlpt.getIndexOf(objlpt.getSelectedItem());
        objlpt.removeAllElements();
        try {
            TDeb.wri("Init LPT boxes");
            java.util.List<Object> portslist;
            if (slave instanceof T_LPT) {
                //on stop => rescan with slave...
                portslist = ((T_LPT) slave).init_interface();
            } else {
                portslist = (new T_LPT(this)).init_interface();
            }
            for (Object port : portslist) {
                if (objlpt.getIndexOf(port) < 0) {
                    objlpt.addElement(port);
                } else {
                    objlpt.setSelectedItem(objlpt.getElementAt(0));
                }
            }
            //setlptboxenable(lptports != null && lptports.length != 0);
            setRS_lights(false);
            if (pos < objlpt.getSize()) {
                objlpt.setSelectedItem(objlpt.getElementAt(pos));
            }

        } catch (Exception ex) {
            TDeb.wri("Pb init_RS Boxes: " + ex.getMessage());
            return -1;
        }
        return 0;
    }

    private int initFILEbox() {
        ///----- Options-----
        java.util.List<File> files = TUtils.getFiles("Keys", null);
        if (files != null) {
            option.setValuefor("SSL_Kfile", new javax.swing.DefaultComboBoxModel<>(files.toArray(new File[]{})));
            option.setValuefor("SSL_Tfile", new javax.swing.DefaultComboBoxModel<>(files.toArray(new File[]{})));
        }
        java.util.List<String> names = TUtils.getFileNames("com/jdcm/editors", null, false);
        java.util.List<String> term_typ = new java.util.ArrayList<>();
        for (String name : names) {
            if (name.startsWith("B_") && name.contains("$") == false) {
                term_typ.add(name);
            }
        }
        term_typ.add(0, "classic");
        option.setValuefor("Term_Type", new javax.swing.DefaultComboBoxModel<>(term_typ.toArray(new String[]{})));
        files = TUtils.getFiles("mapping", null);
        if (files != null) {
            files.add(0, new File("none"));
            option.setValuefor("DB_mapfile", new javax.swing.DefaultComboBoxModel<>(files.toArray(new File[]{})));
        }
        ///----- Ports-----
      /*
         * try { files = TUtils.getFiles("traces"); } catch (Exception ex) { }
         */
        files = TUtils.getFiles("traces", null);
        if (files != null && files.isEmpty() == false) {
            DefaultComboBoxModel<File> c = option.getValuefor("File_traces", new DefaultComboBoxModel<File>());
            c.removeAllElements();
            for (File file : files) {
                c.addElement(file);
            }
            if (c.getSize() > 0) {
                setFile_addr(files.get(0));
            }
            option.setValuefor("File_traces", new javax.swing.DefaultComboBoxModel(files.toArray(new File[]{})));
        }
        TDeb.wri("Normal end for Init_FILEbox ");
        return 0;
    }

    private int initETHbox(int index) {
        T_ETH ethdump;
        TDeb.wri("Init ETH boxes ");
        javax.swing.DefaultComboBoxModel<String> objeth = option.getValuefor("ETH_interfaces", new javax.swing.DefaultComboBoxModel<String>());
        try {
            try {
                // could NOT catch class NOT Found Exception
                ethdump = new T_ETH(this);
            } catch (Throwable ex) {
                TDeb.wri(" jpcap seens not be installed :" + ex.getMessage());
                return -1;
            }
            if (index < 0) {
                objeth.removeAllElements();
                String[] list = ethdump.init_interface();
                //   setethboxenable( list != null && list.length > 0);
                if (list == null) {
                    TDeb.wri("Returned Ethernet devices list is NULL ");
                    return -1;
                }
                if (list.length == 0) {
                    TDeb.wri("Returned Ethernet devices list is empty ");
                    return -1;
                }
                for (String list1 : list) {
                    objeth.addElement(list1);
                }
                objeth.setSelectedItem(objeth.getElementAt(0).toString());
            }
        } catch (Exception ex) {
            return -1;
        }
        TDeb.wri("Normal end for Init_ETHbox");
        return 0;
    }

    private int init_DBbox() {
        DefaultComboBoxModel<String> objdb = option.getValuefor("DB_interfaces", CBstrModel);
        if (objdb.getSize() == 0) {
            String[] dblist = {"SQLServer", "MySQL", "Derby", "Oracle_thin",
                "Oracle_tns", "ODBC_sys", "ODBC_mdb", "ODBC_xls", "HSQL_odb", "HSQL_mem", "Sybase"
            };
            for (String dbname : dblist) {
                try {
                    String dbtype = dbname.split("_")[0];
                    if (getConsole() == null || getConsole().ismedia_loaded(dbtype)) {
                        objdb.addElement(dbname);
                    }
                } catch (Exception ex) {
                    // drier not foubd => normal but not added to the list
                }
            }
            if (objdb.getSize() == 0) {
                // no DB drivers => disable the cab
                modenames.remove("DB");
            } else {
                setDB_type(objdb.getElementAt(0).toString());
            }
        }
        TDeb.wri("Normal end for Init_DBbox");
        return 0;
    }

    @Override
    public void setinitDESCPack(C_Packet p) {
        //throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void setDESC_slid(int pos) {
        //throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public boolean isDESC_EOFstop() {
        return false;//throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    public void write_in(String select__from_acquereurs) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
}
