package com.jdcm.eltrx;

import com.jdcm.common.D_Utils;
import java.net.*;
import javax.net.ssl.*;

/**
 * F_SSL.java Created on 23 decembre 2006, 16:53
 *
 * Display/Edit TCP (&SSL) Socket informations<br>
 *
 * @author Marc GIBOIN
 */
public class F_Socket extends javax.swing.JFrame {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private final D_Debug TDeb;
    private final D_Utils TUtils;
    // if present the port using this socket
    private final C_Port Parent;
    // the socket to display
    private Object sock = null;

    /**
     * frame used to display a TCP with/without SSL
     *
     * @param P the port suppoting this Socket
     * @param cx the Socke to edit
     */
    public F_Socket(C_Port P, Object cx) {
        Parent = P;
        sock = cx;
        TDeb = new D_Debug("Socket: [" + Parent.getIndex() + "]");
        TDeb.setconsole(Parent.getConsole());
        TUtils = new D_Utils();
        if (cx == null) {
            return;
        }
        this.setIconImage(TUtils.getImage("Socket.gif"));
        initComponents();
        this.jTabbedPane1.setEnabled(false);

        if (sock instanceof Socket) {
            setTCP((Socket) sock);
        }
        if (sock instanceof DatagramSocket) {
            setUDP((DatagramSocket) sock);
        }
        if (sock instanceof SSLSocket) {
            setSSL((SSLSocket) sock);
        }
    }

    /**
     * display an SSL context
     *
     * @param ctx SSL context to display
     */
    public void setcontxt(SSLContext ctx) {
        jTextField17.setText(ctx.getProtocol());
        jTextField27.setText(ctx.getProvider().getName());
        jTextField28.setText(ctx.getProvider().getInfo());
        jTextField29.setText("" + ctx.getProvider().getVersion());
        // ctx.
        String[] li = new String[ctx.getProvider().values().size()];
        for (int i = 0; i < li.length; i++) {
            li[i] = ctx.getProvider().values().toArray()[i].toString();
        }
        this.jList5.setModel(getListModel(li));

        String[] li2 = new String[ctx.getProvider().getServices().size()];
        for (int i = 0; i < li2.length; i++) {
            li2[i] = ctx.getProvider().getServices().toArray()[i].toString();
        }
        this.jList10.setModel(getListModel(li2));

        String[] li3 = new String[ctx.getProvider().getServices().size()];
        for (int i = 0; i < li3.length; i++) {
            li3[i] = ctx.getProvider().getServices().toArray()[i].toString();
        }
        this.jList11.setModel(getListModel(li3));
    }

    /**
     * Transfom an array to a ListModel (used for columns) used for
     * chiffersuite/protocols/...
     *
     * @param <U> Type of arrays
     * @param txtsthe array to display
     * @return the litModel
     */
    private <U> javax.swing.DefaultListModel<U> getListModel(U[] txts) {
        javax.swing.DefaultListModel<U> res = new javax.swing.DefaultListModel<>();
        if (txts == null) {
            return res;
        }
        for (U txt : txts) {
            res.addElement(txt);
        }
        return res;
    }

    /**
     * display SSL Socket
     *
     * @param SSLsock Socket to display
     */
    private void setSSL(SSLSocket SSLsock) {
        this.jTabbedPane1.setEnabled(true);
        jTextField26.setText("TCP SSL");
        jCheckBox1.setSelected(SSLsock.getEnableSessionCreation());
        jList1.setModel(getListModel(SSLsock.getEnabledCipherSuites()));
        jList1.setModel(getListModel(SSLsock.getEnabledCipherSuites()));
        jList2.setModel(getListModel(SSLsock.getEnabledProtocols()));
        jList3.setModel(getListModel(SSLsock.getSupportedCipherSuites()));
        jList4.setModel(getListModel(SSLsock.getSupportedProtocols()));
        jCheckBox2.setSelected(SSLsock.getNeedClientAuth());
        jCheckBox4.setSelected(SSLsock.getUseClientMode());
        jCheckBox5.setSelected(SSLsock.getWantClientAuth());
        if (SSLsock.getSession() != null) {
            SSLSession session = SSLsock.getSession();
            jTextField1.setText(session.getCipherSuite());
            jTextField2.setText(session.getPeerHost());
            java.util.Date dt = new java.util.Date();
            dt.setTime(session.getCreationTime());
            jCB_ObjectEditor1.setValue(dt);
            jTextField15.setText("" + session.getApplicationBufferSize());
            //session.getPeerCertificates()[]
            try {
                jList6.setModel(getListModel(SSLsock.getSession().getPeerCertificateChain()));
                jList7.setModel(getListModel(SSLsock.getSession().getLocalCertificates()));
                jList8.setModel(getListModel(SSLsock.getSession().getPeerCertificates()));
                jList9.setModel(getListModel(SSLsock.getSession().getValueNames()));
            } catch (SSLPeerUnverifiedException ex) {
                TDeb.wri("SSL getSession()=>" + ex.getMessage());
            }
            jList6.setCellRenderer(new com.jdcm.editors.ACE_ObjRenderer<javax.security.cert.X509Certificate>());
            jList7.setCellRenderer(new com.jdcm.editors.ACE_ObjRenderer<java.security.cert.Certificate>());
            jList8.setCellRenderer(new com.jdcm.editors.ACE_ObjRenderer<java.security.cert.Certificate>());

            try {
                jTextField16.setText("" + session.getSessionContext().getSessionCacheSize());
            } catch (Exception ex) {
                jTextField16.setEnabled(false);
            }
            try {
                jTextField25.setText("" + session.getSessionContext().getSessionTimeout() / 3600);
            } catch (Exception ex) {
                jTextField25.setEnabled(false);
            }
            java.util.Enumeration<byte[]> e = session.getSessionContext().getIds();
            if (e != null) {
                while (e.hasMoreElements()) {
                    jComboBox1.insertItemAt(TDeb.retire("BYTE", e.nextElement()), 0);
                }
            }
            if (session.getId() != null) {
                if (session.getId().length > 0) {
                    jComboBox1.insertItemAt("=>" + TDeb.retire("BYTE", session.getId()), 0);
                    jComboBox1.setSelectedIndex(0);
                    jComboBox1.setEditable(true);
                } else {
                    jComboBox1.setEnabled(false);
                }
            }
            java.util.Date dt2 = new java.util.Date();
            dt2.setTime(session.getLastAccessedTime());
            jCB_ObjectEditor2.setValue(dt2);
            if (session.getLocalPrincipal() != null) {
                jTextField19.setText(session.getLocalPrincipal().getName());
            } else {
                jTextField19.setEnabled(false);
            }
            jTextField20.setText("" + session.getPacketBufferSize());
            jTextField21.setText(session.getPeerHost());
            jTextField22.setText("" + session.getPeerPort());

            try {
                jTextField23.setText(session.getPeerPrincipal().getName());
            } catch (SSLPeerUnverifiedException ex) {
                jTextField23.setEnabled(false);
            }
            jTextField24.setText("" + session.getProtocol());
            jCheckBox14.setSelected(session.isValid());
        }

    }

    /**
     * Display simple TCP Socket part
     *
     * @param tcpsock the socket to display
     */
    private void setTCP(Socket tcpsock) {
        jTextField26.setText("TCP");
        try {
            jCheckBox3.setSelected(tcpsock.getOOBInline());
        } catch (SocketException ex) {
            jCheckBox3.setEnabled(false);
        }
        jCheckBox6.setSelected(tcpsock.isBound());
        jCheckBox7.setSelected(tcpsock.isClosed());
        jCheckBox8.setSelected(tcpsock.isConnected());
        jCheckBox9.setSelected(tcpsock.isInputShutdown());
        jCheckBox10.setSelected(tcpsock.isOutputShutdown());
        try {
            jCheckBox11.setSelected(tcpsock.getReuseAddress());
        } catch (SocketException ex) {
            jCheckBox11.setEnabled(false);
        }
        try {
            jCheckBox12.setSelected(tcpsock.getTcpNoDelay());
        } catch (SocketException ex) {
            jCheckBox12.setEnabled(false);
        }
        try {
            jCheckBox13.setSelected(tcpsock.getKeepAlive());
        } catch (SocketException ex) {
            jCheckBox13.setEnabled(false);
        }
        jTextField3.setText(tcpsock.getInetAddress().getHostName() + " @ " + tcpsock.getInetAddress().getHostAddress());
        jTextField4.setText("" + tcpsock.getPort());
        //jTextField5.setText(tcpsock.getLocalAddress().getHostName()+" @ "+tcpsock.getLocalAddress().getHostAddress());
        jCB_ObjectEditor3.setValue(tcpsock.getLocalAddress());
        jTextField6.setText("" + tcpsock.getLocalPort());
        try {
            jTextField7.setText("" + tcpsock.getReceiveBufferSize());
        } catch (SocketException ex) {
            jTextField7.setEnabled(false);
        }
        try {
            jTextField8.setText("" + tcpsock.getSendBufferSize());
        } catch (SocketException ex) {
            jTextField8.setEnabled(false);
        }
        try {
            jTextField9.setText("" + tcpsock.getSoLinger());
        } catch (SocketException ex) {
            jTextField9.setEnabled(false);
        }
        try {
            jTextField10.setText("" + tcpsock.getSoTimeout());
        } catch (SocketException ex) {
            jTextField10.setEnabled(false);
        }
        try {
            jTextField11.setText("" + tcpsock.getTrafficClass());
        } catch (SocketException ex) {
            jTextField11.setEnabled(false);
        }
        jTextField12.setText("" + tcpsock.hashCode());
        jTextField13.setText("" + tcpsock.toString());
    }

    /**
     * Display UDP Socket
     *
     * @param sockUDP the UDP Socketto display
     */
    private void setUDP(DatagramSocket sockUDP) {

        jTextField26.setText("UDP");
        jCheckBox6.setSelected(sockUDP.isBound());
        jCheckBox7.setSelected(sockUDP.isClosed());
        jCheckBox8.setSelected(sockUDP.isConnected());
        jCheckBox9.setEnabled(false);
        jCheckBox10.setEnabled(false);
        //jTextField3.setText(sock.getInetAddress().getHostName()+" @ "+sock.getInetAddress().getHostAddress());
        jTextField4.setText("" + sockUDP.getPort());
        //jTextField5.setText(sockUDP.getLocalAddress().getHostName()+" @ "+sockUDP.getLocalAddress().getHostAddress());
        jCB_ObjectEditor3.setValue(sockUDP.getLocalAddress());
        jTextField6.setText("" + sockUDP.getLocalPort());
        try {
            jTextField7.setText("" + sockUDP.getReceiveBufferSize());
        } catch (SocketException ex) {
            jTextField7.setEnabled(false);
        }
        try {
            jTextField8.setText("" + sockUDP.getSendBufferSize());
        } catch (SocketException ex) {
            jTextField8.setEnabled(false);
        }
        jTextField9.setEnabled(false);
        try {
            jTextField10.setText("" + sockUDP.getSoTimeout());
        } catch (SocketException ex) {
            jTextField10.setEnabled(false);
        }
        try {
            jTextField11.setText("" + sockUDP.getTrafficClass());
        } catch (SocketException ex) {
            jTextField11.setEnabled(false);
        }
        jTextField12.setText("" + sockUDP.hashCode());
        jTextField13.setText("" + sockUDP.toString());
    }

    /**
     * Display a certificats in certificat tab
     *
     * @param o the certificat to Display (Object because different class :
     * sun.&jaa.net...).
     */
    public void showcert(Object o) {
        if (o == null) {
            return;
        }
        // Cert.
        ((javax.swing.border.TitledBorder) jPanel7.getBorder()).setTitle(o.getClass().toString());
        p_Texte1.setText(o.toString());// already in title
        p_Texte1.getTextePane().setCaretPosition(0);
        jPanel7.repaint(); // manndatory for borders
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel34 = new javax.swing.JPanel();
        jLabel35 = new javax.swing.JLabel();
        jTextField26 = new javax.swing.JTextField();
        jButton1 = new javax.swing.JButton();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel4 = new javax.swing.JPanel();
        jPanel11 = new javax.swing.JPanel();
        jCheckBox13 = new javax.swing.JCheckBox();
        jCheckBox3 = new javax.swing.JCheckBox();
        jCheckBox6 = new javax.swing.JCheckBox();
        jCheckBox7 = new javax.swing.JCheckBox();
        jCheckBox8 = new javax.swing.JCheckBox();
        jCheckBox9 = new javax.swing.JCheckBox();
        jCheckBox10 = new javax.swing.JCheckBox();
        jCheckBox11 = new javax.swing.JCheckBox();
        jCheckBox12 = new javax.swing.JCheckBox();
        jPanel12 = new javax.swing.JPanel();
        jPanel39 = new javax.swing.JPanel();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jPanel40 = new javax.swing.JPanel();
        jTextField7 = new javax.swing.JTextField();
        jTextField8 = new javax.swing.JTextField();
        jTextField9 = new javax.swing.JTextField();
        jTextField10 = new javax.swing.JTextField();
        jTextField11 = new javax.swing.JTextField();
        jTextField13 = new javax.swing.JTextField();
        jTextField12 = new javax.swing.JTextField();
        jPanel13 = new javax.swing.JPanel();
        jLabel12 = new javax.swing.JLabel();
        jCB_ObjectEditor3 = new com.jdcm.editors.JCB_ObjectEditor();
        jLabel10 = new javax.swing.JLabel();
        jTextField3 = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        jTextField6 = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();
        jTextField4 = new javax.swing.JTextField();
        jPanel3 = new javax.swing.JPanel();
        jPanel14 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList<>();
        jPanel15 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jList2 = new javax.swing.JList<>();
        jPanel16 = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        jList3 = new javax.swing.JList<>();
        jPanel17 = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        jScrollPane4 = new javax.swing.JScrollPane();
        jList4 = new javax.swing.JList<>();
        jPanel2 = new javax.swing.JPanel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jCheckBox2 = new javax.swing.JCheckBox();
        jCheckBox4 = new javax.swing.JCheckBox();
        jCheckBox5 = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        jCheckBox14 = new javax.swing.JCheckBox();
        jPanel6 = new javax.swing.JPanel();
        jLabel31 = new javax.swing.JLabel();
        jTextField25 = new javax.swing.JTextField();
        jButton3 = new javax.swing.JButton();
        jLabel21 = new javax.swing.JLabel();
        jTextField16 = new javax.swing.JTextField();
        jButton2 = new javax.swing.JButton();
        jLabel22 = new javax.swing.JLabel();
        jComboBox1 = new javax.swing.JComboBox<>();
        jPanel18 = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jPanel19 = new javax.swing.JPanel();
        jLabel9 = new javax.swing.JLabel();
        jTextField2 = new javax.swing.JTextField();
        jPanel20 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jCB_ObjectEditor1 = new com.jdcm.editors.JCB_ObjectEditor();
        jLabel23 = new javax.swing.JLabel();
        jCB_ObjectEditor2 = new com.jdcm.editors.JCB_ObjectEditor();
        jPanel22 = new javax.swing.JPanel();
        jLabel28 = new javax.swing.JLabel();
        jTextField22 = new javax.swing.JTextField();
        jLabel25 = new javax.swing.JLabel();
        jTextField15 = new javax.swing.JTextField();
        jPanel23 = new javax.swing.JPanel();
        jLabel29 = new javax.swing.JLabel();
        jTextField23 = new javax.swing.JTextField();
        jLabel24 = new javax.swing.JLabel();
        jTextField24 = new javax.swing.JTextField();
        jPanel24 = new javax.swing.JPanel();
        jLabel27 = new javax.swing.JLabel();
        jTextField19 = new javax.swing.JTextField();
        jLabel20 = new javax.swing.JLabel();
        jTextField20 = new javax.swing.JTextField();
        jPanel21 = new javax.swing.JPanel();
        jLabel26 = new javax.swing.JLabel();
        jTextField21 = new javax.swing.JTextField();
        jPanel5 = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel7 = new javax.swing.JPanel();
        p_Texte1 = new com.jdcm.editors.P_Texte();
        jPanel10 = new javax.swing.JPanel();
        jPanel25 = new javax.swing.JPanel();
        jLabel36 = new javax.swing.JLabel();
        jScrollPane9 = new javax.swing.JScrollPane();
        jList9 = new javax.swing.JList<>();
        jPanel26 = new javax.swing.JPanel();
        jLabel32 = new javax.swing.JLabel();
        jScrollPane6 = new javax.swing.JScrollPane();
        jList6 = new javax.swing.JList<>();
        jPanel27 = new javax.swing.JPanel();
        jLabel33 = new javax.swing.JLabel();
        jScrollPane7 = new javax.swing.JScrollPane();
        jList7 = new javax.swing.JList<>();
        jPanel28 = new javax.swing.JPanel();
        jLabel34 = new javax.swing.JLabel();
        jScrollPane8 = new javax.swing.JScrollPane();
        jList8 = new javax.swing.JList<>();
        jPanel8 = new javax.swing.JPanel();
        jPanel9 = new javax.swing.JPanel();
        jPanel30 = new javax.swing.JPanel();
        jPanel36 = new javax.swing.JPanel();
        jLabel37 = new javax.swing.JLabel();
        jTextField27 = new javax.swing.JTextField();
        jPanel37 = new javax.swing.JPanel();
        jLabel38 = new javax.swing.JLabel();
        jTextField28 = new javax.swing.JTextField();
        jPanel38 = new javax.swing.JPanel();
        jLabel39 = new javax.swing.JLabel();
        jTextField29 = new javax.swing.JTextField();
        jPanel35 = new javax.swing.JPanel();
        jPanel33 = new javax.swing.JPanel();
        jLabel40 = new javax.swing.JLabel();
        jScrollPane10 = new javax.swing.JScrollPane();
        jList5 = new javax.swing.JList<>();
        jPanel32 = new javax.swing.JPanel();
        jLabel41 = new javax.swing.JLabel();
        jScrollPane11 = new javax.swing.JScrollPane();
        jList10 = new javax.swing.JList<>();
        jPanel31 = new javax.swing.JPanel();
        jLabel42 = new javax.swing.JLabel();
        jScrollPane12 = new javax.swing.JScrollPane();
        jList11 = new javax.swing.JList<>();
        jPanel29 = new javax.swing.JPanel();
        jLabel30 = new javax.swing.JLabel();
        jTextField17 = new javax.swing.JTextField();

        setTitle("SOCKET");
        setPreferredSize(new java.awt.Dimension(700, 300));

        jPanel34.setLayout(new javax.swing.BoxLayout(jPanel34, javax.swing.BoxLayout.LINE_AXIS));

        jLabel35.setText("Socket type : ");
        jPanel34.add(jLabel35);

        jTextField26.setEditable(false);
        jTextField26.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        jTextField26.setText("- type -");
        jPanel34.add(jTextField26);

        jButton1.setText("Close");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel34.add(jButton1);

        getContentPane().add(jPanel34, java.awt.BorderLayout.NORTH);

        jPanel4.setLayout(new java.awt.BorderLayout());

        jPanel11.setLayout(new javax.swing.BoxLayout(jPanel11, javax.swing.BoxLayout.Y_AXIS));

        jCheckBox13.setText("getKeepAlive");
        jCheckBox13.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox13.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox13);

        jCheckBox3.setText("getOOBInline");
        jCheckBox3.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox3.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox3);

        jCheckBox6.setText("isBound");
        jCheckBox6.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox6.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox6);

        jCheckBox7.setText("isClosed");
        jCheckBox7.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox7.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox7);

        jCheckBox8.setText("isConnected");
        jCheckBox8.setToolTipText("for UDP see java doc");
        jCheckBox8.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox8.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox8);

        jCheckBox9.setText("isInputShutdown");
        jCheckBox9.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox9.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox9);

        jCheckBox10.setText("isOutputShutdown");
        jCheckBox10.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox10.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox10);

        jCheckBox11.setText("getReuseAddress");
        jCheckBox11.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox11.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox11);

        jCheckBox12.setText("getTcpNoDelay");
        jCheckBox12.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox12.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel11.add(jCheckBox12);

        jPanel4.add(jPanel11, java.awt.BorderLayout.WEST);

        jPanel12.setLayout(new java.awt.BorderLayout());

        jPanel39.setLayout(new java.awt.GridLayout(7, 0));

        jLabel14.setText("getReceiveBufferSize");
        jPanel39.add(jLabel14);

        jLabel15.setText("getSendBufferSize");
        jPanel39.add(jLabel15);

        jLabel16.setText("getSoLinger");
        jPanel39.add(jLabel16);

        jLabel17.setText("getSoTimeout");
        jPanel39.add(jLabel17);

        jLabel18.setText("getTrafficClass");
        jPanel39.add(jLabel18);

        jLabel3.setText("Name : ");
        jPanel39.add(jLabel3);

        jLabel19.setText("hashCode");
        jPanel39.add(jLabel19);

        jPanel12.add(jPanel39, java.awt.BorderLayout.WEST);

        jPanel40.setLayout(new java.awt.GridLayout(7, 0));

        jTextField7.setText("-buff-");
        jPanel40.add(jTextField7);

        jTextField8.setText("-buff-");
        jPanel40.add(jTextField8);

        jTextField9.setText("-SOlinger-");
        jPanel40.add(jTextField9);

        jTextField10.setText("-SOTO-");
        jPanel40.add(jTextField10);

        jTextField11.setText("-traff class-");
        jPanel40.add(jTextField11);

        jTextField13.setText("-tostring-");
        jPanel40.add(jTextField13);

        jTextField12.setText("-hash-");
        jPanel40.add(jTextField12);

        jPanel12.add(jPanel40, java.awt.BorderLayout.CENTER);

        jPanel4.add(jPanel12, java.awt.BorderLayout.CENTER);

        jPanel13.setLayout(new java.awt.GridLayout(2, 0));

        jLabel12.setText("local host");
        jPanel13.add(jLabel12);

        jCB_ObjectEditor3.setEditable(true);
        jPanel13.add(jCB_ObjectEditor3);

        jLabel10.setText("gethost");
        jPanel13.add(jLabel10);

        jTextField3.setText("-adresse-");
        jPanel13.add(jTextField3);

        jLabel13.setText("local port");
        jPanel13.add(jLabel13);

        jTextField6.setText("-loc port-");
        jPanel13.add(jTextField6);

        jLabel11.setText("getPort");
        jPanel13.add(jLabel11);

        jTextField4.setText("-port-");
        jPanel13.add(jTextField4);

        jPanel4.add(jPanel13, java.awt.BorderLayout.SOUTH);

        jTabbedPane1.addTab("Socket", jPanel4);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        jPanel14.setLayout(new javax.swing.BoxLayout(jPanel14, javax.swing.BoxLayout.Y_AXIS));

        jLabel4.setText("getEnabledCipherSuites");
        jPanel14.add(jLabel4);

        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList1MouseReleased(evt);
            }
        });
        jScrollPane1.setViewportView(jList1);

        jPanel14.add(jScrollPane1);

        jPanel3.add(jPanel14);

        jPanel15.setLayout(new javax.swing.BoxLayout(jPanel15, javax.swing.BoxLayout.Y_AXIS));

        jLabel5.setText("getEnabledProtocols");
        jPanel15.add(jLabel5);

        jList2.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList2MouseReleased(evt);
            }
        });
        jScrollPane2.setViewportView(jList2);

        jPanel15.add(jScrollPane2);

        jPanel3.add(jPanel15);

        jPanel16.setLayout(new javax.swing.BoxLayout(jPanel16, javax.swing.BoxLayout.Y_AXIS));

        jLabel6.setText("SupportedCipherSuites");
        jPanel16.add(jLabel6);

        jList3.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList3MouseReleased(evt);
            }
        });
        jScrollPane3.setViewportView(jList3);

        jPanel16.add(jScrollPane3);

        jPanel3.add(jPanel16);

        jPanel17.setLayout(new javax.swing.BoxLayout(jPanel17, javax.swing.BoxLayout.Y_AXIS));

        jLabel7.setText("getSupportedProtocols");
        jPanel17.add(jLabel7);

        jList4.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList4MouseReleased(evt);
            }
        });
        jScrollPane4.setViewportView(jList4);

        jPanel17.add(jScrollPane4);

        jPanel3.add(jPanel17);

        jTabbedPane1.addTab("SSL Cap.", jPanel3);

        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.Y_AXIS));

        jCheckBox1.setText("getEnableSessionCreation");
        jCheckBox1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel2.add(jCheckBox1);

        jCheckBox2.setText("getNeedClientAuth");
        jCheckBox2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel2.add(jCheckBox2);

        jCheckBox4.setText("getUseClientMode");
        jCheckBox4.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox4.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel2.add(jCheckBox4);

        jCheckBox5.setText("getWantClientAuth");
        jCheckBox5.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox5.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jPanel2.add(jCheckBox5);

        jTabbedPane1.addTab("SSL Param.", jPanel2);

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.Y_AXIS));

        jCheckBox14.setText("isValid");
        jCheckBox14.setToolTipText("invalidate : Future connections will not be able to resume or join this session. However, any existing connection using this session can continue to use the session until the connection is closed.");
        jCheckBox14.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox14.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jCheckBox14.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox14ActionPerformed(evt);
            }
        });
        jPanel1.add(jCheckBox14);

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("Context"));
        jPanel6.setLayout(new javax.swing.BoxLayout(jPanel6, javax.swing.BoxLayout.LINE_AXIS));

        jLabel31.setText("Timeout (h)");
        jPanel6.add(jLabel31);

        jTextField25.setText("-getsession TO-");
        jTextField25.setMinimumSize(new java.awt.Dimension(20, 20));
        jPanel6.add(jTextField25);

        jButton3.setToolTipText("setSessionTimeout");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jPanel6.add(jButton3);

        jLabel21.setText("CacheSize");
        jPanel6.add(jLabel21);

        jTextField16.setText("-getSessionCacheSize-");
        jTextField16.setMinimumSize(new java.awt.Dimension(20, 20));
        jPanel6.add(jTextField16);

        jButton2.setToolTipText("setSessionCacheSize");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel6.add(jButton2);

        jLabel22.setText("getId");
        jPanel6.add(jLabel22);

        jComboBox1.setMaximumSize(new java.awt.Dimension(100, 50));
        jPanel6.add(jComboBox1);

        jPanel1.add(jPanel6);

        jPanel18.setLayout(new javax.swing.BoxLayout(jPanel18, javax.swing.BoxLayout.LINE_AXIS));

        jLabel8.setText("getCipherSuite : ");
        jPanel18.add(jLabel8);

        jTextField1.setText("-ciphersuite-");
        jPanel18.add(jTextField1);

        jPanel1.add(jPanel18);

        jPanel19.setLayout(new javax.swing.BoxLayout(jPanel19, javax.swing.BoxLayout.LINE_AXIS));

        jLabel9.setText("getPeerHost :");
        jPanel19.add(jLabel9);

        jTextField2.setText("-host-");
        jPanel19.add(jTextField2);

        jPanel1.add(jPanel19);

        jPanel20.setLayout(new javax.swing.BoxLayout(jPanel20, javax.swing.BoxLayout.LINE_AXIS));

        jLabel2.setText("getCreationTime");
        jPanel20.add(jLabel2);
        jPanel20.add(jCB_ObjectEditor1);

        jLabel23.setText("getLastAccessedTime");
        jPanel20.add(jLabel23);
        jPanel20.add(jCB_ObjectEditor2);

        jPanel1.add(jPanel20);

        jPanel22.setLayout(new javax.swing.BoxLayout(jPanel22, javax.swing.BoxLayout.LINE_AXIS));

        jLabel28.setText("getPeerPrincipal");
        jPanel22.add(jLabel28);

        jTextField22.setText("-Port-");
        jPanel22.add(jTextField22);

        jLabel25.setText("getPacketBufferSize");
        jPanel22.add(jLabel25);

        jTextField15.setText("-A buffer size-");
        jPanel22.add(jTextField15);

        jPanel1.add(jPanel22);

        jPanel23.setLayout(new javax.swing.BoxLayout(jPanel23, javax.swing.BoxLayout.LINE_AXIS));

        jLabel29.setText("getProtocol");
        jPanel23.add(jLabel29);

        jTextField23.setText("-principal name-");
        jPanel23.add(jTextField23);

        jLabel24.setText("getLocalPrincipal");
        jPanel23.add(jLabel24);

        jTextField24.setText("-protocol-");
        jPanel23.add(jTextField24);

        jPanel1.add(jPanel23);

        jPanel24.setLayout(new javax.swing.BoxLayout(jPanel24, javax.swing.BoxLayout.LINE_AXIS));

        jLabel27.setText("getPeerPort");
        jPanel24.add(jLabel27);

        jTextField19.setText("-principale name-");
        jPanel24.add(jTextField19);

        jLabel20.setText("getApplicationBufferSize");
        jPanel24.add(jLabel20);

        jTextField20.setText("-P buff size-");
        jPanel24.add(jTextField20);

        jPanel1.add(jPanel24);

        jPanel21.setLayout(new javax.swing.BoxLayout(jPanel21, javax.swing.BoxLayout.LINE_AXIS));

        jLabel26.setText("getPeerHost");
        jPanel21.add(jLabel26);

        jTextField21.setText("-host-");
        jPanel21.add(jTextField21);

        jPanel1.add(jPanel21);

        jTabbedPane1.addTab("SSL Session", jPanel1);

        jPanel5.setLayout(new java.awt.BorderLayout());

        jSplitPane1.setDividerLocation(350);

        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Certificat"));
        jPanel7.setLayout(new java.awt.BorderLayout());
        jPanel7.add(p_Texte1, java.awt.BorderLayout.CENTER);

        jSplitPane1.setRightComponent(jPanel7);

        jPanel10.setLayout(new javax.swing.BoxLayout(jPanel10, javax.swing.BoxLayout.LINE_AXIS));

        jPanel25.setLayout(new javax.swing.BoxLayout(jPanel25, javax.swing.BoxLayout.Y_AXIS));

        jLabel36.setText("getValue");
        jPanel25.add(jLabel36);

        jScrollPane9.setViewportView(jList9);

        jPanel25.add(jScrollPane9);

        jPanel10.add(jPanel25);

        jPanel26.setLayout(new javax.swing.BoxLayout(jPanel26, javax.swing.BoxLayout.Y_AXIS));

        jLabel32.setText("X509 chain getSigAlgName");
        jPanel26.add(jLabel32);

        jList6.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList6MouseReleased(evt);
            }
        });
        jScrollPane6.setViewportView(jList6);

        jPanel26.add(jScrollPane6);

        jPanel10.add(jPanel26);

        jPanel27.setLayout(new javax.swing.BoxLayout(jPanel27, javax.swing.BoxLayout.Y_AXIS));

        jLabel33.setText("getLocalCertificates");
        jPanel27.add(jLabel33);

        jList7.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList7MouseReleased(evt);
            }
        });
        jScrollPane7.setViewportView(jList7);

        jPanel27.add(jScrollPane7);

        jPanel10.add(jPanel27);

        jPanel28.setLayout(new javax.swing.BoxLayout(jPanel28, javax.swing.BoxLayout.Y_AXIS));

        jLabel34.setText("getPeerCertificates");
        jPanel28.add(jLabel34);

        jList8.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList8MouseReleased(evt);
            }
        });
        jScrollPane8.setViewportView(jList8);

        jPanel28.add(jScrollPane8);

        jPanel10.add(jPanel28);

        jSplitPane1.setLeftComponent(jPanel10);

        jPanel5.add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jTabbedPane1.addTab("SSL Certif.", jPanel5);

        jPanel8.setLayout(new java.awt.BorderLayout());

        jPanel9.setBorder(javax.swing.BorderFactory.createTitledBorder("Provider"));
        jPanel9.setLayout(new java.awt.BorderLayout());

        jPanel30.setLayout(new javax.swing.BoxLayout(jPanel30, javax.swing.BoxLayout.Y_AXIS));

        jPanel36.setLayout(new javax.swing.BoxLayout(jPanel36, javax.swing.BoxLayout.LINE_AXIS));

        jLabel37.setText("Name :");
        jPanel36.add(jLabel37);

        jTextField27.setText("-Name-");
        jPanel36.add(jTextField27);

        jPanel30.add(jPanel36);

        jPanel37.setLayout(new javax.swing.BoxLayout(jPanel37, javax.swing.BoxLayout.LINE_AXIS));

        jLabel38.setText("Info : ");
        jPanel37.add(jLabel38);

        jTextField28.setText("-Info-");
        jPanel37.add(jTextField28);

        jPanel30.add(jPanel37);

        jPanel38.setLayout(new javax.swing.BoxLayout(jPanel38, javax.swing.BoxLayout.LINE_AXIS));

        jLabel39.setText("Version  : ");
        jPanel38.add(jLabel39);

        jTextField29.setText("-Version-");
        jPanel38.add(jTextField29);

        jPanel30.add(jPanel38);

        jPanel9.add(jPanel30, java.awt.BorderLayout.NORTH);

        jPanel35.setLayout(new javax.swing.BoxLayout(jPanel35, javax.swing.BoxLayout.LINE_AXIS));

        jPanel33.setLayout(new javax.swing.BoxLayout(jPanel33, javax.swing.BoxLayout.Y_AXIS));

        jLabel40.setText("Values :");
        jPanel33.add(jLabel40);

        jList5.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList5MouseReleased(evt);
            }
        });
        jScrollPane10.setViewportView(jList5);

        jPanel33.add(jScrollPane10);

        jPanel35.add(jPanel33);

        jPanel32.setLayout(new javax.swing.BoxLayout(jPanel32, javax.swing.BoxLayout.Y_AXIS));

        jLabel41.setText("Services :");
        jPanel32.add(jLabel41);

        jList10.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList10MouseReleased(evt);
            }
        });
        jScrollPane11.setViewportView(jList10);

        jPanel32.add(jScrollPane11);

        jPanel35.add(jPanel32);

        jPanel31.setLayout(new javax.swing.BoxLayout(jPanel31, javax.swing.BoxLayout.Y_AXIS));

        jLabel42.setText("Sets :");
        jPanel31.add(jLabel42);

        jList11.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jList11MouseReleased(evt);
            }
        });
        jScrollPane12.setViewportView(jList11);

        jPanel31.add(jScrollPane12);

        jPanel35.add(jPanel31);

        jPanel9.add(jPanel35, java.awt.BorderLayout.CENTER);

        jPanel8.add(jPanel9, java.awt.BorderLayout.CENTER);

        jPanel29.setLayout(new javax.swing.BoxLayout(jPanel29, javax.swing.BoxLayout.LINE_AXIS));

        jLabel30.setText("getProtocol : ");
        jPanel29.add(jLabel30);

        jTextField17.setText("- getProtocol -");
        jPanel29.add(jTextField17);

        jPanel8.add(jPanel29, java.awt.BorderLayout.PAGE_START);

        jTabbedPane1.addTab("SSLContext", jPanel8);

        getContentPane().add(jTabbedPane1, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void jList1MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MouseReleased
        jList1.setToolTipText(jList1.getSelectedValue().toString());
    }//GEN-LAST:event_jList1MouseReleased

    private void jList2MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList2MouseReleased
        jList2.setToolTipText(jList2.getSelectedValue().toString());
    }//GEN-LAST:event_jList2MouseReleased

    private void jList3MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList3MouseReleased
        jList3.setToolTipText(jList3.getSelectedValue().toString());
    }//GEN-LAST:event_jList3MouseReleased

    private void jList4MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList4MouseReleased
        jList4.setToolTipText(jList4.getSelectedValue().toString());
    }//GEN-LAST:event_jList4MouseReleased

    private void jList5MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList5MouseReleased
        jList5.setToolTipText(jList5.getSelectedValue().toString());
    }//GEN-LAST:event_jList5MouseReleased

    private void jList10MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList10MouseReleased
        jList10.setToolTipText(jList10.getSelectedValue().toString());
    }//GEN-LAST:event_jList10MouseReleased

    private void jList11MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList11MouseReleased
        jList11.setToolTipText(jList11.getSelectedValue().toString());
    }//GEN-LAST:event_jList11MouseReleased

    private void jList8MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList8MouseReleased
        showcert(jList8.getSelectedValue());
    }//GEN-LAST:event_jList8MouseReleased

    private void jList7MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList7MouseReleased
        showcert(jList7.getSelectedValue());
    }//GEN-LAST:event_jList7MouseReleased

    private void jList6MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList6MouseReleased
        showcert(jList6.getSelectedValue());
    }//GEN-LAST:event_jList6MouseReleased

    private void jCheckBox14ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox14ActionPerformed
        if (jCheckBox14.isSelected() && sock instanceof SSLSocket) {
            SSLSocket SSLsock = (SSLSocket) sock;
            if (SSLsock.getSession() != null) {
                SSLsock.getSession().invalidate();
                jCheckBox14.setEnabled(false);
            }

        }
    }//GEN-LAST:event_jCheckBox14ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        if (sock instanceof SSLSocket) {
            SSLSocket SSLsock = (SSLSocket) sock;
            SSLsock.getSession().getSessionContext().setSessionTimeout(3600 * Integer.parseInt(this.jTextField25.getText()));
        }
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        SSLSocket SSLsock = (SSLSocket) sock;
        SSLsock.getSession().getSessionContext().setSessionCacheSize(Integer.parseInt(this.jTextField16.getText()));
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        this.setVisible(false);
    }//GEN-LAST:event_jButton1ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private com.jdcm.editors.JCB_ObjectEditor jCB_ObjectEditor1;
    private com.jdcm.editors.JCB_ObjectEditor jCB_ObjectEditor2;
    private com.jdcm.editors.JCB_ObjectEditor jCB_ObjectEditor3;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBox10;
    private javax.swing.JCheckBox jCheckBox11;
    private javax.swing.JCheckBox jCheckBox12;
    private javax.swing.JCheckBox jCheckBox13;
    private javax.swing.JCheckBox jCheckBox14;
    private javax.swing.JCheckBox jCheckBox2;
    private javax.swing.JCheckBox jCheckBox3;
    private javax.swing.JCheckBox jCheckBox4;
    private javax.swing.JCheckBox jCheckBox5;
    private javax.swing.JCheckBox jCheckBox6;
    private javax.swing.JCheckBox jCheckBox7;
    private javax.swing.JCheckBox jCheckBox8;
    private javax.swing.JCheckBox jCheckBox9;
    private javax.swing.JComboBox<String> jComboBox1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel23;
    private javax.swing.JLabel jLabel24;
    private javax.swing.JLabel jLabel25;
    private javax.swing.JLabel jLabel26;
    private javax.swing.JLabel jLabel27;
    private javax.swing.JLabel jLabel28;
    private javax.swing.JLabel jLabel29;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel30;
    private javax.swing.JLabel jLabel31;
    private javax.swing.JLabel jLabel32;
    private javax.swing.JLabel jLabel33;
    private javax.swing.JLabel jLabel34;
    private javax.swing.JLabel jLabel35;
    private javax.swing.JLabel jLabel36;
    private javax.swing.JLabel jLabel37;
    private javax.swing.JLabel jLabel38;
    private javax.swing.JLabel jLabel39;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel40;
    private javax.swing.JLabel jLabel41;
    private javax.swing.JLabel jLabel42;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JList<String> jList1;
    private javax.swing.JList<String> jList10;
    private javax.swing.JList<String> jList11;
    private javax.swing.JList<String> jList2;
    private javax.swing.JList<String> jList3;
    private javax.swing.JList<String> jList4;
    private javax.swing.JList<String> jList5;
    private javax.swing.JList<javax.security.cert.X509Certificate> jList6;
    private javax.swing.JList<java.security.cert.Certificate> jList7;
    private javax.swing.JList<java.security.cert.Certificate> jList8;
    private javax.swing.JList<String> jList9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel13;
    private javax.swing.JPanel jPanel14;
    private javax.swing.JPanel jPanel15;
    private javax.swing.JPanel jPanel16;
    private javax.swing.JPanel jPanel17;
    private javax.swing.JPanel jPanel18;
    private javax.swing.JPanel jPanel19;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel20;
    private javax.swing.JPanel jPanel21;
    private javax.swing.JPanel jPanel22;
    private javax.swing.JPanel jPanel23;
    private javax.swing.JPanel jPanel24;
    private javax.swing.JPanel jPanel25;
    private javax.swing.JPanel jPanel26;
    private javax.swing.JPanel jPanel27;
    private javax.swing.JPanel jPanel28;
    private javax.swing.JPanel jPanel29;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel30;
    private javax.swing.JPanel jPanel31;
    private javax.swing.JPanel jPanel32;
    private javax.swing.JPanel jPanel33;
    private javax.swing.JPanel jPanel34;
    private javax.swing.JPanel jPanel35;
    private javax.swing.JPanel jPanel36;
    private javax.swing.JPanel jPanel37;
    private javax.swing.JPanel jPanel38;
    private javax.swing.JPanel jPanel39;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel40;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane10;
    private javax.swing.JScrollPane jScrollPane11;
    private javax.swing.JScrollPane jScrollPane12;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane6;
    private javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JScrollPane jScrollPane8;
    private javax.swing.JScrollPane jScrollPane9;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField10;
    private javax.swing.JTextField jTextField11;
    private javax.swing.JTextField jTextField12;
    private javax.swing.JTextField jTextField13;
    private javax.swing.JTextField jTextField15;
    private javax.swing.JTextField jTextField16;
    private javax.swing.JTextField jTextField17;
    private javax.swing.JTextField jTextField19;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JTextField jTextField20;
    private javax.swing.JTextField jTextField21;
    private javax.swing.JTextField jTextField22;
    private javax.swing.JTextField jTextField23;
    private javax.swing.JTextField jTextField24;
    private javax.swing.JTextField jTextField25;
    private javax.swing.JTextField jTextField26;
    private javax.swing.JTextField jTextField27;
    private javax.swing.JTextField jTextField28;
    private javax.swing.JTextField jTextField29;
    private javax.swing.JTextField jTextField3;
    private javax.swing.JTextField jTextField4;
    private javax.swing.JTextField jTextField6;
    private javax.swing.JTextField jTextField7;
    private javax.swing.JTextField jTextField8;
    private javax.swing.JTextField jTextField9;
    private com.jdcm.editors.P_Texte p_Texte1;
    // End of variables declaration//GEN-END:variables
}
