package com.jdcm.eltrx;

import com.jdcm.common.A_Commande;
import com.jdcm.common.D_Utils;
import com.jdcm.common.C_Eventdef;
import com.jdcm.common.C_Actiondef;
import com.jdcm.common.Options_Model;
import com.jdcm.common.XML2tree;
import com.jdcm.common.XMLres;
import com.jdcm.diagram.A_Events;
import com.jdcm.diagram.A_Task;
import java.io.FileNotFoundException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import javax.swing.tree.DefaultTreeModel;

/**
 * Thread that takes in charge the CONF script file an run it .
 *
 * @author Marc GIBOIN
 */
public class T_CONF extends Thread implements I_Command {

    private final D_Debug TDeb;
    private final D_Utils TUtils = new D_Utils();
    private final C_Main Parent;
    private final ArrayList<C_Actiondef> actiondef = new ArrayList<>();
    private long lastModifDartFile = 0;
    private String status = "new";
    private com.jdcm.diagram.A_Events events = null;
    private Options_Model option;

    public T_CONF(C_Main main, com.jdcm.diagram.A_Events config) {
        Parent = main;
        events = config;
        TDeb = new D_Debug("T-Conf");
        TDeb.setconsole(Parent);
        setName(this.getClass().getSimpleName() + ":" + getName());
        initOptions();
    }

    public String getStatus() {
        return status;
    }

    /**
     * initialise Options for the Stacks
     */
    private void initOptions() {
        option = new Options_Model(this);
        // create & feel defaults NAME/VAL (&DEFAULT)/TYPE/DESC/USED/NULLONEMPTY/COMMENT
        option.addOption("Conf_Stat", false, "bool", "false", false, false, "is Conf Statistics collected/displayed");
        option.setTitle(this.getTitle());
    }

    /**
     * not used
     *
     * @param bool
     */
    @A_Commande(help = "set Conf logs enabled & displayed")
    public void setStack_Stat(boolean bool) {
        if (option.getValuefor("Conf_Stat", true) != bool) {
            option.setValuefor("Conf_Stat", bool);
        }

    }

    /**
     * set Pause to the config, if running then actions will have no effect on
     * states
     *
     * @param bool true if Pause is set
     */
    public void setPause(boolean bool) {
        if (status.endsWith("new")) {
            return;
        }
        if (status.endsWith("run") && bool) {
            status = "pause";
        } else {
            status = "waitrun";
        }
    }

    /**
     * the title of the Config for Matrix title/Diagram frame...
     *
     * @return the title string
     */
    public String getTitle() {
        if (events != null) {
            return "Conf: " + events.getTitle();
        } else {
            return "No_Config";
        }
    }

    public int setConfig(java.io.File f) {
        try {
            return setConfig(TUtils.getFileStream(f));
        } catch (Exception ex) {
//            System.out.println("exit 1 " + f + ex.getMessage());
            TDeb.wri("while reading " + f.getName() + "=>" + ex.getMessage());
            return -1;
        }
    }

    public int setConfig(java.io.InputStream xmlFileStr) {
        int res = -1;
        if (xmlFileStr == null) {
            TDeb.wri("T_CONF.setConfig(): stream: " + xmlFileStr);
            return res;
        }
        try {
            events = new com.jdcm.diagram.A_Events(xmlFileStr);
            option.setTitle(getTitle());
            res = 0;
        } catch (Exception ex) {
            TDeb.wri("T_CONF.setConfig(): Pb during config building from:" + xmlFileStr + "=>" + ex.getMessage());
            events = new com.jdcm.diagram.A_Events(Parent);
            res = -1;
        }
        return res;
    }

    public A_Events getEvents() {
        return events;
    }

    public void clear() {
        if (events != null) {
            events.reset();
        }
        option.setTitle(getTitle());
        status = "clear";//new
        Parent.sem.drainPermits();
    }

    public int checkforActions(Object src, Object value) {
        java.lang.StackTraceElement[] traces = Thread.currentThread().getStackTrace();
        //!!!! function is trace[3] because 0 thread, 1 this, !2 Arcv! , 3 the function
        com.jdcm.common.C_Eventdef evt;
        java.lang.reflect.Method meth;
        //if element added and status is "waitrun" stopping Main_CX (and not T_CONF !!!!!)
        if (traces.length >= 3) {
            try {
                meth = src.getClass().getMethod(traces[3].getMethodName(), value.getClass());
            } catch (NoSuchMethodException | SecurityException ex) {
                TDeb.wri("Pb no method for :" + src + "/" + value + "=>" + ex);
                return -1;
            }
            try {
                evt = new com.jdcm.common.C_Eventdef(src, meth, value);
            } catch (Exception ex) {
                TDeb.wri("Pb creating event :" + src + "/" + value + "=>" + ex);
                return -1;
            }
            if (addrunelts(evt) > 0) {
                return 0;
            } else {
                // to much debug...
                //TDeb.wri("no actions for event:" + evt);
                return -1;
            }
        } else {
            TDeb.wri("Pb trace length  :" + traces.length);
            return -1;
        }
    }

    public int addrunelts(C_Eventdef evt) {
        //the source should wait until action is finish
        if (evt == null) {
            TDeb.wri("null Event");
            return -1;
        } else if (status.endsWith("run") == false) {
            TDeb.wri("Status:" + status + " is not OK for add" + evt);
            return -1;
        } else if (events == null) {
            TDeb.wri("Matrix is null" + evt);
            return -1;
        } else {
            // TDeb.wri("adding Asking on : " + evt.toString());
            ArrayList<C_Actiondef> list = events.getTask(evt);
            if (list == null || list.isEmpty()) {
                return 0;
            } else {
                TDeb.wri("adding task(s):" + list);
                actiondef.addAll(list);
                return list.size();
            }
        }
    }

    private java.util.List<I_Command> findPorts(String name) {
        java.util.List<I_Command> res = new java.util.ArrayList<>();
        java.util.List<I_Command> r = Parent.getCMDChilds(name);
        if (r.isEmpty()) {
            if (Parent.getCMDName().compareTo(name) == 0) {
                res.add(Parent);
            } else if (name.endsWith("/X")) {
                r = Parent.getCMDChilds(name.substring(0, name.length() - 2));
                for (I_Command port : r) {
                    if (port instanceof C_Port
                            && ((C_Port) port).getModeName().startsWith("IPL4")
                            && ((C_Port) port).getTCP_Mode().startsWith("TCP_L")
                            && ((C_Port) port).getSlave() instanceof T_IP) {
                        res.addAll(((T_IP) ((C_Port) port).getSlave()).getTCPclients());
                    }
                }
                if (r.isEmpty() == false && res.isEmpty()) {
                    // for DEST/X
                    res.addAll(r);
                }
            }
        } else {
            res.addAll(r);
        }
        return res;
    }

    /**
     * Manage a Dart File that do Action or Task modification during the runing
     * step
     *
     * @param f the file that contains the dart
     */
    private void doDartFile(java.io.File f) {
        if (f == null || !f.canRead()) {
            return;
        }
        java.io.InputStream filestr = null;
        try {
            filestr = new java.io.FileInputStream(f);
        } catch (FileNotFoundException ex) {
            TDeb.wri("Dart stream is null for file : " + f.toString());
        }
        XML2tree t = new XML2tree();
        DefaultTreeModel tm = t.XMLBase(filestr);
        javax.swing.tree.DefaultMutableTreeNode n = (javax.swing.tree.DefaultMutableTreeNode) tm.getRoot();
        while (n != null && n.toString().compareTo("dart") != 0) {
            n = n.getNextNode();
        }
        if (n != null) {
            for (int i = 0; i < n.getChildCount(); i++) {
                javax.swing.tree.DefaultMutableTreeNode acttsknd = (javax.swing.tree.DefaultMutableTreeNode) n.getChildAt(i);
                if (acttsknd.toString().compareTo("C_Actiondef") == 0) {
                    XMLres xmlact = (XMLres) acttsknd.getUserObject();
                    actiondef.add(events.getActionFor(xmlact));
                } else if (acttsknd.toString().compareTo("task") == 0) {
                    XMLres tsk = (XMLres) acttsknd.getUserObject();
                    A_Task task = null;
                    try {
                        int evt = (int) tsk.getAttrVal("event");
                        int st = (int) tsk.getAttrVal("state");
                        task = events.getTasklist()[evt][st];
                    } catch (Exception ex) {
                        TDeb.wri("Dart no task for : " + tsk.toXMLString());
                    }
                    if (task != null) {
                        for (int j = 0; j < acttsknd.getChildCount(); j++) {
                            javax.swing.tree.DefaultMutableTreeNode nodact = (javax.swing.tree.DefaultMutableTreeNode) acttsknd.getChildAt(j);
                            if (nodact.toString().compareTo("C_Actiondef") == 0) {
                                XMLres xmlact = (XMLres) nodact.getUserObject();
                                C_Actiondef act = events.getActionFor(xmlact);
                                int idx = -1;
                                boolean found = false;
                                for (C_Actiondef tskact : task.getActions()) {
                                    idx++;
                                    if (tskact.getDest().equals(act.getDest())
                                            && tskact.getMethodName().compareTo(act.getMethodName()) == 0) {
                                        task.getActions().set(idx, act);
                                        found = true;
                                    }
                                }
                                if (!found) {
                                    task.getActions().add(act);
                                }
                            }
                        }
                    }
                }
            }
            try {
                doActions(actiondef);
            } catch (Exception ex) {
                TDeb.wri("Dart is not apply for file : " + f.toString());
            }
        }
    }

    private C_Actiondef findActionDest(C_Actiondef action) {
        if (action.getDest() instanceof String) {
            String oldName = action.getDest().toString();
            if (oldName.contains("/SRC") && action.getSource() instanceof C_Eventdef) {
                if (((C_Eventdef) action.getSource()).getSrc() instanceof I_Port) {
                    String name;
                    if ((C_Eventdef) action.getSource() instanceof I_Port) {
                        name = ((I_Port) ((C_Eventdef) action.getSource()).getSrc()).getPortName();
                    } else {
                        name = ((C_Eventdef) action.getSource()).getSrc().toString();
                    }
                    if (name.indexOf("_") > 0) {
                        action.setDest(action.getDest().toString().replace("/SRC", name.substring(name.indexOf("_"))));
                    }
                    TDeb.wri("Port labeled: " + oldName + " changed to " + action.getDest());
                }
            }
        }
        return action;
    }

    private void doActions(java.util.ArrayList<C_Actiondef> actions) throws Exception {
        while (actions != null && actions.isEmpty() == false) {
            if (status.compareTo("run") != 0 && status.compareTo("new") != 0) {
                return;
            }
            String cmds = "/start/stop/open/connect/end/";
            C_Actiondef act_init = actions.remove(0);
            C_Actiondef act = act_init.clone();
            TDeb.wri("will do action:" + act);
            java.lang.reflect.Method methode = act.getMethod();
            java.util.List<Object> dests = new ArrayList<>();
            if (act.getDest() instanceof String) {
                act = findActionDest(act);
                dests.addAll(findPorts(act.getDest().toString()));
            } else {
                dests.add(act.getDest());
            }
            // if Parent.getM verif matrice conexion pour applique l'ordre...
            if (dests.isEmpty()) {
                if (act.getDest() != null) {
                    TDeb.wri("Element not found: " + act.getDest());
                    //could be albe to stop if element is not found
                    throw new Exception("Element not found: " + act.getDest());
                } else {
                    throw new Exception("syntaxe Error : no target on:" + act);
                }
            } else if (methode == null && act.getMethodName() != null) {
                String methName = act.getMethodName();
                if (cmds.contains("/" + act.getMethodName() + "/") && dests.get(0) instanceof I_Port) {
                    methName = "CX" + methName;
                }
                if (act.getParamList() != null && act.getParamList().isEmpty() == false) {
                    java.util.List<Class<?>> cls = new java.util.ArrayList<>();
                    for (Object obj : act.getParamList()) {
                        cls.add(obj.getClass());
                    }
                    methode = TUtils.getMethod(dests.get(0).getClass(), methName, cls.toArray(new Class<?>[0]));
                } else {
                    methode = TUtils.getMethod(dests.get(0).getClass(), methName, new Class<?>[0]);
                }

            }
            if (dests.isEmpty()) {
            } else if (methode == null) {
                throw new Exception("Methode not found: " + act);
            } else {
                for (Object dest : dests) {
                    try {
                        //System.out.println("do action:" + dest + "." + methode + "(" + act.getParamList() + ")");
                        TDeb.wri("do action:" + dest + "." + methode + "(" + act.getParamList() + ")");
                        if (act.getParamList() != null && act.getParamList().isEmpty() == false) {
                            for (int idx = 0; idx < act.getParamList().size(); idx++) {
                                Object obj = act.getParamList().get(idx);
                                if (obj != null && obj.toString().contains("/SRC")) {
                                    if ((C_Eventdef) act.getSource() != null && ((C_Eventdef) act.getSource()).getSrc() instanceof I_Port) {
                                        String name = ((I_Port) ((C_Eventdef) act.getSource()).getSrc()).getPortName();
                                        act.getParamList().set(idx, obj.toString().replace("/SRC", name.substring(name.indexOf("_"))));
                                        TDeb.wri("Parametre " + obj + "changed to " + act.getParamList().get(idx));
                                    }
                                }
                            }
                            methode.invoke(dest, act.getParamList().toArray());
                        } else {
                            methode.invoke(dest, new Object[0]);
                        }
                    } catch (IllegalAccessException | IllegalArgumentException ex) {
                        // should comes form a bug inside the function : port unreachable
                        throw new Exception("CONF acc: " + act.getDest() + "." + methode.getName() + "=>" + ex.getMessage());
                        //throw new Exception("Could not invoke: " + dest + "." + methode + "(" + act.getParamList() + ")");
                    } catch (InvocationTargetException ex) {
                        throw new Exception("CONF_fnct: " + act.getDest() + "." + methode.getName() + "=>" + ex.getMessage());
                    }
                }
            }
        }
    }

    @Override
    public javax.swing.ImageIcon getCMDIcon() {
        return ((new com.jdcm.common.D_Utils()).getIcon("event.gif"));
    }

    @Override
    public Options_Model getCMDOptions() {
        return option;
    }

    @Override
    public String getCMDdetails() {
        String res = "";
        res
                = res + "Name : " + this.getCMDName() + " Enabled : " + Parent.isConf_Enabled() + " Size :" + events.toString() + "\r\n";
        return res;
    }

    @Override
    public java.util.List<I_Command> getCMDChilds(String child) {
        return new java.util.ArrayList<>();
    }

    @Override
    public String getCMDName() {
        return "Conf";
    }

    @Override
    public I_Command getCMDParent() {
        if (Parent instanceof I_Command) {
            return (I_Command) Parent;
        }

        return null;
    }

    @Override
    public String getCMDdesc() {
        return getCMDName() + " \t" + this.getStatus() + "\t" + events;
    }

    @Override
    public void run() {
        TDeb.wri("Starting CONF");
        status = "new";
        if (status.compareTo("new") == 0) {
            try {
                doActions(events.getTask("Init"));
                doActions(events.getTask("Start"));
                Parent.sem.release();
                doActions(events.getTask("Run"));
                if (status.startsWith("clear") == false) {
                    status = "waitrun";
                }
            } catch (Exception ex) {
                Parent.sem.release();
                TDeb.wri("abnormal CONFIG end => closing 1 T_CONF " + ex.getMessage());
            }
        }
        while ((status.endsWith("run") || status.compareTo("pause") == 0) && status.compareTo("clear") != 0) {
            try {
                if (actiondef != null && actiondef.isEmpty() == false && status.compareTo("new") != 0) {
                    status = "run";
                }
                while (status.compareTo("pause") == 0) {
                    sleep(10);
                }
                sleep(10);
                java.io.File f = TUtils.getFile(events.getTitle() + ".xml", null);
                if (f != null && f.canRead() && f.lastModified() > lastModifDartFile) {
                    TDeb.wri("a Dart is detected : " + f);
                    lastModifDartFile = f.lastModified();
                    doDartFile(f);
                }
                if (status.compareTo("run") == 0 && actiondef != null) {
                    if (!actiondef.isEmpty()) {
                        TDeb.wri("try : to do " + actiondef.get(0) + " and followings [" + actiondef.size() + "]");
                        doActions(actiondef);
                    }
                    if (actiondef.isEmpty()) {
                        TDeb.wri("actions list is empty, sem will real. :" + Parent.sem.availablePermits());
                        //CARE : Parent.sem.drainPermits(); DO NOT FREE the Tread
                        //     Parent.sem.release();
                    }
                    if (status.compareTo("clear") != 0) {
                        status = "waitrun";
                    }
                }
            } catch (java.lang.InterruptedException ex) {
                Parent.sem.release();
                TDeb.wri("abnormal CONFIG end => closing 2 T_CONF " + ex.getMessage());
            } catch (Exception ex) {
                if (Parent.isConf_ErrExit()) {
                    //   ex.printStackTrace();
                    Parent.sem.release();
                    TDeb.wri("abnormal CONFIG end => closing 3 T_CONF " + ex.getMessage());
                }
            }
        }

        TDeb.wri("END CONFIG");
        if (Parent.isConf_Enabled()) {
            Parent.setConf_Enabled(false);
        }
    }
}
