package com.jdcm.eltrx;

import com.jdcm.analyse.Code_Model;
import java.io.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.List;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import com.jdcm.common.*;

/**
 * T_DB.java Created on 28 decembre 2006, 13:41
 *
 * Slave for DB Port : manage databases Input/output
 *
 * To obtain JDBC DRIVERS http://www.minq.se/products/dbvis/drivers.html#derby
 *
 * TODO : SQL statement timeOut for 'big' resultSet waited...
 *
 * @author Marc GIBOIN
 */
public class T_DB extends Thread implements I_Slave {

    private final C_Port Parent;
    private final D_Debug TDeb;
    private final D_Utils TUtils = new D_Utils();
    private final D_DBUtils dbutils = new D_DBUtils();

    private Connection cxDB = null;
    private List<C_Packet> totreat = null;
    private F_DBConnexion FDBcx;
    private DefaultMutableTreeNode map_act = null;
    //try to load driver by jar but...
    java.sql.Driver dr = null;

    /**
     * Creates a new instance of T_File
     *
     * @param p the port using this thread as in/out/CTRL
     */
    public T_DB(C_Port p) {
        Parent = p;
        TDeb = new D_Debug("T-DB [" + Parent.getIndex() + "] : ");
        TDeb.setconsole(Parent.getConsole());
        totreat = new java.util.ArrayList<>();
        dbutils.setPort(p);
        setName(this.getClass().getSimpleName() + ":" + getName());
    }

    /**
     * Start the slave<br> - check DB drivers is OK
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        Class<?> driver = Parent.getDB_driver();
        if (driver == null) {
            return -1;
        }
        try {
            dr = (java.sql.Driver) driver.newInstance();
        } catch (InstantiationException e) {
            Parent.CXerror("Can not create an instance of JDBC Driver :" + driver);
            return -1;
        } catch (IllegalAccessException e) {
            Parent.CXerror("Can not access to JDBC Driver :" + driver);
            return -1;
        }
        TDeb.wri("BD started with : " + driver);
        return 0;
    }

    /**
     * Open the slave<br> - Open DB with drivers
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        // if not found=> long(qq min)=> should be lanched as thread but semaphores....
        try {
            if (dr.acceptsURL(Parent.getDB_url()) == false) {
                Parent.CXerror("URL refused by driver: " + Parent.getDB_url());
                return -1;
            }
        } catch (SQLException e) {
            TDeb.wri("error testing: " + Parent.getDB_url() + " with :" + dr.toString() + "=>" + e.getMessage());
        }
        // !!!!! PB : DriverManager with loaded driver is not working 
        try {
            DriverManager.registerDriver(dr);
        } catch (SQLException e) {
            Parent.CXerror("Error registring driver :" + dr.toString());
            return -1;
        }
        java.util.Properties propsql = new java.util.Properties();
        propsql.put("charSet", Parent.getDB_charSet().displayName());
        if (Parent.getDB_user() != null && Parent.getDB_user().length() > 0) {
            propsql.setProperty("user", Parent.getDB_user());
            propsql.setProperty("password", Parent.getDB_pwd());
        }
        try {
            TDeb.wri("BD try oppen with : " + Parent.getDB_url() + " / " + propsql.toString());
            cxDB = DriverManager.getConnection(Parent.getDB_url(), propsql);

        } catch (SQLException tr) {
            try {
                TDeb.wri("BD try oppen with : " + Parent.getDB_url() + " width " + Parent.getDB_user() + " / " + Parent.getDB_pwd() + "=>" + tr.getMessage());
                cxDB = DriverManager.getConnection(Parent.getDB_url(), Parent.getDB_user(), Parent.getDB_pwd());
            } catch (SQLException exe) {
                try {
                    TDeb.wri("BD try oppen with : " + Parent.getDB_url() + " (only)" + " => " + exe.getMessage());
                    cxDB = DriverManager.getConnection(Parent.getDB_url());
                } catch (SQLException exee) {
                    Parent.CXerror("Can not connect to : " + Parent.getDB_url() + " => " + exee.getMessage());
                    return -1;
                }
            }
        }
        java.io.PrintWriter logs = null;
        if (Parent.getConsole().isdbgselected("T-DB")) {
            logs = new java.io.PrintWriter(System.out);
        }
        DriverManager.setLogWriter(logs);
        if (cxDB == null) {
            TDeb.wri("JCDB Connection is NULL");
            return -1;
        } else {
            dbutils.setCX(cxDB);
        }

        return 0;
    }

    /**
     * Close the DB <br> - oppened frames<br> - connections <br> - clearing
     * waiting orders
     *
     * @return 0 if DB session is closed correctly
     */
    @Override
    public int sl_close() {
        TDeb.wri("close the slave");
        if (FDBcx != null) {
            FDBcx.setVisible(false);
            FDBcx = null;
        }
        if (totreat != null) {
            totreat.clear();
            totreat = null;
            Parent.setAcc_initsize(-1);
        }
        if (cxDB != null) {
            try {
                cxDB.close();
            } catch (SQLException e) {
                TDeb.wri("Can not close DB connection=>" + e.getMessage());
            }
        }
        cxDB = null;
        dr = null;
        if (Parent.getcxstatus().compareTo("connected") == 0) {
            TDeb.wri("Asking to close the thread");
            Parent.CXstop();
        }
        return 0;
    }

    /**
     *
     * @return the connexion used (made) by this thread
     */
    public Connection getCX() {
        return cxDB;
    }

    public String getSchema() {
        if (cxDB == null) {
            return null;
        }
        try {
            return cxDB.getSchema();
            //  if (txt!=null){
            // txt;
            //  }

        } catch (SQLException ex) {
            try {
                return cxDB.getCatalog();
            } catch (SQLException e) {
                return null;
            }
        }

    }

    /**
     * Send the CX_packet cotents on the DB session <br> byte[]=> SQL /
     * model=>tables / ...
     *
     * @param d the Packet to write in the file
     * @return the length of the writed data
     */
    @Override
    public int write(C_Packet d) {
        // set the packet in a buffer "totreat" and wait until the thread treat it as FIFO
        if (Parent.isslreading()) {
            totreat.add(d);
            Parent.setAcc_position(totreat.size());
        }
        if (d.getsource() != null && d.getsource().isSystem() == false) {
            TDeb.wri("added: " + d.toString() + " to be treated as Number : " + totreat.size());
        }
        return 0;
    }

    public int Trt_in(String sqlreq) {
        C_Packet p = new C_Packet(sqlreq, Parent);
        p = Trt_in(p);
        if (p != null) {
            return p.getSQL_upd();
        } else {
            return -1;
        }
    }

    private C_Packet Trt_in(C_Packet p) {
        //DELETE/UPDATE/INSERT...
        String sqlreq = (String) p.getObj();
        C_Packet res = new C_Packet("", Parent);
        int nbupd;
        if (cxDB == null) {
            return null;
        }//error
        Statement request = null;
        long start = 0;
        long stop = 0;
        try {
            request = cxDB.createStatement();
            start = System.currentTimeMillis();
            nbupd = request.executeUpdate(sqlreq);
            stop = System.currentTimeMillis();
            request.close();
        } catch (SQLException e) {
            TDeb.wri("Pb with SQL request : " + sqlreq + "=>" + e.getMessage());
            if (request != null) {
                try {
                    request.close();
                } catch (SQLException ex) {
                    TDeb.wri("Pb with .close() =>" + ex.getMessage());
                }
            }
            res.setSQL(sqlreq, new java.util.Date(start), new java.util.Date(stop));
            res.setObj("#!SQL Ex =>" + e.getMessage());
            if (Parent.isDB_Errors()) {
                return res;
            } else {
                return null;
            }
        }
        res.setSQL(sqlreq, new java.util.Date(start), new java.util.Date(stop));
        res.setSQL_upd(nbupd);
        res.setObj("Updated : " + nbupd + " lines");
        //TDeb.wri("DB updated : "+nbupd+ " lines");
        return res;
    }

    /**
     * simple request maker
     *
     * @param sqlreq the sql request
     * @return the resultset (SELECT) or null (INSERT/UPDATE/...);
     */
    public ResultSet Trt_out(String sqlreq) {
        //OUT=>SELECT; unUsed DON'T MANAGE UPDATES
        //sqlreq = getstdQuery(sqlreq); // normalize to String
        C_Packet p = new C_Packet(sqlreq, Parent);
        p = Trt_out(p);
        if (p != null && p.getObj() instanceof ResultSet) {
            return (ResultSet) p.getObj();
        } else {
            return null;
        }
    }

    public C_Packet Trt_out(C_Packet d) {
        ///OUT => SELECT
        if (d.isEmpty()) {
            return null;
        }
        String sqlreq = (String) d.getObj();
        C_Packet res = new C_Packet("", Parent);
        if (cxDB == null) {
            return null;
        }//error
        Statement request = null;
        try {
            request = cxDB.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
            //setFetchSize should be better?!
            //http://java.developpez.com/faq/jdbc/?page=resultset
            if (request != null) {
                try {
                    request.setFetchSize(Parent.getDB_fetchnb());
                } catch (SQLException ex) {
                    TDeb.wri("Statement .setFetchSize(int rows) not implemented=>" + ex.getMessage());
                }
            }

            if (request != null) {
                try {
                    request.setMaxRows(Parent.getDB_maxrows());
                } catch (SQLException ex) {
                    TDeb.wri("Statement .setMaxRows(int) not implemented=>" + ex.getMessage());
                }
            }

            if (request != null) {
                try {
                    request.setQueryTimeout(Parent.getDB_TO());
                } catch (SQLException ex) {
                    TDeb.wri("Statement .setQueryTimeout(int) not implemented=>" + ex.getMessage());
                }
            }

        } catch (SQLException e) {
            TDeb.wri("Pb preparing SQL request : " + sqlreq + "=>" + e.getMessage());

            if (request != null) {
                try {
                    request.close();
                } catch (SQLException ex) {
                    TDeb.wri("Pb with .close()=>" + ex.getMessage());
                }
            }

            res.setSQL(sqlreq, new java.util.Date(System.currentTimeMillis()), new java.util.Date(System.currentTimeMillis()));
            res.setObj("#!SQL error".getBytes());
            if (Parent.isDB_Errors()) {
                return res;
            } else {
                return null;
            }
        }
        return Trt_out(request, sqlreq);
    }

    private C_Packet Trt_out(Statement request, String sqlreq) {
        long start = 0;
        long stop = 0;
        C_Packet res = new C_Packet("", Parent);
        ResultSet sqlres;
        try {
            start = System.currentTimeMillis();
            if (request.execute(sqlreq) == false) {
                if (request.getUpdateCount() >= 0) {
                    res.setSQL(sqlreq, new java.util.Date(start), new java.util.Date(System.currentTimeMillis()));
                    res.setObj(request.getUpdateCount());
                    return res;
                }
                return null;
            }
            //sqlres=request.executeQuery(sqlreq);
            //request.close();
            /// NOTE : sqlres is also closed() !!!=> F_DB could not get metadata
            //NOTE : should be closed by closing sqlres later.
            //SHOULD be a good way ...
             /*
             * need to be tryed with CALLABLESTATEMENT while
             * (request.getMoreResults()) { ResultSet res2 =
             * request.getResultSet();} }
             */
        } catch (SQLException e) {
            TDeb.wri("Pb with SQL request : " + sqlreq + "=>" + e.getMessage());
            try {
                request.close();
            } catch (SQLException ex) {
                TDeb.wri("Pb with .close()=>" + ex.getMessage());
            }
            res.setSQL(sqlreq, new java.util.Date(start), new java.util.Date(0));
            res.setObj(("#!SQL error : " + e.getMessage()).getBytes());
            if (Parent.isDB_Errors()) {
                return res;
            } else {
                return null;
            }
        }

        try {
            sqlres = request.getResultSet();
            stop = System.currentTimeMillis();
            /*
             * !!!!excecute & multiple resultset are for stred preocedures... if
             * (request.getMoreResults(Statement.KEEP_CURRENT_RESULT)){ return
             * request.getResultSet(); } else{ return sqlres; }
             */
        } catch (SQLException ex) {
            TDeb.wri("Pb with .getResultSet() =>" + ex.getMessage());
            res.setSQL(sqlreq, new java.util.Date(start), new java.util.Date(stop));
            res.setObj(ex);
            return res;
        }
        res.setSQL(sqlreq, new java.util.Date(start), new java.util.Date(stop));
        res.setObj(getDB_result(sqlres, sqlreq));
        return res;
    }

    private Object getDB_result(ResultSet rs, String sqlreq) {
        if (Parent.getDB_resultType().compareTo("ResultSet") == 0 || rs == null) {
            return rs;
        } else if (Parent.getDB_resultType().compareTo("Tree") == 0) {
            DefaultMutableTreeNode root = new DefaultMutableTreeNode(sqlreq);
            int enr = 0;
            try {
                while (rs.next()) {
                    DefaultMutableTreeNode enrnode = new DefaultMutableTreeNode(enr++);
                    for (int i = 1; i <= rs.getMetaData().getColumnCount(); i++) {
                        DefaultMutableTreeNode attnode = new DefaultMutableTreeNode(rs.getObject(i));
                        enrnode.add(attnode);
                    }
                    root.add(enrnode);
                }
            } catch (SQLException ex) {
                TDeb.wri("=>" + ex);
            }
            return new DefaultTreeModel(root);
        } else if (Parent.getDB_resultType().compareTo("Table") == 0) {
            D_DBUtils.DB_Model mod = new D_DBUtils.DB_Model(rs);
            mod.setQuery(sqlreq);
            return mod;
        }
        return null;
    }

    private void TreatDB() {
        if (totreat == null || cxDB == null || totreat.isEmpty()) {
            return;
        }
        C_Packet d;
        while (totreat != null && totreat.size() > 0) {
            try {
                d = totreat.remove(0);
                boolean treated = false;
                if (d != null && d.getObj() != null) {
                    if (Parent.isDB_Packets()) {
                        TDeb.wri("will treat : " + d.getObj().getClass() + "=>" + d.toString());
                        String msgid = "" + d.getObj().hashCode();//d.getWOtime().getTime();
                        java.util.Map<String, Object> map = new java.util.HashMap<>();
                        map.put("msg_id", msgid);
                        map.put("Source", d.getsource());
                        map.put("destination", d.getdestination());
                        map.put("CREATE_time", d.getCreatetime().toString());
                        map.put("transit", "" + d.getTransittime());
                        map.put("lenght", "" + d.length());
                        if (d.getObj() != null) {
                            map.put("class", "" + d.getObj().getClass());
                        }
                        if (d.getObj() instanceof byte[]) {
                            map.put("data", "" + TUtils.tabTostr((byte[]) d.getObj(), " ".charAt(0)));
                        }
                        dbutils.Insert_datas(getSchema(), "cxPacket", map, java.util.Arrays.asList(new String[]{"msg_id"}));
                        treated = true;
                    } else if (d.getObj() instanceof javax.swing.table.AbstractTableModel
                            || d.getObj() instanceof com.jdcm.common.XMLres
                            || d.getObj() instanceof javax.swing.tree.DefaultTreeModel) {
                        TDeb.wri("will treat : " + d.getObj().getClass() + "=>" + d.toString());
                        TreatDB_model(d.getObj());
                        treated = true;
                    } else if (d.getObj() instanceof byte[] || d.getObj() instanceof String) {
                        TDeb.wri("will treat : " + d.getObj().getClass() + "=>" + d.toString());
                        treated = true;
                        String sqlreq = dbutils.getstdQuery(d.getObj()); // normalize to String
                        if (sqlreq == null) {
                            TDeb.wri("Not a SQL statement received : " + d.getObj().toString());
                            Parent.read_in(new C_Packet("#!SQL Error".getBytes(), Parent));
                        } else if (sqlreq.startsWith("#!SQL")) {
                            TDeb.wri("Not a SQL statement received : " + d.getObj().toString());
                        } else if (sqlreq.trim().length() > 6) {
                            d.setObj(sqlreq);
                            if (sqlreq.trim().toUpperCase().substring(0, 6).compareTo("SELECT") == 0) {
                                //SELECT
                                Parent.read_in(Trt_out(d));
                                //??res.close();
                            } else {
                                //UPDATE/CREATE/INSERT/..
                                Parent.read_in(Trt_in(d));
                            }
                            try {
                                if (cxDB.getAutoCommit() == false) {
                                    cxDB.commit();
                                }
                            } catch (SQLException ex) {
                                TDeb.wri("could not commit DB");
                            }
                        }
                    }

                    if (treated == false) {
                        TDeb.wri(" NOT Treated : " + d.toString());
                    }
                    d.setStoredtime(new java.util.Date(System.currentTimeMillis()));//???
                }
            } catch (ArrayIndexOutOfBoundsException ex) {
            }
            if (totreat == null) {
                Parent.setAcc_initsize(-1);
            } else {
                Parent.setAcc_position(totreat.size());
            }
        }
    }

    private String debugKeysasString(XMLres res) {
        String keyvals = "";
        for (String key : res.getPrimaryKeys()) {
            keyvals += "," + res.getAttrVal(key);
        }
        if (keyvals.length() > 0) {
            keyvals = keyvals.substring(1);
        }
        return keyvals;
    }

    private void TreatDB_model(Object strutobj) {
        //check & add table & field
        //String msgid=""+mod.getWOtime();//.getstarttime().getTime();
//PB WITH MSGID IS HASH??=> NOT  SIGNIFICANT
        if (strutobj instanceof Code_Model) {
            Code_Model mod = (Code_Model) strutobj;
            TDeb.wri("Receive codeModel : " + mod.hashCode() + "  (" + dbutils.getCompteur() + ")");
            java.util.Map<String, Object> map = new java.util.HashMap<>();
            String prev_table = null;
            Long prev_id = null;
            Long msgid = null;
            for (int i = 0; i < mod.getRowCount(); i++) {
                String table = mod.getValueAt(i, "Proto").toString().toLowerCase();
                if (i > 0 && mod.getValueAt(i - 1, "Proto").toString().toLowerCase().compareTo(table) != 0) {
                    prev_table = mod.getValueAt(i - 1, "Proto").toString().toLowerCase();
                    prev_id = msgid;
                    TDeb.wri("new proto: " + mod.getValueAt(i - 1, "Proto").toString() + " id: " + msgid);
                    msgid = dbutils.getNextLong(getSchema(), table, "id_" + table);
                } else if (msgid == null) {
                    msgid = dbutils.getNextLong(getSchema(), table, "id_" + table);
                    if (msgid == null) {
                        msgid = 1L;
                    }
                }
                if (table.length() > 0) {
                    map.clear();
                    map.put("id_" + table, msgid);
                    if (prev_table != null) {
                        map.put("ext_" + prev_table, prev_id);
                        dbutils.createextconstraint(table, "ext_" + prev_table, prev_table, "id_" + prev_table);
                    }
                    if ((Boolean) mod.getValueAt(i, "Visible")) {
                        map.put(mod.getValueAt(i, "Name").toString(), mod.getValueAt(i, "Value"));
                    }
                    dbutils.Insert_datas(getSchema(), table, map, java.util.Arrays.asList(new String[]{"id_" + table}));
                }
            }
        } else if (strutobj instanceof com.jdcm.common.XMLres) {
            com.jdcm.common.XMLres res = (com.jdcm.common.XMLres) strutobj;
            // -----------'lookup_' look for and manage  fields-----------------
            for (String attname : res.getAttrNames()) {
                if (attname.startsWith("lookup_") && res.getAttrVal(attname) != null) {
                    String field = res.getAttrValAsstring(attname).split("=")[0];
                    String txt = res.getAttrValAsstring(attname).split("=")[1];
                    //check if the target field alue is a reference to the transit table/
                    if (res.getAttrVal(field).toString().contains("=")) {
                        txt = "" + txt + "=" + res.getAttrVal(field) + "";
                        Long idx = getLookupId(getSchema(), res.toString(), field, txt);
                        TDeb.wri("found idx=" + idx + " for: " + getSchema() + "." + res.toString() + "." + field + "=" + txt);
                        if (idx != null && idx > 0) {
                            res.setAttrVal(field, idx);
                        }
                    }
                }
            }
            // ---------- need to manage Key/no keys/AutoIncrement/-------------
            // TODO!!! and if lookup_=> teach the good value in transit
            // actually now:
            //if lookup Key, found or getNext()=> +1 
            //else if not specified..perhaps 'auto incr'
            //if specified =>if exists  UPDATE else INSERT

            //------------------DELETE-------------------------------------------
            if (res.getAttrVal("delete_") != null) {
                TDeb.wri("Delete XML element : <" + res + " keys=" + res.getPrimaryKeys() + "=[" + debugKeysasString(res) + "]>  (" + dbutils.getCompteur() + ")");
                dbutils.Delete_datas(getSchema(), res.toString(), res.getAttrsMap(), res.getPrimaryKeys());
            } else {
                //-------------------INSERT-----------------------------------------
                TDeb.wri("Insert XML element : <" + res + " keys=" + res.getPrimaryKeys() + "=[" + debugKeysasString(res) + "]>  (" + dbutils.getCompteur() + ")");
                dbutils.Insert_datas(getSchema(), res.toString(), res.getAttrsMap(), res.getPrimaryKeys());
            }

        } else if (strutobj instanceof javax.swing.tree.DefaultTreeModel) {
            javax.swing.tree.DefaultTreeModel src_tree = (javax.swing.tree.DefaultTreeModel) strutobj;
            String rootName = "" + src_tree.getRoot().toString();
            if (Parent.getDB_mapfile() == null || Parent.getDB_mapfile().length() == 0) {
                TDeb.wri("Receive TreeModel element for buiding: " + rootName + "  (" + dbutils.getCompteur() + ")");
                XMLtoDB(src_tree, null);
            } else {
                TDeb.wri("Receive TreeModel element for mapping: " + rootName + "  (" + dbutils.getCompteur() + ")");
                XML2tree a = new com.jdcm.common.XML2tree();
                javax.swing.tree.DefaultTreeModel maptree = null;
                try {
                    maptree = a.XMLBase(TUtils.getFileStream(Parent.getDB_mapfile()));
                } catch (Exception ex) {
                    TDeb.wri("Pb getting mapping Stream for file:" + Parent.getDB_mapfile().getName());
                }
                if (maptree != null) {
                    init_mapping(maptree, true, src_tree);
                }
            }
        } else if (strutobj instanceof javax.swing.table.AbstractTableModel) {
            javax.swing.table.AbstractTableModel res = (javax.swing.table.AbstractTableModel) strutobj;
            //adding/insert a table to current schema
            // what is the table name???
        }
        if (cxDB == null) {
            TDeb.wri("Abnormal END : DB closed (" + dbutils.getCompteur() + ")");
            return;
        }
        try {
            if (cxDB.getAutoCommit() == false) {
                cxDB.commit();
            }
        } catch (SQLException ex) {
            TDeb.wri("could not commit DB");
        }
        TDeb.wri("END (" + dbutils.getCompteur() + ")");
    }

    /**
     * end of insert_data(...)
     * ------------------------------------------------------------------------
     */
    /**
     * intialise mapping to/from DB
     *
     * @param istodb set to (true) or from DataBase
     * @param map_tree tree is the tree used for mapping
     * @param src_tree set the root of the tree for input, else null
     */
    public void init_mapping(DefaultTreeModel map_tree, boolean istodb, DefaultTreeModel src_tree) {
        java.io.OutputStream out;
        if (map_tree == null) {
            return;
        }
        String tempOutType = Parent.getDB_resultType();
        Parent.setDB_resultType("ResultSet");
        DefaultMutableTreeNode mroot = ((DefaultMutableTreeNode) map_tree.getRoot());
        DefaultMutableTreeNode map;
        for (int i = 0; i < mroot.getChildCount(); i++) {
            if (mroot.getChildAt(0).toString().compareTo("mapping") == 0) {
                map = (DefaultMutableTreeNode) mroot.getChildAt(i);
                while (map != null && map.getChildCount() > 0) {
                    if (map.getChildAt(0).toString().compareTo("file") == 0) {
                        DefaultMutableTreeNode map_node = (DefaultMutableTreeNode) map.getChildAt(0);
                        map_node.removeFromParent();
                        //OPEN OUT FILE
                        String destFileName = null;
                        if (map_node.getUserObject() instanceof XMLres) {
                            destFileName = ((XMLres) map_node.getUserObject()).getAttrValAsstring("destination");
                        }
                        try {
                            if (istodb) {
                                if (destFileName != null && src_tree == null) {
                                    TDeb.wri("XML2DB try to get src_tree from file : " + destFileName);
                                    XML2tree a = new com.jdcm.common.XML2tree();
                                    File f = new java.io.File(destFileName);
                                    if (f.canRead()) {
                                        src_tree = a.XMLBase(TUtils.getFileStream(f));
                                    }
                                }
                                if (src_tree != null) {
                                    DefaultMutableTreeNode src_rt;
                                    src_rt = ((DefaultMutableTreeNode) src_tree.getRoot());
                                    TDeb.wri("XML2DB data: " + src_rt + " via map: " + map_node + ", to: " + getSchema());
                                    //remove all non input fields to avoid loosing time
                                    map_node = TUtils.treefilter(map_node, "sqlin", true, null);
                                    //to debug
                                    // com.jdcm.editors.F_XML f = new com.jdcm.editors.F_XML(false, new javax.swing.tree.DefaultTreeModel(map_node));
                                    map_toDB(src_rt, map_node, null, null);
                                }
                            } else {
                                TDeb.wri("DB2XML map trying: " + map_node);
                                DefaultTreeModel tr = map2XML(map_node);
                                if (destFileName != null) {
                                    out = new java.io.FileOutputStream(destFileName);
                                    TDeb.wri("DB2XML map: " + getSchema() + " to file: " + destFileName);
                                    com.jdcm.common.XML2tree xml = new com.jdcm.common.XML2tree();
                                    xml.savetree2XML(tr, out);
                                } else if (tr != null) {
                                    TDeb.wri("DB2XML map: " + getSchema() + " sending tree..");
                                    Parent.write_in(new C_Packet(tr, Parent));
                                } else {
                                    TDeb.wri("DB2XML null tree");
                                }
                            }
                        } catch (FileNotFoundException ex) {
                            TDeb.wri("PB at initialisation : " + ex.getMessage());
                            return;
                        }
                    } else {
                        ((DefaultMutableTreeNode) map.getChildAt(0)).removeFromParent();
                    }
                }
            }
        }
        Parent.setDB_resultType(tempOutType);
    }

    public void getQuickMap(java.io.InputStream mapstr, java.io.OutputStream outstr, String srcstr, boolean isout) {
        com.jdcm.common.XML2tree xml = new com.jdcm.common.XML2tree();
        DefaultTreeModel map_tree = xml.XMLBase(mapstr);
        if (map_tree != null) {
            DefaultMutableTreeNode mroot = ((DefaultMutableTreeNode) map_tree.getRoot());
            DefaultMutableTreeNode map;
            for (int i = 0; i < mroot.getChildCount(); i++) {
                if (mroot.getChildAt(0).toString().compareTo("mapping") == 0) {
                    map = (DefaultMutableTreeNode) mroot.getChildAt(i);
                    while (map != null && map.getChildCount() > 0) {
                        if (map.getChildAt(0).toString().compareTo("file") == 0) {
                            DefaultMutableTreeNode map_node = (DefaultMutableTreeNode) map.getChildAt(0);
                            map_node.removeFromParent();
                            try {
                                if (isout) {
                                    DefaultTreeModel map_tr = map2XML(map_node);
                                    xml.savetree2XML(map_tr, outstr);
                                } else {
                                    DefaultTreeModel src_tree = xml.XMLBase(srcstr.getBytes());
                                    map_toDB((DefaultMutableTreeNode) src_tree.getRoot(), map_node, null, outstr);
                                }
                            } catch (Exception ex) {
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * try to convert XML to schema
     *
     * @param src_tree set the root of the tree for input
     * @param node
     */
    public void XMLtoDB(DefaultTreeModel src_tree, DefaultMutableTreeNode node) {
        //Parent.write_in(new C_Packet("create schema test",Parent.getIndex()));
        String idtag = "id_";
        String idexttag = "ext_";
        String valtag = "val_";
        DefaultMutableTreeNode parent_node = node;
        if (parent_node == null) {
            parent_node = (DefaultMutableTreeNode) src_tree.getRoot();
            Long idadm = dbutils.getNextLong(null, "admin", "id_admin");
            if (idadm == null) {                                                 //if next index not found (no table)=> try 1..
                idadm = new Long(1);
            }
            DefaultMutableTreeNode root = (DefaultMutableTreeNode) parent_node.getChildAt(0);
            if (root.getUserObject() instanceof XMLres) {
                XMLres r = (XMLres) root.getUserObject();
                Long l = dbutils.getNextLong(null, r.getEltName(), idtag + r.getEltName());
                if (l == null) {
                    l = new Long(1);
                }
                java.util.Map<String, Object> map = new java.util.HashMap<>();
                map.put("id_admin", idadm);
                map.put("createtime", "" + new java.util.Date(System.currentTimeMillis()));
                map.put("root_name", "" + r.getEltName());
                map.put("root_id", "" + l);
                dbutils.Insert_datas(getSchema(), "admin", map, java.util.Arrays.asList(new String[]{"id_admin"}));
            } else {
                // manage non XMLres Trees
                return;
            }
        }
        for (int i = 0; i < parent_node.getChildCount(); i++) {
            DefaultMutableTreeNode child_node = (DefaultMutableTreeNode) parent_node.getChildAt(i);
            XMLres r = (XMLres) child_node.getUserObject();
            Object idval;
            if (r.getAttrVal(idtag + r.getEltName()) != null) {
                idval = r.getAttrValAsstring(idtag + r.getEltName());
            } else {
                Long l = dbutils.getNextLong(null, r.getEltName(), idtag + r.getEltName());
                if (l == null) {
                    l = new Long(1);
                }
                r.addAttrVal(idtag + r.getEltName());
                r.setAttrVal(idtag + r.getEltName(), l);
                idval = l;
            }
            r.setPrimaryKeys(idtag + r.getEltName());

            if (node != null) {
                XMLres parentres = (XMLres) node.getUserObject();
                //TODO : does XML=>DB is it always only one key??
                dbutils.Insert_data(getSchema(), r.getEltName(), idexttag + parentres.getEltName(),
                        idtag + r.getEltName(), idval, parentres.getAttrVal(parentres.getPrimaryKeys().get(0)));
                dbutils.createextconstraint(r.getEltName(), idexttag + parentres.getEltName(), parentres.getEltName(), idtag + parentres.getEltName());
            } else {
                // to create main table (kml/PROJECT/...)
                dbutils.Insert_data(getSchema(), r.getEltName(), idtag + r.getEltName(),
                        idtag + r.getEltName(), idval, idval);
            }
            // Copy the Map ? could it be used lie it ?
            java.util.Map<String, Object> map = new java.util.HashMap<>();
            for (String name : r.getAttrNames()) {
                map.put(name, r.getAttrVal(name));
            }
            dbutils.Insert_datas(getSchema(), r.getEltName(), map, java.util.Arrays.asList(new String[]{idtag + r.getEltName()}));
            if (r.getValue() != null) {
                // manage the tag value (<elt> 'value'</elt>)
                dbutils.Insert_data(getSchema(), r.getEltName(), valtag + r.getEltName(),
                        idtag + r.getEltName(), idval, r.getValue());
            }
            if (child_node.getChildCount() > 0) {
                XMLtoDB(src_tree, child_node);
            }
        }
    }

    /**
     * Return the ID store in transit or define a new id
     *
     * @param dest_table
     * @param dest_schema
     * @param dest_id
     * @param transit
     * @return _1 for error else id or created id for the schema.table.dest with
     * transmit. -3 => Format, -2 => found is not not long , -1=> no transit
     * found index to write later
     */
    public Long getLookupId(String dest_schema, String dest_table, String dest_id, String transit) {
        if (transit.contains("=") == false || transit.contains("@") == false) { // txt=att_dest@port_transit.sh_transit.table_transit.attr_src='id_src=154'
            return -3L;                                                         // could return an Object ??
        }
        String id_src = transit.substring(transit.indexOf("=") + 1);            //Parse the transit string
        transit = transit.substring(0, transit.indexOf("="));
        String[] fields = transit.split("\\.");
        String att_dest = fields[0].split("@")[0];
        String att_trdst = dbutils.getstdColumnName(att_dest);
        String table_tr = dbutils.getstdTableName(fields[1], fields[2]);
        String attr_trsrc = dbutils.getstdColumnName(fields[3]);
        fields[0] = fields[0].split("@")[1];
        java.util.List<I_Command> plist = ((I_Command) Parent.getMain()).getCMDChilds(fields[0]);
        if (plist == null || plist.isEmpty() || plist.size() > 1 || plist.get(0) instanceof C_Port == false) {
            TDeb.wri("getLookupNextId() no valid port for: " + fields[0]);
            return null;
        }
        C_Port port_transit = (C_Port) plist.get(0);
        if (port_transit.getSlave() instanceof T_DB == false) {
            TDeb.wri("getLookupNextId() no DB slave for: " + fields[0]);
            return null;
        }
        T_DB transit_db = (T_DB) port_transit.getSlave();                         //look for transit_db
        // check if an id is already set in the transit table
        java.sql.ResultSet rs = transit_db.Trt_out("SELECT " + att_trdst + " FROM " + table_tr + " WHERE " + table_tr + "." + attr_trsrc + "='" + id_src + "';");
        Object id_destval = null;
        if (rs != null) {
            try {
                try {
                    rs.first();
                } catch (SQLException ex1) {
                }
                id_destval = rs.getObject(1);
            } catch (SQLException ex) {
            }
        }
        if (id_destval != null) {                                               // return id founded id in transit_table
            try {
                TDeb.wri("getLookupNextId(): found " + Long.parseLong("" + id_destval));
                return Long.parseLong("" + id_destval);
            } catch (NumberFormatException ex) {
                TDeb.wri("getLookupNextId(): Pb parsing: " + id_destval + "=>" + ex.getMessage());
                return -2L;
            }
        } else {
            Long id_destlg = dbutils.getNextLong(dest_schema, dest_table, dest_id);       //find next ID (next record ID)and teach it to transit table (for a nexts asking)!;
            if (id_destlg == null) {
                id_destlg = 1L;                                                   //  if Table doest not exist OK but if ID is string??                                
            }
            //  System.out.println("=> getmax(): " + id_destlg + "/" + id_destval);
            XMLres res = new XMLres(fields[2]);
            res.addAttrVal(att_dest);
            res.addAttrVal(fields[3]);
            res.setAttrVal(att_dest, id_destlg);
            res.setAttrVal(fields[3], id_src);
            res.setPrimaryKeys(fields[3]);
            transit_db.TreatDB_model(res);
            //transit_db.Insert_data(fields[2], att_dest, fields[3], id_src, id_destlg); //teach tothe Transit table
            return id_destlg;
        }
        // return -1L;
    }

    /**
     * try to convert schema to DB
     *
     * @param parentnode set the root of the tree for input (null for start with
     * a admin table)
     * @return the node of the XML tree matching
     */
    public DefaultMutableTreeNode DBtoXML(DefaultMutableTreeNode parentnode) {
        String idtag = "id_";
        String idexttag = "ext_";
        String valtag = "val_";
        String where = "";
        String[] mastertables = new String[0];
        try {
            if (parentnode == null) {
                ResultSet rs = Trt_out("SELECT * FROM admin ;");
                String key;
                try {
                    rs.last();
                    mastertables = new String[]{(rs.getString("first_table"))};
                    key = rs.getString("index");
                    where = " WHERE " + idtag + mastertables[0] + "=" + key;
                } catch (SQLException ex) {
                    return null;
                    // } catch (java.sql.SQLFeatureNotSupportedException e){
                    //   return null;
                } catch (NullPointerException ex) {
                    TDeb.wri("No table 'admin'(first_table,index) mandatory for DB => XML");
                }
                parentnode = new DefaultMutableTreeNode(new XMLhdr("1.0", "ISO-8859-1"));
            } else {
                XMLres parentres = (XMLres) parentnode.getUserObject();
                if (parentres.getPrimaryKeys().isEmpty() == false) {
                    //look for tables
                    ResultSet rs = cxDB.getMetaData().getColumns(null, null, null, idexttag + parentres.getEltName());
                    while (rs.next()) {
                        if (rs.getObject("COLUMN_NAME").toString().compareTo(idexttag + parentres.getEltName()) == 0) {
                            mastertables = TUtils.addtab(mastertables, new String[]{rs.getObject("TABLE_NAME").toString()});
                        }
                    }
                    //TODO relations are with ONLY ONE key from XML=> DB?? to check
                    where = " WHERE " + idexttag + parentres.getEltName() + "=" + parentres.getPrimaryKeys().get(0);
                }
            }
            // build an XMLres and add it to node
            for (String tablename : mastertables) {
                ResultSet rs = Trt_out("SELECT * FROM " + tablename + where + ";");
                while (rs.next()) {
                    XMLres res = new XMLres(tablename);
                    for (int i = 0; i
                            < rs.getMetaData().getColumnCount(); i++) {
                        String col = rs.getMetaData().getColumnName(i + 1);
                        if (col.startsWith(idexttag) == false && rs.getObject(col) != null) {
                            // attribute or value
                            if (col.startsWith(idtag)) {
                                res.setElement(col.substring(idtag.length()));
                                res.setPrimaryKeys(rs.getObject(col).toString());
                            } else if (col.compareTo(valtag + res.getEltName()) == 0) {
                                res.setValue(rs.getObject(col).toString());
                            } else {
                                res.addAttrVal(col);
                                res.setAttrVal(col, rs.getObject(col).toString());
                            }
                        }
                    }
                    DefaultMutableTreeNode node = new DefaultMutableTreeNode(res);
                    node = DBtoXML(node);
                    parentnode.add(node);
                }
            }
        } catch (SQLException ex) {
            TDeb.wri(dbutils.getCompteur() + "|DB2XML : [" + ex.getMessage() + "]");
        }
        return parentnode;
    }

    private int checkpath(DefaultMutableTreeNode data_node, DefaultMutableTreeNode map_node, XMLres result) {
        //PB of elt name from DB (poitn/linestring...)
        // for KML zonnes/lines 2 DB
        if (data_node.toString().compareTo("Point") == 0 || data_node.toString().compareTo("LineString") == 0) {
            if (((XMLres) map_node.getUserObject()).getAttrVal("sqlname") != null) {
                ((XMLres) map_node.getUserObject()).addAttribut("name", data_node.toString());
                return 0;
            }
        }
        //COMPARE STRING OF PATH
        DefaultMutableTreeNode map_pere = map_node;
        DefaultMutableTreeNode data_pere = data_node;

        java.util.List<Object[]> vmap = new java.util.ArrayList<>();
        java.util.List<Object[]> vdata = new java.util.ArrayList<>();
        //add characteristics of path for map & data in Lists
        while (data_pere.getParent() != null) {
            String valpere;
            if (data_pere.getUserObject() instanceof XMLres) {
                valpere = data_pere.toString(); // ((XMLres) data_pere.getUserObject()).getValue().toString();
            } else {
                valpere = data_pere.toString();
            }

            vdata.add(new Object[]{data_pere.toString(), (data_pere.getParent()).getIndex(data_pere), valpere});
            data_pere = (DefaultMutableTreeNode) data_pere.getParent();
        }
        while (map_pere.toString().compareTo("file") != 0) {
            XMLres res = (XMLres) map_pere.getUserObject();
            vmap.add(new Object[]{res.getAttrVal("name"), (map_pere.getParent()).getIndex(map_pere), res.getAttrVal("value")});
            map_pere = (DefaultMutableTreeNode) map_pere.getParent();
        } //check characteristics of path differences
        if (vmap.size() != vdata.size()) {
            return -1;
        }
        for (int i = 0; i
                < vmap.size(); i++) {
            if (vmap.get(i)[0].toString().compareTo(vdata.get(i)[0].toString()) != 0 && vmap.get(i)[0].toString().compareTo("/*/") != 0) {
                return -1;
            }
        }
        if (map_act != null) {
            String chk = ((XMLres) map_act.getUserObject()).getAttrValAsstring("checkline");
            if (chk != null && chk.compareTo("index") == 0) {
                for (int i = 0; i
                        < vmap.size(); i++) {
                    if (vmap.get(i)[1] != vdata.get(i)[1]) {
                        return -1;
                    }
                }
            }
        }
        TDeb.wri("matchnig path for : MAP=>" + vmap.get(0)[0].toString() + " TREE=> " + vdata.get(0)[0].toString());
        return 0;

    }

    /**
     * Check the the data node match (or not) with the map node (second loop in
     * the map tree) if "sqlin" then build a result (and !clear! previous,
     * should be better with an array) if result is not null, add sqlkey ,
     * sqlname, sqlvalin...
     *
     * @param data_node the node comming from data
     * @param map_node the node comming from map
     * @param result the previous result
     * @return the result incremented with founded datas
     */
    private XMLres check_map(DefaultMutableTreeNode data_node, DefaultMutableTreeNode map_node, XMLres result) {
        //   TDeb.wri("add_elt from src tree : "+data_node+" with mapp elt : "+map_node+ " set DB result: "+result);
        if (result != null) {
            if (result.getValue() != null) {
                return result;
            }
        }
        if (map_node.toString().compareTo("element") != 0) {
            return result;
        }

        if (checkpath(data_node, map_node, result) == 0) {
            XMLres res_data;
            if (data_node.getUserObject() instanceof XMLres) {
                res_data = (XMLres) data_node.getUserObject();
            } else {
                res_data = new XMLres("string");
                res_data.setValue(data_node.getUserObject().toString());
            }
            XMLres res_map = (XMLres) map_node.getUserObject();
            if (res_map.getAttrVal("sqlin") != null) {
                TDeb.wri("=>" + ((XMLres) map_node.getUserObject()).getName());
                result = new XMLres(res_map.getAttrValAsstring("sqlin"));
                map_act = map_node;
            }
            if (result != null) {
                for (String name : res_map.getAttrNames()) {
                    if (name.compareTo("sqlkey") == 0) {
                        result.addPrimaryKey(res_map.getAttrValAsstring("sqlkey"));
                    }
                }
                if (res_map.getAttrVal("sqlname") != null) {
                    //element name to save
                    result.addAttribut(res_map.getAttrValAsstring("sqlname"), res_data.toString());
                }
                if (res_map.getAttrVal("sqlvalin") != null) {
                    //element value to save
                    String att = res_map.getAttrValAsstring("sqlvalin");
                    if (result.addAttribut(att, res_data.getValue().toString()) == false) {
                        TDeb.wri(" Pb setting attribut: '" + att + "' to value : '" + res_data.getValue().toString() + "' in result : " + result.hashCode());
                    }
                }
                //+++++++++++++++ATTRIBUTS+++++++++++++++++
                for (int j = 0; j
                        < map_node.getChildCount(); j++) {
                    DefaultMutableTreeNode newnode = (DefaultMutableTreeNode) map_node.getChildAt(j);
                    if (newnode.toString().compareTo("attribut") == 0) {
                        res_map = (XMLres) newnode.getUserObject();
                        if (res_map.getAttrVal("sqlvalin") != null) {
                            if (res_data.getAttrVal(res_map.getName()) != null) {
                                result.addAttribut(res_map.getAttrVal("sqlvalin").toString(), res_data.getAttrVal(res_map.getName()));
                            }
                        }
                    }
                }
                result.setValue("end");
                TDeb.wri(" MAP =>" + map_node + " DATA =>" + data_node + " with result : " + result.getAttrValAsstring("ttl"));
            }
        }
        for (int i = 0; i
                < map_node.getChildCount(); i++) {
            DefaultMutableTreeNode mapchild = (DefaultMutableTreeNode) map_node.getChildAt(i);
            result = check_map(data_node, mapchild, result);
            try {
                sleep(1);
            } catch (InterruptedException e) {
            }
        }
        if (result != null) {
            TDeb.wri(" result : " + result.getAttrValAsstring("ttl"));
        }
        return result;
    }

    /**
     * TREE=>DB : build around a recurent scan (map_fromtree)of the data tree
     * and recurent (check_map) scan of the (filtred) map tree.
     *
     * SHOULD BE A BETTER WAY (store map in a "good" data structure? and scan
     * data with the data structure)
     *
     * @param src_node data (xml file) to import in DB
     * @param map_node
     * @param toDB src data returned as XML element to update DB
     * @return return the result
     */
    private XMLres map_toDB(DefaultMutableTreeNode src_node, DefaultMutableTreeNode map_node, XMLres toDB, java.io.OutputStream outstr) {
        int activate = 0;
        if (toDB == null) {
            activate = -1;
        } //avoid root <?....?>
        if (src_node.getParent() == null) {
            src_node = (DefaultMutableTreeNode) src_node.getChildAt(0);
        }
        for (int j = 0; j
                < map_node.getChildCount(); j++) {
            //elements
            DefaultMutableTreeNode mapchild = (DefaultMutableTreeNode) map_node.getChildAt(j);
            toDB = check_map(src_node, mapchild, toDB);
        }
        if (activate == -1 && toDB != null) {
            activate = 1;
        }
        if (toDB != null && toDB.getValue() != null) {
            toDB.setValue(null);
        }
        //avant elt
        for (int i = 0; i < src_node.getChildCount(); i++) {
            DefaultMutableTreeNode datachild = (DefaultMutableTreeNode) src_node.getChildAt(i);
            toDB = map_toDB(datachild, map_node, toDB, outstr);

        }
        if (activate == 1 && toDB != null) {
            map_act = null;
            String keyvals = "[";
            for (String key : toDB.getPrimaryKeys()) {
                keyvals = keyvals + "," + toDB.getAttrVal(key);
            }
            TDeb.wri("===>" + toDB.toString() + ", keys: " + toDB.getPrimaryKeys() + ", keyvals: " + keyvals + "]");
            // for DEBUG
            //Parent.write_in(new C_Packet(toDB, Parent.getIndex()));
            if (outstr == null) {
                write(new C_Packet(toDB, Parent));
            } else {
                try {
                    outstr.write((toDB.toXMLString() + "\r\n").getBytes());
                } catch (IOException ex) {
                    TDeb.wri("Pb wrinting XML in outstream");
                }
            }
            toDB = null;
        } else {
            TDeb.wri("!=>" + toDB + ", src: " + src_node);
        }
        return toDB;
    }

    private DefaultTreeModel map2XML(DefaultMutableTreeNode mapnode) {
        try {
            //  should change header to encode ISO-8859-1??
            com.jdcm.common.XMLhdr hdr = new com.jdcm.common.XMLhdr("1.0", "ISO-8859-1");
            DefaultTreeModel treemodel = new DefaultTreeModel(new DefaultMutableTreeNode(hdr));
            //boolean out=mapnode.
            TDeb.wri("map2XML():start to XML from : " + getSchema());
            for (int i = 0; i < mapnode.getChildCount(); i++) {
                map2XMLNewElement((DefaultMutableTreeNode) mapnode.getChildAt(i), (DefaultMutableTreeNode) treemodel.getRoot(), null, 0);
            }
            if (((XMLres) mapnode.getUserObject()).getAttrVal("out") == null) {
                return treemodel;
            }
        } catch (Exception ex) {
            TDeb.wri("map2XML():Pb write xml destination file ");
        }
        return null;
    }

    /**
     * teach a dstnode (from result tree) from a map-node if map node ask for
     * sqlout the feel sql result and spread it on dest node.
     *
     * @param mapnode first value is the root of the map after the childs of
     * previous nodes
     * @param dstnode first value is the root of the result tree after the
     * created nodes
     * @param sqlres the stored SQLresult to feel the destination tree
     */
    private void map2XMLNewElement(DefaultMutableTreeNode mapnode, DefaultMutableTreeNode dstnode, java.sql.ResultSet sqlres, int maplevel) {
        ResultSet basesql = sqlres;
        if (mapnode.toString().compareTo("element") != 0 || Parent.getDB_CX() == null) {
            return;
        }
        try {
            XMLres maprelt = (XMLres) mapnode.getUserObject();
            if (maprelt.getAttrVal("sqlout") != null) {
                String requette = maprelt.getAttrVal("sqlout") + "";
                //!!! if resultset exist from a father node
                if (basesql != null) {
                    boolean deletebase = false;
                    for (int idx = 0; idx < 5; idx++) {
                        if (maprelt.getAttrVal("sqlup_" + idx) != null && maplevel < mapnode.getLevel()) {
                            Object res = null;
                            try {
                                res = basesql.getObject(maprelt.getAttrValAsstring("sqlup_" + idx));
                            } catch (SQLException ex) {
                            }
                            if (res != null) {
                                requette = requette.replace("/" + idx + "/", "" + res);
                                deletebase = true;
                            } else {
                                TDeb.wri("XMLNewElement(): Not found: SQLup_" + idx + ": " + maprelt.getAttrValAsstring("sqlup_" + idx));
                                return;
                            }
                        }
                    }
                    if (deletebase) {
                        basesql = null;
                    }
                }
                maplevel = mapnode.getLevel();
                //continue with selector
                if (basesql == null) {
                    Statement a = Parent.getDB_CX().createStatement();
                    TDeb.wri("XMLNewElement: try request: " + requette);
                    try {
                        basesql = a.executeQuery(requette);// basesql.first=> pb with FORWARD ONLY
                        if (basesql.next() == false) {
                            return;
                        }

                    } catch (SQLException ex) {
                        TDeb.wri("XMLNewElement: Pb while executeQuery() on : " + getSchema() + "=>" + ex.getMessage());
                        return;
                    }
                    /*
                     * try { String cname =
                     * basesql.getMetaData().getColumnName(1); if (cname !=
                     * null) { // System.out.println("==>"+cname); // Pb on
                     * empty basesql basesql.getObject(cname); } } catch
                     * (Exception ex) { return; }
                     */
                } else {
                    //there is a base =>  new row
                    if (basesql.next() == false) {
                        return;
                    }
                }
            }
            String eltname = maprelt.getName();
            if (maprelt.getAttrVal("sqlname") != null && basesql != null) {
                Object res = basesql.getObject(maprelt.getAttrValAsstring("sqlname"));
                if (res != null) {
                    eltname = res.toString();
                }
            }
            if (eltname == null) {
                return;
            }
            String eltval = "";
            if (maprelt.getAttrValAsstring("value") != null) {
                eltval = maprelt.getAttrValAsstring("value");
            }
            if (maprelt.getAttrVal("sqlvalue") != null && basesql != null) {
                Object res;
                try {
                    res = basesql.getObject(maprelt.getAttrValAsstring("sqlvalue"));
                    if (res != null) {
                        if (res instanceof byte[]) {
                            eltval = eltval + new String((byte[]) res);
                        } else {
                            eltval = eltval + res.toString();
                        }
                    }
                } catch (SQLException ex) {
                    TDeb.wri("XMLNewElement: nothing found for : " + maprelt.getAttrValAsstring("sqlvalue"));
                    // res=ex;
                }
            }
            if (maprelt.getAttrVal("presentif") != null && basesql != null) {
                Object label = basesql.getObject(maprelt.getAttrValAsstring("presentif"));
                if (label == null || label.toString().length() == 0) {
                    return;
                }
            }
            XMLres relt = new XMLres(eltname);
            //+++++++++++++++ATTRIBUTS+++++++++++++++++
            for (int j = 0; j < mapnode.getChildCount(); j++) {
                DefaultMutableTreeNode newnode = (DefaultMutableTreeNode) mapnode.getChildAt(j);
                if (newnode.toString().compareTo("attribut") == 0) {
                    XMLres rc = (XMLres) newnode.getUserObject();
                    String attvalue = rc.getAttrValAsstring("value");
                    if (rc.getAttrVal("sqlvalue") != null && basesql != null) {
                        Object res = null;
                        try {
                            res = basesql.getObject(rc.getAttrValAsstring("sqlvalue")); //maprelt remplaced by rc !!
                        } catch (SQLException ex) {
                        }
                        if (res == null) {
                            TDeb.wri("XMLNewElement: No field '" + rc.getAttrValAsstring("sqlvalue") + "' in resultset");
                        } else {
                            if (res instanceof byte[]) {
                                attvalue = new String((byte[]) res);
                            } else {
                                attvalue = res.toString();
                            }
                        }
                    }
                    if (attvalue != null) {
                        relt.addAttrVal(rc.getName());
                        relt.setAttrVal(rc.getName(), attvalue);
                    }
                }
            }
            //+++++++++++++++VALUES+++++++++++++++++

            if (eltval != null) {
                relt.setValue(eltval);
            }
            for (String name : maprelt.getAttrNames()) {
                if (name.compareTo("sqlkey") == 0) {
                    relt.addPrimaryKey(maprelt.getAttrValAsstring("sqlkey"));
                }
            }
            TDeb.wri("XMLNewElement: add " + relt.toXMLString());
            if (maprelt.getAttrVal("sqlmode") != null) {
                String mode = maprelt.getAttrValAsstring("sqlmode");
                if (mode.compareTo("split") == 0) {
                    Parent.read_in(new C_Packet(relt, Parent));
                }
            }

            //+++++++++++++CHILDS ELTS+++++++++++++++
            DefaultMutableTreeNode newreltnode = new DefaultMutableTreeNode(relt);
            TDeb.wri("add node : " + newreltnode + " to " + dstnode);
            dstnode.add(newreltnode);
            for (int j = 0; j < mapnode.getChildCount(); j++) {
                DefaultMutableTreeNode newnode = (DefaultMutableTreeNode) mapnode.getChildAt(j);
                if (newnode.toString().compareTo("element") == 0) {
                    map2XMLNewElement(newnode, newreltnode, basesql, maplevel);
                }
            }
            //+++++++++++++NEXT DB ELTS+++++++++++++++
            if (basesql != null && maprelt.getAttrVal("sqlout") != null) {
                boolean bool = false;
                try {
                    bool = basesql.isLast();
                } catch (SQLException ex) {
                }
                if (bool == false) {
                    map2XMLNewElement(mapnode, dstnode, basesql, maplevel);
                }
            }

        } catch (SQLException ex) {
            TDeb.wri("XMLNewElement: error in add_xmlelement: " + (XMLres) mapnode.getUserObject() + "for" + "=>" + ex.getMessage());
            // ex.printStackTrace();

        }
    }

    @Override
    public void run() {
        while (Parent.getcxstatus().compareTo("starting") == 0) {
            try {
                sleep(10);
            } catch (InterruptedException ex) {
                TDeb.wri("port starting/ Pb during waiting [" + ex.getMessage() + "]");
            }
        }
        if (Parent.isCX_autoopen() && Parent.isCX_openbyslave()) {
            Parent.CXopen();
        }
        TDeb.wri("---Start of thread DB---");
        while (Parent.isslrunning()) {
            while (Parent.isslreading()) {
                TreatDB();
                try {
                    sleep(10);
                } catch (InterruptedException ex) {
                    TDeb.wri("slave reading/ Pb during waiting [" + ex.getMessage() + "]");
                }
            }
            try {
                sleep(50);
            } catch (InterruptedException ex) {
                TDeb.wri("slave running/ Pb during waiting [" + ex.getMessage() + "]");
            }
        }
        sl_close();
        TDeb.wri("---End of thread---");

    }
}

class DBInputStream extends java.io.InputStream {
    // UNUSED

    private T_DB tdb = null;

    public DBInputStream(T_DB t) {
        super();
        tdb = t;
    }

    @Override
    public int available() {
        if (tdb == null) {
            return -1;
        }
        //  if (tdb.getStreambuff()==null){return 0;}
        //  return tdb.getStreambuff().length;
        return 0;
    }

    @Override
    public int read() throws IOException {
        if (tdb == null) {
            throw new IOException("Terminal not available");
        }
        //    return tdb.readStreambuff();
        return 0;
    }
}
