package com.jdcm.eltrx;

import com.jdcm.analyse.Code_Model;
import com.jdcm.common.C_Packet;
import com.jdcm.common.D_Utils;
import java.io.IOException;
import jpcap.*;
import java.sql.*;

/**
 * Tcpdump.java Created on 12 octobre 2006, 22:53
 *
 * ETHERNET Slave for input/output on Ethernet interface
 *
 * see imeplemented http://netresearch.ics.uci.edu/kfujii/jpcap/doc/index.html
 * Jpcap tested on windows//fedora/mandriva/ubuntu//MacOS
 * Darwin//freeBSD/Solaris !!! FYIexisting see
 * :http://sourceforge.net/projects/jpcap
 *
 * @author Marc GIBOIN
 */
class T_ETH extends Thread implements jpcap.PacketReceiver, I_Slave {
    private final java.util.List<Byte> Streambuff;
    
    private D_Debug TDeb;
    private D_Utils TUtils = new D_Utils();
    private C_Port Parent;
    private JpcapCaptor jpcapt;
    private JpcapSender jpcapSend;
    // private T_Tcpdump handler;
    private NetworkInterface[] devices;


    public T_ETH(C_Port P) {
        Parent = P;
        TDeb = new D_Debug("T-ETH [" + Parent.getIndex() + "]");
        TDeb.setconsole(Parent.getConsole());
        try {
            devices = JpcapCaptor.getDeviceList();
        } catch (Throwable ex) {
            TDeb.wri("Could not use Jpcap/winpcap");
        }
        Streambuff = new java.util.ArrayList<>();
        setName(this.getClass().getSimpleName() + ":" + getName());
    }

    /**
     * Start the slave<br>
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        return 0;
    }

    /**
     * Open the slave<br>
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        Parent.setin(new JpcapInputStream(this));
        //  Parent.setlockout(true);
        return 0;
    }

    /**
     * Stop the slave<br>
     *
     * @return 0 if slave is Stoped correctly
     */
    @Override
    public int sl_close() {
        TDeb.wri("Closing Jpcap");
        // handler=null;
        if (jpcapt != null) {
            jpcapt.breakLoop();
            jpcapt.close();
        }
        if (jpcapSend != null) {
            jpcapSend.close();
        }
        //try {sleep(1000);} catch (InterruptedException ex) { ex.printStackTrace(); }
        jpcapt = null;
        jpcapSend = null;
        return 0;
    }

    public int StreambuffSize() {
        if (Streambuff == null) {
            return -1;
        }
        return Streambuff.size();
    }

    public void setStreambuff(byte[] data) {
        if (Streambuff == null || data == null) {
            return;
        }
        Streambuff.clear();
        for (int i = 0; i < data.length; i++) {
            Streambuff.add(data[i]);
        }
    }

    public int getrcv() {
        if (jpcapt != null) {
            jpcapt.updateStat();
            return jpcapt.received_packets;
        }
        return -1;
    }

    public int getdrp() {
        if (jpcapt != null) {
            jpcapt.updateStat();
            return jpcapt.dropped_packets;
        }
        return -1;
    }

    public int getrdTO() {
        if (jpcapt != null) {
            jpcapt.updateStat();
            return jpcapt.getPacketReadTimeout();
        }
        return -1;
    }

    public void setrdTO(int nb) {
        if (jpcapt != null) {
            jpcapt.setPacketReadTimeout(nb);
        }
    }

    public String geterrMsg() {
        if (jpcapt != null) {
            jpcapt.updateStat();
            return jpcapt.getErrorMessage();
        }
        return null;
    }

    public boolean isnonblocking() {
        if (jpcapt != null) {
            jpcapt.updateStat();
            return jpcapt.isNonBlockinMode();
        }
        return false;
    }

    public void setnonblocking(boolean bool) {
        if (jpcapt != null) {
            jpcapt.setNonBlockingMode(bool);
        }
    }

    public int readStreambuff() {
        if (Streambuff.isEmpty()) {
            return -9999;
        }
        return (int) Streambuff.remove(0);
    }

    /**
     * Write a packet content on the Network Interface<br>
     * NB !! ONLY jpcap.Packet.Packet are allowed
     *
     * @param p the CXpacket to be write on interface
     * @return the number of byte writed
     */
    @Override
    public int write(C_Packet p) {
        jpcap.packet.Packet pack = null;
        if (p.getObj() instanceof byte[]) {
            byte[] dat = (byte[]) p.getObj();
            pack = new jpcap.packet.Packet();
            pack.data = dat;
            pack.header = new byte[0];
            pack.caplen = dat.length;
        }
        if (p.getObj() instanceof Code_Model) {
            Code_Model cm = (Code_Model) p.getObj();
            if (cm.getDESC_Protofirst().getName().startsWith("ETHII")) {
                byte[] dat = cm.getInitBuffer();
                pack = new jpcap.packet.Packet();
                pack.data = dat;
                pack.caplen = dat.length;
            }
        }
        if (p.getObj() instanceof jpcap.packet.Packet) {
            pack = ((jpcap.packet.Packet) p.getObj());
        }
        if (pack != null && Parent.islockout() == false) {
            try {
                jpcapSend.sendPacket(pack);
                return pack.len;
            } catch (Exception e) {
                TDeb.wri("Pb sending (IP?) packet  :" + TUtils.tabTostr(TUtils.addtab(pack.header, pack.data), " ".charAt(0)));
                return -1;
            }
        }
        return 0;
    }

    public jpcap.NetworkInterface getdevice(int i) {
        //  if (i<0 || i>=devices.length){return null;}
        return devices[i];
    }

    public void setdevice() {
        byte[] nhw = new byte[]{0, 1, 2, 3, 4, 5};
        byte[] nad = new byte[]{-20, -10, 0, 13};
        byte[] destadd = new byte[]{-20, -10, 0, 1};
        byte[] mask = new byte[]{-1, -1, -1, 0};
        byte[] broad = new byte[]{-20, -1, -1, -1};
        jpcap.NetworkInterfaceAddress naddr = new jpcap.NetworkInterfaceAddress(nad, mask, broad, destadd);
        jpcap.NetworkInterfaceAddress[] naddrs = new jpcap.NetworkInterfaceAddress[]{naddr};
        jpcap.NetworkInterface nif = new jpcap.NetworkInterface("InterfaceName", "InterfaceDescription", false, "datalink_name", "datalink_description", nhw, naddrs);
    }

    public String[] init_interface() {
        try {
            devices = JpcapCaptor.getDeviceList();
        } catch (Exception ex) {
            TDeb.wri("=>" + ex.getMessage());
        }
        try {
            String res[] = new String[devices.length];
            for (int i = 0; i < devices.length; i++) {
                res[i] = devices[i].description;
                if (res[i] == null || res[i].length() == 0) {
                    res[i] = devices[i].name;
                }

            }
            return res;
        } catch (Throwable e) {
            //javax.swing.JOptionPane.showMessageDialog(null, "winpcap & jpcap seens not be installed", "Tcpdump", javax.swing.JOptionPane.INFORMATION_MESSAGE);
            TDeb.wri("winpcap & jpcap seens not be installed");
            return null;
        }
    }

    public String[] getIPinfo(int i) {

        String[] res = new String[3];
        jpcap.NetworkInterfaceAddress[] add;
        try {
            add = getdevice(Parent.getETH_interfaceidx()).addresses;
        } catch (Exception e) {
            javax.swing.JOptionPane.showMessageDialog(null, "winpcap & jpcap seens not be installed", "Tcpdump", javax.swing.JOptionPane.INFORMATION_MESSAGE);
            TDeb.wri("winpcap & jpcap seens not be installed");
            return null;
        }
        if (add == null) {
            return null;
        }
        if (add.length == 0) {
            return null;
        }
        if (add[i].subnet != null) {
            res[0] = add[i].subnet.toString();
        }
        if (add[i].broadcast != null) {
            res[1] = add[i].broadcast.toString();
        }
        if (add[i].destination != null) {
            res[2] = add[i].destination.toString();
        }
        return res;
    }

    public boolean isEthOK() {
        try {
            //on 64 bit =>java.lang.NoSuchMethodError: setRoutingOption => WEB=>Pb compilation.??
            jpcapt = jpcap.JpcapCaptor.openDevice(getdevice(Parent.getETH_interfaceidx()), 2000, Parent.isETH_promiscuous(), 20);
            jpcapSend = JpcapSender.openDevice(getdevice(Parent.getETH_interfaceidx()));
            // Pb au redemarage, pas test� :  jpcapSend= JpcapSender.openDevice(devicename);
        } catch (IOException e) {
            return false;
        }
        jpcapt.close();
        return true;

    }

    public void Eth_start() {
        if (Parent.getETH_interfaceidx() == -1) {
            Parent.CXerror("No Ethernet interface selected");
            return;
        }
        try {
            //
            jpcapt = jpcap.JpcapCaptor.openDevice(getdevice(Parent.getETH_interfaceidx()), 2000, Parent.isETH_promiscuous(), 20);
            jpcapSend = JpcapSender.openDevice(getdevice(Parent.getETH_interfaceidx()));
            // Pb au redemarage, pas test� :  jpcapSend= JpcapSender.openDevice(devicename);
        } catch (IOException e) {// THROWABLE
            Parent.CXerror("Can not open Eth device Number : " + Parent.getETH_interfaceidx()+"=>"+e.getMessage());
            javax.swing.JOptionPane.showMessageDialog(null, "winpcap & jpcap seens not be installed", "Tcpdump", javax.swing.JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        try {
            if (Parent.getETH_filter() != null) {
                jpcapt.setFilter(Parent.getETH_filter(), true);
            }
        } catch (java.io.IOException ex) { // THROWABLE
            TDeb.wri("could not set filter (incorrect) : " + Parent.getETH_filter()+"=>"+ex.getMessage());
        }
        TDeb.wri("Starting to listner on port ");
        jpcapt.setNonBlockingMode(true);
        // jpcapt.setPacketReadTimeout(10000); NOT SUPPORTED BY WINDOWS
        try {
            //  jpcapt.loopPacket(-1, this);
            while (Parent.isslrunning()) {
                // usefull to proced with 50ns??  
                T_ETH.sleep(1, 50);
                if (Parent.isslreading()) {
                    /// !!!! could lead to bug hs_err in  [wpcap.dll+0x16a05]
                    jpcapt.processPacket(-1, this);
                }
            }
        } catch (InterruptedException e) { // THROWABLE
            TDeb.wri("end of listner on port because : " + e.getMessage());
        }

        // or processPacket ??timeout??
    }

    @Override
    public void receivePacket(jpcap.packet.Packet packet) {
        Timestamp tstamp = new Timestamp(packet.sec);
        if (Parent.isslreading() == true) {
            if (Parent.getETH_Pack().compareTo("PACKET") == 0) {
                TDeb.wri("transmit ETH : hdr/" + packet.header.length + " Data:" + packet.data.length + " Time:" + tstamp.toString() + "." + packet.usec);
                Parent.read_in(new C_Packet(packet, Parent));
            } else {
                byte[] buff = null;
                if (Parent.getETH_Pack().compareTo("BUFFER") == 0) {
                    TDeb.wri("transit ETH Data : rcvd hdr/" + packet.header.length + " Data:" + packet.data.length + " Time:" + tstamp.toString() + "." + packet.usec);
                    buff = packet.data;
                    if (buff.length == 46) {
                        int newl = buff.length;
                        while (buff[newl - 1] == 0 && newl > 0) {
                            newl--;
                        }
                        buff = java.util.Arrays.copyOf(buff, newl);
                    }
                    buff = TUtils.addtab(packet.header, buff);

                } else if (Parent.getETH_Pack().compareTo("TRANSPORT") == 0) {
                    if (packet instanceof jpcap.packet.TCPPacket) {
                        buff = ((jpcap.packet.TCPPacket) packet).data;
                    }
                    if (packet instanceof jpcap.packet.UDPPacket) {
                        buff = ((jpcap.packet.UDPPacket) packet).data;
                    }
                }
                setStreambuff(buff);
                try {
                    Parent.ReadStream(buff.length);
                } catch (Exception ex) {
                    TDeb.wri("Pb with " + buff.length + "=>" + ex.getMessage());
                }
                setStreambuff(null);
            }
        }
    }

    @Override
    public void run() {
        try {
            while (Parent.getcxstatus().compareTo("starting") == 0) {
                sleep(10);
            }
        } catch (InterruptedException ex) {
            Parent.CXerror("port starting/ Pb during waiting [" + ex.getMessage() + "]");
        }
        if (Parent.isCX_autoopen() && Parent.isCX_openbyslave()) {
            Parent.CXopen();
        }
        Eth_start();
    }
}

class JpcapInputStream extends java.io.InputStream {

    private T_ETH dump = null;

    public JpcapInputStream(T_ETH t) {
        super();
        dump = t;
    }

    @Override
    public int available() {
        if (dump == null) {
            return -1;
        }
        if (dump.StreambuffSize() == -1) {
            return -1;
        }
        return dump.StreambuffSize();
    }

    @Override
    public int read() throws java.io.IOException {
        //read() return byte beetwen 0 and 255 (-1 is error)
        if (dump == null) {
            throw new java.io.IOException("TCPdump not available");
        }
        int i = dump.readStreambuff();
        if (i == -9999) {
            throw new java.io.IOException("Read buffer Stream Error");
        }
        if (i < 0) {
            i = i + 256;
        }
        return i;
    }
}
