package com.jdcm.eltrx;

import com.jdcm.analyse.Code_Model;
import com.jdcm.common.C_Packet;
import java.io.*;
import com.jdcm.common.D_Utils;
import java.util.Arrays;

/**
 * T_File.java Created on 28 decembre 2006, 13:41
 *
 * Slave used by FILE port
 *
 * @author Marc GIBOIN
 */
public class T_FS extends Thread implements I_Slave {

    private Long fsize = 0L;
    private final C_Port Parent;
    private final D_Debug TDeb;
    private final D_Utils TUtils = new D_Utils();
    private final Code_Model mod = null;
    
    private boolean shouldRead = true;
    private boolean shouldWrite = true;
    private byte[] openreceived = null;
    private byte[] lastSendBuffer = null;
    private long lastSendTime = 0L;

    /**
     * Creates a new instance of T_File
     * @param P the port supporting this slave
     */
    public T_FS(C_Port P) {
        Parent = P;
        TDeb = new D_Debug("T-File [" + Parent.getIndex() + "] : ");
        TDeb.setconsole(Parent.getConsole());
        setName(this.getClass().getSimpleName() + ":" + getName());
    }

    /**
     * Start the slave<br> - check the r/w possibility of the file <br>
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        if (Parent.getFile_addr() == null) {
            Parent.CXstop();
            return -1;
        }
        shouldRead = Parent.isFile_read();
        if (shouldRead && (Parent.getFile_addr().exists() == false || TUtils.canRead(Parent.getFile_addr()) == false)) {
            // not possible to read
            if (D_Utils.getFileStream(Parent.getFile_addr().getName(), "traces", getClass().getClassLoader()) == null) {
                Parent.CXerror("FILE read: " + Parent.getFile_addr().getName() + " does not exist or is not readable");
                return -1;
            }
        }
        TDeb.wri("Check : reading file :" + Parent.getFile_addr().getName() + ": " + shouldRead);
        shouldWrite = Parent.isFile_write();
        if (shouldWrite && Parent.istFile_addExt()) {
            // ADD file neme timestamp extention 
            java.text.SimpleDateFormat format = new java.text.SimpleDateFormat("-yyyyMMdd_HHmmssSSS");
            String ext = format.format(new java.util.Date(System.currentTimeMillis()));
            try {
                String fn = Parent.getFile_addr().getCanonicalPath();
                String fsep = System.getProperty("file.separator");
                if (fn.lastIndexOf(".") > fn.lastIndexOf(fsep) || (fn.lastIndexOf(".") > 0 && fn.lastIndexOf(fsep) == -1)) {
                    fn = fn.substring(0, fn.lastIndexOf(".")) + ext + fn.substring(fn.lastIndexOf("."));
                } else {
                    fn = fn + ext;
                }
                Parent.setFile_addr(fn);
            } catch (IOException ex) {
                return -1;
            }
        }
        if (shouldWrite && Parent.getFile_addr().exists() && Parent.getFile_addr().canWrite() == false) {
            // not possible to write
            shouldWrite = false;
            Parent.CXerror("FILE write: " + Parent.getFile_addr().getName() + " is not writable error while Stream openning");
            return -1;
        }
        if (Parent.getFile_addr().exists() && Parent.getFile_addr().canWrite()) {
            if (Parent.getMain().isConf_Enabled() == false && Parent.isFile_ClearatStart()) {
                TDeb.wri("check file : ask Ovewriting : " + Parent.getFile_addr().getName());
                if (1 == javax.swing.JOptionPane.showConfirmDialog(null, "Do you want to overwrite file : " + Parent.getFile_addr().getName() + " ?\r\n", "File alredy exist", javax.swing.JOptionPane.YES_NO_OPTION)) {
                    Parent.CXerror("FILE in: " + Parent.getFile_addr().getName() + " is not writable error while Stream openning");
                    return -1;
                }
            }
        }
        return 0;
    }

    /**
     * Open the slave<br> - open the stream of the file<br> - manage
     * isFile_ClearatStart <br>
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        java.io.FileOutputStream out = null;
        InputStream in = null;
        //----------------------in-----------------------------
        if (shouldRead) {
            try {
                in = new FileInputStream(Parent.getFile_addr());
            } catch (FileNotFoundException ex) {
            }
            if (in == null) {
                in = D_Utils.getFileStream(Parent.getFile_addr().getName(), "traces", getClass().getClassLoader());
            }
        }
        //----------------------out-----------------------------
        if (shouldWrite) {
            if (Parent.isFile_ClearatStart() || in == null) {
                out = getoutstream();
            } else {
                /*
                 * Pb :open=> delete file=> need reading befor out!=null...
                 * java.nio.channels.FileChannel fc=out.getChannel();
                 * fc.position(fc.size()); => Pb beacause directly read_in()
                 * without Readstream()
                 }
                 */
                byte[] res = null;
                try {
                    res = new byte[in.available()];
                } catch (IOException ex) {
                }
                if (in != null && res != null) {
                    try {
                        in.read(res);
                    } catch (IOException ex) {
                    }
                }
                out = getoutstream();
                if (res != null && res.length > 0) {
                    openreceived = res;
                    //=> need openned...to seend ;
                    if (out != null) {
                        try {
                            out.write(res);
                        } catch (IOException ex) {
                        }
                    }
                }
            }
        }
        if (in == null && out == null) {
            Parent.CXerror("FILE : could not open In / Out File Stream");
            return -1;
        } else {
            Parent.setin(in);
            Parent.setout(out);
            return 0;
        }
    }

    /**
     * Stop the slave<br>
     *
     * @return 0 if slave is Stoped correctly
     */
    @Override
    public int sl_close() {

        if (Parent.istFile_addExt() && Parent.getFile_addr() != null) {
            // REMOVE file neme timestamp extention 
            try {
                String fn = Parent.getFile_addr().getCanonicalPath();
                if (fn.contains("_") && fn.contains("-")) {
                    if (fn.lastIndexOf("_") - fn.lastIndexOf("-") == "_yyyyMMdd".length()) {
                        Parent.setFile_addr(fn.substring(0, fn.lastIndexOf("-"))
                                + fn.substring(fn.lastIndexOf("-") + "-yyyyMMdd_HHmmssSSS".length()));
                    }
                }
            } catch (IOException ex) {
                TDeb.wri("sl_close(): Pb while checking Ext=>" + ex.getMessage());
            }
        }
        if (Parent.getcxstatus().compareTo("connected") == 0) {
            TDeb.wri("Asking to close the thread");
            Parent.CXstop();
        }
        return 0;
    }

    private FileOutputStream getoutstream() {
        FileOutputStream out;
        try {
            out = new java.io.FileOutputStream(Parent.getFile_addr());
        } catch (FileNotFoundException ex) {
            Parent.CXerror("FILE : checked but no found!!");
            return null;
        }
        return out;
    }

    public boolean isEcho2Clear(byte[] buff) {
        if (lastSendBuffer == null || lastSendTime == 0L) {
            return false;
        }
        long time = System.currentTimeMillis();
        // 50 is not an good value, how to check that this reading "comes from me"???
        // my choice: same value sended 'not to log ago'... and clear values if the same is sended by somewhere else...
        boolean bool = time - lastSendTime < 50 && Arrays.equals(buff, lastSendBuffer);
        lastSendBuffer = null;
        lastSendTime = 0L;
        return bool;
    }

    public Code_Model getmod() {
        if (mod == null) {
            Parent.setcxstatus("error");
            Parent.CXstop();
            return null;
        }
        return mod;
    }

    private void sendbyte(byte[] data) {
        if (data == null) {
            return;
        }
        Parent.write_in(new C_Packet(data, Parent));
    }

    /**
     * Check file before reading/wrinting
     *
     * @param filename path and name of the file to check
     * @param toread true if the file is IN file
     *
     * @return the result of the file check
     *
     */
    public boolean checkfile(String filename, boolean toread) {
        if (filename == null) {/*
             * TDeb.wri("Check : not able to open null file");
             */
            return true;
        }
        if (toread) {
        } else {
            File file = new File(filename);


        }
        return true;
    }

    /**
     * Send the CX_packet cotents on a file <br> NB : Used only for OUT file
     *
     * @param p the Packet to write in the file
     * @return the length of the writed data
     */
    @Override
    public int write(C_Packet p) {
        byte[] res = null;
        if (p.getObj() == null) {
            return 0;
        }
        if (p.getObj() instanceof byte[]) {
            res = (byte[]) p.getObj();
        } else {
            if (Parent.isPar_serial()) {
                res = p.getSerial();
            } else {
                if (p.getObj() instanceof javax.swing.tree.TreeModel) {
                    javax.swing.tree.TreeModel tree = (javax.swing.tree.TreeModel) p.getObj();
                    if (tree.getRoot() instanceof javax.swing.tree.DefaultMutableTreeNode) {
                        javax.swing.tree.DefaultMutableTreeNode root = (javax.swing.tree.DefaultMutableTreeNode) tree.getRoot();
                        if (root.getUserObject() instanceof com.jdcm.common.XMLres) {
                            com.jdcm.common.XML2tree a = new com.jdcm.common.XML2tree();
                            res = a.node2XML(tree, root).getBytes();
                            //  return 0;
                        }
                    }
                } else {
                    res = (p.getObj().toString() + "\r\n").getBytes();
                }
            }
        }
        if (res == null || Parent.getout() == null) {
            return 0;
        }
        try {
            // need to set Position at the end of the file...if somebody else writed
            try {
                java.nio.channels.FileChannel fc = ((java.io.FileOutputStream) Parent.getout()).getChannel();
                fc.position(fc.size());
            } catch (IOException ex) {
            }
            Parent.getout().write(res, 0, res.length);
            lastSendTime = System.currentTimeMillis();
            lastSendBuffer = res;

        } catch (IOException ex) {
            if (p.getsource() == null || p.getsource().isSystem()) {
                TDeb.wri("Write_out error => closing port.");
            }
            Parent.CXstop();
            return res.length;
        }
        return res.length;
    }

    public void wait_State(String state) {
        while (Parent.getcxstatus().compareTo(state) != 0) {
            try {
                sleep(10);
            } catch (InterruptedException ex) {
                //ex.printStackTrace();
            }
        }
    }

    /**
     * Run the File Slave
     *
     * Used only for IN file
     *
     */
    @Override
    public void run() {
        //OUT file are writed without thread???

        wait_State("started");
        if (Parent.isCX_autoopen() && Parent.isCX_openbyslave()) {
            Parent.CXopen();
        }
        if (Parent.isFile_read() == false) {
            return;
        }
        TDeb.wri("---Start of FS thread file---");
        while (Parent.isslrunning()) {
            if (openreceived != null) {
                Parent.read_in(new C_Packet(openreceived, Parent));
            }
            try {
                sleep(5);
            } catch (InterruptedException ex) {
                TDeb.wri("FS slave running/ Pb during waiting [" + ex.getMessage() + "]");
            }
            int size = 0;
            while (Parent.isslrunning() && (Boolean) Parent.getCMDOptions().getValuefor("Par_PadtStr")) {
                //Create an event "Data avalable" FOR PAD
                if (Parent.available() > size) {
                    size = Parent.available();
                 //   System.out.println("File =>"+ size);
                    Parent.ReadStream(-1);
                }
                if (Parent.available() == 0) {
                    size = 0;
                }
                try {
                    sleep(1);
                } catch (InterruptedException ex) {
                    TDeb.wri("FS slave running/ Pb during waiting [" + ex.getMessage() + "]");
                }
            }
            while (Parent.isslreading()) {
                // System.err.println("FS dispo : "+Parent.available());
                try {
                    long nsize = ((FileInputStream) Parent.getin()).getChannel().size();
                    if (nsize < fsize) {
                        // !!! can do better !!!=> how & why...=>options...
                        //if removed byte the position to the end of the file to see net added
                        //CARE if other process DELETE data
                        // bug with windows need CRLF else nsize=0??
                        // ((FileInputStream) Parent.getin()).getChannel().position(0);
                    }
                    //  ((FileInputStream) Parent.getin()).getChannel().position(0);
                    fsize = nsize;
                } catch (IOException ex) {
                }
                // may be a descriptor file => size always 1 & avalable=1!
               //  System.out.println("File =>"+ fsize);
                Parent.ReadStream(-1);
                try {
                    sleep(5);
                } catch (InterruptedException ex) {
                    TDeb.wri("FS slave reading/ Pb during waiting [" + ex.getMessage() + "]");
                }
            }
        }
        sl_close();
        TDeb.wri("---End of FS thread---");
    }
}
