package com.jdcm.eltrx;

import com.jdcm.common.C_Packet;
import java.io.*;
import java.util.*;
import gnu.io.*;

/**
 * T_LTP.java Created on 20 mai 2007, 12:32
 *
 * Slave used On LPT Port NOT TESTED
 *
 * @author Marc GIBOIN
 */
public class T_LPT implements ParallelPortEventListener, I_Slave {

   private final  D_Debug TDeb;
    private final C_Port Parent;
    private ParallelPort lptPort;

    /**
     * Creates a new instance of T_LTP
     * @param P the port supporting this slave
     */
    public T_LPT(C_Port P) {
        Parent = P;
        TDeb = new D_Debug("T-LPT [" + Parent.getIndex() + "]");
        TDeb.setconsole(Parent.getConsole());
        //  init_LPT();
        if (lptPort != null) {
            try {
                lptPort.addEventListener(this);
            } catch (TooManyListenersException e) {
                System.err.println("Pb adding EventListener for Parallele port");
            }
            // ltpPort.notifyOnDataAvailable(true);
        }
    }

    /**
     * Start the slave<br>
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        if (getParallelePort() == null) {
            Parent.setcxstatus("error");
            Parent.CXstop();
            return -1;
        }
        return 0;
    }

    /**
     * Open the slave<br>
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        try {
            Parent.setin(getParallelePort().getInputStream());
            Parent.setout(getParallelePort().getOutputStream());
        } catch (IOException e) {
            Parent.CXerror("Parallele port : Couldn't get the connection to the 'in/out' stream.");
            return -1;
        }
        return 0;
    }

    /**
     * Close the LPT port<br>
     *
     * @return 0 if slave is Stoped correctly
     */
    @Override
    public int sl_close() {
        if (lptPort != null) {
            lptPort.close();
            lptPort = null;
        }
        return 0;
    }

    /**
     * Write a packet content on the interface<br>
     * NOT IMPLEMENTED
     *
     * @param p the CXpacket to be write on interface
     * @return the number of byte writed
     */
    @Override
    public int write(C_Packet p) {
        return 0;
    }

    public ArrayList<Object> init_interface() {
        ArrayList<Object> res = new ArrayList<>();
        Enumeration<gnu.io.CommPortIdentifier> en = null;
        try {
            //javax.comm.PortInUseException
            en = (Enumeration<gnu.io.CommPortIdentifier>) gnu.io.CommPortIdentifier.getPortIdentifiers();
        } catch (Exception ex) {
            TDeb.wri("Pb during Serial port loading : " + ex.getMessage());
        }
        // iterate through the ports.
        if (en != null) {
            gnu.io.CommPortIdentifier portId;
            while (en.hasMoreElements()) {
                portId = en.nextElement();
                TDeb.wri("Serial port found : " + portId.getName());
                if (portId.getPortType() == gnu.io.CommPortIdentifier.PORT_PARALLEL) {
                    if (res.contains(portId) == false) {
                        res.add(portId);
                    }
                }
            }
        }
        return res;
    }

    private void init_LPT() {
        /*  TDeb.wri("initialize LTP : " + Parent.getLPT_Port() + "/" + Parent.getLPT_Mode());
         ParallelPort ltpPort = null;
         CommPortIdentifier portId;
         try {
         portId = CommPortIdentifier.getPortIdentifier(Parent.getLPT_Port());
         } catch (NoSuchPortException ex) {
         ex.printStackTrace();
         return;
         }
         try {
         lptPort = (ParallelPort) portId.open(Parent.getRS_SysLab(), Parent.getRS_OpenTO());
         } catch (PortInUseException e) {
         TDeb.wri("Serial port : " + Parent.getLPT_Port() + " : Pb oppening (already used)");
         lptPort = null;
         return;
         }
         try {
         lptPort.setMode(ltpMode(Parent.getLPT_Mode()));
         } catch (UnsupportedCommOperationException e) {
         TDeb.wri("Serial port : " + Parent.getLPT_Port() + ": Pb configuring ");
         ltpPort = null;
         return;
         }*/
    }

    public int ltpMode(String mode) {
        if (mode.equals("ANY")) {
            return ParallelPort.LPT_MODE_ANY;
        }
        if (mode.equals("ECP")) {
            return ParallelPort.LPT_MODE_ECP;
        }
        if (mode.equals("EPP")) {
            return ParallelPort.LPT_MODE_EPP;
        }
        if (mode.equals("NIBBLE")) {
            return ParallelPort.LPT_MODE_NIBBLE;
        }
        if (mode.equals("PS2")) {
            return ParallelPort.LPT_MODE_PS2;
        }
        if (mode.equals("SPP")) {
            return ParallelPort.LPT_MODE_SPP;
        }
        return ParallelPort.LPT_MODE_ANY;
    }

    public ParallelPort getParallelePort() {
        return lptPort;
    }

    @Override
    public void parallelEvent(ParallelPortEvent pevent) {
        String debug = "LPT Event received : ";
        boolean bool = pevent.getNewValue();
        int a = 1;
        switch (pevent.getEventType()) {
            case ParallelPortEvent.PAR_EV_BUFFER:
                Parent.setRS_lights("B", bool, null);
                break;
            case ParallelPortEvent.PAR_EV_ERROR:
                Parent.setRS_lights("PE", bool, null);
                break;
        }
    }

    @Override
    public void start() {
        throw new UnsupportedOperationException("Not supported yet.");
        //Managed by listner
    }
    /*  public void run() {
     while (Parent.getcxstatus().compareTo("starting")==0){
     try {sleep(10);} catch (InterruptedException ex) { TDeb.wri("port starting/ Pb during waiting ["+ex.getMessage()+"]"); }
     }
     TDeb.wri("---Start of thread file---");
     while (Parent.isslrunning()){
     int size=0;
     try {sleep(50);} catch (InterruptedException ex) {TDeb.wri("slave running/ Pb during waiting ["+ex.getMessage()+"]"); }
     while (Parent.isslrunning() && (Boolean) Parent.getPort_Options().getValuefor("Par_PadtStr")){
     //Create an event "Data avalable" for Pad
     if (Parent.available()>size){
     size=Parent.available();
     Parent.ReadStream(-1);
     }
     if (Parent.available()==0){size=0;}
     try {sleep(1);} catch (InterruptedException ex) { ex.printStackTrace(); }
     }
     while (Parent.isslreading()){
     try {this.sleep(500);} catch (InterruptedException ex) { TDeb.wri("slave reading/ Pb during waiting ["+ex.getMessage()+"]"); }
     }
     }
     close();
     TDeb.wri("---End of thread---");
     }*/
}
