package com.jdcm.eltrx;

import com.jdcm.common.C_Packet;
import com.jdcm.common.A_Commande;
import com.jdcm.common.Options_Model;
import javax.swing.table.AbstractTableModel;
import com.jdcm.modifiers.matrix.mat_base;

/**
 * Matrice.java Created on 13 juillet 2006, 15:14
 *
 * Thread used to transmit data between different port. - use I_mat_elt for data
 * distribution implementation (FULL/RING/...)
 *
 * @author Marc GIBOIN
 */
public class T_Matrice extends Thread implements I_Command {

    private final D_Debug TDeb;
    private final I_Main Parent;
    private final Matrix_Model model;
    private final mat_base matrixtype = null;
    private boolean running;                    //true => running Thread
    private int TO_matrix;                      // => Not used =>just used for Thread ???
    private Options_Model option;

    /**
     * Creates a new instance of Matrice
     * @param P Main suppoting this matrix
     * @param c consol supporting this matrix
     */
    public T_Matrice(I_Main P, I_Console c) {
        running = false;
        Parent = P;
        TDeb = new D_Debug("Matrice");
        TDeb.setconsole(c);
        TO_matrix = 1000;
        initOptions();
        model = new Matrix_Model(this, Parent);
        setName(this.getClass().getSimpleName() + ":" + getName());
    }

    /**
     * provide the Matrix table data Model
     *
     * @return the Matrix data Model
     */
    public AbstractTableModel getmodel() {
        return model;
    }
    // used for module update()

    public void setmodelbools(boolean[][] bools) {
        model.setbools(bools);
    }

    /**
     * initialise Options for the Matrix
     */
    private void initOptions() {
        option = new Options_Model(this);
        // create & feel defaults NAME/VAL (&DEFAULT)/TYPE/DESC/USED/NULLONEMPTY/COMMENT
        option.addOption("Mat_Echo", false, "bool", "0/1", true, false, "true if a port send on itself");
        option.addOption("Mat_Bidir", false, "int", "0/1", true, false, "true if an link A>B is also a B>A");
        option.addOption("Mat_Type", null, "Class instanceof mat_base", "0/1", true, true, "managing the matrix");
        option.addOption("Mat_Stat", false, "bool", "false", true, false, "are Matrix Statistics collected/displayed");
        option.setTitle(getTitle());
    }
    // used for Group module beacause Port flag is in port ...

    public java.util.List<I_Port> getPorts() {
        return Parent.getPorts();
    }

    /**
     * provide the title of the
     *
     * @return
     */
    public String getTitle() {
        String res = "Matrix";
        if (getMatType() != null && getMatType().getClass().getSimpleName().contains("_")) {
            String txt = getMatType().getClass().getSimpleName();
            res += " [" + txt.substring(txt.indexOf("_") + 1) + "]";
        }
        return res;
    }

    @A_Commande(help = "set Matrix Statistics enabled & displayed")
    public void setMat_Stat(boolean bool) {
        if (option.getValuefor("Mat_Stat", true) != bool) {
            option.setValuefor("Mat_Stat", bool);
        }
        if (Parent instanceof C_Main) {
            ((C_Main) Parent).setMat_Stat(bool);
        }
    }

    @A_Commande(help = "set Matrix echo all interfaces input")
    public void setMat_Echo(boolean bool) {
        if (isMat_Echo() != bool) {
            option.setValuefor("Mat_Echo", bool);
            if (model != null) {
                model.update();
            }
            if (Parent instanceof C_Main) {
                ((C_Main) Parent).setMat_Echo(bool);
            }
        }
    }

    @A_Commande(help = "true if Matrix echo all interfaces input")
    public boolean isMat_Echo() {
        return option.getValuefor("Mat_Echo", true);
    }

    @A_Commande(help = "set Matrix bidir")
    public void setMat_Bidir(boolean bool) {
        if (isMat_Bidir() != bool) {
            option.setValuefor("Mat_Bidir", bool);
            if (model != null) {
                model.update();
            }
            if (Parent instanceof C_Main) {
                ((C_Main) Parent).setMat_Bidir(bool);
            }
        }
    }

    @A_Commande(help = "true if Matrix bidir")
    public boolean isMat_Bidir() {
        return option.getValuefor("Mat_Bidir", true);
    }

    /**
     * get the matrix rules
     *
     * @return the object instanceof mat_base managing the matrix or null
     */
    @A_Commande(help = "return the Object managing the matrix (RING/BUS/FULL/FREE/NULL)")
    public mat_base getMatType() {
        Object obj = option.getValuefor("Mat_Type");
        if (obj instanceof mat_base) {
            return (mat_base) obj;
        } else {
            return null;
        }
    }

    @A_Commande(help = "set the type of the matrix (RING/BUS/FULL/FREE/GRP/NULL)")
    public void setMatType(String txt) throws Exception {
        String clname = "com.jdcm.modifiers.matrix.mat_";
        if (txt.compareTo("OFF") == 0) {
            clname += "FULL";
        } else {
            clname += txt;
        }
        Class<?> cl = Class.forName(clname);
        mat_base res = (mat_base) cl.getConstructor(this.getClass()).newInstance(this);
        if (txt.compareTo("OFF") == 0) {
            ((com.jdcm.modifiers.matrix.mat_FULL) res).iscx = false;
        }
        setMatType(res);
    }

    public void removeMatrixCol(int col) {
        if (this.getMatType() instanceof com.jdcm.modifiers.matrix.mat_FREE) {
            for (int c = col + 1; c < model.getColumnCount(); c++) {
                for (int row = 0; row < model.getRowCount(); row++) {
                    Object o = model.getValueAt(row, c);
                    model.setValueAt(o, row, c - 1);
                }
            }
        } else {
            model.update();
            // this.getMatType().mx_init();
        }
        model.fireTableDataChanged();
    }

    public void moveMatrixCol(int col, boolean isUp) {
        if (this.getMatType() instanceof com.jdcm.modifiers.matrix.mat_FREE) {
            int shift = 1;
            if (isUp) {
                shift = -1;
            }

            if ((isUp && col < model.getColumnCount() && col > 1)
                    || (isUp == false && col < model.getColumnCount() - 1 && col > 0)) {
                for (int row = 0; row < model.getRowCount(); row++) {
                    Object stck0 = model.getValueAt(row, col + shift);
                    Object stck1 = model.getValueAt(row, col);
                    if (row + 1 == col) {
                        Object a = model.getValueAt(row, col);
                        Object b = model.getValueAt(row, col + shift);
                        model.setValueAt(model.getValueAt(row + shift, col + shift), row, col);
                        model.setValueAt(model.getValueAt(row + shift, col), row, col + shift);
                        model.setValueAt(b, row + shift, col);
                        model.setValueAt(a, row + shift, col + shift);
                    } else if (row + 1 == col + shift) {
                    } else {
                        model.setValueAt(stck0, row, col);
                        model.setValueAt(stck1, row, col + shift);
                    }

                }
            }
        } else {
            model.update();
        }
    }

    public void setMatType(mat_base matobj) throws Exception {
        //need to update for TCP_ltransit
        if (getMatType() != null && getMatType() == matobj) {
            return;
        }
        option.setValuefor("Mat_Type", matobj);
        option.setTitle(getTitle());
        matobj.mx_init();
        if (model != null) {
            model.update();
        }
    }

    @A_Commande(help = "set source to destination connection ex: >1:0:true")
    public void setConnection(String txt) {
        if (txt == null || txt.length() == 0 || txt.split(":").length < 3) {
            return;
        }
        String[] pars = txt.split(":");
        int src, dest;
        boolean bool = false;
        try {
            src = Integer.parseInt(pars[0]);
            dest = Integer.parseInt(pars[1]);
        } catch (NumberFormatException ex) {
            return;
        }
        if (pars[2].toLowerCase().startsWith("t") || pars[2].toLowerCase().startsWith("y") || pars[2].toLowerCase().startsWith("1")) {
            bool = true;
        }
        if (src >= 0 && dest >= 0) {
            setConnection(src, dest, bool);
        }
    }

    @A_Commande(help = "set source to destivation connection")
    public void setConnection(int src, int dest, boolean bool) {
        matrixtype.setConnection(src, dest, bool);
    }

    public void setTO_matrix(int TOmat) {
        TO_matrix = TOmat;
    }

    public void setTagValue(String portname, String tag) throws Exception {
        /* if (getType().startsWith("GRP") == false) {
         throw new java.lang.Exception("try to assign tags to a no GRP type matrix");
         }
         int targetidx = -1;
         for (I_Port port : Parent.getPorts()) {
         if (port.getPortName().compareTo(portname) == 0) {
         targetidx = port.getIndex();
         }
         }
         if (targetidx == -1) {
         throw new java.lang.Exception("port not found: " + portname);
         }
         model.setValueAt(tag, 0, targetidx+1);*/
    }

    public void mx_model(C_Packet d) {
        // 2 routes mechanism :
        //      - matrix (filled whith FULL/BUS/RING/FREE/GRP
        //       - if matrix is checked, check route in packet (set by port route)
        if (d == null) {
            return;
        }
        Parent.addpacket("" + d.hashCode(), System.currentTimeMillis(), "Matrice", d.toString());
        boolean dropped = true;
        for (int j = 0; j < Parent.getPorts().size(); j++) {
            int row_idx = d.getsource().getIndex();
            row_idx += getMatType().getRowCount(new boolean[][]{});
            if (model.getValueAt(row_idx, j + 1) instanceof Boolean) {
                if ((Boolean) (model.getValueAt(row_idx, j + 1))) {
                    if (isPortopenned(Parent.getPorts().get(j))) {
                        C_Packet e = (C_Packet) d.clone();//should be cloned ??
                        e.setdestination(Parent.getPorts().get(j));
                        TDeb.wri("<= [" + d.getsource() + ">" + j + "]/" + d.toString());
                        Parent.addpacket("" + e.hashCode(), System.currentTimeMillis(), "Matrice [" + d.hashCode() + "]", e.toString());
                        Parent.addMatStat(e);
                        Parent.write(e);
                        dropped = false;
                    }
                }
            }
        }
        if (dropped) {
            d.setdestination(null);
            Parent.addMatStat(d);
        }
        // if not found shoud addMatlog(d) "not routed"
    }

    /**
     * Check if port is able to receive data from matrix<br> NB : for exemple to
     * be connected later
     *
     * @param cx the sender port
     * @return true if oppenned
     *
     */
    public boolean isPortopenned(I_Port cx) {
        //tell matrix if ports is able to receive datas
        boolean res;
        if (cx == null) {
            return true;
        } // admin ports Cap reader...
        if (cx instanceof C_Port) {
            // NB : if Listner => should not be more than started...
            if (((C_Port) cx).getTCP_Mode().startsWith("TCP_L")) {
                return false;
            }
        }
        res = (cx.getcxstatus().compareTo("opened") == 0)
                || (cx.getcxstatus().compareTo("connected") == 0)
                || (cx.getcxstatus().compareTo("disconnected") == 0);
        return res;
    }

    private void cyclic() {
        while (running) {
            try {
                Thread.sleep(TO_matrix, 0);
            } catch (InterruptedException ex) {
                //ex.printStackTrace();
            }

        }
    }

    public void stopmat() {
        running = false;
    }

    @Override
    public javax.swing.ImageIcon getCMDIcon() {
        return ((new com.jdcm.common.D_Utils()).getIcon("Matrix.gif"));
    }

    @Override
    public Options_Model getCMDOptions() {
        return option;
    }

    @Override
    public java.util.List<I_Command> getCMDChilds(String child) {
        java.util.List<I_Command> res = new java.util.ArrayList<>();
        java.util.List<I_Command> fulllist = new java.util.ArrayList<>();//no childs
        for (I_Command chld : fulllist) {
            if (chld.getCMDName().compareTo(child) == 0) {
                res.add(chld);
            }
        }
        return res;
    }

    @Override
    public I_Command getCMDParent() {
        if (Parent instanceof I_Command) {
            return (I_Command) Parent;
        }
        return null;
    }

    @Override
    public String getCMDdesc() {
        return "Matrice \t" + getMatType().getClass().getSimpleName() + "\t" + "echo/bidir: " + isMat_Echo() + "/" + isMat_Bidir();
    }

    @Override
    public String getCMDName() {
        return "Matrix";
    }

    @Override
    public String getCMDdetails() {
        return "Mode :\t" + getMatType() + "\r\nis echo : \t" + this.isMat_Echo() + "\r\n detailled matrice table\r\n";
    }

    @Override
    public void run() {
        running = true;
        cyclic();
    }
}

class Matrix_Model extends AbstractTableModel {
    // for serializable interface

    private static final long serialVersionUID = 1L;
    private final I_Main Parent;
    private final T_Matrice Mat;
    private boolean[][] boolArray = null;

    public Matrix_Model(T_Matrice mat, I_Main m) {
        Parent = m;
        Mat = mat;
        boolArray = new boolean[Parent.getPorts().size()][Parent.getPorts().size()];
    }

    public void setbools(boolean[][] b) {
        boolArray = b;
    }

    public void update() {
        if (boolArray.length != Parent.getPorts().size()) {
            // add port=> DataStructure change
            boolean[][] bools = new boolean[Parent.getPorts().size()][Parent.getPorts().size()];
            Mat.getMatType().update(boolArray, bools);
        } else {
            Mat.getMatType().update(null, null);
        }
    }

    @Override
    public String getColumnName(int column) {
        if (column == 0) {
            if (Parent.getPorts().isEmpty()) {
                return "No Ports";
            }
            return "From";
        }
        if (Parent.getPorts().isEmpty()) {
            return "?Empty?";
        } else if (Parent.getPorts().size() <= column - 1) {
            return "?Absent?";
        } else if (Parent.getPorts().get(column - 1) == null) {
            return "?NULL?";
        }
        return Parent.getPorts().get(column - 1).getPortName();
    }

    @Override
    public int getColumnCount() {
        if (boolArray == null) {
            return 0;
        }
        return boolArray.length + 1;
    }

    @Override
    public int getRowCount() {
        if (boolArray == null) {
            return 0;
        } else if (Mat.getMatType() != null) {
            return Mat.getMatType().getRowCount(boolArray);
        } else {
            return boolArray.length;
        }
    }

    @Override
    public Object getValueAt(int aRow, int aColumn) {

        Object typeObj = Mat.getMatType().getValueAt(aRow, aColumn);
        if (typeObj != null) {
            //used for additional row (GRP
            return typeObj;
        }
        aRow = aRow - (getRowCount() - Parent.getPorts().size());
        if (aColumn == 0) {
            // titles of rows (should be a specific Model...
            if (Parent.getPorts().size() > aRow) {
                return Parent.getPorts().get(aRow);//.getPortName();
            }
            return "No Ports";
        } else {
            for (int idx : new Integer[]{aRow, aColumn - 1}) {
                if (Parent.getPorts().size() > idx) {
                    I_Port port = Parent.getPorts().get(idx);
                    if (port instanceof C_Port) {
                        if (((C_Port) port).getModeName().startsWith("IPL4")
                                && ((C_Port) port).getTCP_Mode().startsWith("TCP_L")) {
                            return "-";
                        }
                    }
                }
            }
            if (boolArray == null) {
                return "error";
            }
            if (aRow >= boolArray.length) {
                return "error row : " + aRow + "should be deleted";
            }
            if (aRow > 0 && aColumn - 1 >= boolArray[0].length) {
                return "error column: " + aColumn + "should be deleted";
            }
        }
        return boolArray[aRow][aColumn - 1];

    }

    @Override
    public boolean isCellEditable(int row, int column) {
        if (column == 0) {
            return false;
        }
        return Mat.getMatType().isCellEditable(row, column);
    }

    @Override
    public void setValueAt(Object value, int aRow, int aColumn) {
        if (aColumn == 0) {
            return;
        }
        Mat.getMatType().setValueAt(value, aRow, aColumn);
        aRow = aRow - (getRowCount() - Parent.getPorts().size());
        if (value instanceof Boolean) {
            //How does F_Machine is able to be a Value ???
            if (boolArray.length > 0 && boolArray[0].length >= 0 && aRow >= 0 && aColumn > 0) {
                if (aRow < boolArray.length && (aColumn - 1 < boolArray[0].length)) {
                    boolArray[aRow][aColumn - 1] = (Boolean) value;
                    if (Mat.isMat_Bidir()) {
                        boolArray[aColumn - 1][aRow] = (Boolean) value;
                    }
                    try {
                        this.fireTableDataChanged();
                    } catch (Exception ex) {
                        System.err.println("Matrix : setValueAt() =>" + ex.getMessage());
                    }
                }
            }
        }
    }
}
