package com.jdcm.eltrx;

import com.jdcm.common.C_Packet;
import java.io.*;
import java.util.*;
import gnu.io.*;

/**
 * Com.java Created on 19 janvier 2005, 10:17
 *
 * The serial interface management Slave
 * *http://christophej.developpez.com/tutoriel/java/javacomm/ see JDK 1.2 and
 * the Java(tm) Communications API Version 2.0 implemented bellow
 * http://java.sun.com/developer/Books/javaprogramming/cookbook/11.pdf=>
 * detailed explaination
 *
 * @author Marc GIBOIN
 */
public class T_RS implements SerialPortEventListener, RS485PortEventListener, I2CPortEventListener, RawPortEventListener, CommPortOwnershipListener, I_Slave {

   private final D_Debug TDeb;
    private final C_Port Parent;
    
    private RXTXPort serialPort; //SerialPort
    private boolean avalable;

    /**
     * au cas ou on veuille utiliser les evenelents plutot que le pooling de
     * "10ms"
     * @param P Port supporting this slave
     */
    public T_RS(C_Port P) {
        Parent = P;
        TDeb = new D_Debug("T-RS [" + Parent.getIndex() + "]");
        TDeb.setconsole(Parent.getConsole());

    }

    /**
     * Start the RS slave<br> - check serial port exist<br> - set lights<br> -
     * Update table port
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        init_Serial();
        if (serialPort == null) {
            Parent.CXerror("Couldn't start null port");
            return -1;
        }
        try {
            serialPort.addEventListener(this);
        } catch (TooManyListenersException e) {
            Parent.CXerror("serial port : Pb adding EventListener for serial port");
            //   System.err.println("Pb adding EventListener for serial port");
        }
        serialPort.notifyOnDataAvailable(true);
        Parent.setRS_lights(Parent.isRS_lights());
        setDTR_RTS();
        return 0;
    }

    /**
     * Open the RS slave<br>
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        // RS232 nb pas de SoTimeout pour liberer un read().
        try {
            Parent.setin(getSerialPort().getInputStream());
            Parent.setout(getSerialPort().getOutputStream());
        } catch (IOException e) {
            Parent.CXerror("serial port : Couldn't get the connection to the 'in/out' stream.");
            return -1;
        }
        return 0;
    }

    /**
     * Stop the RS slave<br>
     *
     * @return 0 if slave is Stoped correctly
     */
    @Override
    public int sl_close() {

        if (serialPort == null) {
            return 0;
        }
        try {
            serialPort.clearCommInput();
            serialPort.removeEventListener();
            Thread.sleep(50);
            serialPort.close();
            ((CommPortIdentifier) Parent.getRS_Port()).removePortOwnershipListener(this);
        } catch (UnsupportedCommOperationException | InterruptedException ex) {
            TDeb.wri("sl_close() =>" + ex.getMessage());
        }
        serialPort = null;
        Parent.initRSbox();
        return 0;
    }

    public SerialPort getSerialPort() {
        return serialPort;
    }

    private int init_Serial() {
        serialPort = null;
        CommPortIdentifier portId;
        try {
            portId = (CommPortIdentifier) Parent.getRS_Port();
        } catch (Exception ex) {
            TDeb.wri("Serial Port :" + Parent.getRS_Port() + " not found in port list");
            return -1;
        }
        if (init_interface().contains(portId) == false) {
            TDeb.wri("Serial Port :" + Parent.getRS_Port() + "is not found in avalable Ports");
            serialPort = null;
            return -1;
        }
        try {
            portId.addPortOwnershipListener(this);
        } catch (Exception ex) {
            TDeb.wri("Serial Port seems to be null");
            return -1;
        }
        TDeb.wri("initialize RS232 : " + portId.getName() + "/" + Parent.getRS_Rate() + "/" + Parent.getRS_Databit() + "/" + Parent.getRS_Parity() + "/" + Parent.getRS_Stopbit() + "/" + Parent.getRS_FlowCtrl());
        if (portId.getPortType() != CommPortIdentifier.PORT_SERIAL) {
            TDeb.wri("Serial Port :" + Parent.getRS_Port() + " is not a SERIAL PORT");
            return -1;
        }
        if (portId.isCurrentlyOwned()) {
            TDeb.wri("Serial port : " + portId.getName() + " : isCurrentlyOwned by " + portId.getCurrentOwner());
            serialPort = null;
            return -1;
        }
        try {
            serialPort = (RXTXPort) portId.open(Parent.getRS_SysLab(), Parent.getRS_OpenTO()); //SerialPort
            try {
                if (serialPort != null) {
                    // Set port settings.
                    serialPort.setSerialPortParams(
                            new Integer(Parent.getRS_Rate()),
                            Databits("" + Parent.getRS_Databit()),
                            Stopbits(Parent.getRS_Stopbit()),
                            Parity(Parent.getRS_Parity()));
                    // Set flow control.
                    try {
                        serialPort.setFlowControlMode(FlowControl(Parent.getRS_FlowCtrl()));
                    } catch (Exception e) {
                        TDeb.wri("Serial port : " + portId.getName() + " : Pb Unsupported flow control : " + Parent.getRS_FlowCtrl());
                    }
                }
            } catch (UnsupportedCommOperationException e) {
                TDeb.wri("Serial port : " + portId.getName() + ": Pb configuring =>" + e.getMessage());
                serialPort = null;
                return -1;
            }
        } catch (PortInUseException e) {
            TDeb.wri("Serial port : " + portId.getName() + " : Pb oppening =>" + e.getMessage());
            serialPort = null;
            return -1;
        }
        TDeb.wri("Serial port : " + portId.getName() + ": configuring seems OK");
        return 0;
    }

    public String getRXTXinfo() {
        if (serialPort == null) {
            return "";
        }
        try {
            return "UART type : " + serialPort.getUARTType();
        } catch (UnsatisfiedLinkError | UnsupportedCommOperationException er) {
            return "UART unknown";
        }
    }

    public Object getPort(String name) {
        ArrayList<gnu.io.CommPortIdentifier> list = init_interface();
        for (gnu.io.CommPortIdentifier port : list) {
            if (port.getName().compareTo(name) == 0) {
                return port;
            }
        }
        return null;
    }

    public static ArrayList<gnu.io.CommPortIdentifier> init_interface() {
        ArrayList<gnu.io.CommPortIdentifier> res = new ArrayList<>();
        java.util.Enumeration<gnu.io.CommPortIdentifier> en = null;
        try {
            //javax.comm.PortInUseException
            en = gnu.io.CommPortIdentifier.getPortIdentifiers();
        } catch (Exception ex) {
            System.err.println("Pb during Serial port loading : " + ex.getMessage());
        }
        // iterate through the ports.
        if (en != null) {
            //     could be:   res=Collections.list(en);
            gnu.io.CommPortIdentifier portId;
            while (en.hasMoreElements()) {
                portId = en.nextElement();
                //  System.out.println("Serial port found : " + portId.getName());
                if (portId.getPortType() == gnu.io.CommPortIdentifier.PORT_SERIAL
                        || portId.getPortType() == gnu.io.CommPortIdentifier.PORT_RAW
                        || portId.getPortType() == gnu.io.CommPortIdentifier.PORT_RS485
                        || portId.getPortType() == gnu.io.CommPortIdentifier.PORT_I2C) {
                    if (res.contains(portId) == false) {
                        res.add(portId);
                    }
                }
            }
        }
        return res;
    }

    public String COMtypes(int typeidx) {
        if (typeidx == gnu.io.CommPortIdentifier.PORT_SERIAL) {
            return "SERIAL";
        } else if (typeidx == gnu.io.CommPortIdentifier.PORT_RAW) {
            return "RAW";
        } else if (typeidx == gnu.io.CommPortIdentifier.PORT_RS485) {
            return "RS485";
        } else if (typeidx == gnu.io.CommPortIdentifier.PORT_I2C) {
            return "I2C";
        } else {
            return "Unkbown";
        }
    }

    public int Databits(String databits) {
        if (databits.equals("5")) {
            return SerialPort.DATABITS_5;
        }
        if (databits.equals("6")) {
            return SerialPort.DATABITS_6;
        }
        if (databits.equals("7")) {
            return SerialPort.DATABITS_7;
        }
        if (databits.equals("8")) {
            return SerialPort.DATABITS_8;
        }
        return SerialPort.DATABITS_8;
    }

    public int Stopbits(String stopbits) {
        if (stopbits.equals("1")) {
            return SerialPort.STOPBITS_1;
        }
        if (stopbits.equals("1.5")) {
            return SerialPort.STOPBITS_1_5;
        }
        if (stopbits.equals("2")) {
            return SerialPort.STOPBITS_2;
        }
        return SerialPort.STOPBITS_1;
    }

    /**
     * Converts a
     * <code>String</code> describing a flow control type to an
     * <code>int</code> type defined in
     * <code>SerialPort</code>.
     *
     * @param flowControl A <code>string</code> describing a flow control type.
     * @return An <code>int</code> describing a flow control type.
     */
    private int FlowControl(String flowControl) {
        if (flowControl.toLowerCase().equals("none")) {
            return SerialPort.FLOWCONTROL_NONE;
        }
        if (flowControl.toLowerCase().equals("xon/xoff out")) {
            return SerialPort.FLOWCONTROL_XONXOFF_OUT;
        }
        if (flowControl.toLowerCase().equals("xon/xoff in")) {
            return SerialPort.FLOWCONTROL_XONXOFF_IN;
        }
        if (flowControl.toLowerCase().equals("rts/cts in")) {
            return SerialPort.FLOWCONTROL_RTSCTS_IN;
        }
        if (flowControl.toLowerCase().equals("rts/cts out")) {
            return SerialPort.FLOWCONTROL_RTSCTS_OUT;
        }
        return SerialPort.FLOWCONTROL_NONE;
    }

    public int Parity(String parity) {
        if (parity.toLowerCase().equals("none")) {
            return SerialPort.PARITY_NONE;
        }
        if (parity.toLowerCase().equals("even")) {
            return SerialPort.PARITY_EVEN;
        }
        if (parity.toLowerCase().equals("odd")) {
            return SerialPort.PARITY_ODD;
        }
        if (parity.toLowerCase().equals("space")) {
            return SerialPort.PARITY_SPACE;
        }
        if (parity.toLowerCase().equals("mark")) {
            return SerialPort.PARITY_MARK;
        }
        return SerialPort.PARITY_NONE;
    }

    public void setDTR_RTS() {

        try {
            if (serialPort.isDTR() != Parent.isRS_DTR()) {
                serialPort.setDTR(Parent.isRS_DTR());
                TDeb.wri("DTR modified to :" + Parent.isRS_DTR());
                Parent.setRS_lights("DTR", serialPort.isDTR(), null);
            }
            if (serialPort.isRTS() != Parent.isRS_RTS()) {
                serialPort.setRTS(Parent.isRS_RTS());
                TDeb.wri("RTS modified to :" + Parent.isRS_RTS());
                Parent.setRS_lights("RTS", serialPort.isRTS(), null);
            }
            Parent.setRS_lights("DTR", serialPort.isDTR(), null);
            Parent.setRS_lights("RTS", serialPort.isRTS(), null);
        } catch (Exception ex) {
            TDeb.wri("DTR/RTS modify error [" + ex.getMessage() + "]");
        }

    }

    public boolean isavailable() {
        return avalable;
    }

    public void clearavailable() {
        //used after reading to set the LED to false
        avalable = false;
    }

    public void setlights(boolean bool) {
        try {
            TDeb.wri("set lights notifications : " + bool);
            serialPort.notifyOnBreakInterrupt(bool);
            serialPort.notifyOnCTS(bool);
            serialPort.notifyOnDSR(bool);
            serialPort.notifyOnCarrierDetect(bool);
            serialPort.notifyOnRingIndicator(bool);
            serialPort.notifyOnParityError(bool);
            serialPort.notifyOnOverrunError(bool);
            serialPort.notifyOnOutputEmpty(bool);
            serialPort.notifyOnFramingError(bool);

            // serialPort.notifyOnDataAvailable(true);=> made on constructor
            //Init lights
            Parent.setRS_lights("CTS", serialPort.isCTS(), null);
            Parent.setRS_lights("DSR", serialPort.isDSR(), null);
            Parent.setRS_lights("DCD", serialPort.isCD(), null);
            Parent.setRS_lights("RI", serialPort.isRI(), null);
            Parent.setRS_lights("RTS", serialPort.isRTS(), null);
            Parent.setRS_lights("DTR", serialPort.isDTR(), null);
            Parent.setRS_lights("PE", false, null);
            Parent.setRS_lights("FE", false, null);
            Parent.setRS_lights("OE", false, null);
            Parent.setRS_lights("OBE", false, null);
            Parent.setRS_lights("DRX", false, null);
            Parent.setRS_lights("BI", false, null);
        } catch (Exception e) {
            TDeb.wri("RS Couldn't notify (port closed?)");
        }
    }

    /**
     * Write a packet content on the serial port
     *
     * @param p the CXpacket to be write on interface
     * @return the number of byte writed
     */
    @Override
    public int write(C_Packet p) {
        byte[] res = null;
        if (p.getObj() instanceof byte[]) {
            res = (byte[]) p.getObj();
        }
        if (res == null || Parent.getout() == null) {
            return 0;
        }
        try {
            Parent.setRS_lights("OBE", false, "/sending : " + res.length);
            //!!! blocking if hw control serial port N/A ! or bad drivers or...
            //=> TODO : Sheduld a Timeout..(5 sec?)
            Parent.getout().write(res, 0, res.length);
        } catch (IOException ex) {
            if (p.getsource() != null || p.getsource().isSystem() == false) {
                TDeb.wri("Write_out error (closing port) =>" + ex.getMessage());
            }
            Parent.CXstop();
            return res.length;
        }
        return res.length;
    }

    @Override
    public void ownershipChange(int i) {
        String typ = "unknown";
        switch (i) {
            case CommPortOwnershipListener.PORT_OWNED:
                typ = "PORT_OWNED";
                break;
            case CommPortOwnershipListener.PORT_OWNERSHIP_REQUESTED:
                typ = "PORT_OWNERSHIP_REQUESTED";
                break;
            case CommPortOwnershipListener.PORT_UNOWNED:
                typ = "PORT_UNOWNED";
                break;
        }
        TDeb.wri("PortOwnershipEvent:" + typ);
    }

    @Override
    public void RS485Event(RS485PortEvent rspe) {
        TDeb.wri("COM RS485 serial Event received : [" + rspe.getEventType() + "=>" + rspe.getNewValue() + "]");
        COMEvent(rspe.getNewValue(), rspe.getEventType());
    }

    @Override
    public void serialEvent(SerialPortEvent event) {
        TDeb.wri("COM serial Event received : [" + event.getEventType() + "=>" + event.getNewValue() + "]");
        COMEvent(event.getNewValue(), event.getEventType());
    }

    @Override
    public void I2CEvent(I2CPortEvent icpe) {
        TDeb.wri("COM I2C Event received : [" + icpe.getEventType() + "=>" + icpe.getNewValue() + "]");
        COMEvent(icpe.getNewValue(), icpe.getEventType());
    }

    @Override
    public void RawEvent(RawPortEvent rpe) {
        TDeb.wri("COM Raw Event received : [" + rpe.getEventType() + "=>" + rpe.getNewValue() + "]");
        COMEvent(rpe.getNewValue(), rpe.getEventType());
    }

    private void COMEvent(boolean bool, int evtnb) {
        // !!?? all Serial EventType use same Values for BI/OE/... :
        //http://show.docjava.com/book/cgij/jdoc/constant-values.html#gnu.io.RS485PortEvent.BI
        switch (evtnb) {
            case SerialPortEvent.BI:
                Parent.setRS_lights("BI", bool, null);
                break;
            case SerialPortEvent.OE:
                Parent.setRS_lights("OE", bool, null);
                break;
            case SerialPortEvent.FE:
                Parent.setRS_lights("FE", bool, null);
                break;
            case SerialPortEvent.PE:
                Parent.setRS_lights("PE", bool, null);
                break;
            case SerialPortEvent.CD:
                Parent.setRS_lights("DCD", serialPort.isCD(), null);
                break;
            case SerialPortEvent.CTS:
                Parent.setRS_lights("CTS", serialPort.isCTS(), null);
                Parent.setRS_lights("RTS", serialPort.isRTS(), null);
                break;
            case SerialPortEvent.DSR:
                Parent.setRS_lights("DSR", serialPort.isDSR(), null);
                break;
            case SerialPortEvent.RI:
                Parent.setRS_lights("RI", serialPort.isRI(), null);
                break;
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                Parent.setRS_lights("OBE", bool, null); //serialPort.isRI()
                break;
            case SerialPortEvent.DATA_AVAILABLE:
                TDeb.wri("Data available : " + bool);
                avalable = bool;
                Parent.setRS_lights("DRX", bool, "/disp : " + Parent.available());
                // if slave reading=> read data (managed by slreading  in ReadStream)
                Parent.ReadStream(-1);
                break;

        }
    }

    @Override
    public void start() {
        //Managed by Serial event Litner
    }
    /*
     public void run() {
     // de l'utilitee du thread ???'
     while (Parent.getcxstatus().compareTo("starting")==0){
     //     try {sleep(10);} catch (InterruptedException ex) { TDeb.wri("port starting/ Pb during waiting ["+ex.getMessage()+"]"); }
     }
     TDeb.wri("---Start of thread file---");
     while (Parent.isslrunning()){
     //     try {sleep(50);} catch (InterruptedException ex) {TDeb.wri("slave running/ Pb during waiting ["+ex.getMessage()+"]"); }
     while (Parent.isslreading()){
     try {wait(500);} catch (InterruptedException ex) { TDeb.wri("slave reading/ Pb during waiting ["+ex.getMessage()+"]"); }
     //    TDeb.wri("----RS loop----"); //!!! closing after 500 ms
     }
     }
     TDeb.wri("---End of thread---");
     }*/
}
