package com.jdcm.eltrx;

import com.jdcm.common.C_Packet;
import javax.sound.sampled.*;

/**
 * T_SND.java Created on 09/05/2011
 *
 * Slave for SOUND interface
 * http://download.oracle.com/javase/tutorial/sound/sampled-overview.html
 *
 * @author Marc GIBOIN
 */
public class T_SND extends Thread implements I_Slave, javax.sound.sampled.LineListener {

   private final D_Debug TDeb;
       private final C_Port Parent;
       
   private TargetDataLine targetline = null;
   private SourceDataLine sourceline = null;
    private boolean avalable;

    /**
     * au cas ou on veuille utiliser les evenelents plutot que le pooling de
     * "10ms"
     */
    public T_SND(C_Port P) {
        Parent = P;
        TDeb = new D_Debug("T-SND [" + Parent.getIndex() + "]");
        TDeb.setconsole(Parent.getConsole());
        setName(this.getClass().getSimpleName() + ":" + getName());
    }

    /**
     * Start the SND slave<br> - nothing<br>
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        return 0;
    }

    /**
     * Open the SND slave<br>
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        try {
            //linein.addLineListener(this);
            // linein.open();
            if (Parent.getSND_linein() != null) {
                if (Parent.getSND_linein() instanceof Mixer) {
                    targetline = AudioSystem.getTargetDataLine(null, ((Mixer) Parent.getSND_linein()).getMixerInfo());
                } else if (Parent.getSND_lineout() instanceof TargetDataLine) {
                    targetline = (TargetDataLine) Parent.getSND_linein();
                }
                targetline.open();
                //  tl.getControls()[0].getType();
                Parent.setSND_formatin(targetline.getFormat());
                Parent.setin(new javax.sound.sampled.AudioInputStream(targetline));
                targetline.start();
            }
        } catch (Exception e) {
            Parent.CXerror("Sound pb while opening input (Targetline): " + e.getMessage());
            return -1;
        }
        try {
            if (Parent.getSND_lineout() != null) {
                if (Parent.getSND_lineout() instanceof Mixer) {
                    sourceline = AudioSystem.getSourceDataLine(null, ((Mixer) Parent.getSND_lineout()).getMixerInfo());
                } else if (Parent.getSND_lineout() instanceof SourceDataLine) {
                    sourceline = (SourceDataLine) Parent.getSND_lineout();
                }
                Parent.setSND_formatout(sourceline.getFormat());
                sourceline.open();
                sourceline.start();
            }
        } catch (Exception e) {
            Parent.CXerror("Sound pb while opening output (SourceLine)" + e.getMessage());
            return -1;
        }

        return 0;
    }

    /**
     * Stop the RS slave<br>
     *
     * @return 0 if slave is Stoped correctly
     */
    @Override
    public int sl_close() {
        if (targetline != null) {
            targetline.stop();
            targetline.close();
            targetline = null;
        }
        if (sourceline != null) {
            sourceline.stop();
            sourceline.close();
            sourceline = null;
        }
        return 0;
    }

    public AudioFormat[] initFormat() {
        Float rate = Float.valueOf("44100").floatValue();
        int chanels = 1;// (mono/stereo)
        int samplesize = 8;
        boolean bigendian = false;
        AudioFormat af1;
        try {
            af1 = new AudioFormat(AudioFormat.Encoding.ALAW, rate, samplesize, chanels, chanels * (samplesize / 8), rate, bigendian);
            return new AudioFormat[]{af1};
        } catch (Exception ex) {
            TDeb.wri("Pb creating format :" + ex.getMessage());
            return null;
        }
    }

    public javax.sound.sampled.Line[] init_interface(String type) {
        java.util.List<javax.sound.sampled.Line> res = new java.util.ArrayList<>();
        Mixer.Info[] mixers = AudioSystem.getMixerInfo();
        // res.add("null");
        for (int i = 0; i < mixers.length; i++) {
            try {
                Mixer tempmix = AudioSystem.getMixer(mixers[i]);
                if (type.startsWith("linesin") && tempmix.getTargetLineInfo().length > 0) {
                    if (tempmix.getClass().toString().endsWith("PortMixer") == false) {
                        res.add(tempmix);
                        for (Line.Info inf : tempmix.getTargetLineInfo()) {
                            Line.Info[] a = AudioSystem.getTargetLineInfo(inf);
                            for (Line.Info b : a) {
                                if (b.getLineClass().toString().startsWith("interface javax.sound.sampled.TargetDataLine")) {
                                    //      res.add(AudioSystem.getLine(b));
                                }
                            }
                        }
                    }
                } else if (type.startsWith("linesout") && tempmix.getSourceLineInfo().length > 0) {
                    if (tempmix.getClass().toString().endsWith("PortMixer") == false) {
                        res.add(tempmix);
                        for (Line.Info inf : tempmix.getSourceLineInfo()) {
                            Line.Info[] a = AudioSystem.getSourceLineInfo(inf);
                            for (Line.Info b : a) {
                                if (b.getLineClass().toString().startsWith("interface javax.sound.sampled.SourceDataLine")) {
                                    //  res.add(AudioSystem.getLine(b));
                                }
                            }
                        }
                    }
                }

            } catch (Exception ex) {
            }
        }
        return res.toArray(new javax.sound.sampled.Mixer[]{});

    }

    public boolean isavailable() {
        return avalable;
    }

    public void clearavailable() {
        avalable = false;
    }

    /**
     * Write a packet content on the serial port
     *
     * @param p the CXpacket to be write on interface
     * @return the number of byte writed
     */
    @Override
    public int write(C_Packet p) {
        byte[] res = null;
        if (p.isbuffer()) {
            res = p.getBytes();
        }
        if (res != null && sourceline != null) {
            try {
                sourceline.write(res, 0, res.length);
            } catch (Exception ex) {
                if (p.getsource() != null || p.getsource().isSystem() == false) {
                    TDeb.wri("Write_out error => closing port.");
                }
                Parent.CXstop();
            }
            return res.length;
        } else {
            return 0;
        }
    }

    public int wait_State(String state) {
        String nowstate = Parent.getcxstatus();
        while (nowstate.compareTo(state) != 0) {
            nowstate = Parent.getcxstatus();
            if (nowstate.startsWith("clos") || nowstate.startsWith("stop")) {
                return -1;
            }
            try {
                sleep(10);
            } catch (InterruptedException ex) {
                TDeb.wri("sleep error: " + ex.getMessage());
                return -1;
            }
        }
        return 0;
    }

    @Override
    public void run() {
        TDeb.wri("---Start of thread file---");
        if (Parent.isCX_autoopen() && Parent.isCX_openbyslave()) {
            if (wait_State("started") == 0) {
                TDeb.wri("started reached => try to open");
                Parent.CXopen();
            } else {
                Parent.CXerror("Abnormal SND slave terminaison while waiting for started");
                return;
            }
        }

        while (Parent.isslrunning()) {
            try {
                sleep(1);
            } catch (InterruptedException ex) {
                TDeb.wri("slave running/ Pb during waiting [" + ex.getMessage() + "]");
            }
            while (Parent.isslreading()) {
                if (targetline.available() > 0) {
                    Parent.ReadStream(-1);
                }
                try {
                    sleep(1);
                } catch (InterruptedException ex) {
                    TDeb.wri("slave reading/ Pb during waiting [" + ex.getMessage() + "]");
                }
            }
        }
        TDeb.wri("---End of thread---");
    }

    @Override
    public void update(LineEvent event) {
        TDeb.wri(event.toString());
    }
}
