package com.jdcm.eltrx;

import com.jdcm.common.C_Packet;
import java.io.IOException;
import usb.core.*;

/**
 * USB slave for USB PORT on TEST !!
 *
 * http://www.tel.fer.hr/_download/repository/JavaUSBforWindowsWeb.pdf
 * http://www.steelbrothers.ch/jusb/
 *
 * @author Marc GIBOIN
 */
public class T_USB extends Thread implements I_Slave {

    private final D_Debug TDeb;
    private final C_Port Parent;
    private usb.core.Interface interf;
    private usb.core.Endpoint endp;

    /**
     * au cas ou on veuille utiliser les evenelents plutot que le pooling de
     * "10ms"
     *
     * @param P the port supporting this slave
     */
    public T_USB(C_Port P) {
        Parent = P;
        TDeb = new D_Debug("T-USB [" + Parent.getIndex() + "]");
        TDeb.setconsole(Parent.getConsole());
        setName(this.getClass().getSimpleName() + ":" + getName());
    }

    /**
     * Start the USB slave<br>
     *
     * @return 0 if slave is started correctly
     */
    @Override
    public int sl_start() {
        interf = null;
        endp = null;
        Object ob = Parent.getUSB_Element();
        if (ob instanceof usb.core.Interface) {
            interf = (usb.core.Interface) Parent.getUSB_Element();
        } else if (ob instanceof usb.core.Endpoint) {
            endp = (usb.core.Endpoint) Parent.getUSB_Element();
        } else {
            Parent.CXerror("no interface selected");
            return -1;
        }

        return 0;
    }

    /**
     * Open the USB slave<br>
     *
     * @return 0 if slave is Opened correctly
     */
    @Override
    public int sl_open() {
        if (interf == null && endp == null) {
            Parent.CXerror("no selected interface");
            return -1;
        }
        try {
            if (interf != null) {
                for (int i = 0; i < interf.getNumEndpoints(); i++) {
                    Endpoint ep = interf.getEndpoint(i);
                    if (ep.getType().compareTo("bulk") == 0) {
                        if (ep.isInput() && Parent.getin() == null) {
                            Parent.setin(ep.getInputStream());
                            TDeb.wri("USB input openned with :" + ep.getEndpointAddress());
                        }
                        if (ep.isInput() == false && Parent.getout() == null) {
                            Parent.setout(ep.getOutputStream());
                            TDeb.wri("USB output openned with :" + ep.getEndpointAddress());
                        }
                    }
                }
            } else {
                if (endp.getType().compareTo("bulk") == 0) {
                    if (endp.isInput() && Parent.getin() == null) {
                        Parent.setin(endp.getInputStream());
                        TDeb.wri("USB input openned with :" + endp.getEndpointAddress());
                    }
                    if (endp.isInput() == false && Parent.getout() == null) {
                        Parent.setout(endp.getOutputStream());
                        TDeb.wri("USB output openned with :" + endp.getEndpointAddress());
                    }
                }
            }
            // on ouvre les flux des connect
        } catch (IOException e) {
            Parent.CXerror("USB Couldn't get I/O for interface: " + e.getMessage());
            return -1;
        }
        return 0;
    }

    /**
     * Stop the USB slave<br>
     *
     * @return 0 if slave is Stoped correctly
     */
    @Override
    public int sl_close() {
        interf = null;
        return 0;
    }

    /**
     * Write a packet content on the USB port
     *
     * @param p the CXpacket to be write on interface
     * @return the number of byte writed
     */
    @Override
    public int write(C_Packet p) {
        if (Parent.getout() == null) {
            return -1;
        }
        if (p.getObj() instanceof byte[]) {
            try {
                if (Parent.getout() != null) {
                    Parent.getout().write(p.getBytes());
                } else if (endp.isInput() == false && endp.getType().startsWith("interrupt")) {
                    endp.sendInterrupt(p.getBytes());
                }
            } catch (IOException ex) {
                TDeb.wri("Write Exception: " + ex.getMessage());
            }
        }

        return 0;
    }

    public Host getUSBhost() {
        try {
            return HostFactory.getHost();

            /*
             long start = System.currentTimeMillis();
             long end = (OBSERVATION_TIME * 1000) + start;
             while (System.currentTimeMillis() <= end) {
             continue;
             }
             */
        } catch (java.lang.UnsatisfiedLinkError | java.io.IOException ex) {
            // should be library (jusb.dll/.so) missing
            // include USBException
            TDeb.wri(ex.getMessage());
        } catch (SecurityException e) {
            TDeb.wri("USB permissions problem:");
            TDeb.wri(e.getMessage());
            //System.exit(1);
        }
        return null;
    }

    public int wait_State(String state) {
        String nowstate = Parent.getcxstatus();
        while (nowstate.compareTo(state) != 0) {
            nowstate = Parent.getcxstatus();
            if (nowstate.startsWith("clos") || nowstate.startsWith("stop")) {
                return -1;
            }
            try {
                sleep(10);
            } catch (InterruptedException ex) {
                TDeb.wri("sleep error: " + ex.getMessage());
                return -1;
            }
        }
        return 0;
    }

    @Override
    public void run() {
        TDeb.wri("Start Slave USB");
        if (Parent.isCX_autoopen() && Parent.isCX_openbyslave()) {
            if (wait_State("started") == 0) {
                TDeb.wri("started reached => try to open");
                Parent.CXopen();
            } else {
                Parent.CXerror("Abnormal USB slave terminaison while waiting for started");
                return;
            }
        }
        while (Parent.isslrunning()) {
            while (Parent.isslreading()) {
                if (endp != null && endp.isInput() && endp.getType().startsWith("interrupt")) {
                    try {
                        byte[] by = endp.recvInterrupt();
                        if (by != null && by.length > 0) {
                            Parent.read_in(new C_Packet(by, Parent));
                        }
                    } catch (IOException ex) {
                        TDeb.wri("interrupt reading: " + ex.getMessage());
                    }
                } else if (Parent.getin() != null) {
                    Parent.ReadStream(-1);
                }
                try {
                    sleep(5);
                } catch (InterruptedException ex) {
                    TDeb.wri(" USB slave reading/ Pb during waiting [" + ex.getMessage() + "]");
                }
            }
        }
        sl_close();
        TDeb.wri("---End of USB thread---");
    }
}
