package com.jdcm.modifiers.stacks;

import com.jdcm.common.D_Utils;
import java.util.List;
import java.util.Arrays;
import java.net.InetAddress;
import com.jdcm.common.C_Packet;
import com.jdcm.common.O_MACaddress;

/**
 * ARP is used to manage IP adressses on Ethernet link - ARP table is set as
 * OptionsModel() entry (then editable)
 *
 * @author Marc GIBOIN
 */
public class Stack_ARP extends Stack_base {

    private final D_Utils util = new D_Utils();
    private final List<EthPack> waitting = new java.util.ArrayList<>();

    public Stack_ARP() {
        // byte[] gw = {-64, -88, 0x78, 0x01};
        super();
        byte[] net = {0x00, 0x01};
        byte[] prot = {0x08, 0x00};
        int netsz = 6;
        int protsz = 4;
        boolean isstoreall = true;
        getOptions().addOption("arp_storeall", isstoreall, "boolean", "", true, true, "keep all arp mac addr");
        getOptions().addOption("arp_hw", net, "Byte[]", "", true, true, "Hardware identification (01>ETH)");
        getOptions().addOption("arp_prot", prot, "Byte[]", "", true, true, "Protocol identification");
        getOptions().addOption("arp_hwsize", netsz, "int", "", true, true, "Hardware address size");
        getOptions().addOption("arp_protsize", protsz, "int", "", true, true, "Protocol address size");
        javax.swing.JButton b = new javax.swing.JButton("clear");
        b.setMargin(new java.awt.Insets(0, 0, 0, 0));
        b.addActionListener(new java.awt.event.ActionListener() {

            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeArpEntries();
            }
        });
        getOptions().addOption("arp_clear", b, "button", "", true, true, "ask to clear ARP entries");
    }

    public static String getComment() {
        return "Address Resolution Protocol";
    }

    public static double getosi() {
        return 2.5;
    }

    @Override
    public void StkArcv(String message, String type) {
    }

    @Override
    public void onClock() {
    }

    private void removeArpEntries() {
        toLog(this, "remove ARP table", true);
        for (int row = 0; row < getOptions().getRowCount(); row++) {
            String label = getOptions().getValueAt(row, 0).toString();
            if (label.startsWith("arp_@")) {
                getOptions().removeOption(label);
                row = row - 1;
            }
        }
        while (waitting.isEmpty() == false) {
            waitting.remove(0);
        }
        getOptions().fireTableDataChanged();
    }

    private byte[] getarpbegin() {
        byte[] buff = null;
        buff = util.addtab(buff, (byte[]) getOptions().getValuefor("arp_hw"));
        buff = util.addtab(buff, (byte[]) getOptions().getValuefor("arp_prot"));
        int hwl = (Integer) getOptions().getValuefor("arp_hwsize");
        int protl = (Integer) getOptions().getValuefor("arp_protsize");
        byte[] sizes = new byte[2];
        if (hwl < 128 && protl < 128) {
            sizes[0] = (byte) hwl;
            sizes[1] = (byte) protl;
        } else {
            return null;
        }
        buff = util.addtab(buff, sizes);
        return buff;
    }

    private List<byte[]> getadresses(byte[] buff) {
        List<byte[]> v = new java.util.ArrayList<>();
        int hwl = (Integer) getOptions().getValuefor("arp_hwsize");
        int protl = (Integer) getOptions().getValuefor("arp_protsize");
        byte[] hwsnd = java.util.Arrays.copyOfRange(buff, 0, hwl);
        byte[] prtsnd = java.util.Arrays.copyOfRange(buff, hwl, hwl + protl);
        byte[] hwrcv = java.util.Arrays.copyOfRange(buff, hwl + protl, 2 * hwl + protl);
        byte[] protrcv = java.util.Arrays.copyOfRange(buff, 2 * hwl + protl, 2 * hwl + 2 * protl);
        v.add(hwsnd);
        v.add(prtsnd);
        v.add(hwrcv);
        v.add(protrcv);
        return v;
    }

    private void checkwaiting(byte[] iptgt, O_MACaddress macsnd) {
        byte[] hwdest = macsnd.getValue();
        int protl = (Integer) getOptions().getValuefor("arp_protsize");
        for (EthPack e : waitting) {
            if (e.getaddress().length == protl) {
                O_MACaddress macaddr;
                try {
                    macaddr = new O_MACaddress(hwdest);
                } catch (Exception ex) {
                    return;
                }
                if (Arrays.equals(iptgt, e.getaddress())) {
                    getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_dest", macaddr);
                    getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_sproto", e.getproto());
                    //port is column -1!!!
                    e.getpacket().setsource(getport());
                    e.getpacket().setdestination(getport());
                    toLog(this, "found " + macsnd.toString() + " for " + e.getaddress(), true);
                    getStack().reply_out(this, e.getpacket());
                    waitting.remove(e);
                } else if (Arrays.equals(iptgt, e.getgateway())) {
                    getOptions().setValuefor("arp_@" + e.getaddress(), macaddr);
                    toLog(this, "added " + "arp_@" + util.Formate_Hexa(e.getaddress(), true) + "=" + macaddr.toString(), true);
                    getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_dest", macaddr);
                    getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_sproto", e.getproto());
                    e.getpacket().setsource(getport());
                    e.getpacket().setdestination(getport());
                    getStack().reply_out(this, e.getpacket());
                    waitting.remove(e);
                }
            }
        }
    }

    /**
     * check packet if IP/MAC is known, if not stock it Answer to request for my
     * IP
     *
     * @param packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> liste = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        byte[] res = (byte[]) getStack().getinfos(getport(), "ETHII", "eth2_sproto", SEARCH_DOWN, this);
        if (Arrays.equals(res, new byte[]{0x08, 0x06}) == false) {
            liste.add(packet);
            return liste;
        }
        toLog(this, "ARP pack" + new String(pack), true);
        res = java.util.Arrays.copyOfRange(pack, 0, 6);
        if (Arrays.equals(res, getarpbegin())) {
            res = java.util.Arrays.copyOfRange(pack, 6, 8);
            List<byte[]> addrs = getadresses(Arrays.copyOfRange(pack, 8, pack.length));
            if (Arrays.equals(res, new byte[]{0x00, 0x01})) {
                try {
                    if (Arrays.equals(addrs.get(3), ((InetAddress) getStack().getinfos(getport(), "IP", "ip_src", SEARCH_UP, this)).getAddress())) {
                        // this is a request of my ip send ARP reply
                        toLog(this, "ARP request for :" + util.Formate_Hexa(addrs.get(3), true) + "=>" + new String(pack), true);
                        byte[] buff = getarpbegin();
                        buff = util.addtab(buff, new byte[]{0x00, 0x02});
                        buff = util.addtab(buff, ((O_MACaddress) getStack().getinfos(getport(), "ETHII", "eth2_src", SEARCH_DOWN, this)).getValue());
                        buff = util.addtab(buff, ((InetAddress) getStack().getinfos(getport(), "IP", "ip_src", SEARCH_UP, this)).getAddress());
                        buff = util.addtab(buff, addrs.get(0));
                        buff = util.addtab(buff, addrs.get(1));
                        getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_dest", new O_MACaddress(addrs.get(0)));
                        //Parent.setinfos(this, "ETHII", "eth2_sproto", new byte[] , pack.getsource()){0x08, 0x06});
                        C_Packet e = new C_Packet(buff, packet.getsource());
                        e.setdestination(getport());
                        getStack().reply_out(this, e);
                        return null;
                    }
                } catch (Exception ex) {
                    toLog(this, "NO IP=> no need of sending a request", true);
                }
            } else if (Arrays.equals(res, new byte[]{0x00, 0x02}) || (Boolean) getOptions().getValuefor("arp_storeall")) {
                // this is a reply  => store if unknown MAC (used or not!!)
                InetAddress ipsend = (InetAddress.getByAddress(addrs.get(1)));
                if (getOptions().getValuefor("arp_@" + ipsend.toString()) == null) {
                    O_MACaddress sndmac = new O_MACaddress(addrs.get(0));
                    getOptions().addOption("arp_@" + ipsend, sndmac, "MAC addr", "", true, true, "MAC entry");
                    checkwaiting(ipsend.getAddress(), sndmac);
                }
                InetAddress iptgt = InetAddress.getByAddress(addrs.get(3));
                if (getOptions().getValuefor("arp_@" + iptgt.toString()) == null) {
                    getOptions().addOption("arp_@" + iptgt.toString(), new O_MACaddress(addrs.get(2)), "MAC addr", "", true, true, "MAC entry");
                }
                toLog(this, "ARP store for :" + util.Formate_Hexa(iptgt.getAddress(), true) + "=>" + new String(pack), true);
            }

        }
        packet.setObj(pack);
        liste.add(packet);
        return liste;
    }

    /**
     * if asked address is known assign MAC, else send a ARP request to look for
     * MAC
     *
     * @param packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        byte[] buff = null;
        List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();

        InetAddress ipdest = (InetAddress) getStack().getinfos(getport(), "IP", "ip_dest", SEARCH_UP, this);
        // if adress is not in the list
        if (ipdest == null) {
            res.add(packet);
            return res;
        }

        O_MACaddress macdest = (O_MACaddress) getOptions().getValuefor("arp_@" + ipdest);
        //if out of net=> gateway in the mac
        if (macdest == null || macdest.getValue().length == (Integer) getOptions().getValuefor("arp_protsize")) {
            toLog(this, "Unknown MAC for IP:" + util.Formate_Hexa(ipdest.getAddress(), true), false);
            buff = util.addtab(buff, getarpbegin());
            //forward an ARP request
            buff = util.addtab(buff, new byte[]{0x00, 0x01});
            buff = util.addtab(buff, ((O_MACaddress) getStack().getinfos(getport(), "ETHII", "eth2_src", SEARCH_DOWN, this)).getValue());
            buff = util.addtab(buff, ((InetAddress) getStack().getinfos(getport(), "IP", "ip_src", SEARCH_UP, this)).getAddress());
            byte[] mac = new byte[(Integer) getOptions().getValuefor("arp_hwsize")];
            Arrays.fill(mac, (byte) 0x00);
            buff = util.addtab(buff, mac);
            InetAddress tempip = ipdest;
            EthPack e = new EthPack((C_Packet) packet.clone(), ipdest.getAddress(), (byte[]) getStack().getinfos(getport(), "ETHII", "eth2_sproto", SEARCH_DOWN, this));
            if (macdest != null) {
                //tempip is gateway
                buff = util.addtab(buff, macdest.getValue());
                e.setgateway(macdest.getValue());
            } else {
                buff = util.addtab(buff, tempip.getAddress());
            }
            waitting.add(e);
            getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_sproto", new byte[]{0x08, 0x06});
            mac = new byte[(Integer) getOptions().getValuefor("arp_hwsize")];
            Arrays.fill(mac, (byte) -1);
            getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_dest", new com.jdcm.common.O_MACaddress(mac));
            packet.setObj(buff);
            toLog(this, "ARP looking for:" + util.Formate_Hexa(ipdest.getAddress(), true) + "=>" + new String(pack), true);
            //need to wait answer and send packet
        } else {
            //addrip MAC is known =>change ETH MAC dest & forward=>unchanged packet
            getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_dest", macdest);
            toLog(this, "ARP found MAC:" + util.Formate_Hexa(macdest.getValue(), true) + "=>" + new String(pack), true);
        }
        res.add(packet);
        return res;
    }

    class EthPack {

        private final C_Packet p;
        private final byte[] address;
        private final byte[] proto;

        private byte[] gateway;

        public EthPack(C_Packet pa, byte[] addr, byte[] pro) {
            p = pa;
            address = addr;
            proto = pro;
        }

        public C_Packet getpacket() {
            return p;
        }

        public byte[] getaddress() {
            //192.168.120.2
            return address;
        }

        public void setgateway(byte[] gw) {
            gateway = gw;
        }

        public byte[] getgateway() {
            //192.168.120.2
            return gateway;
        }

        public byte[] getproto() {
            //Ex : {0x08,0x00}
            return proto;
        }
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        byte[] sign = new byte[]{0, 1, 8, 0, 6, 4};
        double res = 0.0;
        if (pack.isbuffer()) {
            if (pack.getBytes().length > sign.length) {
                boolean isok = true;
                for (int i = 0; i < sign.length; i++) {
                    if (sign[i] != pack.getBytes()[i]) {
                        isok = false;
                    }
                }
                if (isok) {
                    res = 0.99934640;
                }

            }
        }

        return res;
    }
}
