/*
 *
 */
package com.jdcm.modifiers.stacks;

import com.jdcm.common.D_Utils;
import com.jdcm.analyse.*;
import com.jdcm.common.C_Packet;
import com.jdcm.common.XMLres;
import com.jdcm.eltrx.C_Port;
import java.util.List;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

/**
 * First : Read a stream with a Code_model and ProtoMatrix
 *
 * next goal : ?? Write the stream wih CodeModel & DB
 *
 * @author M.Marc GIBOIN
 */
public class Stack_Analyse extends Stack_base {

    private Code_Model CodeMod = null;
    private ProtoEventMatrix ProtoEvents = null;
    private D_Utils TUtils = new D_Utils();
    java.util.Map<String, Object> val_map = new java.util.HashMap<>();
    java.util.Map<String, byte[]> field_map = new java.util.HashMap<>();
    java.util.Map<String, String> string_map = new java.util.HashMap<>();

    public Stack_Analyse() {
        super();
        getOptions().addOption("ana_proto", "auto", "String", "protocol", true, true, "name of the protocol managed by this Stack");
        getOptions().addOption("ana_model", CodeMod, "Code Model", "model", true, true, "Code Model used to study packet");
        getOptions().addOption("ana_conv", ProtoEvents, "ProtoEvents", "eventMatrix", true, true, "Event Matrix used to follow the conversation");
    }

    public static String getComment() {
        return "provide stack reading streams according to Desciptor files";
    }

    public void setProtocol(String txt) {
        getOptions().setValuefor("ana_proto", txt);
    }

    public static double getosi() {
        return 0;
    }

    /**
     * Decapsule packet, check header and trailer and provide payload
     *
     * @param packet
     * @return
     * @throws Exception
     */
    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> res = new java.util.ArrayList<>();
        if (CodeMod == null) {
            String proto = getOptions().getValuefor("ana_proto").toString();
            CodeMod = new Code_Model((C_Port) getport(), packet, proto);
            if (proto.compareTo("auto") == 0) {
                getOptions().setValuefor("ana_proto", CodeMod.getDESC_Protofirst().getName());
            }
            CodeMod.setDESC_Protolast(null);
            if (CodeMod.waitCodeEnds(10000) < 0) {
                return null;
            }
            ProtoEvents = ProtoEventMatrix.getProtoEventMatrix(getOptions().getValuefor("ana_proto", ""));
            getOptions().setValuefor("ana_model", CodeMod);
            getOptions().setValuefor("ana_conv", ProtoEvents);
        }
        CodeMod.clear();
        CodeMod.setbuffer(packet.getBytes());
        CodeMod.setinitbuffer();
        CodeMod.createDec();
        if (CodeMod.waitCodeEnds(10000) < 0) {
            return null;
        }
        for (int row = 0; row < CodeMod.getRowCount(); row++) {
            // EventMatrix part :
            if (ProtoEvents != null) {
                ProtoEvents.addActions(CodeMod, row, new C_Packet[]{packet}, null);
            }
            // Protocol Option Part :
            String fieldName = CodeMod.getValueAt(row, "Name").toString();
            String fieldVDesc = CodeMod.getValueAt(row, "VDesc", "");
            String fieldComment = CodeMod.getValueAt(row, "Description", "");
            Object fieldVal = CodeMod.getValueAt(row, "Value");
            if (fieldVDesc != null && fieldVDesc.length() > 0) {
                fieldComment = fieldComment + " => " + fieldVDesc;
            }
            if (fieldName.endsWith("data")) {
                C_Packet payloadPack = (C_Packet) packet.clone();
                payloadPack.setObj(fieldVal);
                res.add(payloadPack);
                System.out.println("Stack_Analyse : " + fieldName + ":" + row);
            } else {
                // remove informations 'not on a leaf'
                for (int i = getOptions().getRowCount() - 1; i >= 0; i--) {
                    if (fieldName.contains(":")
                            && getOptions().isPresent(fieldName.substring(0, fieldName.lastIndexOf(":")))) {//fieldName.startsWith(getOptions().getLabelAt(row)+":")){
                        getOptions().removeOption(fieldName.substring(0, fieldName.lastIndexOf(":")));
                    }
                }
                if (getOptions().isPresent(fieldName)) {
                    if (getOptions().getValuefor(fieldName) instanceof java.util.List == false) {
                        java.util.List<Object> list = new java.util.ArrayList<>();
                        list.add(getOptions().getValuefor(fieldName));
                        list.add(fieldVal);
                        getOptions().setValuefor(fieldName, list);
                    } else {
                        java.util.List<Object> list = (java.util.List) getOptions().getValuefor(fieldName);
                        list.add(fieldVal);
                    }
                } else if (fieldVal != null) {
                    getOptions().addOption(fieldName, fieldVal, fieldVal.getClass().getSimpleName(),
                            "ato-generate", true, true, fieldComment);
                } else {
                    getOptions().addOption(fieldName, fieldVal, "null",
                            "ato-generate", true, true, fieldComment);
                }
            }
        }
        return res;
    }

    /**
     * encpsule payload in header to form packet
     *
     * @param packet the payload
     * @return the list of C_Packet to send
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        List<C_Packet> res = new java.util.ArrayList<>();
        byte[] buffer ;
// no fragmentation for starting...

        if (packet.getBytes() != null) {
            val_map.put("data", packet.getBytes());
            field_map.put("data", packet.getBytes());
        }
        // GET Protocol Field name & Values
        for (int row = 0; row < getOptions().getRowCount(); row++) {
            if (getOptions().getLabelAt(row).startsWith("_") == false
                    && getOptions().getLabelAt(row).startsWith("ana_") == false) {
                val_map.put(getOptions().getLabelAt(row), getOptions().getValuefor(row));
            }
        }
        // GET Protocol TREE

        DefaultTreeModel treeModel = CodeMod.getcurrentprotoTree();
        if (treeModel == null || treeModel.getRoot() == null) {
            return null;
        } else {
            DefaultMutableTreeNode roottn = (DefaultMutableTreeNode) treeModel.getRoot();
            DefaultMutableTreeNode leaf = roottn;
            // --- BROWSE protocol to find fields definitions (AND GO INSIDE THE TREE)

            // BROWSE protocol by branch to get Buffer

            buffer = inspectBranch(roottn, null, "");
            packet.setObj(buffer);
            res.add(packet);
            return res;
        }
    }

    private byte[] inspectBranch(DefaultMutableTreeNode root, String subs, String baseFieldName) {
        DefaultMutableTreeNode branch = null;
        byte[] buffer = new byte[]{};
        String fieldName = "";
        int memonxt = -1; // used to avoid loop when next is used
        for (int i = 0; i < root.getChildCount(); i++) {
            XMLres child = (XMLres) ((DefaultMutableTreeNode) root.getChildAt(i)).getUserObject();
            if (subs == null && child.toString().compareTo("protocol") == 0) {
                branch = (DefaultMutableTreeNode) root.getChildAt(i);
            } else if (child.toString().compareTo("subproto") == 0
                    && subs != null && child.getName().compareTo(subs) == 0) {
                branch = (DefaultMutableTreeNode) root.getChildAt(i);
            }
        }
        DefaultMutableTreeNode initBranch = branch;
        if (branch == null) {
            return null;
        }

        for (int i = 0; i < branch.getChildCount(); i++) {
            XMLres field = (XMLres) ((DefaultMutableTreeNode) branch.getChildAt(i)).getUserObject();
            if (field.toString().startsWith("field")) {
                if (val_map.containsKey(baseFieldName + field.getName())) {
                    fieldName = baseFieldName + field.getName();
                    feelFieldBuffer(field, fieldName);
                    buffer = TUtils.addtab(buffer, field_map.get(fieldName));
                    if (branch.getChildAt(i).getChildCount() > 0) {
                        branch = (DefaultMutableTreeNode) branch.getChildAt(i);
                        i = -1;
                    }
                }
                if (field.getName().startsWith("TLV")) {
                    //skip TLVv...managed in TLVt
                    // OR change TLVv-01 => TLVt-TLVv=01 OR TLVt=> TLVv OR manage "next"...
                    //si TLVt => manage sub node => should check after Value if TLVv.. and sub proto...
                    if (field.getName().compareTo("TLVt") == 0) {
                        DefaultMutableTreeNode tvltBranch = (DefaultMutableTreeNode) branch.getChildAt(i);
                        DefaultMutableTreeNode tvlvBranch = (DefaultMutableTreeNode) branch.getChildAt(i + 1);
                        for (int validx = 0; validx < tvltBranch.getChildCount(); validx++) {
                            DefaultMutableTreeNode tvlBranch = (DefaultMutableTreeNode) tvltBranch.getChildAt(validx);
                            XMLres tlvval = (XMLres) tvlBranch.getUserObject();
                            if (tlvval.toString().compareTo("value") == 0) {
                                String tagstr = getStringFromValue(tlvval.getAttrValAsstring("value"), field.getAttrValAsstring("code"), field.getAttrValAsstring("class"));
                                byte[] tagbuff = getBufferFromString(tagstr, field.getAttrValAsstring("code"));
                                tagbuff = getFieldFromBuffer(tagbuff, field.getAttrValAsstring("size"), field.getAttrValAsstring("sizecode"));
                                String tvlName = baseFieldName + "TLVv-" + tagstr;
                                if (val_map.containsKey(tvlName)) {
                                    
                                    if (tvlBranch.getChildCount() > 0
                                            && ((XMLres) ((DefaultMutableTreeNode) tvlBranch.getChildAt(0)).getUserObject()).getName().compareTo("TLVv") == 0) {
                                     feelFieldBuffer(((XMLres) ((DefaultMutableTreeNode) tvlBranch.getChildAt(0)).getUserObject()), tvlName);
                                    } else {
                                        feelFieldBuffer((XMLres) tvlvBranch.getUserObject(), tvlName);
                                    }
                                    buffer = TUtils.addtab(buffer, tagbuff); // TAG
                                    buffer = TUtils.addtab(buffer, field_map.get(tvlName));///VALUE
                                }
                            }
                        }
                    }
                }
                String sproto = field.getAttrValAsstring("sproto");
                if (sproto != null && sproto.length() > 0) {
                    fieldName = baseFieldName + field.getName();
                    boolean explorProto = false;
                    for (String key : val_map.keySet()) {
                        if (key.startsWith(fieldName + ":")) {
                            explorProto = true;
                        }
                    }
                    if (explorProto) {
                        byte[] sprotoBuff = inspectBranch(root, sproto, fieldName + ":");
                        sprotoBuff = getFieldFromBuffer(sprotoBuff, field.getAttrValAsstring("size"), field.getAttrValAsstring("sizecode"));
                        buffer = TUtils.addtab(buffer, sprotoBuff);
                    }
                }
            } else if (field.toString().compareTo("value") == 0) {
                if (field.getAttrValAsstring("value").compareTo(string_map.get(fieldName)) == 0
                        || field.getAttrValAsstring("value").compareTo("/*/") == 0) {
                    branch = (DefaultMutableTreeNode) branch.getChildAt(i);
                    i = -1;
                }
            } else {
                System.err.println("not avalable field :" + field.toXMLString() + " in " + getOptions().getValuefor("ana_proto", ""));
            }
            if (i >= 0) {

                //if no othere then return at prevous level
                while (branch.getLevel() >= 3
                        && (i == branch.getChildCount() - 1)) { //Analyse & protocol &field
                    i = branch.getParent().getIndex(branch);
                    branch = (DefaultMutableTreeNode) branch.getParent();
                    if (branch.getUserObject().toString().compareTo("field") == 0) {
                        if (((XMLres) branch.getUserObject()).getAttrValAsstring("next") != null) {
                            if (memonxt < 0 || buffer.length > memonxt) {
                                memonxt = buffer.length;
                                String nxtreq = ((XMLres) branch.getUserObject()).getAttrValAsstring("next");
                                // find the node in subtree proto/sproto
                                DefaultMutableTreeNode next = initBranch;
                                while (next!=null && next.getNextNode() != null) {
                                    next = next.getNextNode();
                                    if (next.getUserObject() instanceof XMLres) {
                                        if (((XMLres) next.getUserObject()).getName().compareTo(nxtreq) == 0) {
                                            branch = (DefaultMutableTreeNode) next.getParent();
                                            i = branch.getIndex(next) - 1;
                                        }
                                    }
                                }
                            }
                        } else {
                            i = branch.getParent().getIndex(branch);
                            branch = (DefaultMutableTreeNode) branch.getParent();
                        }
                    }
                }
            }
        }
        return buffer;
    }

    private void feelFieldBuffer(XMLres field, String fullName) {
        Object fieldVal;
        if (val_map.get(fullName) instanceof java.util.List) {
            java.util.List<Object> list = (java.util.List) val_map.get(fullName);
            fieldVal = list.remove(0);
            if (list.isEmpty()) {
                val_map.remove(fullName);
            }
        } else {
            fieldVal = val_map.get(fullName);
            val_map.remove(fullName);
        }
        byte[] buff;
        String strVal;
        if (fieldVal instanceof byte[] && field.getAttrValAsstring("code").compareTo("BYTE") == 0) {
            // already a byte => just Format ascii (00:01:02) first byte for <value>
            buff = (byte[]) fieldVal;
            strVal = TUtils.Formate_Hexa(buff, false).trim();
        } else {
            strVal = getStringFromValue(fieldVal, field.getAttrValAsstring("code"), field.getAttrValAsstring("class"));
            buff = getBufferFromString(strVal, field.getAttrValAsstring("code"));
        }
        buff = getFieldFromBuffer(buff, field.getAttrValAsstring("size"), field.getAttrValAsstring("sizecode"));
        field_map.put(fullName, buff);
        string_map.put(fullName, strVal);
    }

    /**
     * Format a buffer according to a field definition.
     *
     * @param fvalue the Value to format
     * @param fcode the code used ASCII/DEC/BCD...
     * @param fclass the class used to change Byte[]/String <=> Object
     * @param fsize the size of the field
     * @param fsizecode the code used to read the size of the field.
     * @return the buffer matching with Value & definitions
     */
    private String getStringFromValue(Object fvalue, String fcode, String fclass) {
        // int 1 => "1" (ou 1/1/1970)
        String sres = "";
        if (fclass == null || fclass.compareTo("String") == 0) {
            sres = fvalue.toString();
        } else {
            sres = fvalue.toString();
        }
        if (fcode.compareTo("ASCII") == 0) {
            sres = fvalue.toString();
        } else if (fcode.compareTo("BYTE") == 0) {
            try {
                sres = TUtils.Formate_Hexa((byte[]) fvalue, true);
            } catch (Exception ex) {
            }
        } else {
            sres = fvalue.toString();
        }
        return sres;
    }

    private byte[] getBufferFromString(String fvalue, String fcode) {
        //"1" => 0x31 (ou 0x01) ou 1
        byte[] res = null;
        if (fcode.compareTo("ASCII") == 0) {
            res = fvalue.getBytes();
        } else if (fcode.compareTo("BYTE") == 0) {
            try {
                res = TUtils.string2byte(fvalue);
            } catch (Exception ex) {
            }
        } else if (fcode.compareTo("DEC") == 0) {
            res = TUtils.prend("DEC", fvalue);
        } else {
            res = fvalue.getBytes();
        }
        return res;
    }

    private byte[] getFieldFromBuffer(byte[] buff, String fsize, String fsizecode) {
        //0x31 => 0x1:0x31 (si Var/1)
        byte[] res = buff;
        if (fsize.startsWith("Var/")) {
            byte[] szbuff = getsizePart(res.length, Integer.parseInt(fsize.substring("Var/".length())), fsizecode);
            res = TUtils.addtab(szbuff, res);
        } else if (fsize.startsWith("Var[")) {
            int pos = Integer.parseInt(fsize.substring("Var[".length(), fsize.indexOf("]")));
            byte[] szbuff = getsizePart(res.length, Integer.parseInt(fsize.substring(fsize.indexOf("/") + 1)), fsizecode);
            if (res.length < pos + szbuff.length) {
                res = java.util.Arrays.copyOf(res, pos + szbuff.length);
            }
            System.arraycopy(szbuff, 0, res, pos, szbuff.length);

        } else if (fsize.startsWith("Var-")) {
            int offset = Integer.parseInt(fsize.substring("Var-".length(), fsize.indexOf("/")));
            byte[] szbuff = getsizePart(res.length + offset, Integer.parseInt(fsize.substring(fsize.indexOf("/") + 1)), fsizecode);
            res = TUtils.addtab(szbuff, res);
        } else if (fsize.startsWith("FS")) {
        } else {
            try {
                int size = Integer.parseInt(fsize);
                res = java.util.Arrays.copyOf(res, size);
            } catch (NumberFormatException ex) {
                 System.out.println("Stack_Analyse : getFieldFromBuffer() =>"+ex.getMessage());
            }
        }
        return res;
    }

    private byte[] getsizePart(int nb, int szsz, String format) {
        if (format == null) {
            format = "DEC";
        }
        byte[] sz = TUtils.prend(format, nb);
        if (sz.length <= szsz) {
            while (sz.length < szsz) {
                sz = TUtils.addtab(new byte[]{0}, sz);
            }
        } else {
            // size is biger than size of the size
            return null;
        }
        return sz;
    }
}
