package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import com.jdcm.common.C_Actiondef;
import com.jdcm.common.C_Eventdef;
import com.jdcm.diagram.A_Events;
import com.jdcm.common.D_Utils;
import java.lang.reflect.Field;

/**
 * STX...ETX... Protocol use for simple serial transmissions - managed by an
 * event diagram
 *
 * @author Marc GIBOIN http://www.frameip.com/entete-udp/
 */
public class Stack_C0C1 extends Stack_baseStart {

    private D_Utils TUtils = new D_Utils();
    java.util.List<byte[]> ctrls = new java.util.ArrayList<>();
    // private int session_flag = 0;
    private static final byte[] CTRL_EOT = new byte[]{0x4};
    private static final byte[] CTRL_ACK = new byte[]{0x6};
    private static final byte[] CTRL_ENQ = new byte[]{0x5};
    private static final byte[] CTRL_NACK = new byte[]{0x15};
    private static final byte[] CTRL_STX = new byte[]{0x2};
    private static final byte[] CTRL_ETX = new byte[]{0x3};
    private static final byte[] CTRL_DLE = new byte[]{0x10};
    public static final int CLOSED = 0;
    public static final int OPENNING = 1;
    public static final int CLOSING = 3;
    public static final int OPENNED = 4;
    private int rcvd_msg = 0;
    private int sent_msg = 0;

    // private int wait_idx = 0;
    public Stack_C0C1() {
                super();
        for (java.lang.reflect.Field fs : this.getClass().getDeclaredFields()) {
            if (fs.getName().startsWith("CTRL_")) {
                try {
                    ctrls.add((byte[]) fs.get(fs));
                } catch (IllegalArgumentException | IllegalAccessException ex) {
                }
            }
        }
        setOptions();
    }

    public static String getComment() {
        return "STX/ETX framming protocol";
    }

    public static double getosi() {
        return 2.0;
    }

    private void setOptions() {
        getOptions().addOption("c0c1_waitmsg", new java.util.ArrayList<>(), "waiting message list", "", true, true, "waiting messages");
        getOptions().addOption("c0c1_framebuffer", new byte[0], "received bytes", "", true, true, "received bytes");
        getOptions().addOption("c0c1_ManageDLE", true, "Boolean", "", true, true, "manage DEL before 0x10 & before ETX");
        getOptions().addOption("c0c1_cxbyPort", true, "Boolean", "", true, true, "askON or askOFF on port connected/stopped");
        getOptions().addOption("c0c1_maxrcvd", 0, "int", "", true, true, "nb of messages waitted befor close");
        getOptions().addOption("c0c1_maxsent", 0, "int", "", true, true, "nb of messages acknowlegded befor close");
        try {
            A_Events eventmx2 = new A_Events(D_Utils.getFileStream("evt_c0c1.xml", "mapping", getClass().getClassLoader()));
            eventmx2.setStatusAsFields(this);
            getOptions().addOption("c0c1_StatMat", eventmx2, "A_Events", "", true, true, "Event/status matrix");
        } catch (Exception ex) {
            //  ex.printStackTrace();
        }
    }

    private String getStatus() {
        A_Events evts = (A_Events) getOptions().getValuefor("c0c1_StatMat");
        return ((Field) evts.getActivStatus().getcontent()).getName();
    }

    @Override
    public boolean onStart() {
        try {
            String portstate = getport().getcxstatus();
            if (portstate.compareTo("stopped") == 0 || portstate.compareTo("creating") == 0) {
                getport().CXstart();
            }
            receive(new C_Packet("ASKON", getport()));
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public boolean onStop() {
        try {
            String portstate = getport().getcxstatus();
            if (!portstate.startsWith("stopp") && !portstate.startsWith("closed")) {
                getport().CXstop();
            }
            receive(new C_Packet("ASKOFF", getport()));
        } catch (Exception ex) {
            return false;
        }
        return true;
    }
    /*
     * @Override public boolean setStack_isStart(boolean bool) { String
     * txttarget=super.setStack_start(txt); if (txttarget!=null) { String
     * portstate = getport().getcxstatus(); try { if (txt.startsWith("start")) {
     *
     * } else {
     *
     * }
     * } catch (Exception ex) { } } return txt;
     }
     */

    @Override
    public void StkArcv(String message, String type) {
        if (getOptions().getValuefor("c0c1_cxbyPort", true) && type.compareTo("Status") == 0) {
            if (message.compareTo("stopping") == 0) {
                setStack_isStart(false);
            } else if (message.compareTo("connected") == 0) {
                setStack_isStart(true);
            }
        }
    }

    @Override
    public void onClock() {
    }

    private java.lang.reflect.Method getcurentMethod(Class<?> cl) {
        java.lang.StackTraceElement[] traces = Thread.currentThread().getStackTrace();
        if (traces.length >= 2) {
            try {
                //      if (this.isfrommedia) {
                return this.getClass().getMethod(traces[2].getMethodName(), cl);
                /*
                 * } else { if (traces[2].getMethodName().endsWith("receive")) {
                 * return this.getClass().getMethod("transmit", cl); } else {
                 * return this.getClass().getMethod("receive", cl); }
                 }
                 */
            } catch (NoSuchMethodException | SecurityException ex) {
                return null;
            }
        }
        return null;
    }

    /**
     * decapsule packet, check header and trailer and provide payload
     *
     * @param packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        if (pack != null) {
            toLog(this, new String(pack), true);
        } else {
            toLog(this, "" + packet.getObj(), true);
        }
        if (pack != null && pack.length > 1 &&(pack[0] == 6 || pack[0] == 5 || pack[0] == 4)) {
            // if 06 04 => understand ACK ,EOT and not {06,04}, could be donefor each byte...
            C_Packet pack1 = ((C_Packet) packet.clone());
            pack1.setObj(new byte[]{pack[0]});
            res.addAll(receive(pack1));
            for (C_Packet p : res) {
                getStack().reply_out(this, p);
            }
            res = new java.util.ArrayList<>();
            byte[] pack2 = new byte[pack.length - 1];
            System.arraycopy(pack, 1, pack2, 0, pack2.length);
            packet.setObj(pack2);
            res.addAll(receive(packet));
            for (C_Packet p : res) {
                getStack().reply_out(this, p);
            }
            return null;
        }
        A_Events eventmatrice = (A_Events) getOptions().getValuefor("c0c1_StatMat");
        C_Eventdef evt = new com.jdcm.common.C_Eventdef(this, getcurentMethod(packet.getClass()), packet.getObj());
        java.util.ArrayList<C_Actiondef> list = eventmatrice.getTask(evt);
        if (list != null && list.isEmpty() == false) {
            for (C_Actiondef actions : list) {
                // SEND CTRLS
                if (actions.getParam() instanceof byte[]) {
                    C_Packet crtl_out = new C_Packet(actions.getParam(), getport());
                    getStack().reply_out(this, crtl_out);
                } else if (actions.getParam() instanceof Field) {
                    Object o = ((Field) actions.getParam()).get(actions.getParam());
                    C_Packet crtl_out = new C_Packet(o, getport());
                    getStack().reply_out(this, crtl_out);

                } else if (actions.getParam().equals("ACK")) {
                    int max = (Integer) getOptions().getValuefor("c0c1_maxsent");
                    sent_msg = sent_msg + 1;
                    toLog(this, "acknoleged nb:" + sent_msg, true);
                    if (max > 0 && sent_msg >= max) {
                        receive(new C_Packet("ASKOFF", getport()));
                        return res;
                    }
                } else if (actions.getParam().equals("BEGIN")) {
                    getOptions().setValuefor("c0c1_framebuffer", new byte[0]);
                    send_waiting();
                } else if (actions.getParam().equals("END")) {
                    ((java.util.List) getOptions().getValuefor("c0c1_waitmsg")).clear();
                    eventmatrice.reset();
                    getOptions().setValuefor("c0c1_framebuffer", new byte[0]);
                }
            }
        } else {
            if (getStatus().compareTo("OPENNED") == 0 && packet.isbuffer()) {
                C_Packet d = new C_Packet(new byte[]{}, getport());

                byte[] buffer = (byte[]) getOptions().getValuefor("c0c1_framebuffer");
                if (buffer != null) {
                    toLog(this, "init buffer[" + new String(buffer) + "]", true);
                }
                if (pack != null) {
                    toLog(this, "received[" + new String(pack) + "]", true);
                }
                buffer = TUtils.addtab(buffer, pack);

                byte[] checked;
                try {
                    toLog(this, "before check[" + new String(buffer) + "]", true);
                    checked = chkedframe(buffer);
                    if (checked == null) {
                        if (buffer != null) {
                            toLog(this, "new buffer[" + new String(buffer) + "]", true);
                        }
                        getOptions().setValuefor("c0c1_framebuffer", buffer);
                        pack = null;
                    } else {
                        toLog(this, "checked[" + new String(checked) + "]", true);
                        int max = (Integer) getOptions().getValuefor("c0c1_maxrcvd");
                        pack = checked;
                        getOptions().setValuefor("c0c1_framebuffer", new byte[0]);
                        d.setObj(Stack_C0C1.CTRL_ACK);
                        getStack().reply_out(this, d);
                        rcvd_msg = rcvd_msg + 1;
                        toLog(this, "received nb:" + rcvd_msg, true);
                        if (max > 0 && rcvd_msg >= max) {
                            receive(new C_Packet("ASKOFF", getport()));
                        }
                    }
                } catch (Exception ex) {
                    d.setObj(Stack_C0C1.CTRL_NACK);
                    toLog(this, "Exception " + ex.getMessage(), true);
                    getStack().reply_out(this, d);
                    pack = null;
                }
                if (pack != null) {
                    packet.setObj(pack);
                    res.add(packet);
                }
                return res;
            }
        }

        return res;
    }

    /**
     * encpsule payload in header and trailers to form packet
     *
     * @param packet the payload
     * @return the packet
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        if (pack != null) {
            toLog(this, new String(pack), false);
        } else {
            toLog(this, "" + packet.getObj(), false);
        }
        A_Events eventmatrice = (A_Events) getOptions().getValuefor("c0c1_StatMat");
        C_Eventdef evt = new com.jdcm.common.C_Eventdef(this, getcurentMethod(packet.getClass()), packet.getObj());
        java.util.ArrayList<C_Actiondef> list = eventmatrice.getTask(evt);
        for (C_Actiondef actions : list) {
            if (actions.getParam() != null) {
                if (actions.getParam().equals("TRANSMIT")) {
                    packet.setObj(createframe(pack));
                    finalres.add(packet);
                } else if (actions.getParam().equals("STORE")) {
                    getOptions().getValuefor("c0c1_waitmsg", new java.util.ArrayList<byte[]>()).add(pack);
                    pack = null;
                } else if (actions.getParam().equals("BEGIN")) {
                    getOptions().setValuefor("c0c1_framebuffer", new byte[0]);
                    C_Packet d = new C_Packet(new byte[]{}, getport());
                    d.setObj(Stack_C0C1.CTRL_ENQ);
                    getStack().reply_out(this, d);
                    pack = null;
                }
            }
        }
        return finalres;
    }

    private void send_waiting() {
        java.util.List<byte[]> cb = getOptions().getValuefor("c0c1_waitmsg", new java.util.ArrayList<byte[]>());
        while (cb.isEmpty() == false) {
            byte[] waited = cb.get(0);
            /*
             * if (cb.get(0) instanceof byte[]) { waited = (byte[]) cb.get(0);
             }
             */
            cb.remove(0);
            if (waited != null) {
                try {
                    java.util.List<C_Packet> list = transmit(new C_Packet(waited, getport()));
                    for (C_Packet pack : list) {
                        pack.setdestination(getport());
                        getStack().reply_out(this, pack);
                    }
                } catch (Exception ex) {
                }
            }
        }
    }

    private byte[] createframe(byte[] buff) {
        byte[] bs = buff;
        if (getOptions().getValuefor("c0c1_ManageDLE", true)) {
            bs = addDLE(bs);
        }
        bs = TUtils.addtab(bs, Stack_C0C1.CTRL_ETX);
        bs = TUtils.addtab(bs, new byte[]{lrc(bs)});
        bs = TUtils.addtab(Stack_C0C1.CTRL_STX, bs);
        return bs;
    }

    /**
     * check the received data
     *
     * @param buff the buffered data to check (started with STX)
     * @return null if not checked, data if checked
     * @throws Exception
     */
    private byte[] chkedframe(byte[] buff) throws Exception {
        if (buff.length == 0 || buff[0] != Stack_C0C1.CTRL_STX[0]) {
            if (buff.length > 0) {
                getOptions().setValuefor("c0c1_framebuffer", new byte[0]);
                throw new Exception("no STX at Start : " + buff[0] + ";clear buffer");
            } else {
                throw new Exception("empty buffer");
            }
        }
        if (buff.length < 2) {
            //no checked, waited other data
            return null;
        }
        int pos = 1; // checked that buff[0]= STX
        if (getOptions().getValuefor("c0c1_ManageDLE", true)) {
            while (pos < buff.length && !(buff[pos - 1] == Stack_C0C1.CTRL_DLE[0] && buff[pos] == Stack_C0C1.CTRL_ETX[0])) {
                pos++;
            }
        } else {
            while (pos < buff.length && !(buff[pos] == Stack_C0C1.CTRL_ETX[0])) {
                pos++;
            }
        }
        if (pos < buff.length - 1) { // -1 for LRC
            // DLE/ETX found calcul LRC without STX with ETX
            byte[] datapacket = new byte[pos - 1];
            System.arraycopy(buff, 1, datapacket, 0, pos - 1);
            byte[] lrcpacket = TUtils.addtab(datapacket, Stack_C0C1.CTRL_ETX);
            if (buff[pos + 1] == lrc(lrcpacket)) {
                if (getOptions().getValuefor("c0c1_ManageDLE", true)) {
                    return removeDLE(datapacket);
                } else {
                    return datapacket;
                }
            } else {
                // if no DLE => No 03 for data => No exception for nothing !
                throw new Exception("red LRC is :" + buff[pos + 1] + ";computed LRC :" + lrc(lrcpacket));
            }
        } else {

            return null;
        }
    }

    private byte[] addDLE(byte[] buff) {
        java.util.ArrayList<Byte> reslist = new java.util.ArrayList<>();
        for (byte b : buff) {
            boolean isctrl = false;
            for (byte[] ctr : ctrls) {
                if (ctr[0] == b) {
                    isctrl = true;
                }
            }
            if (isctrl) {
                reslist.add(Stack_C0C1.CTRL_DLE[0]);
            }
            reslist.add(b);
        }
        byte[] res = new byte[reslist.size() + 1];
        for (int i = 0; i < res.length - 1; i++) {
            res[i] = reslist.get(i);
        }
        res[reslist.size()] = Stack_C0C1.CTRL_DLE[0];
        return res;
    }

    /**
     * remove DLE escape caracteres NB : DLE DLE=DLE
     *
     * @param buff byte array of DATA (without STX,ETX,LRC)
     * @return DATA whithout excape DLE ,may be data 10 (DLE)
     */
    private byte[] removeDLE(byte[] buff) {
        java.util.List<Byte> reslist = new java.util.ArrayList<>();
        boolean previsdle = false;
        // last should be DLE before ETX; is removed
        for (int idx = 0; idx < buff.length - 1; idx++) {
            byte b = buff[idx];
            if (b != CTRL_DLE[0]) {
                reslist.add(b);
                previsdle = false;
            } else {
                if (previsdle) {
                    reslist.add(b);
                    previsdle = false;
                } else {
                    previsdle = true;
                }
            }
        }
        byte[] res = new byte[reslist.size()];

        for (int i = 0; i < res.length; i++) {
            res[i] = reslist.get(i);
        }
        return res;
    }

    private byte lrc(byte[] buff) {
        byte lrc = 0;
        for (byte oct : buff) {
            lrc ^= oct;
        }
        return lrc;
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            byte[] frame = pack.getBytes();
            if (frame.length == 1) {
                if (frame[0] == 6 || frame[0] == 4 || frame[0] == 5 || frame[0] == 15) {
                    // could not check the checksum beacause of peudosession??
                    res = 0.81;
                }
            } else if (frame.length > 3) {
                if (frame[0] == 2 && frame[frame.length - 2] == 3) {
                    res = 0.9980;
                }
            }
        }
        return res;
    }
}
/*
 * To change this template, choose Tools | Templates and open the template in
 * the editor.
 */
