package com.jdcm.modifiers.stacks;

import com.jdcm.common.D_Utils;
import java.util.Arrays;
import java.net.NetworkInterface;
import com.jdcm.eltrx.I_Port;
import com.jdcm.common.C_Packet;
import com.jdcm.common.O_MACaddress;

/**
 * known as 802.3 ETHERNET used for LLC protocols
 *
 * @author Marc GIBOIN
 *
 */
public class Stack_ETHI extends Stack_baseIP {


    public Stack_ETHI() {
        super();
        byte[] ethsrc = {0x00, 0x1b, -4, 0x33, 0x55, 0x47};
        byte[] ethdst = {0x00, 0x0f, 0x1f, -90, -59, 0x68};
        try {
            getOptions().addOption("eth1_src", new O_MACaddress(ethsrc), "MAC address", "", true, true, "source MAC adress");
            getOptions().addOption("eth1_dest", new O_MACaddress(ethdst), "MAC address", "", true, true, "destination MAC adress");
        } catch (Exception ex) {
        }
        // no sense except for local com???
        getOptions().addOption("eth1_copy", false, "Boolean", "", true, true, "will copy msg dest mac & proto on stack");
        getOptions().addOption("eth1_chksrc", false, "Boolean", "", true, true, "chk msg dest MAC with eth1_src");
        getOptions().addOption("eth1_chkdest", false, "boolean", "", true, true, "filter by eth1_dest");
        javax.swing.JButton b = new javax.swing.JButton("src");
        b.setMargin(new java.awt.Insets(0, 0, 0, 0));
        b.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                getsystemsinfo();
            }
        });
        getOptions().addOption("eth_loadSys", b, "button", "", true, true, "ask for System ETH up adress");
    }

    public static String getComment() {
        return "Ethernet I";
    }

    public static double getosi() {
        return 2.0;
    }

    /**
     * set the PC eth interface UP in eth2_src
     */
    private void getsystemsinfo() {
        NetworkInterface neth;
        java.util.Enumeration<NetworkInterface> ent = null;
        try {
            ent = NetworkInterface.getNetworkInterfaces();
        } catch (java.net.SocketException ex) {
        }
        if (ent != null) {
            while (ent.hasMoreElements()) {
                neth = ent.nextElement();
                try {
                    if (neth.isUp()) {
                        byte[] addr = neth.getHardwareAddress();
                        if (addr != null && addr.length == ((O_MACaddress) getOptions().getValuefor("eth1_src")).getValue().length) {
                            getOptions().setValuefor("eth1_src", new O_MACaddress(addr));
                        }
                    }
                } catch (java.net.SocketException ex) {
                } catch (Exception t) {
                }
            }
        }

    }

    @Override
    public void setParents(I_Port idx, com.jdcm.eltrx.C_Stack modmangt) {
        super.setParents(idx, modmangt);
        if (getport().getModeName().compareTo("ETH") == 0) {
            if (getport().getClass().getName().endsWith("IF_Port")) {
                com.jdcm.eltrx.C_Port p = (com.jdcm.eltrx.C_Port) getport();
                try {
                    byte[] mac = p.getETH_MAC();
                    if (mac != null) {
                        getOptions().setValuefor("eth1_src", new O_MACaddress(mac));
                    }
                } catch (Exception ex) {
                }
            }
        }
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param  packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        byte[] frame = packet.getBytes();
        if (frame == null || frame.length < 15) {
            toLog(this, "size < 15 " + new String(frame), true);
            return null;
            // throw new java.lang.Exception("packet lenght is to short to be an ETHII Packet, size : " + packet.length + "");
        }
        byte[] msgdst = java.util.Arrays.copyOfRange(frame, 0, 6);
        byte[] msgsrc = java.util.Arrays.copyOfRange(frame, 6, 12);
        byte[] msgsize = java.util.Arrays.copyOfRange(frame, 12, 14);
        int size = (int) this.array2dec(msgsize);
        byte[] msgdata = java.util.Arrays.copyOfRange(frame, 14, 14 + size);
        if (msgdata.length == 46) {// <46??
            // remove null at the begining of packet
            //!!!!! if packet length less than 46 and strting by 0X00 !!!!!!!!!=> at the end ???
            int pos = -1;
            for (int i = 0; msgdata[i] == 0 || i < msgdata.length; i++) {
                pos = i;
            }
            msgdata = java.util.Arrays.copyOfRange(msgdata, pos, msgdata.length);
        }
        toLog(this, "dst:" + TUtils.Formate_Hexa(msgdst, true) + ",src:" + TUtils.Formate_Hexa(msgsrc, true) 
                + ",size:" + size + ",data:" + TUtils.Formate_Hexa(msgdata, true), true);
        // check dest of tho packet is the source of the module (Reception) : ismy || is brdcast || not checked
        boolean bool = getOptions().getValuefor("eth1_chkdest", true);
        byte[] ethsrc = ((O_MACaddress) getOptions().getValuefor("eth1_src")).getValue();
        byte[] ethdst = ((O_MACaddress) getOptions().getValuefor("eth1_dest")).getValue();
        if (Arrays.equals(msgdst, ethsrc) || Arrays.equals(msgdst, new byte[]{-1, -1, -1, -1, -1, -1}) || bool == false) {
            if (getOptions().getValuefor("eth1_chksrc", true) == false || Arrays.equals(msgsrc, ethdst)) {
                // if (Arrays.equals(res, (byte[]) options.getValuefor("eth1_dest")) ) {

                if (getOptions().getValuefor("eth1_copy", true)) {
                    getOptions().setValuefor("eth1_dest", new O_MACaddress(msgdst));
                    getOptions().setValuefor("eth1_src", new O_MACaddress(msgsrc));
                }
                packet.setObj(msgdata);
                liste.add(packet);
                return liste;
            }
        }
        return liste;
    }

    /**
     * Encpsule payload in header to form packet
     *
     * @param packet the payload
     * @return the packets
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) {
        //encapsule
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        int i = 0;
        //building fragments in fragments List if trame > 1500 (datagram max size)
        java.util.List<byte[]> fragments = new java.util.ArrayList<>();
        while (i < pack.length) {
            int mtu = 1500;
            if (mtu > pack.length - i) {
                fragments.add(Arrays.copyOfRange(pack, i, pack.length));
                i = pack.length;
            } else {
                fragments.add(Arrays.copyOfRange(pack, i, i + mtu));
                i += mtu;
            }
        }
        java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
        for (byte[] data : fragments) {
            if (data.length < 46) {
                // minimum size = 46 => foirreux car les 00 sont mis en debut du dernier paquet.
                // pas de bourrages plus malin ??
                byte[] buff = new byte[46 - data.length];
                java.util.Arrays.fill(buff, (byte) 0x00);
                data = TUtils.addtab(data, buff);
                // data = filltosize(data, 46);
            }
            //preambule : 7 octets of 10101010
            //Start delimiter : 1 octet of 10101011
            byte[] res = null;
            res = TUtils.addtab(res, ((O_MACaddress) getOptions().getValuefor("eth1_dest")).getValue());
            res = TUtils.addtab(res, ((O_MACaddress) getOptions().getValuefor("eth1_src")).getValue());
            byte[] szbuff = TUtils.prend("DEC", data.length);
            if (szbuff.length == 1) {
                szbuff = TUtils.addtab(new byte[]{0x00}, szbuff);
            }
            res = TUtils.addtab(res, szbuff);
            res = TUtils.addtab(res, data);
            toLog(this, "dst: " + TUtils.Formate_Hexa(((O_MACaddress) getOptions().getValuefor("eth1_dest")).getValue(), true)
                    + ",src: " + TUtils.Formate_Hexa(((O_MACaddress) getOptions().getValuefor("eth1_src")).getValue(), true)
                    + ",size: " + TUtils.Formate_Hexa(szbuff, true)
                    + ",data: " + TUtils.Formate_Hexa(data, false), false);
            /* crc32.update(res);
             byte[] crc=util.prend("DEC", crc32.getValue());
             if (crc.length < 4) {
             crc=filltosize(crc,4);
             }
             res = util.addtab(res, crc );*/
            packet.setObj(res);
            finalres.add(packet);
        }
        return finalres;
    }

    /**
     * Apply Options() modification to store receved data and disable checks
     *
     * @param bool
     */
    @Override
    public void setCopyMode(boolean bool) {
        getOptions().setValuefor("eth1_copy", bool);
        getOptions().setValuefor("eth1_chksrc", !bool);
        getOptions().setValuefor("eth1_chkdest", !bool);
    }

    /**
     * evaluate if the packet list match with this protocol
     *
     * @param pack the list of messages to test 1 is OK
     * @return the 'probabilitie to
     */
    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            byte[] frame = pack.getBytes();
            if (frame.length < 14) {
                res = 0.0;
            } else if (!(pack.getProtocols() == null || pack.getProtocols().isEmpty()
                    || (pack.getProtocols().size()==1 && pack.getProtocols().get(0).getOsi()==0.0))) {
                res = 0.1;
            } else {
                byte[] buff = java.util.Arrays.copyOfRange(frame, 12, 14);
                int size = 256 * getInt(buff[0]) + getInt(buff[1]);
                if (size + 14 > 1514) {
                    res = 0.0;
                } else {
                    res=0.2;
                    if (frame.length == size + 14) {
                        res = 0.99;
                    } else if (frame.length<size+14){
                        //no fragmentation possible on ETHERNET
                        res=0.0;
                    }else if (frame.length <= size + 14 + 16) {
                        //Trailer max 16??
                        boolean isNull = true;
                        for (int i = size + 14; i < frame.length && isNull; i++) {
                            isNull = frame[i] == 0;
                        }
                        if (isNull) {
                            return 0.8;
                        }
                    }
                }
            }
        }
        return res;
    }
}
