package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Format;
import com.jdcm.common.C_Packet;
import com.jdcm.common.D_Utils;

/**
 * Follow ONLY tcp or udp flow of DATA from a byte array
 *
 * @author Marc Giboin
 */
public class Stack_FollowStr extends Stack_base {

    private final D_Utils TUtils = new D_Utils();

    public Stack_FollowStr() {
        super();
        getOptions().addOption("flw_port", -1, "int", "", true, true, "TCP/UDP port to follow");
        getOptions().addOption("flw_isTcp", true, "boolean", "", true, true, "follow TCP Streams");
        getOptions().addOption("flw_isUdp", false, "boolean", "", true, true, "follow UDP Streams");
        getOptions().addOption("flw_addHdr", false, "boolean", "", true, true, "add header [,,] or <> with time,src,dst,size,..  ");
        getOptions().addOption("flw_toXML", false, "boolean", "", true, true, "send packet as XML");
        // getOptions().addOption("flw_chgSrc", false, "boolean", "", true, true, "avoid source if Tcp dest port match");
    }

    public static String getComment() {
        return "Followig tcp/udp";
    }

    private int getport(byte[] ar) {
        int res = -1;
        //System.out.println("port=>"+ar[0]+","+ar[1]);
        if (ar != null && ar.length == 2) {
            if (ar[0] < 0) {
            }
            res = 256 * getPositiveByte(ar[0]) + getPositiveByte(ar[1]);
        }
        return res;
    }

    private int getPositiveByte(byte b) {
        if (b < 0) {
            return 256 + b;
        } else {
            return b;
        }
    }

    public static double getosi() {
        return 4.0;
    }

    /**
     * encapsule payload in header to form packet
     *
     * @param packet the payload
     * @return the packets
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        //liste.add(packet);
        return liste;
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param  packet
     * @return the payloads
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        boolean istcp = getOptions().getValuefor("flw_isTcp", true);
        boolean isudp = getOptions().getValuefor("flw_isUdp", true);
        int port = (Integer) getOptions().getValuefor("flw_port");
        //treated as string to avoid pb if jpcap not loaded
        byte[] res = null;
        com.jdcm.common.XMLres res2 = new com.jdcm.common.XMLres("PacketCap");
        int portdst = -1;
        int portsrc = -1;
        if (packet.getObj().getClass().getCanonicalName().startsWith("jpcap.packet.")) {
            // ------------JPCAP CAPTURE PACKET------------------------
            if (packet.getObj() instanceof jpcap.packet.TCPPacket
                    || packet.getObj() instanceof jpcap.packet.TCPPacket) {
                if (istcp && packet.getObj() instanceof jpcap.packet.TCPPacket) {
                    jpcap.packet.TCPPacket tcp = (jpcap.packet.TCPPacket) packet.getObj();
                    portsrc = tcp.src_port;
                    portdst = tcp.dst_port;
                    if (port < 0 || portsrc == port || portdst == port) {
                        res = tcp.data;
                        if (getOptions().getValuefor("flw_toXML", true)) {
                            res2.addAttribut("trsp_data", TUtils.Formate_Hexa(tcp.data, true));
                        }
                    }
                } else if (isudp && packet.getObj() instanceof jpcap.packet.UDPPacket) {
                    jpcap.packet.UDPPacket udp = (jpcap.packet.UDPPacket) packet.getObj();
                    portsrc = udp.src_port;
                    portdst = udp.dst_port;
                    if (port < 0 || portsrc == port || portdst == port) {
                        res = udp.data;
                        if (getOptions().getValuefor("flw_toXML", true)) {
                            res2.addAttribut("trsp_data", TUtils.Formate_Hexa(udp.data, true));
                        }
                    }
                }
            }
            jpcap.packet.Packet pack = (jpcap.packet.Packet) packet.getObj();
            java.sql.Timestamp ts = new java.sql.Timestamp(1000 * pack.sec + (Long) (pack.usec / 1000));
            if (getOptions().getValuefor("flw_addHdr", true)) {
                if (getOptions().getValuefor("flw_toXML", true)) {
                    //create a mapping file for Objects...
                    res2.addAttribut("id", packet.getsource().getPortName() + "_" + pack.sec + "_" + pack.usec);
                    res2.setPrimaryKeys("id");
                    res2.addAttribut("timestamp", ts);
                    res2.addAttribut("usec", pack.usec);
                    res2.addAttribut("port", packet.getsource().getPortName());
                    res2.addAttribut("size", pack.caplen);
                    if (pack.datalink instanceof jpcap.packet.EthernetPacket) {
                        jpcap.packet.EthernetPacket link = (jpcap.packet.EthernetPacket) pack.datalink;
                        res2.addAttribut("link_type", "Ethernet");
                        res2.addAttribut("link_src", link.getSourceAddress());
                        res2.addAttribut("link_dest", link.getDestinationAddress());
                        res2.addAttribut("link_type", link.frametype);
                    }
                    if (packet.getObj() instanceof jpcap.packet.TCPPacket
                            || packet.getObj() instanceof jpcap.packet.TCPPacket) {
                        if (packet.getObj() instanceof jpcap.packet.TCPPacket) {
                            jpcap.packet.TCPPacket tcp = (jpcap.packet.TCPPacket) packet.getObj();
                            res2.addAttribut("trsp_type", "TCP");
                            res2.addAttribut("rx_src", tcp.src_ip);
                            res2.addAttribut("rx_dest", tcp.dst_ip);
                            res2.addAttribut("trsp_src", tcp.src_port);
                            res2.addAttribut("trsp_dest", tcp.dst_port);

                        } else {
                            jpcap.packet.UDPPacket udp = (jpcap.packet.UDPPacket) packet.getObj();
                            res2.addAttribut("trsp_type", "UDP");
                            res2.addAttribut("rx_src", udp.src_ip);
                            res2.addAttribut("rx_dest", udp.dst_ip);
                            res2.addAttribut("trsp_src", udp.src_port);
                            res2.addAttribut("trsp_dest", udp.dst_port);
                        }
                    }
                } else {
                    // Header for byte[]
                    String hdr = "["
                            + packet.getsource().getPortName() + "_" + pack.sec + "_" + pack.usec
                            + "," + portsrc
                            + "," + portdst
                            + "," + ts
                            + "]";
                    res = TUtils.addtab(hdr.getBytes(), res);
                    C_Format c = new C_Format();
                    c.setvalue("" + hdr.getBytes().length);
                    res = TUtils.addtab(TUtils.fill_left(c.B_Prend("INT"), (byte) 0, 4), res);
                }
            }
        } else if (packet.getBytes() != null) {
            // ------------ETHERNET BYTE ARRAYS ------------------------
            byte[] buff = packet.getBytes();
            int firsttcp = 34;
            if (buff[14] >= 0x60) {
                //  checks ipV6 or ipV4 
                firsttcp = firsttcp + 20;
            }
            if (buff.length >= firsttcp + 4 && buff[12] == 8 && buff[13] == 0) {
                // System.out.println("tcp=>"+buff[33]+","+buff[34]+","+buff[35]+","+buff[36]+","+buff[37]+","+buff[38]+","+buff[39]);
                portdst = getport(new byte[]{buff[firsttcp + 2], buff[firsttcp + 3]});
                portsrc = getport(new byte[]{buff[firsttcp], buff[firsttcp + 1]});
                // System.out.println("port Source : " + portsrc + "; Dest : " + portdst);
                boolean isGoodPort = port < 0 || port == portdst || port == portsrc;
                if (isGoodPort && istcp && buff.length >= 54 && buff[23] == 6) {
                    res = java.util.Arrays.copyOfRange(buff, 54, buff.length);
                    if (getOptions().getValuefor("flw_toXML", true)) {
                        res2.addAttribut("trsp_data", TUtils.Formate_Hexa(res, true));
                    }
                }
                if (isGoodPort && isudp && buff[23] == 17) {
                    res = java.util.Arrays.copyOfRange(buff, 38, buff.length);
                    if (getOptions().getValuefor("flw_toXML", true)) {
                        res2.addAttribut("trsp_data", TUtils.Formate_Hexa(res, true));
                    }
                }
                if (getOptions().getValuefor("flw_addHdr", true)) {
                    long sec = Math.round(packet.getCreatetime().getTime() / 1000);
                    long usec = 1000 * (packet.getCreatetime().getTime() - (sec * 1000));
                    if (getOptions().getValuefor("flw_toXML", true)) {
                        //create a mapping file for Objects...
                        res2.addAttribut("id", packet.getsource().getPortName()
                                + "_" + sec
                                + "_" + usec);
                        res2.setPrimaryKeys("id");
                        res2.addAttribut("timestamp", packet.getCreatetime());
                        res2.addAttribut("usec", sec * 1000);
                        res2.addAttribut("port", packet.getsource().getPortName());
                        if (res!=null){
                        res2.addAttribut("size", res.length);
                        }else{
                            res2.addAttribut("size", 0);
                        }
                        if (istcp) {
                            res2.addAttribut("trsp_type", "TCP");
                        } else {
                            res2.addAttribut("trsp_type", "UDP");
                        }
                        res2.addAttribut("trsp_src", portsrc);
                        res2.addAttribut("trsp_dest", portdst);
                    } else {
                        // Header for byte[]
                        String hdr = "["
                                + packet.getsource().getPortName() + "_" + sec + "_" + usec
                                + "," + portsrc
                                + "," + portdst
                                + "," + packet.getCreatetime()
                                + "]";
                        res = TUtils.addtab(hdr.getBytes(), res);
                        C_Format c = new C_Format();
                        c.setvalue("" + hdr.getBytes().length);
                        res = TUtils.addtab(TUtils.fill_left(c.B_Prend("INT"), (byte) 0, 4), res);
                    }
                }
            }
        }
        if (getOptions().getValuefor("flw_toXML", true)) {
            // output as XMLres()
            if ( res2.length() > 0) {
                packet.setObj(res2);
                liste.add(packet);
            }
        } else {
            // output as byte []
            if (res != null && res.length > 0) {
                packet.setObj(res);
                liste.add(packet);
            }
        }
        return liste;
    }
}
