package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import java.util.List;
import java.util.Arrays;

/**
 * SEND/RECEIVE ICMP packets on Ethernet
 *
 * Internet Control Management Protocol !! see faille de Fernando Gont
 * http://www.frameip.com/entete-tcp/
 *
 * @author Marc Giboin
 */
public class Stack_ICMP extends Stack_baseIP {

    // buffer is the list of current asked ping
    // asked ping are "forgeted" if "icmp_to" reached or reply receved
    private final java.util.List<C_Packet> buffer = new java.util.ArrayList<>();

    public Stack_ICMP() {
        super();
        byte icmptyp = 0x08;
        byte icmpcode = 0x00;
        byte[] icmpid = new byte[]{0x01, 0x00};
        int icmpto = 1500;
        getOptions().addOption("icmp_ver", -1, "int", "", true, true, "version of  ICMPv4 or ICMPv6 packet");
        getOptions().addOption("icmp_typ", icmptyp, "byte", "", true, true, "type of ICMP packet");
        getOptions().addOption("icmp_cod", icmpcode, "byte", "", true, true, "code of ICMP packet");
        getOptions().addOption("icmp_id", icmpid, "byte[]", "", true, true, "icmp Identification");
        getOptions().addOption("icmp_to", icmpto, "int", "", true, true, "icmp reply waiting time(ms)");
        getOptions().addOption("icmp_folw", false, "bool", "0/1", true, true, "transmit icmp data");
    }

    public void seticmp_ver(int ver) {
        setName(getClass().getSimpleName().substring(6) + "_V" + ver);
    }

    public static String getComment() {
        return "Internet Control Maganement Protocol";
    }

    public static double getosi() {
        return 3.2;
    }

    @Override
    public void onClock() {
        int to = (Integer) getOptions().getValuefor("icmp_to");
        for (C_Packet p : buffer) {
            if (p.getStoredtime().getTime() < System.currentTimeMillis() - to) {
                buffer.remove(p);
            }
        }
    }

    /**
     * check packet change MAC ? according to route Answer to request for my IP
     *
     * @param packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        // routting protocol => do nothing except if routing protocol ...
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        int sipproto = ((Integer) getStack().getinfos(getport(), "IP", "ip_sproto", SEARCH_DOWN, this));
        if (sipproto != 0x01) {
            getOptions().setValuefor("icmp_ver", 4);
        } else if (sipproto != 0x3a) {
            getOptions().setValuefor("icmp_ver", 6);
        } else {
            liste.add(packet);
            return liste;
        }
        if (pack.length <= 8) {
            throw new Exception("packet length is " + pack.length + "+ is not enought for ICMP");
        }
        byte[] id = Arrays.copyOfRange(pack, 4, 6);
        byte[] nbsec = Arrays.copyOfRange(pack, 6, 8);
        byte res = java.util.Arrays.copyOfRange(pack, 0, 1)[0];
        if (res == (Byte) getOptions().getValuefor("icmp_typ")) {
            // echo request
            byte[] data = Arrays.copyOfRange(pack, 8, pack.length);
            byte[] temp = TUtils.addtab(id, TUtils.addtab(nbsec, data));
            byte[] buff = new byte[]{0x00, 0x00};
            int crc = ip_sum_calc(TUtils.addtab(buff, temp));
            byte[] l = TUtils.prend("DEC", crc);
            if (l.length < 2) {
                l = TUtils.addtab(new byte[]{0x00}, l);
            }
            buff = TUtils.addtab(buff, l);
            buff = TUtils.addtab(buff, temp);
            C_Packet d = new C_Packet(buff, packet.getsource());
            d.setdestination(getport());
            getStack().reply_out(this, d);
            if (getOptions().getValuefor("icmp_folw", true)) {
                packet.setObj(data);
                liste.add(packet);
                return liste;
            }
        } else if (res == 0x00) {
            // echo reply should check id & secquencenb
            //when using for(pack:buffer) => don't continue after the loop !!
            boolean removed = false;
            for (int i = 0; buffer.isEmpty() == false && i < buffer.size(); i++) {
                if (removed) {
                    i -= 1;
                }
                C_Packet p = buffer.get(i);
                removed = false;
                byte[] trame = p.getBytes();
                if (trame != null & trame.length > 8) {
                    byte[] id2 = Arrays.copyOfRange(trame, 4, 6);
                    byte[] nbsec2 = Arrays.copyOfRange(trame, 6, 8);
                    if (Arrays.equals(id2, id) && Arrays.equals(nbsec2, nbsec)) {
                        long rtt = System.currentTimeMillis() - p.getStoredtime().getTime();
                        //... quid de i?? i--?
                        buffer.remove(p);
                        removed = true;
                    }
                }
            }
            packet.setObj(Arrays.copyOfRange(pack, 8, pack.length));
            liste.add(packet);
            return liste;
        }
        return null;
    }

    /**
     *
     * @param packet to  transmit
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        //Pb : if UDP or TCP > How to simply formard packet here ??
        List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        byte[] icmptyp = {(Byte) getOptions().getValuefor("icmp_typ")};
        byte[] icmpcod = {(Byte) getOptions().getValuefor("icmp_cod")};

        byte[] id = (byte[]) getOptions().getValuefor("icmp_id");
        int nbsec = (int) Math.round(Math.random() * 0xFFFF);
        byte[] sec = TUtils.prend("DEC", nbsec);
        if (sec.length < 2) {
            sec = TUtils.addtab(new byte[]{0x00}, sec);
        }
        byte[] buff = TUtils.addtab(icmptyp, icmpcod);
        byte[] temp = TUtils.addtab(id, TUtils.addtab(sec, pack));
        // ON IPV4 CRC is made with packet : ICMP & data, 
        // ON IPV6 need ip pseudo sesion & ICMP
        int crc = ip_sum_calc(TUtils.addtab(buff, temp));
        byte[] lcrc = TUtils.prend("DEC", crc);
        if (lcrc.length < 2) {
            lcrc = TUtils.addtab(new byte[]{0x00}, lcrc);
        }
        buff = TUtils.addtab(buff, lcrc);
        buff = TUtils.addtab(buff, temp);
        packet.setObj(buff);
        C_Packet stored = (C_Packet) packet.clone();
        stored.setStoredtime(new java.util.Date(System.currentTimeMillis()));
        buffer.add(stored);
        res.add(packet);
        if (getStack() != null) {
            if (getOptions().getValuefor("ipcmp_ver",1) == 6) {
                getStack().setinfos(getport(), "IP4", SEARCH_DOWN, this, "ip_sproto", 0x3a);
            } else {
                getOptions().setValuefor("ipcmp_ver", 4);
                getStack().setinfos(getport(), "IP4", SEARCH_DOWN, this, "ip_sproto", 0x01);
            }
        }
        return res;
        //need to way answer and send packet
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer() && pack.getBytes().length > 8) {
            java.util.List<I_stack_elt> prots = pack.getProtocols();
            if (prots != null && prots.size() > 0 && prots.get(prots.size() - 1) instanceof Stack_IP) {
                Stack_IP ip = (Stack_IP) prots.get(prots.size() - 1);
                if (ip.getOptions().getValuefor("ip_sproto", 1) == 1) {
                    getOptions().setValuefor("icmp_ver", 4);
                    return 0.9;
                } else if (ip.getOptions().getValuefor("ip_sproto", 1) == 0x3a) {
                    getOptions().setValuefor("icmp_ver", 6);
                    return 0.9;
                }
            }
            res = 0.1;
            byte[] frame = pack.getBytes();
            int type = getInt(frame[0]);
            int code = getInt(frame[1]);
            if (type < 41) { // uselly 18
                //ICMP V4 or V6 possible answers
                if (type == 3 && code <= 15) {
                    res = 0.2;
                }
                if ((type == 0 || type == 4 || type > 8) && code == 0) {
                    // code in echo is always 0
                    res = 0.2;
                }
                if (type == 5 && code < 4) {
                    // code in echo is always 0
                    res = 0.2;
                }
            } else if (type >= 100) {
                //ICMPV6 source : http://en.wikipedia.org/wiki/ICMPv6
                if (code == 0 && type < 155) {
                    res = 0.3;
                } else if (code == 1 && type >= 138 && type <= 140) {
                    res = 0.3;
                } else if (code == 2 && type >= 139 && type <= 140) {
                    res = 0.3;
                } else if (code == 255 && type == 138) {
                    res = 0.3;
                }
            }
            if (res == 0.2) {
                // NOT working on ICMPV6=> need pseudoIP
                if (ip_sum_calc(frame) == 0) {
                    res = 0.999;
                }
            }
            if (pack.getProtocols() != null && pack.getProtocols().isEmpty() == false) {
                if (pack.getProtocols().get(pack.getProtocols().size() - 1) instanceof Stack_IP) {
                    Stack_IP elt = (Stack_IP) pack.getProtocols().get(pack.getProtocols().size() - 1);
                    if (elt.getOptions().getValuefor("ip_sproto", 1) == 58) { //0x3a=> ICMPv6
                        res = 0.9999;
                    }
                }
            }
        }
        return res;
    }
}
