package com.jdcm.modifiers.stacks;

import com.jdcm.eltrx.I_Port;
import java.util.Arrays;
import java.net.NetworkInterface;
import java.util.List;
import java.net.InetAddress;
import com.jdcm.common.C_Packet;
import static com.jdcm.modifiers.stacks.Stack_base.SEARCH_DOWN;
import java.net.SocketException;
import java.net.UnknownHostException;

/**
 * SEND/RECEIVE IP packet on Ethernet
 *
 * http://www.frameip.com/ NO OPTION MANAGEMNENT TODO : if IP on IP (labels are
 * IP & IP*, what's about set/getinfos(..)? TODO : IP V6 TODO : IP V4 option
 * management
 *
 * @author Marc GIBOIN
 */
public class Stack_IP extends Stack_baseIP {

    public Stack_IP() {
        super();
        InetAddress ipsrc = null;
        InetAddress ipdst = null;
        try {
            ipsrc = InetAddress.getByName("192.168.110.110");
            ipdst = InetAddress.getByName("192.168.110.254");//;new byte[]{-64, -88, 0x78, 0x01}
        } catch (UnknownHostException ex) {
        }
        getOptions().addOption("ip_chksproto", true, "Boolean", "", true, true, "check ip sub-proto(tcp/udp/...) match with ip_proto");
        getOptions().addOption("ip_chksrc", true, "Boolean", "", true, true, "check received ip src is my id_dest");
        getOptions().addOption("ip_chkdest", false, "Boolean", "", true, true, "check received ip dest is my ip_src");
        getOptions().addOption("ip_copy", false, "Boolean", "", true, true, "copy incomming addr(src&dst) & proto ");
        getOptions().addOption("ip_readme", false, "Boolean", "", true, true, "Allow reading of msg emitted with my ip_src");
        getOptions().addOption("ip_vers", -1, "int", "", true, true, "IP version ");
        getOptions().setused("ip_vers", false);
        getOptions().addOption("ip_defver", 4, "int", "", true, true, "default IP version");
        getOptions().addOption("ip_src", ipsrc, "InetAddress", "", true, true, "source IP adress");
        getOptions().addOption("ip_dest", ipdst, "InetAddress", "", true, true, "destination IP adress");
        getOptions().addOption("ip_sproto", 1, "int", "", true, true, "Level4 protocol reference");
        getOptions().addOption("ip_tos", new byte[]{0x00}, "Byte[]", "", true, true, "IP Type of service");
        getOptions().addOption("ip_ttl", new byte[]{0x40}, "Byte[]", "", true, true, "IP Time To Live");
        getOptions().addOption("ip_mtu", 500, "int", "", true, true, "Maximum Transmission Unit");
        javax.swing.JButton b = new javax.swing.JButton("src & mtu");
        b.setMargin(new java.awt.Insets(0, 0, 0, 0));
        b.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                getsystemsinfo();
            }
        });
        getOptions().addOption("ip_loadSys", b, "button", "", true, true, "ask for System IP adress & mtu");
    }

    public static String getComment() {
        return "Internet protocol";
    }

    public static double getosi() {
        return 3.1;
    }

    private void getsystemsinfo() {
        NetworkInterface neth;
        java.util.Enumeration<NetworkInterface> ent = null;
        try {
            ent = NetworkInterface.getNetworkInterfaces();
        } catch (java.net.SocketException ex) {
        }
        if (ent != null) {
            boolean trouve = false;
            while (ent.hasMoreElements() && trouve == false) {
                neth = ent.nextElement();
                try {
                    if (neth.isUp()) {
                        int mtu = neth.getMTU();
                        java.util.Enumeration<InetAddress> addr = neth.getInetAddresses();
                        if (mtu > 0 && neth.isLoopback() == false) {
                            trouve = true;
                            getOptions().setValuefor("ip_mtu", mtu);
                            while (addr.hasMoreElements()) {
                                java.net.InetAddress adrs = addr.nextElement();
                                if (adrs instanceof java.net.Inet4Address) {
                                    getOptions().setValuefor("ip_src", adrs);
                                }
                            }
                        }
                    }
                } catch (SocketException t) {
                }
            }
        }

    }

    @Override
    public void setParents(I_Port idx, com.jdcm.eltrx.C_Stack modmangt) {
        super.setParents(idx, modmangt);
        if (getport().getModeName().compareTo("ETH") == 0) {
            if (getport().getClass().getName().endsWith("IF_Port")) {//!!!
                com.jdcm.eltrx.C_Port p = (com.jdcm.eltrx.C_Port) getport();
                getOptions().setValuefor("ip_src", p.getETH_firstIP());
            }
        }
    }

    @Override
    public void StkArcv(String message, String type) {
    }

    @Override
    public void onClock() {
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param packet
     * @return the payloads
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        int optionSize=0;
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        byte[] res;
        if (getStack() != null) {
            // check if IP is waited by previous stack
            res = (byte[]) getStack().getinfos(getport(), "ETHII", "eth2_sproto", SEARCH_DOWN, this);
           Object r= getStack().getinfos(getport(), "PPP", "ppp_sproto", SEARCH_DOWN, this);
            if (res!=null && Arrays.equals(res, new byte[]{0x08, 0x00}) == false
                    ||r!=null && ((byte)r)==0x21) {
                return null;
            }
        }
        byte oct = java.util.Arrays.copyOfRange(pack, 0, 1)[0];
        // check dest of tho packet is the source of the module (Reception)

        int ipver = getOptions().getValuefor("ip_vers", 1);
        if (ipver < 0 || (oct >> 4) == ipver) {
            ipver = (oct >> 4);
            int hdrsz = (oct & 0x0f) * 4;
            res = java.util.Arrays.copyOfRange(pack, 2, 4);
            byte[] proto = java.util.Arrays.copyOfRange(pack, 9, 10);
            byte[] msgsrc = java.util.Arrays.copyOfRange(pack, 12, 16);
            byte[] msgdst = java.util.Arrays.copyOfRange(pack, 16, 20);
            if (ipver == 6) {
                hdrsz = 40;
                res = java.util.Arrays.copyOfRange(pack, 4, 6);
                proto = java.util.Arrays.copyOfRange(pack, 6, 7);
                msgsrc = java.util.Arrays.copyOfRange(pack, 8, 24);
                msgdst = java.util.Arrays.copyOfRange(pack, 24, 40);
            }
            int taille = Integer.parseInt(TUtils.retire("DEC", res));
            int sproto = (int) proto[0];
            if (ipver == 6) {
                // OPTIONS
                if (pack[6] == 0 || pack[6] == 43 || pack[6] == 51 || pack[6] == 60) { //hop by hop ; routage;AH; destination 
                    optionSize=pack[41] + 2;
                    sproto = pack[40];
                } else if (pack[6] == 44) {//frazgmentation
                    optionSize=8;
                     sproto = pack[40];
                } else if (pack[6] == 50) {//ESP
                     optionSize=1;
                     sproto = pack[40];
                }
            }
            if (sproto == getOptions().getValuefor("ip_sproto", 1)
                    || getOptions().getValuefor("ip_chksproto", true) == false) {
                boolean ischkipsrc = getOptions().getValuefor("ip_chksrc", true);
                // does I have to accepts packet with my address of sour address (default no)
                if (Arrays.equals(msgsrc, ((InetAddress) getOptions().getValuefor("ip_src")).getAddress()) == false
                        || getOptions().getValuefor("ip_readme", true)) {
                    //Does this message is comming from my destination addr (if checked) ?
                    if (Arrays.equals(msgsrc, ((InetAddress) getOptions().getValuefor("ip_dest")).getAddress())
                            || getOptions().getValuefor("ip_chkdest", true) == false) {

                        if (hdrsz > 20 && ipver == 4) {
                            byte[] opt = java.util.Arrays.copyOfRange(pack, 20, hdrsz);
                        }
                        //!!! options (route reccord/source route/..)not imlemented
                        if (Arrays.equals(msgdst, ((InetAddress) getOptions().getValuefor("ip_src")).getAddress())
                                || ischkipsrc == false) {
                            if (getOptions().getValuefor("ip_copy", true)) {
                                getOptions().setValuefor("ip_vers", ipver);
                                getOptions().setValuefor("ip_src", InetAddress.getByAddress(msgsrc));
                                getOptions().setValuefor("ip_dest", InetAddress.getByAddress(msgdst));
                                getOptions().setValuefor("ip_sproto", sproto);
                            }
                            if (ipver == 6) {
                                //payload
                                packet.setObj(java.util.Arrays.copyOfRange(pack, hdrsz+optionSize, hdrsz + taille));
                            } else {
                                //length
                                packet.setObj(java.util.Arrays.copyOfRange(pack, hdrsz, taille)); //packet.length
                            }
                            liste.add(packet);
                            return liste;
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * encpsule payload in header to form packet several packet if fragmented
     *
     * @param packet the payload
     * @return the packet(s)
     * @throws java.lang.Exception
     */
    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        byte[] opt = new byte[0];
        int i = 0;
        //identification building (befor fragmentation)
        byte[] idt = TUtils.prend("DEC", (int) (0xFFFF * Math.random()));
        if (idt.length < 2) {
            idt = TUtils.addtab(new byte[]{0x00}, idt);
        }
        //building fragments in fragments List
        List<byte[]> fragments = new java.util.ArrayList<>();
        while (i < pack.length) {
            int mtu = getOptions().getValuefor("ip_mtu", 1);
            //Fragment payload size must be a multiple of 8 bytes, except for the last one
            mtu = (mtu / 8) * 8;
            if (mtu > pack.length - i) {
                fragments.add(Arrays.copyOfRange(pack, i, pack.length));
                i = pack.length;
            } else {
                fragments.add(Arrays.copyOfRange(pack, i, i + mtu));
                i += mtu;
            }
        }

        int fragment_ofset = 0;
        //building fragmented packet in temp
        List<C_Packet> temp = new java.util.ArrayList<>();
        for (byte[] bmtu : fragments) {
            //2
            int vers = getOptions().getValuefor("ip_vers", 1);
            if (vers < 0) {
                vers = getOptions().getValuefor("ip_defver", 1);
            }
            byte[] res = new byte[]{(byte) ((vers << 4) + (20 + opt.length) / 4)};
            res = TUtils.addtab(res, getOptions().getValuefor("ip_tos", new byte[]{}));
            //2 total lenght
            byte[] l = TUtils.prend("DEC", bmtu.length + 20 + opt.length);
            if (l.length < 2) {
                l = TUtils.addtab(new byte[]{0x00}, l);
            }
            res = TUtils.addtab(res, l);
            //2 identifiaction (d'un fragment de packet
            res = TUtils.addtab(res, idt);
            //2flag (3bit) & ofset(13bit) /2 oct
            //see with fragments
            int frg_field = fragment_ofset / 8;
            if (bmtu.length + fragment_ofset < pack.length) {
                //more fragment
                frg_field ^= 0x2000;
            } else {
                frg_field ^= 0x4000;
            }
            byte[] frg = TUtils.prend("DEC", frg_field);
            //  frg=new byte[]{0x40, 0x00};
            res = TUtils.addtab(res, frg);
            //2
            res = TUtils.addtab(res, getOptions().getValuefor("ip_ttl", new byte[]{}));
            int sproto = getOptions().getValuefor("ip_sproto", 1);
            res = TUtils.addtab(res, new byte[]{(byte) sproto});
            // CRC/2oct
            //10
            res = TUtils.addtab(res, new byte[]{0x00, 0x00});
            res = TUtils.addtab(res, ((InetAddress) getOptions().getValuefor("ip_src")).getAddress());
            res = TUtils.addtab(res, ((InetAddress) getOptions().getValuefor("ip_dest")).getAddress());
            //!!! options (route reccord/source route/..)not imlemented
            int crc = ip_sum_calc(res);
            l = TUtils.prend("DEC", crc);
            if (l.length < 2) {
                l = TUtils.addtab(new byte[]{0x00}, l);
            }
            res = java.util.Arrays.copyOf(res, res.length - 4 - 4 - 2);
            res = TUtils.addtab(res, l);
            res = TUtils.addtab(res, ((InetAddress) getOptions().getValuefor("ip_src")).getAddress());
            res = TUtils.addtab(res, ((InetAddress) getOptions().getValuefor("ip_dest")).getAddress());
            res = TUtils.addtab(res, bmtu);
            fragment_ofset += bmtu.length;
            C_Packet packt = (C_Packet) packet.clone();
            packt.setObj(res);
            //new C_Packet(res,dat.getsource());
            temp.add(packt);
        }
        if (getStack() != null) {
            getStack().setinfos(getport(), "ETHII", SEARCH_DOWN, this, "eth2_sproto", new byte[]{0x08, 0x00});
        }
        return temp;
    }

    /**
     * Apply Options() modification to store receved data and disable checks
     *
     * @param bool
     */
    @Override
    public void setCopyMode(boolean bool) {
        getOptions().setValuefor("ip_copy", bool);
        getOptions().setValuefor("ip_readme", bool);
        getOptions().setValuefor("ip_chksproto", !bool);
        getOptions().setValuefor("ip_chksrc", !bool);
        getOptions().setValuefor("ip_chkdest", !bool);
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            byte[] frame = pack.getBytes();
            java.util.List<I_stack_elt> prots = pack.getProtocols();
            if (prots != null && prots.size() > 0 && prots.get(prots.size() - 1) instanceof Stack_ETHII) {
                Stack_ETHII tcp = (Stack_ETHII) prots.get(prots.size() - 1);
                byte[] prot = tcp.getOptions().getValuefor("eth2_sproto", new byte[]{});
                if (prot != null && prot.length == 2 && prot[0] == 8 && prot[1] == 0) {
                    res = 0.999;
                }
            }else  if (prots != null && prots.size() > 0 && prots.get(prots.size() - 1) instanceof Stack_PPP) {
                Stack_PPP ppp = (Stack_PPP) prots.get(prots.size() - 1);
              //  byte prot =;
                if (  ppp.getOptions().getValuefor("ppp_sproto", (byte)0)== 0x21) {
                    res = 0.999;
                }
            }
            if (frame.length > 24 && (frame[0] >= 0x45 || frame[0] >= 0x46) && frame[0] <= 0x4f) {
                //6<l<15 for ip  getOptions().. unused??
                //IPV4
                if (frame.length == array2dec(new byte[]{frame[2], frame[3]})) {
                    res = 0.9980;
                }
            } else if (frame.length > 40 && frame[0] >= 0x60 && frame[0] < 0x70) {
                if (frame.length == array2dec(new byte[]{frame[4], frame[5]}) + 40 ) { // 40 is IPV6 size without options
                    // to check with IPV6 options 
                    res = 0.9980;
                }
            }
        }
        return res;
    }
}
