package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;

/**
 *
 * SEND/RECEIVE Link Layer Protocol (LLC) on Ethernet
 * near HDLC protocol and used by NETBEUI (microsoft) older than TCP/IP used to complete OSI layer2 from MAC layer
 * 
 * UI:	Unumbered Information.,Information non numérotée	
 * SABME:	Set Asynchronous Balanced Mode Extended. (Initialisation en mode ABM Etendu.)
 * TEST	TEST: (Trame de Test.	)
 * I:	Information. (Trame transportant des informations)
 * XID	eXchange IDentification. (echange d'identifiant)	
 * disc	DISConnect (Déconnexion)
 * RR	Receive Ready (Réception OK)	
 * UA	Unnumbered Acknoledgement (Acquittement de trame non numérotée)
 * RNR	Receive Not Ready (Réception non OK)	
 * DM	Disconect Mode (Mode de déconnexion)
 * REJ	Reject (Rejet de trame)
 * FRMR	FRaMe Rejet (Rejet de Trame)
 * 
 * @author Marc GIBOIN 
 *
 */
public class Stack_LLC extends Stack_baseIP {

    public Stack_LLC() {
        super();
        getOptions().addOption("llc_dsap", 0, "int", "", true, true, "Service Access Point,IGbit");
        getOptions().addOption("llc_ssap", 0, "int", "", true, true, "SSAP,CR BIT");
        getOptions().addOption("llc_ctrl", 0, "int", "", true, true, "Control Field");
        getOptions().addOption("llc_org", new byte[]{}, "int", "", true, true, "Organisation Code");
        getOptions().addOption("llc_pid", 0, "int", "", true, true, "Sub Protocol ID");
        getOptions().addOption("llc_copy", false, "boolean", "", true, true, "will copy msg information");
    }

    public static String getComment() {
        return "Link Layer Protocol";
    }

    public static double getosi() {
        return 2.0;
    }

    /**
     * decapsule packet, check header and provide payload
     * Short implmentation to get STP protocol...
     * @param  packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        byte[] frame = packet.getBytes();
        if (frame == null || frame.length < 8) {
            toLog(this, "size < 8 " + new String(frame), true);
            return null;
        }
        getOptions().setValuefor("llc_dsap", frame[0]);
        getOptions().setValuefor("llc_ssap", frame[1]);
        getOptions().setValuefor("llc_ctrl", frame[2]);
     /*   getOptions().setValuefor("llc_org", java.util.Arrays.copyOfRange(frame, 3, 6));
        getOptions().setValuefor("llc_pid", 256 * frame[6] + frame[7]);*/
        byte[] msgdata = java.util.Arrays.copyOfRange(frame, 3, frame.length);
        packet.setObj(msgdata);
        liste.add(packet);
        return liste;
    }

    /**
     * encpsule payload in header to form packet
     * NOT IMPLEMENTED
     * @param packet the payload
     * @return the packet
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) {
        //encapsule
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        int i = 0;
        //building fragments in fragments List if trame > 1500 (datagram max size)

        return null;
    }

    /**
     * Apply Options() modification to store receved data and disable checks
     *
     * @param bool
     */
    @Override
    public void setCopyMode(boolean bool) {
        getOptions().setValuefor("llc_copy", bool);
    }

    /**
     * evaluate if the packet list match with this protocol
     *
     * @param pack the list of messages to test 1 is OK
     * @return the 'probabilitie to
     */
    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            byte[] frame = pack.getBytes();
            if (frame.length < 8) {
                res = 0.0;
            } else {
                if (pack.getProtocols() != null && pack.getProtocols().size()==1) {
                    if (pack.getProtocols().get(0) instanceof Stack_ETHI) {
                        Stack_ETHI elt = (Stack_ETHI) pack.getProtocols().get(0);
                        if (elt.getOptions().getValuefor("eth1_copy", true) != null) {
                            res = 0.9999;
                        }
                    }
                } else if (frame.length > 10 && frame[8] == 0 && frame[9] == 0) {
                    //  protocol identifier STP
                 //   res = 0.7;
                }
            }
        }
        return res;
    }
}
