package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import java.util.ArrayList;
import java.util.List;

/**
 * Simple modem emulation as MODEM (isemu) or as MODEM Recever
 *
 * @author Marc GIBOIN
 *
 */
public class Stack_Modem extends Stack_base {

    // pad start with command line, padbuffer is used for this
    private String padbuffer = "";

    public Stack_Modem() {
        super();
        init_option();
    }

    private void init_option() {
        getOptions().addOption("modm_cxmsg", "\r\nCONNECT\r\n".getBytes(), "byte[]", "", true, false, "string to send on connection");
        getOptions().addOption("modm_init", "ATS0=1\r\n".getBytes(), "byte[]", "", true, false, "string to port CXStart");
        getOptions().addOption("modm_dcxmsg", "\r\nNO CARRIER\r\n".getBytes(), "byte[]", "", true, false, "string waited to return sleep mode");
        getOptions().addOption("modm_isemu", true, "boolean", "", true, false, "true command the modem (AT waited or CONNECT waited)");
        getOptions().addOption("modm_escape", "+++", "String", "", true, true, "escape string for back to command mode");
        getOptions().addOption("modm_status", "sleep", "String", "", true, true, "status of this process");
        setmodm_isemu(getOptions().getValuefor("modm_isemu", true));
    }

    public static String getComment() {
        return "provide a modem emulation & command mode";
    }

    public static double getosi() {
        return 2.0;
    }

    public void setmodm_isemu(boolean bool) {
        getOptions().setused("modm_init", !bool);
        getOptions().setused("modm_dcxmsg", !bool);
        getOptions().setused("modm_escape", bool);
        getOptions().setused("modm_cxmsg", bool);
    }

    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        String status = getOptions().getValuefor("modm_status", "");
        toLog(this, new String(pack), false);
        String esc = getOptions().getValuefor("modm_escape", "");
        if (pack == null || pack.length == 0) {
            return null;
        }

        if (status.startsWith("connected") == false) {
            padbuffer = "" + padbuffer + new String(pack);
        } else if (esc != null && (new String(pack)).length() >= esc.length()
                && (new String(pack)).startsWith(esc)
                && getOptions().getValuefor("modm_isemu", true)) {
            // Manage escape (+++) reception emullation
            getOptions().setValuefor("modm_status", "escaped");
            status = "escaped";
            pack = java.util.Arrays.copyOfRange(pack, esc.length(), pack.length);
            if (pack.length == 0) {
                return null;
            }
        }

        if (status.compareTo("connected") == 0) {
            if (getOptions().getValuefor("modm_isemu", true)) {
                res.add(packet);
            } else {
                // NO buffer, it should append in the same packet..
                if (java.util.Arrays.equals(getOptions().getValuefor("modm_dcxmsg", new byte[]{}), packet.getBytes())) {
                    getOptions().setValuefor("modm_status", "sleep");
                } else {
                    res.add(packet);
                }
            }

        } else if (status.compareTo("sleep") == 0) {
            if (padbuffer.endsWith("\r") == false && padbuffer.endsWith("\n") == false) {
                return res;
            }
            padbuffer = padbuffer.trim();
            if (getOptions().getValuefor("modm_isemu", true) == false) {
                if (padbuffer.startsWith("CONNECT")) {
                    getOptions().setValuefor("modm_status", "connected");
                }
            } else {
                if (padbuffer.startsWith("ATO")) {
                    connect();
                } else if (padbuffer.startsWith("ATD")) {
                    packet.setObj(("\r\nDialing '" + padbuffer.substring("ATD".length()) + "'\r\n").getBytes());
                    getStack().reply_out(this, packet);
                    connect();
                } else {
                    if (padbuffer.startsWith("ATI")) {
                        // to reach Parent care id CMD is no more implmented
                        packet.setObj(("\r\n" + "modem emulation of " + getStack().getCMDParent().toString() + "\r\nOK\r\n").getBytes());
                    } else if (padbuffer.startsWith("ATZ")) {
                        getOptions().setValuefor("modm_status", "sleep");
                        packet.setObj(("\r\n" + "modem reseted\r\nOK\r\n").getBytes());
                    } else if (padbuffer.startsWith("AT&F")) {
                        init_option();
                        packet.setObj(("\r\n" + "modem factory conf\r\nOK\r\n").getBytes());
                    } else if (padbuffer.startsWith("AT")) {
                        packet.setObj("\r\nOK\r\n".getBytes());
                    } else {
                        packet.setObj("\r\nNOK\r\n".getBytes());
                    }
                    getStack().reply_out(this, packet);
                }
            }
            padbuffer = "";
        } else if (status.compareTo("escaped") == 0) {
            if (esc != null && esc.length() > 0 && esc.compareTo(new String(pack)) == 0) {
                getOptions().setValuefor("modm_status", "connected");
            } else {
                if (new String(pack).startsWith("ATZ")) {
                    packet.setObj(("\r\n" + "modem reseted\r\nOK\r\n").getBytes());
                    getOptions().setValuefor("modm_status", "sleep");
                    getStack().reply_out(this, packet);
                } else if (new String(pack).startsWith("ATH")) {
                    packet.setObj("\r\nOK\r\n".getBytes());
                    getOptions().setValuefor("modm_status", "sleep");
                    getStack().reply_out(this, packet);
                }
            }
        }

        return res;
    }

    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        // no need of upper Protocols
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        String status = getOptions().getValuefor("modm_status").toString();
        if (status.startsWith("connected")) {
            res.add(packet);
        }
        return res;

    }

    private void connect() {
        C_Packet packet = new C_Packet("", getport());
        packet.setObj("\r\nOK\r\n".getBytes());
        getStack().reply_out(this, packet);
        getOptions().setValuefor("modm_status", "connected");
        String txt = new String((byte[]) getOptions().getValuefor("modm_cxmsg"));
        if (txt.length() > 0) {
            packet.setObj(txt.getBytes());
            getStack().reply_out(this, packet);
        }
        try {
            I_stack_elt target = getStack().getTarget(getport(), "PAD");
            if (target != null) {
                ((Stack_PAD) target).setStack_isStart(false);
                ((Stack_PAD) target).setStack_isStart(true);
            }
        } catch (Exception ex) {
        }
    }

    @Override
    public void StkArcv(String message, String type) {
        if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            getOptions().setValuefor("modm_status", "sleep");
            padbuffer = "";
        }
        if (message.compareTo("connected") == 0 && type.compareTo("Status") == 0) {
            if (getOptions().getValuefor("modm_isemu", true) == false) {
                C_Packet packet = new C_Packet("", getport());
                byte[] initstr = getOptions().getValuefor("modm_init", new byte[]{});
                packet.setObj(initstr);
                if (getStack() != null) {
                    getStack().reply_out(this, packet);
                }
            }
        }
    }
    @Override
        public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            String txt=new String(pack.getBytes());
            if ( txt.length()>0 ){
                if (txt.compareTo("ATO\n")==0){
                    res=0.9;
                }else     if (txt.compareTo("+++")==0){
                    res=0.9;
                }else     if (txt.compareTo("\r\nOK\r\n")==0){
                    res=0.9;
                }else     if (txt.startsWith("\r\nCONNECT")&&txt.endsWith("\r\n") && txt.length()<=17){
                    res=0.9;
                }else     if (txt.compareTo("\r\nRING\r\n")==0){
                    res=0.9;
                }  if (txt.compareTo("\r\nBUSY\r\n")==0){
                    res=0.9;
                }if (txt.compareTo("\r\nNO CARRIER\r\n")==0){
                    res=0.9;
                }else if (txt.startsWith("\r\nERROR")&&txt.endsWith("\r\n")){
                    res=0.9;
                }  else  if (txt.startsWith("AT") && (txt.endsWith("\r")||txt.endsWith("\r\n"))){
                    res=0.9;
                }
                
            }
        }
        return res;
        }
}
