package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import com.jdcm.common.O_X25address;
import com.jdcm.common.D_Utils;
import com.jdcm.eltrx.I_Port;
import java.util.ArrayList;
import java.util.List;

/**
 * Simple pad layer, in fact just for emulation should manage assembly by CRLF/
 * by TO / by (see SET X:X X:X...).
 *
 * @author Marc GIBOIN
 *
 */
public class Stack_PAD extends Stack_baseStart {

    // pad start with command line, padbuffer is used for this
    private String padbuffer = "";
    D_Utils util = new D_Utils();
    String CR = "\r\n\u0000\u0000\u0000\u0000";

    public Stack_PAD() {
        super();
        getOptions().addOption("pad_banner", (CR + "TRANSPAC" + CR).getBytes(), "bytes", "", true, true, "the pad banner");
        getOptions().addOption("pad_prompt", "pad >", "String", "", true, true, "the commands prompt");
        getOptions().addOption("pad_com", (CR + "COM" + CR).getBytes(), "byte[]", "", true, true, "the COM message");
        getOptions().addOption("pad_commands", "SET?/SET/RESET", "String", "", true, true, "command for call");
        getOptions().addOption("pad_escape", "+++", "String", "", true, true, "escape string for back to command mode");
        getOptions().addOption("pad_status", "sleep", "String", "", true, true, "status of this process");
        getOptions().addOption("pad_even", false, "bool", "", true, true, "set comm in 7 bit even (SET21:1)");
    }

    public static String getComment() {
        return "provide a packet system (dis)assember";
    }

    public static double getosi() {
        return 5.0;
    }

    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        String status = getOptions().getValuefor("pad_status").toString();
        toLog(this, new String(pack), false);
        String esc = getOptions().getValuefor("pad_escape").toString();
        if (pack == null || pack.length == 0) {
            return null;
        }
        if (status.startsWith("connect") == false) {
            padbuffer = "" + padbuffer + new String(pack);
        } else if (esc != null && esc.length() > 0 && esc.compareTo(new String(pack)) == 0) {
            getOptions().setValuefor("pad_status", "escaped");
        }

        if (status.compareTo("connected") == 0) {
            if ((Boolean) getOptions().getValuefor("pad_even")) {
            }
            res.add(packet);
        } else if (status.compareTo("sleep") == 0) {
            if (padbuffer.endsWith("\n") == false && padbuffer.endsWith("\r") == false) {
                return res;
            }
            padbuffer = padbuffer.replaceAll("\r", "").replaceAll("\n", "");
            String cmd = getOptions().getValuefor("pad_commands").toString();

            if (cmd.indexOf(padbuffer.split(" ")[0]) >= 0) {
                // is a PAD command (or bintec minipad??)
                if (padbuffer.startsWith("SET?")) {
                    String txt = "\r\n1:1,2:2,3:3,4:4\r\n"
                            + "\r\n" + getOptions().getValuefor("pad_prompt").toString();
                    packet.setObj(txt.getBytes());
                    getStack().reply_out(this, packet);
                } else {
                    String txt = "unmanaged command\r\n[" + cmd + "]"
                            + "\r\n" + getOptions().getValuefor("pad_prompt").toString();
                    packet.setObj(txt.getBytes());
                    getStack().reply_out(this, packet);
                }
            } else {
                // this will be a CALL
                if (padbuffer.toLowerCase().startsWith("call ")) {
                    padbuffer = padbuffer.substring("call ".length());
                }
                try {
                    String[] addrcud = new String[]{padbuffer};
                    int cudpos = padbuffer.indexOf("D");
                    if (cudpos == -1) {
                        cudpos = padbuffer.indexOf("B");
                    }
                    if (cudpos != -1 && padbuffer.length() > cudpos) {
                        addrcud = new String[]{padbuffer.substring(0, cudpos), padbuffer.substring(cudpos + 1, padbuffer.length())};
                    }
                    Long.parseLong(addrcud[0]);
                    O_X25address addrsdest;
                    try {
                        addrsdest = new O_X25address(addrcud[0]);
                    } catch (Exception ex) {
                        String txt = "adresse is not Valid '" + addrcud[0] + "'"
                                + "\r\n" + getOptions().getValuefor("pad_prompt").toString();
                        packet.setObj(txt.getBytes());
                        getStack().reply_out(this, packet);
                        return res;
                    }
                    try {
                        getStack().setinfos(getport(), "X25", SEARCH_UP, this, "x25_dest", addrsdest);
                    } catch (Exception ex) {
                        String txt = "no X25 Stack on port:" + getport().getPortName()
                                + "\r\n" + getOptions().getValuefor("pad_prompt").toString();
                        packet.setObj(txt.getBytes());
                        getStack().reply_out(this, packet);
                        return res;
                    }
                    String srcaddr = "01000000";
                    byte[] cud = new byte[0];
                    if (addrcud.length > 1) {
                        cud = addrcud[1].getBytes();
                    }
                    if (addrcud.length > 1 && padbuffer.indexOf("B") > 0) {
                        srcaddr = addrcud[1].substring(0, 8);
                        addrcud[1] = addrcud[1].substring(8);
                        cud = getcudfromtxt(addrcud[1]);
                    }
                    getStack().setinfos(getport(), "X25", SEARCH_UP, this, "x25_src", new O_X25address(srcaddr));
                    if (cud.length > 1) {
                        getStack().setinfos(getport(), "X25", SEARCH_UP, this, "x25_cud", cud);
                    } else {
                        getStack().setinfos(getport(), "X25", SEARCH_UP, this, "x25_cud", new byte[0]);
                    }
                    getOptions().setValuefor("pad_status", "connected");
                    packet.setObj(getStack().getinfos(getport(), "X25", "x25_nodata", SEARCH_UP, this).toString().getBytes());
                    res.add(packet);
                } catch (Exception ex) {
                    packet.setObj(("\r\n" + getOptions().getValuefor("pad_prompt").toString()).getBytes());
                    getStack().reply_out(this, packet);
                }
            }
            padbuffer = "";
        } else if (status.compareTo("escaped") == 0) {
            if (esc != null && esc.length() > 0 && esc.compareTo(new String(pack)) == 0) {
                getOptions().setValuefor("pad_status", "connected");
            } else {
                //treat escape (as sleep?)
            }
        }
        return res;

    }

    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        // no need of upper Protocols
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        if ((new String(pack)).compareTo("X25CALLCONF") == 0) {
            if (((byte[]) getOptions().getValuefor("pad_com")).length > 0) {
                packet.setObj(getOptions().getValuefor("pad_com"));
                res.add(packet);
            }
        } else {
            res.add(packet);
        }
        toLog(this, new String(pack), true);

        return res;
    }

    private byte[] getcudfromtxt(String cud) {
        if (cud.length() > 1) {
            for (int i = cud.length(); i > 0; i = i - 2) {
                cud = cud.substring(0, i) + " " + cud.substring(i);
            }
            try {
                return util.string2byte(cud);
            } catch (Exception ex) {
                return null;
            }
        }
        return null;
    }

    @Override
    public boolean onStart() {
        try {
            C_Packet packet = new C_Packet("", getport());
            String t = new String((byte[]) getOptions().getValuefor("pad_banner"))
                    + "\r\n" + getOptions().getValuefor("pad_prompt").toString();
            if (t.length() > 0) {
                packet.setObj(t.getBytes());
                getStack().reply_out(this, packet);
            }
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public boolean onStop() {
        try {
            getOptions().setValuefor("pad_status", "sleep");
            padbuffer = "";
            try {
                getStack().setinfos(getport(), "X25", SEARCH_UP, this, "x25_status", "cleared");
            } catch (Exception ex) {
            }
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public void setParents(I_Port idx, com.jdcm.eltrx.C_Stack modmangt) {
        super.setParents(idx, modmangt);
        getOptions().setValuefor("pad_banner", (CR + "TRANSPAC from " + getStack().getCMDParent().toString() + CR).getBytes());
    }

    @Override
    public void StkArcv(String message, String type) {
        if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            setStack_isStart(false);
        }
        if (message.compareTo("connected") == 0 && type.compareTo("Status") == 0) {
            setStack_isStart(true);
        }
    }
    @Override
        public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            String txt=new String(pack.getBytes());
            if ( txt.length()>0 ){
                 if (txt.compareTo("\r\n\0\0\0\0\r\n\0\0\0\0COM\r\n\0\0\0\0")==0){
                    res=0.95;
                }else if (txt.startsWith("\r\n\0\0\0\0")&&txt.endsWith("\r\n\0\0\0\0")){
                    res=0.9;
                }  else if (txt.startsWith("SET")&&txt.contains(":")
                        && (txt.endsWith("\r")||txt.endsWith("\r\n"))){
                    // could do better
                    res=0.9;
                } else if (txt.startsWith("CALL")
                        && (txt.endsWith("\r")||txt.endsWith("\r\n"))){
                    // could do better
                    res=0.9;
                } else if (txt.startsWith("LIB")
                        && (txt.endsWith("\r")||txt.endsWith("\r\n"))){
                    // could do better
                    res=0.9;
                } 
            }
            }
        return res;
    }
}
