/*

 * 
 */
package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import java.util.ArrayList;
import java.util.List;

/**
 * RFC1086 implmentation as bridge from stream to pseudo-X25 (not implmented)
 * http://www.faqs.org/rfcs/rfc1086.html
 *
 * WHY Stack_baseIP ??
 * 
 * @author Marc GIBOIN
 */
public class Stack_RFC1086 extends Stack_baseIP {

    public Stack_RFC1086() {
        super();
        getOptions().addOption("rfc_addrL2", "000000000000", "String", "", true, true, "adress of physical device");
        getOptions().addOption("rfc_typeL2", "01", "String", "", true, true, "type of physical device (0:ETH)");
         getOptions().addOption("rfc_typeL3", "01", "String", "", true, true, "type of network  (3:X25)");
        getOptions().addOption("rfc_addrL3", "196123456", "String", "", true, true, "Network Address");
        getOptions().addOption("rfc_protoL3", "196123456", "String", "", true, true, "L3 protocol");
        getOptions().addOption("rfc_cudL3", "196123456", "String", "", true, true, "L3 User Data");
        getOptions().addOption("rfc_facL3", "196123456", "String", "", true, true, "L3 Facilities");
        getOptions().addOption("rfc_maker", "SAGEM", "String", "", true, true, "RFC plus declared maker");
        getOptions().addOption("rfc_sn", "FT11111111", "String", "", true, true, "RFC plus declared serial number");
        getOptions().addOption("rfc_pn", "12345678", "String", "", true, true, "RFC plus declared serial number");
        getOptions().addOption("rfc_addX25", true, "Boolean", "", true, true, "true adding X25 pseudo protocol else only data are sent");
        getOptions().addOption("rfc_isplus", true, "Boolean", "", true, true, "true if RFC1086 with concert implmentation");
    }

    public static String getComment() {
        return "ISO-TP0 bridge between TCP and X.25";
    }

    public static double getosi() {
        return 4.0;
    }
    /**
     *  decapsule packet, check header and trailer and provide payload
     * @param packet
     * @return
     * @throws Exception
     */
    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> liste = new ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        //    String x25status = Parent.getinfos(getportidx(), "X25", "x25_status").toString();
        if (pack != null && pack.length > 1) {
                if (pack[0] == 4 && pack.length>68) {//
                    // need pseudo X25 implementation for incomming CALL
                    // for 'CopyMode' to feel with construtor, equipment,....
                   getOptions().setValuefor("rfc_isplus",true);
                    byte[] buff=new byte[pack[0x0c]];
                   System.arraycopy(pack,2, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_maker", new String(buff));
                    buff=new byte[pack[0x17]];
                   System.arraycopy(pack,0x0d, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_pn", new String(buff));
                    buff=new byte[pack[0x2c]];
                   System.arraycopy(pack,0x18, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_sn", new String(buff));
                       buff=new byte[pack[0x41]];
                   System.arraycopy(pack,0x2d, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_addrL2", new String(buff));                 
                    getOptions().setValuefor("rfc_typeL2", pack[0x42]);                     
                    pack = java.util.Arrays.copyOfRange(pack, 0x44, pack.length);
                    //RFC1086+ removed follow with RFC1086
                  }            
                if (pack[0] == 1 && pack.length>48) {
                    //rfc simple not tested (40 is a joke)
                     getOptions().setValuefor("rfc_typeL3", 256*pack[0x1]+pack[2]); 
                    byte[] buff=new byte[pack[19]];
                   System.arraycopy(pack,3, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_addrL3", new String(buff));
                    buff=new byte[pack[25]];
                   System.arraycopy(pack,21, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_protoL3", buff);
                  buff=new byte[pack[41]];
                   System.arraycopy(pack,25, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_cudL3", buff);
                                        buff=new byte[pack[48]];
                   System.arraycopy(pack,42, buff, 0, buff.length);
                    getOptions().setValuefor("rfc_facL3", buff);
                    pack = java.util.Arrays.copyOfRange(pack, 49, pack.length);
                    //RFC1086 removed follow with reste of the packet
                } 
               while(pack.length > 3 && pack[0] == 3) {
                    // no call conf with rfc : first data=> call conf
                    byte[] header = "X25DATA".getBytes();
                    byte[] res = java.util.Arrays.copyOfRange(pack, 1, 4);
                    int len = Integer.parseInt(TUtils.retire("DEC", res));
                    //len include RFC header (03....)
                    res = java.util.Arrays.copyOfRange(pack, 4, len);
                    if (getOptions().getValuefor("rfc_addX25", true)) {
                        res = TUtils.addtab(header, res);
                    }
                    C_Packet newpack = new C_Packet(res, getport());
                    newpack.setdestination(packet.getdestination());
                    liste.add(newpack);
                    pack = java.util.Arrays.copyOfRange(pack, len, pack.length);
                } 
        }
        return liste;
    }

    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        List<C_Packet> liste = new ArrayList<>();
        byte[] res = new byte[0];
        if (packet.isbuffer() == false) {
            return liste;
        }
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        // no need of upper Protocols
        if (getOptions().getValuefor("rfc_addX25", true) == false) {
            packet.setObj(makeData(pack));
            liste.add(packet);
        } else if ((new String(pack)).startsWith("X25CALL:")) {
            String callstr = (new String(pack)).substring("X25CALL:".length() + 1);
            //1     connect to a particular X.25 host
            //2     listen for incoming X.25 connections
            //4     Extended connextion 
            if ((Boolean) getOptions().getValuefor("rfc_isplus")) {
                C_Packet p = (C_Packet) packet.clone();
                //p.setObj(new byte[]{4}); // => SAGEM sent 04 before 01... Specifications are ambigus
                //getStack().reply_out(this, p);
                res = new byte[]{4, 1};
                res = TUtils.addtab(res, makeField(10, ((String) getOptions().getValuefor("rfc_maker")).getBytes()));
                res = TUtils.addtab(res, makeField(10, ((String) getOptions().getValuefor("rfc_pn")).getBytes()));
                res = TUtils.addtab(res, makeField(20, ((String) getOptions().getValuefor("rfc_sn")).getBytes()));
                res = TUtils.addtab(res, makeField(20, ((String) getOptions().getValuefor("rfc_addrL2")).getBytes()));
                res = TUtils.addtab(res, makeField(4, new byte[]{0, 1, 1}));
            } else {
                res = new byte[]{1};
            }
            /*String addr = Parent.getinfos(getportidx(), "X25", "x25_dest").toString();
             String cud = Parent.getinfos(getportidx(), "X25", "x25_cud").toString();
             String fac = Parent.getinfos(getportidx(), "X25", "x25_facilities").toString();*/
            int addrsl = pack["X25CALL:".length()];
            byte[] cud = new byte[0];
            String fac = "";
            if (addrsl < 0) {
                addrsl = 256 + addrsl;
            }
            int srcl = addrsl / 16;
            String addr = callstr.substring(0, addrsl - (16 * srcl));
            callstr = callstr.substring(srcl + addr.length());
            int facl = pack["X25CALL:".length() + 1 + srcl + addr.length()];
            if (facl > 0) {
                fac = callstr.substring(1, facl);
            }
            if (pack.length > "X25CALL:".length() + 1 + srcl + addr.length() + 1 + facl) {
                cud = java.util.Arrays.copyOfRange(pack, "X25CALL:".length() + 1 + srcl + addr.length() + 1 + facl, pack.length);
            }

            if (addr.length() < 20 && cud.length < 20 && fac.length() < 20) {
                res = TUtils.addtab(res, makeField(16, addr.getBytes()));
                res = TUtils.addtab(res, makeField(4, new byte[0]));
                cud = TUtils.addtab(new byte[]{1, 0, 0, 0}, cud);//IS IT X25SRC? (0100000 forPAD) 
                res = TUtils.addtab(res, makeField(16, cud));
                res = TUtils.addtab(res, makeField(6, fac.getBytes()));
            } else {
                System.err.println("bad parametrer length for addr:" + addr + "; cud:" + cud + "; facilities:" + fac);
                return null;
            }
            //need to complet packet sent before CALLCONF => reply_out should be a 'afterTrasmit()' listner
            packet.setObj(res);
            getStack().reply_out(this, packet);
            getStack().reply_in(this, new C_Packet("X25CALLCONF".getBytes(), getport()));
        } else if ((new String(pack)).startsWith("X25DATA")) {
            pack = java.util.Arrays.copyOfRange(pack, "X25DATA".length(), pack.length);
            res = TUtils.addtab(res, makeData(pack));
            packet.setObj(res);
            liste.add(packet);
        } else {
            // CALL CONF / CLEAR / CLEARCONF => do nothing
        }
        return liste;
    }

    private byte[] makeField(int size, byte[] value) {
        byte[] res = new byte[0];
        res = TUtils.addtab(res, value);
        res = java.util.Arrays.copyOfRange(res, 0, size + 1);
        int datasz = value.length;
        if (datasz < Byte.MAX_VALUE && datasz > Byte.MIN_VALUE) {
            res[res.length - 1] = (byte) datasz;
        } else {
            //field size too large
            return null;
        }
        return res;
    }

    private byte[] makeData(byte[] value) {
        byte[] res = new byte[]{3};
        byte[] szbuff = TUtils.prend("DEC", value.length + 4);
        if (szbuff.length == 1) {
            szbuff = TUtils.addtab(new byte[]{0x00, 0x00}, szbuff);
        } else if (szbuff.length == 2) {
            szbuff = TUtils.addtab(new byte[]{0x00}, szbuff);
        }
        res = TUtils.addtab(res, szbuff);
        res = TUtils.addtab(res, value);
        return res;
    }

    @Override
    public void StkArcv(String message, String type) {
        if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            getStack().reply_in(this, new C_Packet("X25CLEAR".getBytes(), getport()));
        }
    }

    @Override
    public void setCopyMode(boolean bool) {
        getOptions().setValuefor("rfc_addX25", !bool);
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        if (pack.isbuffer()) {
            if (pack.getBytes().length > 4) {
                byte[] frame = pack.getBytes();
                if (frame[0] == 3) {
                    int pointer = 0;
                    while (pointer < frame.length) {
                        if (frame[pointer] == 3) {
                            pointer += array2dec(new byte[]{frame[pointer + 1], frame[pointer + 2], frame[pointer + 3]});
                            if (pointer == frame.length) {
                                return 0.9990;
                            } else if (pointer < frame.length) {
                                return 0.6;
                            }
                        } else {
                            //pointer = frame.length + 1;
                            return 0.0;
                        }
                    }
                    if (pointer == frame.length) {
                        return 0.9990;
                    }
                } else if (frame[0] == 4 && frame[1] == 1 
                        && frame[12] < 10// constructror size
                        && frame[23] < 10// P/N size
                        && frame[44] < 20
                        && frame[0x41] < 20) {// S/N size
                    //rfc+
                    // last byte of constructor & productuct ...
                    if (frame[11] == 00 && frame[22] == 00) {
                        res = 0.9;
                    } else if (frame.length == 121||frame.length == 117) {
                        // nacked RFC1086+
                        res = 0.9;
                    } else if (frame.length > 121) {
                        res = 0.7;
                        if (frame.length >= 124 && frame[121] == -62 && frame[122] + 124 == frame.length) {
                            // CBCOM implementation! !!! care ( -55 for CHPR but no data over RFC)
                            res = 0.9;
                        }
                    }
                } else if (frame[0] == 1 && frame.length > 5) {
                    // call message for french (196...) X25 addres
                    if (frame[1] == 0 && frame[2] == 3 && frame[3] == 0x31
                            && frame[4] == 0x39 && frame[5] == 0x36) {
                        //rfc simple 
                        res = 0.6;
                    }
                }
            } else if (pack.getBytes().length == 1 && pack.getBytes()[0] == 4) {
                res = 0.8;
            }
        }
        return res;
    }
}
