package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import java.util.List;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * Change MAC source & destination to go to gateway if not in the mask
 *
 * @author Marc GIBOIN
 */
public class Stack_RouteIP extends Stack_base {

    public Stack_RouteIP() {
        super();
        java.net.InetAddress rtgw = null;
        java.net.InetAddress rtmask = null;
        try {
            rtgw = java.net.InetAddress.getByName("192.168.120.254");
            rtmask = java.net.InetAddress.getByName("255.255.255.0");
        } catch (UnknownHostException ex) {
        }
        getOptions().addOption("rt_gw", rtgw, "InetAddress", "", true, true, "default gateway");
        getOptions().addOption("rt_mask", rtmask, "InetAddress", "", true, true, "mask");
    }

    public static String getComment() {
        return "IP routing";
    }

    public static double getosi() {
        return 3.0;
    }

    /**
     * check packet change MAC ? according to route Answer to request for my IP
     *
     * @param packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        // routting protocol => do nothing except if routing protocol ...
        java.util.List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        res.add(packet);
        return res;
    }

    /**
     * if IPdest is in the mask of IPsrc then forward else add an ARP entry of
     * IP_dest with IP gateway MAC
     *
     *
     * @param  packet
     * @return the payloads
     * @throws java.lang.Exception
     */
    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        // if not a local adress => add a new entry { WAN IP address/MAC gateway }
        List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        res.add(packet);
        InetAddress ipdest = (InetAddress) getStack().getinfos(getport(), "IP", "ip_dest", SEARCH_DOWN, this);
        InetAddress ipsrc = (InetAddress) getStack().getinfos(getport(), "IP", "ip_src", SEARCH_DOWN, this);
        InetAddress mask = (InetAddress) getOptions().getValuefor("rt_mask");
        boolean bool = true;
        for (int i = 0; i < ipdest.getAddress().length; i++) {
            int des = ipdest.getAddress()[i] & mask.getAddress()[i];
            int src = ipsrc.getAddress()[i] & mask.getAddress()[i];
            bool = bool && (des == src);
        }
        if (ipdest == null || bool) {
            //not an ip protocol or in the mask
            return res;
        } else {
            //not in the mask
            InetAddress ipgw = (InetAddress) getOptions().getValuefor("rt_gw");
            byte[] gwmac = (byte[]) getStack().getinfos(getport(), "ARP", "arp_" + ipgw, SEARCH_DOWN, this);
            if (gwmac == null) {
                //look for Gateway MAC??
                getStack().setinfos(getport(), "ARP", SEARCH_DOWN, this, "arp_" + ipdest, ipgw.getAddress());
            } else {
                getStack().setinfos(getport(), "ARP", SEARCH_DOWN, this, "arp_" + ipdest, gwmac);
            }
        }
        return res;
        //need to way answer and send packet

    }
}
