package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;

import java.util.ArrayList;
import java.util.List;

/**
 * Provide a Shell session with prompt and status implementaiton - only minipad
 * command is implmented
 *
 * @author Marc GIBOIN
 *
 */
public class Stack_Shell extends Stack_baseStart {

    // pad start with command line, padbuffer is used for this
    private String padbuffer = "";

    public Stack_Shell() {
        super();
        init_option();
    }

    private void init_option() {
        getOptions().addOption("shl_prompt", "\r\rrouteur>".getBytes(), "byte[]", "", true, false, "x25 user data");
        getOptions().addOption("shl_status", "sleep", "String", "", true, true, "status of this process");
        getOptions().addOption("shl_cmde", "", "String", "", true, true, "the running command");
    }

    public static String getComment() {
        return "provide a shell emulation";
    }

    public static double getosi() {
        return 7.0;
    }

    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        String status = getOptions().getValuefor("shl_status", "");
        toLog(this, new String(pack), false);
        if (pack == null || pack.length == 0) {
            return null;
        }
        if (status.startsWith("connect") == false) {
            padbuffer = "" + padbuffer + new String(pack);
        }

        if (status.compareTo("connected") == 0) {
            res.add(packet);
        } else if (status.compareTo("sleep") == 0) {
            if (padbuffer.endsWith("\r") == false && padbuffer.endsWith("\n") == false) {
                return res;
            }
            padbuffer = padbuffer.trim();
            packet.setObj(getOptions().getValuefor("shl_prompt", new byte[]{}));
            if (padbuffer == null || padbuffer.length() == 0) {
                getStack().reply_out(this, packet);
            } else if (padbuffer.startsWith("minipad")) {
                res = minipad(padbuffer);
                getOptions().setValuefor("shl_cmde", "minipad");
            } else {
                getStack().reply_out(this, packet);
            }
            padbuffer = "";
        }

        return res;
    }

    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        // no need of upper Protocols
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        String status = getOptions().getValuefor("shl_status").toString();
        if (status.startsWith("connected")) {
            if (pack[0] != -61 && "minipad".compareTo(getOptions().getValuefor("shl_cmde", "")) == 0) {
                // BINTEC minipad command DO NOT transmit CBCOM openning (C3 XXXX) after "call connected to ..."
                res.add(packet);
            }
        } else if (status.startsWith("connecting")) {
            if (packet.isbuffer()) {
                if (new String(packet.getBytes()).contains("COM")) {
                    getOptions().setValuefor("shl_status", "connected");
                    packet.setObj("Call connected to <XXXXXXXX>.\r\nEscape character processing inhibited.\r\n".getBytes());
                    res.add(packet);
                } else {
                    packet.setObj("Call Aborted".getBytes());
                    res.add(packet);
                    getStack().reply_out(this, packet);
                    getOptions().setValuefor("shl_status", "Sleep");
                    getOptions().setValuefor("shl_cmde", "");
                }
            }
        }
        return res;

    }

    private List<C_Packet> minipad(String commande) {
        String dest;
        // pase pad command to get adresses &optioins (cud/fac/..)
        //minipad -i -t -h 01000000C209050412011130080101 196372527
        /// to parse
        String padopts[] = commande.split(" ");
        //Parent.PAD_start() OR shouldbe started...
        if (padopts != null && padopts.length > 4) {
            dest = padopts[padopts.length - 1];
        } else {
            C_Packet packet = new C_Packet("format need -i -t -h cud addr".getBytes(), getport());
            getStack().reply_out(this, packet);
            return null;
        }
        String cud = "";
        if (padopts != null && padopts.length > 4) {
            cud = padopts[4];
        }
        if (cud.length() > 0) {
            dest = dest + "B" + cud; //shouldbe D but CUD format is not ASCII
        }
        dest = dest + "\r\n";

        //echo de minipad...
        C_Packet packet = new C_Packet((commande + "\r\n").getBytes(), getport());
        getStack().reply_out(this, packet);
        getOptions().setValuefor("shl_status", "connecting");
        List<C_Packet> res = new java.util.ArrayList<>();
        res.add(new C_Packet(dest.getBytes(), getport()));
        return res;
    }

    @Override
    public boolean onStart() {
        try {
            /*
             * if (portstate.compareTo("stopped") == 0 || portstate.compareTo("creating") == 0) {
             * getStack().getPort(portidx).CXstart();}
             */
            C_Packet res = new C_Packet(getOptions().getValuefor("shl_prompt"), getport());
            getStack().reply_out(this, res);
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public boolean onStop() {
        try {
            /*
             * if (portstate.startsWith("stopp")==false && portstate.startsWith("clos")==false) {
             * getStack().getPort(portidx).CXstop();  }
             */
            getStack().reply_out(this, new C_Packet("Session closed", getport()));
            getOptions().setValuefor("shl_status", "sleep");
            getOptions().setValuefor("shl_cmde", "");
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public void StkArcv(String message, String type) {
        if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            getOptions().setValuefor("shl_status", "sleep");
            getOptions().setValuefor("shl_cmde", "");
            padbuffer = "";
        }
        if (message.compareTo("connected") == 0 && type.compareTo("Status") == 0) {
        }
    }
}
