package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;

/**
 * Provide a TCP implementation (care IP pseudo session)
 *
 * http://www.frameip.com/entete-tcp/ CONNECTED PROTOCOL => USE PORT STATES ??
 * (OPEN/CONNECTED/..)???????? TCP is NOT ISO, pseudo IP is null if IP is not
 * present : getpseudoIP() NO OPTION MANAGEMNET, NO SLIDING WINDOWS MANAGEMENT,
 * NO RETRANSMISSION,...
 *
 *
 * @author Marc GIBOIN
 */
public class Stack_TCP extends Stack_baseIP {
    private final java.util.List<C_Packet> buffer = new java.util.ArrayList<>();
    
    private long dist_seq = 0;
    private long dist_ack = 0;
    private int dist_swind = 0;
    private String status = "new";// conneting/connected/closing/new
    private byte lastflag = 0;


    public Stack_TCP() {
        super();
        long secq = 10;//first SYN
        long ack = 0;//first SYN
        int flg = 0x02; //SYN
        boolean listner = false;
        byte[] upt = new byte[]{0x00, 0x00};
        byte[] win = new byte[0];
        byte[] wout = new byte[0];
        byte[] opt = new byte[]{0x02, 0x04, 0x05, -76, 0x01, 0x03, 0x03, 0x03, 0x01, 0x01, 0x04, 0x02};
        getOptions().addOption("tcp_copy", listner, "boolean", "", true, true, "copy received settings in the model");
        getOptions().addOption("tcp_chkdest", false, "boolean", "", true, true, "check received dest TCP port");
        getOptions().addOption("tcp_chksrc", false, "boolean", "", true, true, "check received source TCP port");
        getOptions().addOption("tcp_src", 1000, "int", "", true, true, "source TCP port");
        getOptions().addOption("tcp_dest", 2000, "int", "", true, true, "destination TCP port");
        getOptions().addOption("tcp_secq", secq, "long", "", true, true, "TCP secquence nb");
        getOptions().addOption("tcp_ack", ack, "long", "", true, true, "TCP acknowlege nb");
        getOptions().addOption("tcp_flag", flg, "int", "", true, true, "TCP Flag");
        getOptions().addOption("tcp_urgpt", upt, "int", "", true, true, "TCP Urgent Pointer");
        getOptions().addOption("tcp_slwin", win, "byte[]", "", true, false, "TCP windows  for input");
        getOptions().addOption("tcp_slwout", wout, "byte[]", "", true, false, "TCP windows for output");
        getOptions().addOption("tcp_options", opt, "byte[]", "", true, false, "TCP options at connection");
        
    }

    public static String getComment() {
        return "Transmision Control Protocol";
    }

    public static double getosi() {
        return 4.0;
    }

    @Override
    public void StkArcv(String message, String type) {
        try {
            if (type.compareTo("Etat") == 0) {
                if (message.compareTo("connected") == 0) {
                    //SYN
                    TCP_askopenning();
                }
                if (message.compareTo("stopping") == 0) {
                    //RST
                    TCP_askclosing();
                }
                if (message.compareTo("stopped") == 0) {
                    //RST
                    TCP_reset();
                }
            }
        } catch (Exception ex) {
        }
    }

    @Override
    public void onClock() {
    }

    private void TCP_askopenning() {
        status = "connecting";
        //init windows/...
        try {
            getOptions().setValuefor("tcp_flag", 0x02);
            getOptions().setValuefor("tcp_options", new byte[]{0x02, 0x04, 0x05, -76, 0x01, 0x03, 0x03, 0x03, 0x01, 0x01, 0x04, 0x02});
            C_Packet d = new C_Packet(new byte[0], getport());
            d.setdestination(getport());
            getStack().reply_out(this, transmit_frame(d));
            getOptions().setValuefor("tcp_secq", 1 + (Long) getOptions().getValuefor("tcp_secq"));
        } catch (Exception ex) {
            status = "new";
        }
    }

    private void TCP_replyopenning() {
        //init windows/...
        if (lastflag == 2) {
            try {
                getOptions().setValuefor("tcp_ack", 1 + (Long) getOptions().getValuefor("tcp_ack"));
                getOptions().setValuefor("tcp_options", new byte[]{0x02, 0x04, 0x05, -76, 0x01, 0x03, 0x03, 0x03, 0x01, 0x01, 0x04, 0x02});
                getOptions().setValuefor("tcp_flag", 2 | 16);
                C_Packet d = new C_Packet(new byte[0], getport());
                d.setdestination(getport());
                getStack().reply_out(this, transmit_frame(d));
                getOptions().setValuefor("tcp_secq", 1 + (Long) getOptions().getValuefor("tcp_secq"));
                status = "connecting";
            } catch (Exception ex) {
                status = "new";
            }
        } else {
            //Errors
            TCP_reset();
        }
    }

    private void TCP_open() {
        //init windows/...
        if (lastflag == (2 | 16)) {
            //SYN,ACK receved => ACK
            try {
                getOptions().setValuefor("tcp_options", new byte[0]);
                getOptions().setValuefor("tcp_flag", 0x10);//ACK
                getOptions().setValuefor("tcp_slwout", new byte[dist_swind]);
                getOptions().setValuefor("tcp_ack", 1 + dist_seq);
                C_Packet d = new C_Packet(new byte[0], getport());
                d.setdestination(getport());
                getStack().reply_out(this, transmit_frame(d));
                status = "connected";
            } catch (Exception ex) {
                TCP_reset();
            }
        } else if (lastflag == (16)) {
            //ACK receved after SYN
            status = "connected";
            getOptions().setValuefor("tcp_options", new byte[0]);
        } else {
            //Errors
            TCP_reset();
        }
        if (status.compareTo("connected") == 0 && buffer.size() > 0) {
            try {
                getOptions().setValuefor("tcp_flag", 0x08 | 0x10);//PUSH ACK
                while (buffer.isEmpty() == false) {
                    C_Packet p = buffer.remove(0);
                    int datasz = p.getBytes().length;
                    p.setdestination(getport());
                    getStack().reply_out(this, transmit_frame(p));
                    getOptions().setValuefor("tcp_secq", datasz + (Long) getOptions().getValuefor("tcp_secq"));
                }
            } catch (Exception ex) {
            }
        }
    }

    private void TCP_askclosing() {
        if (status.compareTo("new") != 0) {
            status = "closing";
            //init windows/...
            try {
                getOptions().setValuefor("tcp_flag", 0x01);
                C_Packet d = new C_Packet(new byte[0], getport());
                d.setdestination(getport());
                getStack().reply_out(this, transmit_frame(d));
            } catch (Exception ex) {
                status = "new";
            }
        }
    }

    private void TCP_replyclosing() {
        status = "closing";
        //init windows/...
        C_Packet d = new C_Packet(new byte[0], getport());
        d.setdestination(getport());
        try {
            getOptions().setValuefor("tcp_ack", 1 + (Long) getOptions().getValuefor("tcp_ack"));
            getOptions().setValuefor("tcp_flag", 0x10);
            getStack().reply_out(this, transmit_frame(d));
            getOptions().setValuefor("tcp_flag", 0x01 | 0x10);
            getStack().reply_out(this, transmit_frame(d));
            status = "new";
        } catch (Exception ex) {
            status = "new";
        }
    }

    private void TCP_close() {
        //FIN=> ACK
        if (status.compareTo("new") != 0) {
            if ((lastflag & 1) == 1) {
                try {
                    getOptions().setValuefor("tcp_ack", 1 + (Long) getOptions().getValuefor("tcp_ack"));
                    getOptions().setValuefor("tcp_flag", 0x10);
                    C_Packet d = new C_Packet(new byte[0], getport());
                    d.setdestination(getport());
                    getStack().reply_out(this, transmit_frame(d));
                    status = "new";
                } catch (Exception ex) {
                    TCP_reset();
                }
            }
        }
    }

    private void TCP_reset() {
        //feel input windows/ & check acknoleges...
        if (status.compareTo("conneccted") == 0 || status.compareTo("connecting") == 0) {
            buffer.clear();
            try {
                //send the acknolege
                getOptions().setValuefor("tcp_options", new byte[0]);
                getOptions().setValuefor("tcp_flag", 0x04);
                C_Packet d = new C_Packet(new byte[0], getport());
                d.setdestination(getport());
                getStack().reply_out(this, transmit_frame(d));
            } catch (Exception ex) {
                status = "new";
            }
        }
        status = "new";
    }

    private void TCP_receive() {
        //feel input windows/ & check acknoleges...
        if ((lastflag & 1) == 1) {
            TCP_replyclosing();
        }
        if ((lastflag & 4) == 4) {
            TCP_reset();
            return;
        }
        if ((lastflag & 16) == 16) {
            //ACK
        }
        if ((lastflag & 8) == 8) {
            //PUSH
        }
    }

    private void processTCP(boolean isin) {
        if (isin == false) {
            //beefore transmit data to lower laeyrs
            if (status.compareTo("new") == 0) {
                // transmission => open=>SYN
                TCP_askopenning();
            }
            if (status.compareTo("closing") == 0) {
                // transmission => set the PUSH
                TCP_close();
            }
        }
        if (isin) {
            //received packet from lower layers
            if (status.compareTo("new") == 0) {
                //is a SYN ?=> SYN ACK
                TCP_replyopenning();
                return;
            }
            if (status.compareTo("connecting") == 0) {
                // as a SYNACK=> ACK //ACK=> Connected// else reset
                TCP_open();
                return;
            }
            if (status.compareTo("connected") == 0) {
                // transmission => send !
                TCP_receive();
            }
        }
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        if (pack == null || pack.length < 20) {
            throw new Exception("no enouth byte for TCP header");
        }
        int proto = -1;
        if (getStack() != null) {
            proto = (Integer) getStack().getinfos(getport(), "IP", "ip_sproto", SEARCH_DOWN, this);
        }
        if (proto > 0 && proto != 0x06) {
            //liste.add(packet);
            return liste;
        } else if (pack.length > 20) {
            byte[] res = java.util.Arrays.copyOfRange(pack, 0, 2);
            // check dest of tho packet is the source of the module (Reception)
            int tcpdest = (int) array2dec(res);
            if (tcpdest == getOptions().getValuefor("tcp_dest", 1) || getOptions().getValuefor("tcp_chkdest", true) == false) {
                res = java.util.Arrays.copyOfRange(pack, 2, 4);
                int tcpsrc = (int) array2dec(res);
                if (tcpsrc == getOptions().getValuefor("tcp_src", 1) || getOptions().getValuefor("tcp_chksrc", true) == false) {
                    dist_seq = array2dec(java.util.Arrays.copyOfRange(pack, 4, 8));
                    dist_ack = array2dec(java.util.Arrays.copyOfRange(pack, 8, 12));
                    byte[] len = java.util.Arrays.copyOfRange(pack, 12, 13);
                    // xxxx 0000=> xxxx : size of head by 32 bits (8*4)
                    int a = Math.abs(len[0] >> 4);
                    int lhdr = a * 4;
                    lastflag = java.util.Arrays.copyOfRange(pack, 13, 14)[0];
                    dist_swind = (int) array2dec(java.util.Arrays.copyOfRange(pack, 14, 16));

                    int crc = (int) array2dec(java.util.Arrays.copyOfRange(pack, 16, 18));
                    byte[] upt = java.util.Arrays.copyOfRange(pack, 18, 20);
                    //!options & Urget Pointers
                    if (getOptions().getValuefor("tcp_copy", true)) {
                        getOptions().setValuefor("tcp_dest", tcpdest);
                        getOptions().setValuefor("tcp_src", tcpsrc);
                        getOptions().setValuefor("tcp_ack", dist_ack);
                        getOptions().setValuefor("tcp_secq", dist_seq);
                        getOptions().setValuefor("tcp_flag", lastflag);
                        getOptions().setValuefor("tcp_slwin", dist_swind);
                        getOptions().setValuefor("tcp_urgpt", array2dec(upt));
                    }
                    if (status.startsWith("pass") == false) {
                        processTCP(true);
                    }
                    if (pack.length > lhdr) {
                        byte[] data = java.util.Arrays.copyOfRange(pack, lhdr, pack.length);
                        if (status.startsWith("pass") == false) {
                            try {
                                //send the acknolege
                                getOptions().setValuefor("tcp_ack", data.length + getOptions().getValuefor("tcp_ack", 1L));
                                getOptions().setValuefor("tcp_flag", 0x10);
                                C_Packet d = new C_Packet(new byte[0], getport());
                                d.setdestination(getport());
                                getStack().reply_out(this, transmit_frame(d));
                            } catch (Exception ex) {
                                status = "new";
                            }
                        }
                        packet.setObj(data);
                        liste.add(packet);
                        return liste;
                    }
                }
            }
        }
        return null;
    }

    /**
     * encapsule payload in header to form packet
     *
     * @param packet the payload
     * @return the packet
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        processTCP(false);
        if (status.compareTo("connected") == 0) {
            getOptions().setValuefor("tcp_flag", 0x08 | 0x10);
            java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
            finalres.add(transmit_frame(packet));
            getOptions().setValuefor("tcp_secq", pack.length + getOptions().getValuefor("tcp_secq", 1L));
            return finalres;
        } else {
            packet.setsource(getport());
            buffer.add(packet);
        }
        return null;

    }

    public C_Packet transmit_frame(C_Packet dat) throws Exception {
        byte[] data = dat.getBytes();
        byte[] opt = (byte[]) getOptions().getValuefor("tcp_options");
        byte[] res = null;
        if (getStack() != null) {
            getStack().setinfos(getport(), "IP", SEARCH_DOWN, this, "ip_sproto", 0x06);
        }
        byte[] tcpsrc = TUtils.prend("DEC", getOptions().getValuefor("tcp_src", 1));
        byte[] tcpdst = TUtils.prend("DEC", getOptions().getValuefor("tcp_dest", 1));
        res = TUtils.addtab(res, filltosize(tcpsrc, 2));
        res = TUtils.addtab(res, filltosize(tcpdst, 2));
        byte[] tcpseq = TUtils.prend("INT", getOptions().getValuefor("tcp_secq", 1L));
        byte[] tcpack = TUtils.prend("INT", getOptions().getValuefor("tcp_ack", 1L));
        res = TUtils.addtab(res, filltosize(tcpseq, 4));
        res = TUtils.addtab(res, filltosize(tcpack, 4));
        byte[] l = TUtils.prend("DEC", 4 * (opt.length + 20));
        res = TUtils.addtab(res, l);
        //!! ++addresses IP / taille IP / proto IP
        int flag = (Integer) getOptions().getValuefor("tcp_flag");
        byte[] tcpflag = new byte[]{(byte) flag};
        res = TUtils.addtab(res, tcpflag);
        int wsout = ((byte[]) getOptions().getValuefor("tcp_slwout")).length;
        byte[] tcpwsout = TUtils.prend("DEC", wsout);
        res = TUtils.addtab(res, filltosize(tcpwsout, 2));
        byte[] crcres = TUtils.addtab(getpseudoIP(data.length + opt.length + 20), res);
        crcres = TUtils.addtab(crcres, opt);
        crcres = TUtils.addtab(crcres, data);
        int crc = ip_sum_calc(crcres);
        byte[] crcbuff = TUtils.prend("DEC", crc);
        if (crcbuff.length < 2) {
            crcbuff = TUtils.addtab(new byte[]{0x00}, l);
        }
        res = TUtils.addtab(res, crcbuff);
        res = TUtils.addtab(res, (byte[]) getOptions().getValuefor("tcp_urgpt"));
        res = TUtils.addtab(res, opt);
        res = TUtils.addtab(res, data);

        // set the data & configure for lower levels
        dat.setObj(res);

        return dat;
    }

    /**
     * fill whith 0x00 before initbuff
     *
     */
    private byte[] filltosize(byte[] initbuff, int newsize) {
        if (initbuff.length < newsize) {
            byte[] buff = new byte[newsize - initbuff.length];
            java.util.Arrays.fill(buff, (byte) 0x00);
            initbuff = TUtils.addtab(buff, initbuff);
        }
        return initbuff;
    }

    /**
     * Apply Options() modification to store receved data and disable checks
     *
     * @param bool
     */
    @Override
    public void setCopyMode(boolean bool) {
        getOptions().setValuefor("tcp_copy", bool);
        getOptions().setValuefor("tcp_chkdest", !bool);
        getOptions().setValuefor("tcp_chksrc", !bool);
        if (bool) {
            status = "pass";
        } else {
            status = "new";
        }
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        //always asked by IP
        if (pack.isbuffer()) {
            byte[] frame = pack.getBytes();
            if (frame.length >= 20) {
                res = 0.1;
                if (pack.getProtocols() != null && pack.getProtocols().isEmpty() == false) {
                 Object obj=pack.getProtocols().get(pack.getProtocols().size() - 1);
                    if ( obj instanceof Stack_IP) {
                        Stack_IP elt = (Stack_IP)obj;
                        if (elt.getOptions().getValuefor("ip_sproto", 1) == 6) {
                            res = 0.9999;
                        }
                    }
                }
                //   if (frame[12] == 0x50 || frame[12] == 0x70 || getInt(frame[12]) == 0x80) {
                // could not check the checksum beacause of peudosession need IP??
                //        res = 0.8;
                //    }
            }
        }
        return res;
    }
}
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
