package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;
import java.util.ArrayList;
import java.util.List;
import java.lang.reflect.Field;
import com.jdcm.common.C_Actiondef;
import javax.swing.DefaultComboBoxModel;
import com.jdcm.diagram.A_Events;
import com.jdcm.diagram.A_Element;
import com.jdcm.common.D_Utils;
import com.jdcm.eltrx.I_Port;

/**
 * Provide a TELNET protocol implmentation (NOT STABLE)
 *
 * not implemented : negociations management (with values..) cisco => windows
 * size management Cisco => terminal type
 *
 * @author Marc GIBOIN
 */
public class Stack_Telnet extends Stack_base {

    private final D_Utils TUtils = new D_Utils();
    public static final int ST_MAND = 0;
    public static final int ST_COULD = 1;
    public static final int ST_MANDNOT = 2;
    public static final int ST_APPLY = 3;
    public static final int ST_NOAPPLY = 4;
    public static final int ST_ASK = 5;
    public static final int ST_ASKNO = 6;
    static final byte OPT_WILL = -5;
    static final byte OPT_WONT = -4;
    static final byte OPT_DO = -3;
    static final byte OPT_DONT = -2;
    static final byte OPT_SUBBEG = -6;
    static final byte OPT_SUBEND = -16;
    static final int Version = 1;
    private final String[] optlist = new String[]{"lecho", "recho", "lsgoahead", "rsgoahead", "lbinary", "rbinary", "lwindsize", "rwindsize", "lauth", "rauth", "lnewEnv", "rnewEnv", "lttype", "rttype"};
    private final byte[] vallist = new byte[]{1, 1, 3, 3, 0, 0, 31, 31, 37, 37, 39, 39, 24, 24};
    private final int[] statelist = new int[]{ST_ASK, ST_ASK, ST_COULD, ST_COULD, ST_COULD, ST_COULD, ST_COULD, ST_MANDNOT, ST_MANDNOT, ST_MANDNOT, ST_MANDNOT, ST_MANDNOT, ST_MANDNOT, ST_MANDNOT};
    // private A_Events eventmx = null;
    private final java.util.List<A_Element> cbst = new java.util.ArrayList<>();
    private final java.util.List<Field> cbop = new java.util.ArrayList<>();

    /**
     * http://www.iana.org/assignments/telnet-options
     *
     * veut utiliser (DO) 253 -3 refuse d'utiliser (DON'T) 254 -2 veut que
     * l'autre extrémité utilise (WILL) 251 -5 refuse que l'autre extrémité
     * utilise (WON'T) 252 -4 sub option negiciation 250 -6 SE End of
     * subnegotiation parameters. 240 -16
     *
     *
     * Go ahead 249	The GA signal. -7 Erase Line	248	The function EL. -8 Erase
     * character	247	The function EC -9 Are You There	246	The function AYT. -10
     * Abort output	245	The function AO -11 Interrupt Process 244	The function
     * IP. -12 Break 243	NVT character BRK. -13 Data Mark	242	The data stream
     * portion of a Synch. -14 NOP 241	No operation -15
     *
     *
     */
    public Stack_Telnet() {
        super();
        for (Field fs : this.getClass().getDeclaredFields()) {
            if (fs.getName().startsWith("ST_")) {
                cbst.add(new A_Element(fs));
            } else if (fs.getName().startsWith("OPT_")) {
                cbop.add(fs);
            }
        }
        Object[] evt_in = new Object[]{OPT_WILL, OPT_WONT, OPT_DO, OPT_DONT, "START_L", "START_R"};
        /*eventmx = new A_Events(cbst, java.util.Arrays.asList(evt_in),new A_Task[][]{
         //{ST_MAND ST_COULD ST_MANDNOT ST_APPLY ST_NOAPPLY ST_ASK ST_ASKNO}
         { new A_Task(ST_MAND,OPT_DO),   new A_Task(ST_APPLY,OPT_DO) , new A_Task(ST_MANDNOT,OPT_DONT),   new A_Task(),                     new A_Task(ST_APPLY,OPT_DO),   new A_Task(ST_APPLY,OPT_DO),      new A_Task(ST_APPLY,OPT_DO)},
         { new A_Task(ST_MAND,OPT_DO),   new A_Task(ST_NOAPPLY,OPT_DO) , new A_Task(ST_MANDNOT,OPT_DONT), new A_Task(ST_NOAPPLY,OPT_DONT),  new A_Task(),                  new A_Task(ST_NOAPPLY,OPT_DONT),  new A_Task(ST_NOAPPLY,OPT_DONT),},
         { new A_Task(ST_MAND,OPT_WILL), new A_Task(ST_APPLY,OPT_WILL), new A_Task(ST_MANDNOT,OPT_WONT),  new A_Task(),                     new A_Task(ST_APPLY,OPT_WILL), new A_Task(ST_APPLY,OPT_WILL),    new A_Task(ST_APPLY,OPT_WILL)},
         { new A_Task(ST_MAND,OPT_WILL), new A_Task(ST_APPLY,OPT_WONT),new A_Task(ST_MANDNOT,OPT_WONT),   new A_Task(ST_NOAPPLY,OPT_WONT),  new A_Task(),                  new A_Task(ST_NOAPPLY,OPT_WONT),  new A_Task(ST_NOAPPLY,OPT_WONT)},
         { new A_Task(),                new A_Task(),                  new A_Task(),                      new A_Task(),                     new A_Task(),                  new A_Task(ST_APPLY,OPT_DO),      new A_Task(ST_NOAPPLY,OPT_DONT)},
         { new A_Task(),                new A_Task(),                  new A_Task(),                      new A_Task(),                     new A_Task(),                  new A_Task(ST_APPLY,OPT_WILL),    new A_Task(ST_NOAPPLY,OPT_WONT)}});*/
        A_Events eventmx = new A_Events(D_Utils.getFileStream("evt_telnet.xml", "mapping", getClass().getClassLoader()));
        eventmx.setStatusAsFields(this);
        getOptions().addOption("telnet_banner", ("This is the Telnet (V:" + Version + ") banner of JDatacom\r\n--------------------------------------\r\n").getBytes(), "byte[]", "", true, true, " telnet banner ");
        getOptions().addOption("telnet_issrv", true, "boolean", "", true, true, "true if telnet run as server");
        getOptions().addOption("telnet_event", eventmx, "Eventsmx", "", true, true, "the state event matrix");
        getOptions().addOption("telnet_winsize", new java.awt.Rectangle(0, 0, 80, 25), "size", "", true, true, "the size of windows in nego");
        getOptions().addOption("telnet_isrefused", false, "boolean", "", true, true, "true if telnet run as server");
        for (int i = 0; i < optlist.length; i++) {
            DefaultComboBoxModel<A_Element> cbm = new javax.swing.DefaultComboBoxModel<>(cbst.toArray(new A_Element[]{}));
            getOptions().addOption("telnet_" + optlist[i], cbm, "int", "", true, true, optlist[i] + " value is " + vallist[i]);
            setState("telnet_" + optlist[i], statelist[i]);
        }
    }

    private void init_options() {
        getOptions().setValuefor("telnet_isrefused", false);
        for (int i = 0; i < optlist.length; i++) {
            //   DefaultComboBoxModel<A_Element> cbm = new javax.swing.DefaultComboBoxModel<A_Element>(cbst.toArray(new A_Element[]{}));
            setState("telnet_" + optlist[i], statelist[i]);
        }
    }

    public static String getComment() {
        return "provide a simple telnet layer";
    }

    public static double getosi() {
        return 7.0;
    }

    private void manageOptions(byte[] opt, boolean fromrcv) {
        Field optrcv = null;
        for (Field f : cbop) {
            try {
                if (f.getByte(f) == opt[0]) {
                    optrcv = f;
                }
            } catch (IllegalArgumentException | IllegalAccessException ex) {
            }
        }
        String optname = getValueasString(opt[1], "option");

        if (optname == null) {
            //not managed case
            toLog(this, getValueasString(opt[0], "action") + " unmanaged opt: [" + opt[1] + "] ", fromrcv);
            if (opt[0] == OPT_WILL || opt[0] == OPT_WONT) {
                getStack().reply_out(this, new C_Packet(new byte[]{-1, OPT_DONT, opt[1]}, getport()));
                toLog(this, "OPT_DONT [" + opt[1] + "] ", fromrcv == false);
            } else if (opt[0] == OPT_DO) {
                getStack().reply_out(this, new C_Packet(new byte[]{-1, OPT_WONT, opt[1]}, getport()));
                toLog(this, "OPT_WONT [" + opt[1] + "] ", fromrcv == false);
            }
            return;
        }
        if (opt[0] == OPT_WILL || opt[0] == OPT_WONT) {
            optname = "telnet_r" + optname;
        } else if (opt[0] == OPT_DO || opt[0] == OPT_DONT) {
            optname = "telnet_l" + optname;
        } else {
            toLog(this, "Unknown action : " + opt[0], fromrcv);
            return;
        }
        toLog(this, getValueasString(opt[0], "action") + " " + optname + " ", fromrcv);

        // ---------STATE  MODIFICATIONS-----------------------------

        A_Events eventmx = (A_Events) getOptions().getValuefor("telnet_event");
        eventmx.reset();
        A_Element nxt = null;
        for (A_Element a : eventmx.getStatuslist()) {
            if (a.getcontent().equals(getStateField(optname))) {
                nxt = a;
            }
        }
        for (A_Element elt : eventmx.getStatuslist()) {
            if (elt.getcontent() instanceof Field) {
                if ((Field) elt.getcontent() == nxt.getcontent()) {
                    eventmx.setStatus(null, elt);
                }
            }
        }
        //A_Task a = eventmx.getRcvActions(opt[0]).get(0);
        com.jdcm.common.C_Actiondef act;
        try {
            act = (eventmx.getTask(new com.jdcm.common.C_Eventdef("telnet", "receive", optrcv.getName()))).get(0);
        } catch (Exception ex) {
            return;
        }
        toLog(this, "I AM" + getStateField(optname).getName() + "=>" + nxt.getcontent(), fromrcv);
        Field f0 = (Field) nxt.getcontent();//(Field)eventmx.getPreviousStatusField();
        Field f1 = (Field) eventmx.getActivStatus().getcontent();
        setState(optname, eventmx.getActivStatus());
        if (act.getParam() != null && (f0 != f1 || f0.getName().startsWith("ST_MAND"))) {
            byte resact = getAction(act.getParam());
            if (resact != -1) {
                getStack().reply_out(this, new C_Packet(new byte[]{-1, resact, opt[1]}, getport()));
                toLog(this, "Reply :" + getValueasString(resact, "action") + " " + optname + " ", fromrcv == false);
                byte[] sub = doSubNego(opt[1], resact);
                if (sub != null) {
                    getStack().reply_out(this, new C_Packet(sub, getport()));
                    toLog(this, "Reply :" + optname + " SUBNEGO data", fromrcv == false);
                }
            }
        }
    }

    private byte[] doSubNego(byte option, byte action) {
        byte[] start = new byte[]{-1, -6, option};
        byte[] fin = new byte[]{-1, -16};
        byte[] data = null;
        if (action == OPT_WILL && option == getOptionValue("windsize")) {
            java.awt.Rectangle p = (java.awt.Rectangle) getOptions().getValuefor("telnet_winsize");
            data = TUtils.addtab(getdec(p.width, 2), getdec(p.height, 2));
        }
        if (data != null) {
            data = TUtils.addtab(start, data);
            data = TUtils.addtab(data, fin);
        }
        return data;
    }

    public void setissrv(boolean bool) {
        if (bool) {
            setState("telnet_lecho", ST_ASK);
            setState("telnet_recho", ST_ASK);
        } else {
            setState("telnet_lecho", ST_MANDNOT);
            setState("telnet_recho", ST_MANDNOT);
        }
    }

    private byte[] getdec(int x, int s) {
        com.jdcm.common.C_Format f = new com.jdcm.common.C_Format();
        f.setvalue("" + x);
        byte[] res = f.B_Prend("DEC");
        if (res.length > s) {
            return null;
        }
        while (res.length < s) {
            res = TUtils.addtab(new byte[]{0}, res);
        }
        return res;
    }

    private int getState(String optname) {
        Field state = (Field) (((A_Element) ((DefaultComboBoxModel) getOptions().getValuefor(optname)).getSelectedItem()).getcontent());
        try {
            return state.getInt(state);
        } catch (IllegalArgumentException | IllegalAccessException ex) {
        }
        return -1;
    }

    private Field getStateField(String optname) {
        try {
            Field state = (Field) ((A_Element) ((DefaultComboBoxModel) getOptions().getValuefor(optname)).getSelectedItem()).getcontent();
            return state;
        } catch (Exception ex) {
        }
        return null;
    }

    private void setState(String optname, int nb) {
        DefaultComboBoxModel<A_Element> cbm = getOptions().getValuefor(optname, new DefaultComboBoxModel<A_Element>());
        try {
            for (int i = 0; i < cbm.getSize(); i++) {
                Field f = (Field) cbm.getElementAt(i).getcontent();
                if (f.getInt(f) == nb) {
                    cbm.setSelectedItem(cbm.getElementAt(i));
                    return;
                }
            }
        } catch (IllegalArgumentException | IllegalAccessException ex) {
        }
    }

    private void setState(String optname, A_Element elt) {
        DefaultComboBoxModel<A_Element> cbm = getOptions().getValuefor(optname, new DefaultComboBoxModel<A_Element>());
        if (elt == null) {
            return;
        }
        Object target = elt.getcontent();
        for (int i = 0; i < cbm.getSize(); i++) {
            Object ob = cbm.getElementAt(i).getcontent();
            if (target.equals(ob)) {
                cbm.setSelectedItem(cbm.getElementAt(i));
                break;
            }
            /*      if (cbm.getIndexOf(fld) >= 0) {
             cbm.setSelectedItem(fld);
             }*/

        }
    }
    // le status => action=> etat=> action
    // ici action=> etat!s! => action!s!

    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> liste = new ArrayList<>();
        byte[] pack = packet.getBytes();
        if ((Boolean) getOptions().getValuefor("telnet_isrefused")) {
            return null;
        }
        packet.setObj(null);
        int state = getState("telnet_rbinary");
        if (state == ST_APPLY) {
            /* ----------------------------------------------
             * is it MS bug, ?? send: ff fd 18 (termina l type? after binary...)
             * 
             */
            if (pack != null && pack.length == 3 && pack[0] == -1 && pack[2] == 24) {
                getStack().reply_out(this, new C_Packet(new byte[]{-1, -4, 24}, getport()));
            }
            //------------------------------------------------
            packet.setObj(pack);
        } else {
            while (pack != null && pack.length > 0) {
                if (pack[0] == -1 && pack.length > 2) {
                    byte[] res;
                    if (pack[1] == OPT_SUBBEG) {
                        int pos = 1;
                        while ((pack[pos] == -1 && pack[pos + 1] == -16) && pos + 1 < pack.length) {
                            pos++;
                        }
                        res = java.util.Arrays.copyOfRange(pack, 1, pos - 1);
                        // managesubOptions(res);
                        toLog(this, "subOption /" + res.length, true);
                        pack = java.util.Arrays.copyOfRange(pack, pos + 1, pack.length);
                    } else {
                        res = java.util.Arrays.copyOfRange(pack, 1, 3);
                        manageOptions(res, true);
                        pack = java.util.Arrays.copyOfRange(pack, 3, pack.length);
                    }
                } else {
                    state = getState("telnet_recho");
                    if (state == ST_APPLY) {
                        getStack().reply_out(this, new C_Packet(pack, getport()));
                    }
                    packet.setObj(pack);
                    pack = null;
                }
            }
        }
        if (packet.getObj() != null) {
            liste.add(packet);
        }
        return liste;
    }

    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        List<C_Packet> liste = new ArrayList<>();
        byte[] pack = packet.getBytes();
        if ((Boolean) getOptions().getValuefor("telnet_isrefused")) {
            return null;
        }
        int state = getState("telnet_lbinary");
        if (state == ST_APPLY) {
            //  no changge
        } else {
            //need to remove non printable char (!CRLF), manage "effacement"...
            state = getState("telnet_lsgoahead");
            if (state == ST_APPLY) {
                String txt = new String(pack);
                //txt.replaceAll("\r\n", "\r");
                packet.setObj(txt.getBytes());
            }
            state = getState("telnet_lecho");
            if (state == ST_APPLY) {
                getStack().reply_in(this, new C_Packet(pack, getport()));
            }
        }
        liste.add(packet);
        return liste;
    }

    private byte getOptionValue(String txt) {
        for (int i = 0; i < optlist.length; i++) {
            if (optlist[i].substring(1).compareTo(txt) == 0) {
                return vallist[i];
            }
        }
        return -1;
    }

    private byte getAction(Object txt) {
        if (txt == null) {
            return -1;
        }
        for (Field f : cbop) {
            if (f.getName().compareTo(txt.toString()) == 0) {
                try {
                    return f.getByte(f);
                } catch (IllegalArgumentException | IllegalAccessException ex) {
                }
            }
        }
        return -1;
    }

    private String getValueasString(int value, String txt) {
        if (txt.startsWith("state")) {
            if (value == ST_NOAPPLY) {
                return "ST_NOAPLY";
            } else if (value == ST_MANDNOT) {
                return "ST_MANDNOT ";
            } else if (value == ST_MAND) {
                return "ST_MAND";
            } else if (value == ST_COULD) {
                return "ST_COULD";
            } else if (value == ST_APPLY) {
                return "ST_APPLY";
            }
        } else if (txt.startsWith("action")) {
            if (value == OPT_WILL) {
                return "OPT_WILL";
            } else if (value == OPT_WONT) {
                return "OPT_WONT";
            } else if (value == OPT_DO) {
                return "OPT_DO";
            } else if (value == OPT_DONT) {
                return "OPT_DONT";
            }
        } else if (txt.startsWith("option")) {
            for (int i = 0; i < vallist.length; i++) {
                if (vallist[i] == value) {
                    return optlist[i].substring(1);
                }
            }
        }
        return null;
    }

    @Override
    public void setParents(I_Port idx, com.jdcm.eltrx.C_Stack modmangt) {
        super.setParents(idx, modmangt);
        getOptions().setValuefor("telnet_banner", ("This is the Telnet banner of"
                + getStack().getCMDParent().toString()
                + "\r\n--------------------------------------\r\n").getBytes());
    }

    @Override
    public void StkArcv(String message, String type) {
        if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            init_options();
        }
        if (message.compareTo("connected") == 0 && type.compareTo("Status") == 0) {
            if ((Boolean) getOptions().getValuefor("telnet_issrv")) {
                byte[] banner = (byte[]) getOptions().getValuefor("telnet_banner");
                getStack().reply_out(this, new C_Packet(banner, getport()));
                toLog(this, "init : send banner", false);

                for (int i = 0; i < getOptions().getRowCount(); i++) {
                    String title = getOptions().getValueAt(i, 0).toString();
                    if (title.startsWith("telnet_l") || title.startsWith("telnet_r")) {
                        String option = title.substring("telnet_r".length());
                        A_Element state = (A_Element) ((DefaultComboBoxModel) getOptions().getValuefor(title)).getSelectedItem();
                        if (state != null) {
                            java.util.ArrayList<C_Actiondef> list = null;
                            A_Events eventmx = (A_Events) getOptions().getValuefor("telnet_event");
                            for (A_Element elt : eventmx.getStatuslist()) {
                                if (elt.getcontent() instanceof Field) {
                                    if (elt.getcontent().equals(state.getcontent())) {
                                        eventmx.setStatus(null, elt);
                                    }
                                }
                            }

                            if (title.startsWith("telnet_r")) {
                                list = eventmx.getTask(new com.jdcm.common.C_Eventdef("telnet", "receive", "START_R"));
                            } else if (title.startsWith("telnet_l")) {
                                list = eventmx.getTask(new com.jdcm.common.C_Eventdef("telnet", "receive", "START_L"));
                            }
                            setState(title, eventmx.getActivStatus());
                            if (list != null && list.isEmpty() == false) {
                                byte res = getAction(list.get(0).getParam());
                                if (res != -1) {
                                    getStack().reply_out(this, new C_Packet(new byte[]{-1, res, getOptionValue(option)}, getport()));
                                    toLog(this, "init : " + getValueasString((int) res, "action") + " " + title, false);
                                }
                            }
                        }
                    }
                }

            }
        }
    }

    @Override
    public void onClock() {
    }

    public void settelnet_lecho(Field f) {
        //  System.out.println("set telnet lecho as:"+f);
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        // only check ff fb 0x at the begining
        double res = 0.0;
        if (pack.isbuffer()) {
            byte[] buff = pack.getBytes();
            if (buff.length > 2 && buff[0] == -1 && (buff[1] < -1 && buff[1] > -7)) {
                // broadcats Ethernet addr
                if (buff.length > 4 && java.util.Arrays.equals(new byte[]{buff[0], buff[1], buff[2], buff[3]}, new byte[]{-1, -1, -1, -1})) {
                    return 0.0;
                }
                return 1.0;
            }
        }
        return res;
    }
}
