package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;

/**
 * Provide ans UDP protocol (care IP pseudo session)
 *
 * http://www.frameip.com/entete-udp/ UDP is NOT ISO, pseudo IP is null if IP is
 * not present : getpseudoIP()
 *
 * @author Marc GIBOIN
 */
public class Stack_UDP extends Stack_baseIP {

    public Stack_UDP() {
        super();
        getOptions().addOption("udp_copy", false, "boolean", "", true, true, "copy receied data on this stack");
        getOptions().addOption("udp_chkdest", false, "boolean", "", true, true, "check received dest UDP port");
        getOptions().addOption("udp_chksrc", false, "boolean", "", true, true, "check received source UDP port");
        getOptions().addOption("udp_src", 1000, "integer", "", true, true, "source UDP port");
        getOptions().addOption("udp_dest", 1067, "integer", "", true, true, "destination UDP port");
    }

    public static String getComment() {
        return "User Datagram protocol";
    }

    public static double getosi() {
        return 4.0;
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param  packet
     * @return the payloads
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> liste = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        if (pack == null || pack.length < 8) {
            throw new Exception("no enouth byte for UDP header");
        }
        int proto = -1;
        try {
            proto = (Integer) getStack().getinfos(getport(), "IP", "ip_sproto", SEARCH_DOWN, this);
        } catch (Exception ex) {
            // no IP at all!!!
        }
        if (proto >= 0 && proto != 0x11) {
            //** protocol is set but bad 
            //liste.add(packet);
            return liste;
        } else if (pack.length > 8) {
            // received packet is UDP
            byte[] res = java.util.Arrays.copyOfRange(pack, 0, 2);
            // check dest of tho packet is the source of the module (Reception)
            int udpsrc = (int) array2dec(res);
            if (udpsrc == getOptions().getValuefor("udp_dest", 1) || getOptions().getValuefor("udp_chkdest", true) == false) {
                res = java.util.Arrays.copyOfRange(pack, 2, 4);
                int udpdst = (int) array2dec(res);
                if (udpdst == getOptions().getValuefor("udp_src", 1) || getOptions().getValuefor("udp_chksrc", true) == false) {
                    res = java.util.Arrays.copyOfRange(pack, 4, 6);
                    int len = Integer.parseInt(TUtils.retire("DEC", res));
                    if (len <= pack.length) {
                        res = java.util.Arrays.copyOfRange(pack, 6, 8);
                        int crc = Integer.parseInt(TUtils.retire("DEC", res));
                        //if (crc==util.ip_sum_calc()  !!! care incomming pseudo IP
                        if (getOptions().getValuefor("udp_copy", true)) {
                            getOptions().setValuefor("udp_dest", udpdst);
                            getOptions().setValuefor("udp_src", udpsrc);
                        }
                        packet.setObj(java.util.Arrays.copyOfRange(pack, 8, len));//packet.length
                        liste.add(packet);
                        return liste;
                    }
                }
            }
        }
        return null;
    }

    /**
     * encpsule payload in header to form packet
     *
     * @param packet the payload
     * @return the packets
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        try {
            getStack().setinfos(getport(), "IP", SEARCH_DOWN, this, "ip_sproto", 0x11);
        } catch (Exception ex) {
            // no IPV4 to update..
        }
        byte[] udpsrc = TUtils.prend("DEC", (Integer) getOptions().getValuefor("udp_src"));
        if (udpsrc.length < 2) {
            udpsrc = TUtils.addtab(new byte[]{0x00}, udpsrc);
        }
        byte[] udpdst = TUtils.prend("DEC", (Integer) getOptions().getValuefor("udp_dest"));
        if (udpdst.length < 2) {
            udpdst = TUtils.addtab(new byte[]{0x00}, udpdst);
        }
        byte[] res = udpsrc;
        res = TUtils.addtab(res, udpdst);
        byte[] l = TUtils.prend("DEC", pack.length + 8);
        if (l.length < 2) {
            l = TUtils.addtab(new byte[]{0x00}, l);
        }
        res = TUtils.addtab(res, l);
        //!! ++addresses IP / taille IP / proto IP
        byte[] crcres = TUtils.addtab(getpseudoIP(pack.length + 8), res);
        crcres = TUtils.addtab(crcres, pack);
        int crc = ip_sum_calc(crcres);
        byte[] crcbuff = TUtils.prend("DEC", crc);
        if (crcbuff.length < 2) {
            crcbuff = TUtils.addtab(new byte[]{0x00}, l);
        }
        res = TUtils.addtab(res, crcbuff);
        res = TUtils.addtab(res, pack);
        packet.setObj(res);
        java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
        finalres.add(packet);
        return finalres;
    }

    /**
     * Apply Options() modification to store receved data and disable checks
     *
     * @param bool
     */
    @Override
    public void setCopyMode(boolean bool) {
        getOptions().setValuefor("udp_copy", bool);
        getOptions().setValuefor("udp_chkdest", !bool);
        getOptions().setValuefor("udp_chksrc", !bool);
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        double res = 0.0;
        // always asked by IP
        if (pack.isbuffer()) {
            if (pack.getBytes().length >= 8) {
                byte[] frame = pack.getBytes();
                if (pack.getProtocols() != null && pack.getProtocols().isEmpty() == false) {
                    Object obj=pack.getProtocols().get(pack.getProtocols().size() - 1);
                    if ( obj instanceof Stack_IP) {
                        Stack_IP elt = (Stack_IP)obj;
                        if (elt.getOptions().getValuefor("ip_sproto", 1) == 17) { //0x11=>UDP
                           return 0.9999;
                        }
                    }else if (pack.getProtocols().get(pack.getProtocols().size() - 1) instanceof Stack_ICMP) {
                       // res = 0.6;
                    }
                } 
                if (frame.length == (int) array2dec(new byte[]{frame[4], frame[5]})) {
                    res = 0.7;
                }
            }
        }
        return res;
    }
}
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
