package com.jdcm.modifiers.stacks;

import com.jdcm.common.D_Utils;
import com.jdcm.common.C_Packet;
import com.jdcm.common.O_X25address;
import java.util.ArrayList;
import java.util.List;

/**
 * Provide pseudo-X25 basics 
 * 
 * used for RFC management missing RESET, real
 * x25 could need Layer 2 (include RR/RNR/REJ..) ?? could need a new stack
 * routing x25 (VP/VC managt)
 *
 *  @author Marc GIBOIN 
 */
public class Stack_X25 extends Stack_baseStart {

    private D_Utils util = new D_Utils();
    private List<byte[]> waitting = new java.util.ArrayList<>();

    public Stack_X25() {
         super();
        O_X25address src = null;
        try {
            src = new O_X25address("010000000");
        } catch (Exception ex) {
        }
        getOptions().addOption("x25_src", src, "adresse X25", "", true, true, "X25 source adresse");
        getOptions().addOption("x25_dest", new O_X25address(), "adresse X25", "", true, true, "X25 destination adresse");
        getOptions().addOption("x25_cud", new byte[0], "byte[]", "", true, false, "x25 user data");
        getOptions().addOption("x25_cudasdata", false, "bool", "", true, true, "cud are transmitted in data");
        getOptions().addOption("x25_facilities", "", "String", "", true, false, "x25 facilities String");
        getOptions().addOption("x25_nodata", "CALL_ASKED", "String", "", true, true, "define null data for call without data");
        getOptions().addOption("x25_status", "cleared", "String", "", true, true, "x25 facilities String");
    }

    public static String getComment() {
        return "provide a simple X25 layer (for RFC)";
    }

    public static double getosi() {
        return 3.0;
    }

    @Override
    public List<C_Packet> receive(C_Packet packet) throws Exception {
        List<C_Packet> res = new ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), true);
        // res.add(packet);
        String status = getOptions().getValuefor("x25_status").toString();
        if (status.startsWith("cleared")) {
            if ((new String(pack)).startsWith("X25CALL:")) {
                getStack().reply_out(this, new C_Packet("X25CALLCONF".getBytes(), getport()));
                byte[] calldata = java.util.Arrays.copyOfRange(pack, "X25CALL:".length(), pack.length);
                List<byte[]> args = getcalldatas(calldata);
                getOptions().setValuefor("x25_dest", new String(args.get(0)));
                getOptions().setValuefor("x25_src", new String(args.get(1)));
                getOptions().setValuefor("x25_facilities", new String(args.get(2)));
                getOptions().setValuefor("x25_cud", (new String(args.get(3))).getBytes());
                packet.setObj(args.get(3));
                getOptions().setValuefor("x25_status", "connected");
                res.add(packet);
                //res.add(new C_Packet(args.get(3), portidx));
            }
            // else trash
        } else if (status.startsWith("connecting")) {
            if ((new String(pack)).compareTo("X25CALLCONF") == 0) {
                getOptions().setValuefor("x25_status", "connected");
                res.add(packet);
                sendWaiting();
            }
            // else trash
        } else if (status.startsWith("connected")) {
            if ((new String(pack)).startsWith("X25DATA")) {
                byte[] data = java.util.Arrays.copyOfRange(pack, "X25DATA".length(), pack.length);
                packet.setObj(data);
                res.add(packet);
            }
            if ((new String(pack)).compareTo("X25CLEAR") == 0) {
                getStack().reply_out(this, new C_Packet("X25CLEARCONF".getBytes(), getport()));
                getOptions().setValuefor("x25_status", "cleared");
            }
            // if clear rcvd => cleared + send clear conf
        } else if (status.startsWith("clearing")) {
            if ((new String(pack)).compareTo("X25CLEARCONF") == 0) {
                getOptions().setValuefor("x25_status", "cleared");
                waitting.clear();
            }
            //clear conf=> sleep
        }
        return res;
    }

    @Override
    public List<C_Packet> transmit(C_Packet packet) throws Exception {
        // no need of upper Protocols
        List<C_Packet> res = new ArrayList<>();
        if (packet == null || packet.length() == 0) {
            return null;
        }
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        String status = getOptions().getValuefor("x25_status").toString();
        if (status.startsWith("cleared")) {
            toLog(this, "waiting for call", false);
            // ask call => ...data=> buffering
            byte[] call = "X25CALL:".getBytes();
            call = util.addtab(call, getaddresses());
            call = util.addtab(call, getfacilities());
            if ((Boolean) getOptions().getValuefor("x25_cudasdata")) {
                waitting.add((byte[]) getOptions().getValuefor("x25_cud"));
            } else {
                call = util.addtab(call, (byte[]) getOptions().getValuefor("x25_cud"));
            }
            packet.setObj(call);
            res.add(packet);
            // getStack().reply_out(this,new C_Packet(call, portidx));
            if ((new String(pack)).compareTo(getOptions().getValuefor("x25_nodata").toString()) != 0) {
                waitting.add(pack);
            }
            getOptions().setValuefor("x25_status", "connecting");
            // else trash
        } else if (status.startsWith("connecting")) {
            waitting.add(pack);
        } else if (status.startsWith("connected")) {
            sendWaiting();
            pack = util.addtab("X25DATA".getBytes(), pack);
            packet.setObj(pack);
            res.add(packet);
            // if I ask clear?? => clearing + send clear
        } else if (status.startsWith("clearing")) {
            // wait cleared
        }
        return res;

    }

    private void sendWaiting() {
        while (waitting.isEmpty() == false) {
            byte[] data = waitting.remove(0);
            data = util.addtab("X25DATA".getBytes(), data);
            getStack().reply_out(this, new C_Packet(data, getport()));
        }
    }

    @Override
    public boolean onStart() {
        try {
            /* if (portstate.compareTo("stopped") == 0 || portstate.compareTo("creating") == 0) {
             * getStack().getPort(portidx).CXstart(); }
             */
            List<C_Packet> res = transmit(new C_Packet(getOptions().getValuefor("x25_nodata").toString().getBytes(), getport()));
            for (C_Packet p : res) {
                getStack().reply_out(this, p);
            }
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public boolean onStop() {
        try {
            /* if (portstate.startsWith("stopp")==false && portstate.startsWith("clos")==false) {
             * getStack().getPort(portidx).CXstop();}
             */
            getStack().reply_out(this, new C_Packet("X25CLEAR".getBytes(), getport()));
            getOptions().setValuefor("x25_status", "cleared");
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    private byte[] getfacilities() {
        String fac = getOptions().getValuefor("x25_facilities").toString();
        if (fac == null || fac.length() == 0) {
            return new byte[]{0};
        } else if (fac.length() > 255) {
            // seems too log (length on a byte)
            return new byte[]{0};
        } else {
            byte[] res = new byte[1];
            res[0] = (byte) fac.length();
            res = util.addtab(res, fac.getBytes());
            return res;
        }
    }

    private byte[] getaddresses() {
        String src = getOptions().getValuefor("x25_src").toString();
        String dst = getOptions().getValuefor("x25_dest").toString();
        if (src == null || src.length() == 0 || dst == null || dst.length() == 0) {
            return new byte[]{0};
        } else if (dst.length() > 15 || src.length() > 15) {
            return new byte[]{0};
        } else {
            byte[] res = new byte[1];
            res[0] = (byte) (dst.length() + 16 * src.length()); // dst lenght/4 bit & src lenght/4 bit
            res = util.addtab(res, dst.getBytes());
            res = util.addtab(res, src.getBytes());
            return res;
        }
    }

    private java.util.List<byte[]> getcalldatas(byte[] buff) {
        java.util.List<byte[]> res = new java.util.ArrayList<>();
        int lsrc = buff[0] / 16;
        int ldst = buff[0] - lsrc;
        int lfac = buff[lsrc + ldst];
        byte[] dst = java.util.Arrays.copyOfRange(buff, 1, ldst);
        byte[] src = java.util.Arrays.copyOfRange(buff, ldst, 1 + ldst + lsrc);
        byte[] fac = java.util.Arrays.copyOfRange(buff, 1 + ldst + lsrc + 1, 2 + lfac);
        byte[] cud = java.util.Arrays.copyOfRange(buff, 2 + ldst + lsrc + lfac, buff.length);
        res.add(dst);
        res.add(src);
        res.add(fac);
        res.add(cud);
        return res;
    }

    @Override
    public void StkArcv(String message, String type) {
        String status = getOptions().getValuefor("x25_status").toString();
        if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            if (status.startsWith("cleared") == false) {
                setStack_isStart(false);
            }
        }
        if (message.compareTo("connected") == 0 && type.compareTo("Status") == 0) {
            // care if previous layer stack in port...
            //   setx25_start("start");
        }
    }
}
