package com.jdcm.modifiers.stacks;

import com.jdcm.common.C_Packet;

/**
 * Get a basic CBCOM Link
 *
 * @author M.Marc GIBOIN
 */
public class Stack_ZCBCOM extends Stack_base {

    public Stack_ZCBCOM() {
        super();
         setName("CBCOM");
    }

    public static String getComment() {
        return "CBCOM protocol is a session protocol for french payment systems";
    }

    public static double getosi() {
        return 5.0;
    }

    /**
     * decapsule packet, check header and provide payload NOT IMPLMENTED
     * (TRANSPARENT)
     *
     * @param  packet the
     * @return the payloads
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        if (packet.isbuffer() == false) {
            return null;
        }
        toLog(this, new String(pack), true);
        java.util.Map<Byte, byte[]> m0 = getTLVFields(packet.getBytes());
        for (java.util.Map.Entry<Byte, byte[]> ent : m0.entrySet()) {
            int hdrsz = 2 + ent.getValue().length;
            if (ent.getKey() == -60 || ent.getKey() == -61 || ent.getKey() == -62||ent.getKey() == -55) {//c1...c4 & c9
                if (hdrsz > packet.getBytes().length) {
                    packet.setObj(java.util.Arrays.copyOfRange(packet.getBytes(), hdrsz, packet.getBytes().length));
                } else {
                    packet.setObj(null);
                }
            }
            if (m0.containsKey((byte) -63)) {
                java.util.Map<Byte, byte[]> m1 = getTLVFields(m0.get((byte) -63));
                byte tagsz = 7;
                if (m1.containsKey(tagsz)) {
                    int size = (int) array2dec(m1.get(tagsz));
                   // int hdrsz2 = 2 + m0.get((byte) -63)[0];
                    packet.setObj(java.util.Arrays.copyOfRange(packet.getBytes(), hdrsz, hdrsz + size));
                }else{
                    packet.setObj(java.util.Arrays.copyOfRange(packet.getBytes(), hdrsz, packet.getBytes().length));
                }
            }
        }
        res.add(packet);
        return res;
    }

    /**
     * Encpsule payload in header to form packet NOT IMPLMENTED (TRANSPARENT)
     *
     * @param packet the payload
     * @return the packets
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        if (pack != null) {
            packet.setObj(pack);
        }
        finalres.add(packet);
        return finalres;
    }

    /**
     * retrieve a TLV with tag lenght=1 and size length=1
     *
     * @param buffer
     * @return
     */
    private java.util.Map<Byte, byte[]> getTLVFields(byte[] buffer) {
        java.util.Map<Byte, byte[]> res = new java.util.HashMap<>();
        byte[] sbuff = buffer;
        try {
            while (sbuff != null && sbuff.length > 2) {
                if (sbuff[1] > sbuff.length - 2) {
                    return null;
                }

                byte[] field = java.util.Arrays.copyOfRange(sbuff, 2, 2 + sbuff[1]);
                byte tag = sbuff[0];
                if (tag <= -60 && tag >= -63) {
                    //only one field in first level (C1/C2/C3/C4)
                    sbuff = null;
                } else {
                    sbuff = java.util.Arrays.copyOfRange(sbuff, sbuff[1] + 2, sbuff.length);
                }
                res.put(tag, field);
                // res.putAll(getCSVFields(field));
            }
        } catch (Exception ex) {
            return null;
        }
        if (res.isEmpty()) {
            return res;
        }
        return res;
    }

    @Override
    public double matchProtocol(C_Packet pack) {
        if (pack.isbuffer()) {
            if (pack.getBytes().length > 4) {
                byte[] frame = pack.getBytes();
                if (frame[0] <= -60 && frame[0] >= -63) {
                    java.util.Map<Byte, byte[]> m0 = getTLVFields(frame);
                    if (m0 != null && m0.isEmpty() == false) {
                        if (m0.size() == 1) {
                            java.util.Map<Byte, byte[]> m1 = getTLVFields((byte[])(m0.values().toArray()[0]));
                            if (m1 != null && m1.isEmpty() == false) {
                            if (m0.containsKey((byte) -60) || m0.containsKey((byte) -61) 
                                    || m0.containsKey((byte) -62)||m0.containsKey((byte) -63)||m0.containsKey((byte) -55)) {
                                if (m1.containsKey((byte)5)||m1.containsKey((byte)6)||m1.containsKey((byte)1)) {
                                    return 0.99;
                                }else if (m0.containsKey((byte) -63) &&m1.containsKey((byte)7)) { 
                                            int size = (int) array2dec(m1.get((byte)7));
                                            if (2 + m0.get((byte) -63).length + size == frame.length) {
                                               // CHPN always ends with 0x978 (EURO)
                                               if ( pack.getBytes()[ pack.getBytes().length-2]==0x09
                                                       && pack.getBytes()[ pack.getBytes().length-1]==0x78 ){
                                                   return 0.999;
                                               }
                                               return 0.99;
                                            }
                                    }
                            }
                            }
                        }
                    }
                }

            }
        }
        return 0.0;
    }
}
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
