package com.jdcm.modifiers.stacks;

import com.jdcm.common.Options_Model;
import com.jdcm.eltrx.C_Stack;
import com.jdcm.eltrx.I_Port;
import com.jdcm.common.C_Packet;
import com.jdcm.common.XMLres;
import com.jdcm.modifiers.modif_base;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

/**
 * Abstract Basic implementation of a stack protocol
 *
 * @author Marc GIBOIN
 */
public abstract class Stack_base extends modif_base implements I_stack_elt {

    private final Options_Model options = new Options_Model(this);
    
        private C_Stack Parent = null;
    private double level = -1.0;
    private I_Port portidx = null;
    private String stackName = "Dumy";
    public static boolean SEARCH_UP = true;
    public static boolean SEARCH_DOWN = false;

    public Stack_base() {
        getOptions().addOption("_FromMedia", true, "bool", "true/false", true, true, "received is applied on packet comming from media ");
        getOptions().addOption("_isDebug", false, "bool", "true/false", true, true, "Ask to print debug logs for it");
        int pos = getClass().getSimpleName().indexOf("_");
        if (pos > 0 && pos < getClass().getSimpleName().length() - 1) {
            setName(getClass().getSimpleName().substring(pos + 1));
        }
    }

    public static String getComment() {
        return "Empty basic stack";
    }

    public void set_label(String txt) {
        options.setTitle(txt);
    }

    @Override
    public double getlevel() {
        return level;
    }

    public String getStackSignature() {
        // get "tcp_"
        String baseName = stackName.toLowerCase() + "_";
        if (baseName.compareTo("ethii_") == 0) {
           // exception between options() and stackName
            baseName = "eth2_";
        }
        String sgn = "";
        DefaultMutableTreeNode rt = getProtoTree();
        if (rt != null && rt.getUserObject() instanceof XMLres) {
            XMLres x = (XMLres) rt.getUserObject();
            if (x.getAttrVal("convSignature") != null) {
                sgn = x.getAttrVal("convSignature").toString();
            }
        }
        if (sgn != null && sgn.length() > 0) {
            //  tcp=> convSignature="'src','dst'"
            String[] flds = sgn.split(",");
            for (int i = 0; i < flds.length; i++) {

                flds[i] = baseName + flds[i].replaceAll("'", "");
            }
            for (String fld : flds) {
                if (getOptions().isPresent(fld)) {
                    Object val = getOptions().getValuefor(fld);
                    if (val != null) {
                        sgn = sgn.replaceAll("'" + fld.substring(baseName.length()) + "'", val.toString());
                    }
                }
            }
        }
        return sgn;
    }

    private DefaultMutableTreeNode getProtoTree() {
        DefaultTreeModel tr = null;
        String className=this.getClass().getSimpleName();
        if (className.startsWith("Stack_")){
            className=className.substring("Stack_".length());
        }
        if (className != null) {
            java.io.InputStream inStream = com.jdcm.common.D_Utils.getFileStream(className + ".xml", "Descriptors", null);
            if (inStream != null) {
                com.jdcm.common.XML2tree a = new com.jdcm.common.XML2tree();
                tr = a.XMLBase(inStream);
            }
        }
        javax.swing.tree.DefaultMutableTreeNode rt = null;
        if (tr != null && tr.getRoot() instanceof javax.swing.tree.DefaultMutableTreeNode) {
            javax.swing.tree.DefaultMutableTreeNode hdr = (javax.swing.tree.DefaultMutableTreeNode) tr.getRoot();

            java.util.LinkedList l = new java.util.LinkedList<>(java.util.Arrays.asList(new String[]{"pdml", "Analyse", "protocol"}));
            while (l.isEmpty() == false) {
                if (hdr != null && hdr.getChildCount() > 0) {
                    int j = 0;
                    while (j < hdr.getChildCount() && (rt == null || l.contains(rt.toString()) == false)) {
                        rt = (javax.swing.tree.DefaultMutableTreeNode) hdr.getChildAt(j++);
                    }
                    hdr = rt;
                    l.removeFirst();
                }
            }
        }
        return rt;
    }

    public java.awt.Color getStackColor() {
        java.awt.Color res = java.awt.Color.white;
        DefaultMutableTreeNode rt = getProtoTree();
        if (rt != null && rt.getUserObject() instanceof XMLres) {
            XMLres x = (XMLres) rt.getUserObject();
            res = (java.awt.Color) x.getAttrVal("color");
            if (x.getEltName().compareTo("protocol") == 0 && res != null) {
                return res;
            }
        }

        return res;
    }

    @Override
    public boolean isfrommedia() {
        return getOptions().getValuefor("_FromMedia", true);
    }

    @Override
    public void setfrommedia(boolean bool) {
        getOptions().setValuefor("_FromMedia", bool);
    }

    @Override
    public boolean isdebug() {
        return getOptions().getValuefor("_isDebug", true);
    }

    @Override
    public void setdebug(boolean bool) {
        getOptions().setValuefor("_isDebug", bool);
    }

    @Override
    public void setlevel(double d) {
        level = d;
    }

    @Override
    public double getOsi() {
        // because static is not accessible from stacks
        return Stack_base.getosi();
    }
    //  @Override

    public static double getosi() {
        return 8.0;
    }

    @Override
    public final void setName(String txt) {
        stackName = txt;
        getOptions().setTitle(txt);
    }

    @Override
    public String toString() {
        return stackName;
    }

    @Override
    public I_Port getport() {
        return portidx;
    }

    public C_Stack getStack() {
        return Parent;
    }

    public void toLog(I_stack_elt ob, String txt, boolean dir) {
        if (Parent != null) {
            Parent.dbgwri(ob, txt, dir);
        }
    }

    @Override
    public void setParents(I_Port idx, C_Stack modmangt) {
        Parent = modmangt;
        portidx = idx;
    }

    @Override
    public final Options_Model getOptions() {
        return options;
    }

    @Override
    public void StkArcv(String message, String type) {
    }

    @Override
    public void onClock() {
    }

    public long array2dec(byte[] buff) {
        long res = 0;
        int temp;
        if (buff != null && buff.length > 0) {
            for (int i = 0; i < buff.length; i++) {
                temp = (buff[i] >= 0) ? buff[i] : 256 + buff[i];
                res += temp * (long) Math.pow(256.0, buff.length - i - 1);
            }
        }
        if (res < 0) {
            toLog(this, "TCP warning : dec value is negatif", true);
        }
        return res;
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param  packet
     * @return the payload
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        Parent.dbgwri(this, new String(pack), true);
        if (pack != null) {
            packet.setObj(pack);
        }
        res.add(packet);
        return res;
    }

    /**
     * encpsule payload in header to form packet
     *
     * @param packet
     * @return the packet
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        Parent.dbgwri(this, new String(pack), false);
        if (pack != null) {
            packet.setObj(pack);
        }
        finalres.add(packet);
        return finalres;
    }

    @Override
    public void setCopyMode(boolean bool) {
    }

    @Override
    public double matchProtocol(C_Packet list) {
        return 0.0;
    }
}
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
