package com.jdcm.modifiers.stacks;

/**
 * Abstract Basic IP implementation of a stack protocol (implment Pseudo IP)
 *
 * @author Marc GIBOIN
 *
 */
public class Stack_baseIP extends Stack_base {

    com.jdcm.common.D_Utils TUtils = new com.jdcm.common.D_Utils();

    public Stack_baseIP() {
        super();
    }

    public static String getComment() {
        return "Tookkit for IP";
    }

    /**
     * provide a pseudo session ip for checksum tcp/udp/icmpv6 processing
     *
     * @param udpsize the length of the tcp/upd/icmpv6 packet
     * @return
     */
    public byte[] getpseudoIP(int udpsize) {
        if (getStack() == null) {
            return null;
        }
        try {
            byte[] res = ((java.net.InetAddress) getStack().getinfos(getport(), "IP", "ip_src", SEARCH_DOWN, this)).getAddress();
            res = TUtils.addtab(res, ((java.net.InetAddress) getStack().getinfos(getport(), "IP", "ip_dest", SEARCH_DOWN, this)).getAddress());
            res = TUtils.addtab(res, new byte[]{0x00});//MB
            int sproto = (Integer) getStack().getinfos(getport(), "IP", "ip_sproto", SEARCH_DOWN, this);
            res = TUtils.addtab(res, new byte[]{(byte) sproto});
            byte[] l = TUtils.prend("DEC", udpsize);
            int lgsz = 2;
            if (sproto == 6) {
                lgsz = 4;
            }
            while (l.length < lgsz) {
                l = TUtils.addtab(new byte[]{0x00}, l);
            }
            res = TUtils.addtab(res, l);
            return res;
        } catch (Exception ex) {
            return new byte[]{0, 0};
            // return null;
        }
    }

    /**
     * return the checksum of an IP frame buff used with pseudo IP in
     * TCP/UDP/ICMPv6 and like it in ICMPv4
     *
     * @param buff
     * @return
     */
    public int ip_sum_calc(byte[] buff) {
        int word16;
        long sum = 0;
        int i;
        //adjust len of buff if not pair
        if (buff.length / 2 != (double) buff.length / 2) {
            buff = TUtils.addtab(buff, new byte[]{0x00});
        }
        int len_ip_header = buff.length;
        // make 16 bit words out of every two adjacent 8 bit words in the packet
        // and add them up
        for (i = 0; i < len_ip_header; i = i + 2) {
            word16 = ((buff[i] << 8) & 0xFF00) + (buff[i + 1] & 0xFF);
            sum = sum + (long) word16;
        }

        // take only 16 bits out of the 32 bit sum and add up the carries
        while (sum >> 16 > 0) {
            sum = (sum & 0xFFFF) + (sum >> 16);
        }
        // one's complement the resulta
        sum = 0xFFFF - sum;
        return ((int) sum);
    }

    /**
     * User for all java byte[-128-128] instead of [0-255]
     *
     * @param by
     * @return
     */
    public int getInt(byte by) {
        return (by >= 0) ? by : 256 + by;
    }
}
