package com.jdcm.modifiers.stacks;

import javax.swing.DefaultComboBoxModel;
import com.jdcm.common.C_Packet;
import com.jdcm.common.O_Amount;
import com.jdcm.eltrx.C_Port;
import com.jdcm.eltrx.C_Stack;
import com.jdcm.eltrx.I_Port;
import java.util.Date;

/**
 * Used as Cheque reader manager
 *
 * @author Marc GIBOIN
 */
public class Stack_zcheque extends Stack_baseStart {

    public Stack_zcheque() {
        super();
        String[] rep = new String[]{";:eject", "0:Annul. action", "1:absorbtion", "2:abs.+lect.",
            "7:impr. chq.", "9:impr.bor."};
        DefaultComboBoxModel<String> cmdes = new DefaultComboBoxModel<>(rep);
        cmdes.setSelectedItem("1:absorbtion");
        getOptions().addOption("chq_RSport", new DefaultComboBoxModel<>(new String[]{}), "PortCOM list", "", true, true, "used serial port");
        getOptions().addOption("chq_cmde", cmdes, "liste cmdes", "", true, true, "Cheque action commande");
        getOptions().addOption("chq_date", new Date(System.currentTimeMillis()), "Date", "", true, true, "Date pour impr.");
        getOptions().addOption("chq_number", "-", "String", "", true, true, "Cheque protocol");
        getOptions().addOption("chq_lieu", "LIEU", "String", "", true, true, "Lieu pour impr.[12-24]");
        getOptions().addOption("chq_benef", "BENEF", "String", "", true, true, "Benef pour impr. [25-32]");
        getOptions().addOption("chq_text", "texte", "String", "", true, true, "Benef pour impr.[<85]");
        getOptions().addOption("chq_montant", new O_Amount(12.34), "montant", "", true, true, "Montant pour impr.");
    }

    @Override
    public void setParents(I_Port idx, C_Stack modmangt) {
        super.setParents(idx, modmangt);
        DefaultComboBoxModel<Object> cb = ((C_Port) getport()).getCMDOptions().getValuefor("RS_interfaces", new DefaultComboBoxModel<>());
        if (cb != null && cb.getSize() >= 0) {
            getOptions().setValuefor("chq_RSport", cb);
        }
    }

    public void settrs_RSport(Object port) {
        DefaultComboBoxModel<Object> cb = getOptions().getValuefor("chq_RSport", new DefaultComboBoxModel<>());
        ((C_Port) getport()).setRS_Port(cb.getSelectedItem());
    }

    public static String getComment() {
        return "Manage ELC terminals";
    }

    public static double getosi() {
        return 8.0;
    }

    @Override
    public boolean onStart() {
        try {
            String portstate = getport().getcxstatus();
            if (portstate.compareTo("stopped") == 0 || portstate.compareTo("creating") == 0) {
                getport().CXstart();
            }
            getStack().setinfos(getport(), "C0C1", SEARCH_DOWN, this, "c0c1_maxrcvd", 1);
            getStack().setinfos(getport(), "C0C1", SEARCH_DOWN, this, "c0c1_maxsent", 1);
            DefaultComboBoxModel<String> c = getOptions().getValuefor("chq_cmde", new DefaultComboBoxModel<String>());
            String cmd = c.getSelectedItem().toString().substring(0, c.getSelectedItem().toString().indexOf(":"));
            C_Packet d = new C_Packet(cmd.getBytes(), getport());
            if (cmd.compareTo("7") == 0) {
                d.setObj(getPrintingFrame());
            }
            d.setdestination(getport());
            getStack().reply_out(this, d);
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    @Override
    public boolean onStop() {
        try {
            if (getport().getcxstatus().compareTo("stopped") != 0) {
                getport().CXstop();
            }
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    private String PadBenefLieu(String init, int min, int max) {
        if (init.length() > max) {
            init = init.substring(0, max - 1);
        } else if (init.length() < min) {
            byte[] b = new byte[min - init.length()];
            java.util.Arrays.fill(b, " ".getBytes()[0]);
            init += new String(b);
        }
        return init;
    }

    private byte[] getPrintingFrame() {
        String res = "7";
        java.text.NumberFormat form;
        O_Amount montant = (O_Amount) getOptions().getValuefor("chq_montant");
        form = new java.text.DecimalFormat("0000000000");
        String montantstr = "a" + form.format(montant.AMOUNT * 100);
        Date dt = (Date) getOptions().getValuefor("chq_date");
        java.text.DateFormat formd = new java.text.SimpleDateFormat("ddmmyy");
        String datestr = "b" + formd.format(dt);
        String li = getOptions().getValuefor("chq_lieu").toString();
        if (li != null && li.length() > 0) {
            li = "c" + PadBenefLieu(li, 12, 24);
        }
        String bf = getOptions().getValuefor("chq_benef").toString();
        if (bf != null && bf.length() > 0) {
            bf = "d" + PadBenefLieu(bf, 25, 32);
        }
        String tx = getOptions().getValuefor("chq_text").toString();
        if (tx != null && tx.length() > 0) {
            tx = "e" + PadBenefLieu(tx, 1, 85);
        }
        res = res + montantstr + datestr + li + bf + tx;
        return res.getBytes();

    }

    @Override
    public void StkArcv(String message, String type) {
        if (message.compareTo("connected") == 0 && type.compareTo("Status") == 0) {
            DefaultComboBoxModel<String> c = getOptions().getValuefor("chq_cmde", new DefaultComboBoxModel<String>());
            String cmd = c.getSelectedItem().toString().substring(0, c.getSelectedItem().toString().indexOf(":"));
            C_Packet d = new C_Packet(cmd.getBytes(), getport());
            d.setdestination(getport());
            if (cmd.compareTo("7") == 0) {
                d.setObj(getPrintingFrame());
            }
            getStack().reply_out(this, d);
        } else if (message.compareTo("stopped") == 0 && type.compareTo("Status") == 0) {
            setStack_isStart(false);
        }
    }

    /**
     * decapsule packet, check header and provide payload
     *
     * @param packet C_Packet to be received
     * @return the payload
     * @throws java.lang.Exception thrown on errors
     */
    @Override
    public java.util.List<C_Packet> receive(C_Packet packet) throws Exception {
        java.util.List<C_Packet> res = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        if (pack != null) {
            toLog(this, new String(pack), true);
        } else {
            toLog(this, "" + packet.getObj(), true);
        }
        if (pack != null) {
            packet.setObj(pack);
        }
        res.add(packet);
        return res;
    }

    /**
     * encpsule payload in header to form packet
     *
     * @param packet  C_Packet to be send
     * @return the list of C_Packets to send
     * @throws java.lang.Exception
     */
    @Override
    public java.util.List<C_Packet> transmit(C_Packet packet) throws Exception {
        java.util.List<C_Packet> finalres = new java.util.ArrayList<>();
        byte[] pack = packet.getBytes();
        toLog(this, new String(pack), false);
        if (pack != null) {
            packet.setObj(pack);
        }
        finalres.add(packet);
        return finalres;
    }
}
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
